package Modules::Catalog::Tree;
#################################################################################
# Tree: a data structure used for building category trees in Catalog module
#
# Usage:
# 1. Create a tree node
# 2. Populate data record
# 3. Set parent
# 4. Set children
#
# e.g.
# var $x = new Modules::Catalog::Tree();
# $x->data($data);  # pass in category record
# $x->prev($parent); # set parent node
# $x->AddChild($child); # add child nodes.
#
# -------------------------------------------------------------------------------
#
# To get all child nodes in an array reference:
#	$x->next();
#
# To get the primary key value from a category record(need to set a 
# category record in a node using $x->data($data) first):
#	$x->id();
#
# To get the number of child nodes:
# 	$x->ChildCount();
#
# To get the table rowspan value:
#	$x->RowSpan();
#
# To set the table rowspan value:
#	$x->RowSpan(<rowspan value>);
#
# To get the title of a category:
#	$x->Title();
#
# Add a child node:
#	$x->AddChild(<Tree object>);
#
# Set a category record:
#	$x->data(<category data>);
#
#################################################################################
use strict;
use Class::Struct;

use vars qw(@ISA);

struct("Modules::Catalog::Tree"=>
	{
		next=>"*@", # next nodes
		prev=>"\$", # parent node
		depth=>"\$", # depth
		HasProduct=>"\$",
		data=>"\$",
		productlist_html=>"\$",
		RowSpan=>"\$",
		count_flag=>"\$",
	}
);
sub SetValue{
	my ($this,$key,$value) = @_;
	if (!$value){
		$this->{$key} = $this->{$key} || 0;
	}else{
		$this->{$key} = $value;
	}
	return $this->{$key};
}
# Return/set child nodes
sub next{
	my ($this,$value) = @_;
	if (!$value){
		$this->{next} = $this->{next} || [];
	}else{
		$this->{next} = $value;
	}
	return $this->{next};
}
# Return/set the parent node
sub prev{
	my ($this,$value) = @_;
	return $this->SetValue("prev",$value);
}
# Return/set the primary key value
sub id{
	my ($this) = @_;
	return $this->data->{catalog_category_id};
}
# Flag the node as counted(used for counting rowspan)
sub count_flag{
	my ($this,$value) = @_;
	return $this->SetValue("count_flag",$value);
}
# Return the number of child nodes
sub ChildCount{
	my ($this) = @_;
	return @{$this->next()};
}
# Return/set rowspan value
sub RowSpan{
	my ($this,$value) = @_;
	return $this->SetValue("RowSpan",$value);
}
# Return the category title
sub Title{
	my ($this) = @_;
	return $this->data->{title};
}
# Add a child node
sub AddChild{
	my ($this,$node) = @_;
	return if !$node;
	# {next} is an array reference
	push(@{$this->{next}},$node);
	return;
}
#=========================================================================
# Figure out table span
sub FigureOutTableSpan{
	my ($this,$node) = @_;
	return if !$node;
	# We count the rowspan from the last child node to the root node
	if (!$node->ChildCount()){
		# We've reached the last child node.  
		my $rows = 1;
		my $prev = $node->prev();
		# Start counting the rowspan
		$this->CalculateTableSpan($node->prev(),$rows);
	}else{
		# We aren't at the last child node yet.
		foreach my $nod(@{$node->next()}){
			$this->FigureOutTableSpan($nod);
		}
	}
	return;
}
# Calculate table span
sub CalculateTableSpan{
	my ($this,$node,$rowspan) = @_;
	return if !$node;
	# Add 1 to the row span if the node has products and child categories
	if ($node->HasProduct() && !$node->count_flag() && $node->ChildCount()){		
		$rowspan += 1;
	}
	# Flag the node as counted. Avoid counting a node twice.
	$node->count_flag(1);
	# Save rowspan
	$node->RowSpan($node->RowSpan() + $rowspan);
	# We've reached the root node.  Stop counting
	return if !$node->prev();
	$this->CalculateTableSpan($node->prev(),$rowspan);
	return;
}
# Is this node a grand parent ?
sub IsGrandParent{
	my ($this,$node) = @_;
	# If no $node is specified, use the current object as the node
	$node = $node || $this;
	# No child nodes
	return 0 if ($node->ChildCount() < 1);
	foreach my $child (@{$node->next()}){
		# One of the child nodes has children.  This node is a grand parent.
		return 1 if $child->ChildCount() > 0;
	}
	return 0;
}
# Maximum depth of a tree
sub MaxDepth{
	my ($this,$node) = @_;
	# If no $node is specified, use the current object as the node
	$node = $node || $this;
	return 0 if !$node;
	my $depth = 1;
	foreach my $nod(@{$node->next()}){
		my $newdepth = 1 + $this->MaxDepth($nod);
		if ($depth < $newdepth){
			$depth = $newdepth;
		}
	}
	return $depth;
}
sub GetParentWithChildNode{
	my ($this,$node) = @_;
	$node = $node || $this;
	return (undef,undef) if !$node;
	return $this->GetParentWithChildNodeRecursive(undef,$node);
}
sub GetParentWithChildNodeRecursive{
	my ($this,$parent,$child) = @_;
	my $prev = $child->prev();
	return (undef,undef) if !$prev;
	if (!$prev->HasProduct()){
		return $prev->GetParentWithChildNodeRecursive(undef,$prev);
	}else{
		return ($prev,$child);
	}
}
sub product_data{
	my ($this,$data) = @_;
	if (!$data){
		return ref($this->{product_data}) =~ /array/i ? $this->{product_data} : [];
	}else{
		$this->{product_data} = $data;
		return $data;
	}
}
sub print_out{
	my ($this,$sep) = @_;
	my $go = "->";
	$sep = $sep || $go;
	print "$sep ".$this->data->{title}."<br>";
	if (@{$this->next()} > 0){
		foreach my $node(@{$this->next()}){
			$node->print_out($sep.$go);
		}
	}
	return;
}
1;
