package Modules::GoogleMaps;

#----------------------------------------------------------------------------
#   This plug-in uses HTML adapted from 
#   http://www.google.com/apis/maps/documentation/#The_Hello_World_of_Google_Maps
#   That HTML code is Copyright Google.
#-----------------------------------------------------------------------
#
#   Everything else is:
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#-----------------------------------------------------------------------
# GoogleMaps : tool to add Google Maps to a web page
#
# Important Note:
# If you use this googlemap plugin on a page where other .onload events occur,
# they must be combined into one function after the plugin is loaded. 
# DMenu is an example of a plugin that uses onload events.
# Example:
# <script type="text/javascript">
# window.onload=function() {
#  cbeInitialize("DIV", "SPAN"); 
#  if (window.windowOnload) window.windowOnload();
#  init(); // google maps onload event
# }
# </script>
#
# Options:
# key=KEY			Google-provided Google Maps key 
# lon=LON			Longitude for the map center (in degrees)
# lat=LAT			Latitude for the map center (in degrees)
# markerLonN=LON		Longitude for a pin marker (in degrees)
# markerLatN=LAT		Latitude for a pin marker (in degrees)
# typeControl=true|false	Turn the "Map/Satellite/Hybrid" option on/off
#
# new options added Robert Long aug 2006
# also updated map creation code to Google Maps v2
# height=H			Set the height of the map in pixels
# width=W			Set the width of the map in pixels
# markerTextN=HTML		Set popup text for a pin marker
# 
# All of these options can be provided through the GoogleMaps.conf, 
# as an oyster option or through the query string
# Options precedence is as follows: 
# 1. Query string
# 2. Oyster options
# 3. Configuration file
# 
# A Google Maps key can be aquired at the following address:
# http://www.google.com/apis/maps/signup.html
#-----------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Misc;

# For new()
use base 	qw(Modules::BaseDCD);

# Write google maps div
sub write() {
	my $this = shift @_;
	my $paramList = shift @_;

	my %query = %{$this->{input}};
	my %param = &DecodeString($paramList);
	# Set up some defaults... 
	# QUERY takes precedence over other

	# Merge the two 3 hashes, making sure that the QUERY_STRING
	# takes precedence over local options and that local options take precedence
	# over config file
	if (exists $config{'GoogleMaps'}) {
		%param = (%{$config{'GoogleMaps'}}, %param, %query);
	}
	else
	{
		%param = (%param, %query);
	}

	my %options = (
		'zoom' => $param{'zoom'} || 15,
		'lat' => $param{'lat'} || 49.26,
		'lon' => $param{'lon'} || -123.12,
		'key' => $param{'key'},
		#height/width setter
		'height' => $param{'height'} || 400, 
		'width' => $param{'width'} || 500,
	);

	$options{'pan'} = $this->recenterOrPanToLatLng(\%param);
	$options{'additionalControls'} = $this->mapControls(\%param);
	$options{'marker'} = $this->markers(\%param);

	my $out = <<"DONE";
	<!-- Google Maps -->
		<script src="http://maps.google.com/maps?file=api&v=2&key=$options{key}" type="text/javascript"></script>
		<script type="text/javascript">
			//<![CDATA[
			function init() {
				if (GBrowserIsCompatible()) { 
					var map = new GMap2(document.getElementById("map"));
					map.addControl(new GSmallMapControl());
					$options{additionalControls}
	//				map.centerAndZoom(new GPoint($options{lon}, $options{lat}), $options{zoom});
					map.setCenter(new GLatLng($options{lat}, $options{lon}), $options{zoom});
$options{pan}
$options{marker}					
				}
			}
			//]]>
			window.onload = init;
			window.onunload = GUnload;
		</script>
		<div id="map" style="width: $options{width}; height: $options{height}"></div>
	<!-- Google Maps -->
DONE
	return $out;
}


sub ioctl($$) {
	my $this = shift @_;
	my $request = shift @_;

	if($request =~ /ModuleInfo/) {
		return <<"INFO"

INFO
	}
	return undef; 
}

# Match google Maps API functions here
sub recenterOrPanToLatLng() {
	my $this = shift @_;
	my $options = shift @_;
	
	my $panLon = $options->{'panLon'};
	my $panLat = $options->{'panLat'};
	return "" unless $panLon && $panLat;

	return <<"DONE";
window.setTimeout(function() {
	map.recenterOrPanToLatLng(new GPoint($panLon, $panLat));
}, 2000);
DONE
}

# Used for build-in map controls... maybe another method could
# be used for user-defined map controls?
# I'm assuming most controls are going to be requested by simple
# switches, which is why I think this single method makes sense
sub mapControls($) {
	my $this = shift @_;
	my $options = shift @_;

	my $out = "";
	if($options->{'typeControl'} =~ /true|1/) {
		$out .= "map.addControl(new GMapTypeControl());\n";
	}
	return $out;
}

sub markers($) {
	my $this = shift @_;
	my $options = shift @_ || {};

	my @markers = ();
	my @markerCoords = grep { /^marker/ } keys %{$options};

	return "" if scalar(@markerCoords) == 0;
	
	# Index marker data using a hash
	my %markers;
	# Get marker data from the $option hash
	for(my $i = 0; $i < @markerCoords; $i++) {
		# We are only interested in gathering longitude and latitude information
		next if $markerCoords[$i] !~ /marker(Lat|Lon)(\d*)/;
		$markerCoords[$i] =~ /marker(Lat|Lon)(\d*)/;
		my $markerid = $2;
		my $markertype = lc($1);
		$markers{$markerid}{$markertype} = $options->{$markerCoords[$i]};
		# Get marker popup text 
		my $markerText = $options->{"markerText$markerid"};
		if ($markerText){
			$markers{$markerid}{text} = $markerText;
		}
	}

	my $out = "";
	# Output html
	foreach my $id(keys(%markers)){
		my $marker = $markers{$id};
		my $marker_name = "marker${id}";
		# Insert marker onto the map
		$out .= <<END;
					var $marker_name = new GMarker(new GLatLng($marker->{lat}, $marker->{lon}));
					map.addOverlay($marker_name);
END
		# Add marker popup text
		if ($marker->{text}){
			$out .= <<END;
					GEvent.addListener($marker_name, "click", function() {$marker_name.openInfoWindowHtml("$marker->{text}");});	
END
		}

	}
	return $out;
}
1;
