#!/usr/bin/perl
package Modules::PhotoForum;

#==========================================================================
# PhotoForum : a tool to share and comment on photos provided by users
#
# Uses the Zine framework to handle indexing, archiving, publishing, etc.
# We simply override a few admin/access control functions to allow anyone
# to post to the photoblog.
#==========================================================================


use strict;
use ExSite::Config;
use ExSite::Report;
use ExSite::Misc;
use ExSite::Util;
use ExSite::Mail;
use ExSite::HTML;
use ExSite::Crypt;
use ExSite::Image;
use Modules::Zine;

use vars qw(@ISA %label);
@ISA = qw(Modules::Zine);

%label = (author=>"Posted by");

# new : called implicitly once per page when a DCM is invoked

sub new {
    my $this = shift;
    my $obj = {};
    my $class = ref($this) || $this;
    bless $obj, $class;

    # setup base object 
    $obj->initialize_object;

    # read the module configuration file
    $obj->read_conf;
    $obj->read_conf("Zine");

    # automatically read the default input source
    $obj->read;

    # set ourselves as the current module
    $share{current_module} = $this->module_name;

    return $obj;
}

sub ioctl {
    my ($this) = shift;
    $_ = shift;

    if (/ControlPanel/) {
	return \&Modules::Zine::ctrl_panel;
    }
    elsif (/isRestricted/) {
	return $config{PhotoForum}{isRestricted};
    }
    elsif (/isService/) {
	return $config{PhotoForum}{isService};
    }
    elsif (/Publish/) {
	return \&Modules::Zine::publish;
    }
}

# display a complete article

sub show {
    my $this = shift;
    my $out;

    # show the article

    if ($this->{article}{type} =~ /comment/) {
	$out .= $this->show_comment;
    }
    else {
	$out .= $this->show_article;
    }
    $out .= $this->tools($this->{article});

    my $footer;
    if ($this->{reference}) {
	my $refurl = $this->link(zine=>"show",aid=>$this->{reference}{article_id});
	$footer = "<p>[<a href=\"$refurl\">Back to \"".$this->reftitle."\"</a>]</p>\n";
    }

    # show the regular article index
    $out .= $this->index("article",(archive=>$this->{input}{archive}));

    # show the index of comments

    $out .= $this->index("comment",(archive=>$this->{input}{archive}));

    # show the shortcuts out of here
    $out .= $footer;

    return $out;
}

sub footer {
    my $this = shift;
    my $out;
    my $author = $this->author;
    if ($this->type =~ /article/) {
	$out = "\nPosted on ".$this->show_data("ctime")." by $author<br>\n";
    }
    $out .= $this->show_data("about");
    if ($out) {
	return "<address>$out</address>\n";
    }
    return undef;
}

sub allow_post {
    my ($this,$data) = @_;
    if ($data->{article_id} > 0) {
	# we appear to be editing an existing article
	if ($data->{type} eq "comment") {
	    if (! $share{DB}->is_admin) {
		if ($data->{owner} == $share{DB}->my_uid()) {
		    my $t = new ExSite::Time;
		    my $t1 = $t->write("unix_timestamp");
		    $t->set($data->{ctime},"sql_timestamp");
		    my $t2 = $t->write("unix_timestamp");
		    if ($t1 - $t2 > $config{Zine}{edit_my_comment_timeout}) {
			return "<p>This comment is now frozen.</p>\n";
		    }
		    # else, edit comment is allowed
		    return undef;
		}
		return "<p>You don't have permission to edit this comment.</p>\n";
	    }
	}
	if ($data->{type} eq "article" && ! $this->owns) {
	    # user does not own the Zine
	    return "<p>Sorry, you do not have permission to manage this article.</p>\n";
	}
	if ($data->{type} !~ /article|comment/ && ! $this->{admin}) {
	    # Zine management only from control panel
	    return "<p>Zines can only be managed from the administrator control panel.</p>\n";
	}
    }
    else {
	# we appear to be creating a new article
	if ($data->{type} eq "comment" && ! $this->allow_comments) {
	    # comments are disabled under this Zine
	    if ($config{Zine}{$this->reftype}{comment_policy}{member}) {
		return "<p>Sorry, only members can post comments here.</p>\n";
		# login link might be nice...
	    }
	    return "<p>Sorry, comments are not permitted.</p>\n";
	}
	if ($data->{type} eq "article") {
	    if ($config{PhotoForum}{access} =~ /public/) {
		return undef;
	    }
	    elsif ($config{PhotoForum}{access} =~ /members/ && $share{DB}->authorize) {
		return undef;
	    }
	    return "<p>You must be logged in to post pictures here.</p>\n";
	}
	if ($data->{type} !~ /article|comment/ && ! $this->{admin}) {
	    # Zine creation only from control panel	
	    return "<p>Sorry, you do not have permission to create new Zines.</p>\n";
	}
    }
    return undef;
}

sub index_list {
    my ($this,$mode,$itype,%opt) = @_;

    my $out;
    my $indexlimit = $opt{indexlimit} || $this->indexlimit($mode);
    my $indexage = $opt{indexage} || $this->indexage($mode);
    my $indexformat = $opt{archive} ? "list" : "photoindex";
#	($opt{indexformat} || $this->indexformat($mode));
    my $comment_policy = $opt{comment_policy} || $this->comment_policy($mode);
    my %match = ( parent_id=>$this->{article_id}, status=>"active" );
    if ($opt{type}) { $match{type} = $opt{type}; }

    my @art = $share{DB}->fetch_match("article",
				      \%match,
				      {"article_id"=>"desc"},
				      );

    my $archive;  # if set, means there are archived articles
    my $n = 0;
    if (@art > 0) {
	my $article_save = $this->{article};
	foreach my $art (@art) {
	    next if ($art->{type} eq "comment" and $mode eq "article");
	    next if ($art->{type} ne "comment" and $mode ne "article");
	    $this->set($art);
	    if ($indexage && ($this->age > $indexage) && !$opt{archive}) {
		$archive = 1;
		next;
	    }
	    if ($indexformat eq "photoindex") {
		my $thumb = $this->thumb || "<img src=\"$config{server}{HTMLpath}/_Modules/Zine/nophoto.gif\" border=\"0\">";
		my $summary = $this->summarize("teaser");
		my $url = $this->link(zine=>"show",aid=>$this->{article_id});
		$out .= "<tr><td width=\"100\" align=\"center\" class=\"ZineThumb\"><a href=\"$url\">$thumb</a></td><td>$summary</td></tr>\n";
	    }
	    else {  # simple list
		$out .= "<p>\n";
		$out .= $this->summarize($indexformat);
		if ($indexformat eq "full" || $this->{admin}) {
		    #$out .= $this->index_summary(); # already called in summarize()
		    $out .= $this->tools($art);
		}
		$out .= "</p>\n";
	    }
	    $n++;
	    if ($n == $indexlimit && ! $opt{archive}) {
		$archive = 1;
		last;
	    }
	}
	$this->set($article_save);
    }
    if ($out) {
	if ($indexformat eq "photoindex") {
	    $out = "<table class=\"ZineIndex\">\n$out";
	}
	elsif ($indexformat eq "gallery") {
	    $out = "<table class=\"ZineIndex\"><tr>\n$out</tr>";
	}
	elsif ($indexformat eq "table") {
	    $out = "<table class=\"ZineIndex\">\n<tr><th colspan=\"3\">Archives</th></tr>\n<tr><th>Title</th><th>Author</th><th>Date Posted</th></tr>\n$out";
	}
	if ($indexformat =~ /photoindex|gallery|table/) {
	    $out .= "</table>\n";
	}
    }
    if ($archive) {
	my $archive_url = $this->link(archive=>1,zine=>"show");
	my $archive_anchor = $config{PhotoForum}{archive_anchor} || "Show archived images";
	$out .= "<p><a href=\"$archive_url\">$archive_anchor</a>.</p>\n";
    }
    elsif ($opt{archive} && $n > 0) {
	my $archive_url = $this->link(archive=>undef);
	my $current_anchor = $config{PhotoForum}{current_anchor} || "Show current images";
	$out .= "<p><a href=\"$archive_url\">$current_anchor</a>.</p>\n";
    }
    return $out;
}

sub tools {
    my ($this,$article) = @_;
    my ($out,$url,$ntool);
    if (! exists $this->{tools}) { $this->{tools} = {}; }
    my $aid = $article->{article_id};

    if ($article->{type} eq "comment" && $this->allow_comments) {
	$url = $this->link(zine=>"reply",aid=>$aid);
	$out .= "<td align=\"center\">\n<a href=\"$url\"><img src=\"$config{server}{HTMLpath}/_Modules/PhotoForum/reply.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"reply\" title=\"reply to this\">";
	if ($config{Zine}{icon_labels}) { $out .= "<br>reply"; }
	$out .= "</a></td>\n";
	$ntool++;
    }
    else {
	$url = $this->link(zine=>"new",aid=>$aid);
	if ($article->{type} =~ /article|forum/ && $this->allow_comments("article")) {
	    $out .= "<td align=\"center\">\n<a href=\"$url\"><img src=\"$config{server}{HTMLpath}/_Modules/PhotoForum/comment.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"new comment\" title=\"new comment\">";
	    if ($config{Zine}{icon_labels}) { $out .= "<br>comment"; }
	    $out .= "</a></td>\n";
	    $ntool++;
	}
    }

    if ($this->{admin}) {

	# new article

	if ($article->{type} !~ /comment|forum/) {
	    my $newart = 1;
	    if ($article->{type} eq "article") {
		# only show the newarticle link if this is a book
		my $ref = $share{DB}->fetch("article",$article->{reference});
		$newart = ($ref->{type} eq "book");
	    }
	    if ($newart) {
		$url = $this->link(zine=>"newarticle",aid=>$article->{article_id});
		$out .= "<td align=\"center\">\n<a href=\"$url\"><img src=\"$config{server}{HTMLpath}/_Modules/PhotoForum/new.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"new photo\" title=\"new photo\">";
		if ($config{Zine}{icon_labels}) { $out .= "<br>new photo"; }
		$out .= "</a></td>\n";
		$ntool++;
	    }
	}

	# edit article

	$url = $this->link(zine=>"edit",aid=>$article->{article_id});
	$out .= "<td align=\"center\">\n<a href=\"$url\"><img src=\"$config{server}{HTMLpath}/_Modules/PhotoForum/edit.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"edit\" title=\"edit\">";
	if ($config{Zine}{icon_labels}) { $out .= "<br>edit"; }
	$out .= "</a></td>\n";
	$ntool++;

	# configure article

	$url = $this->link(zine=>"conf",aid=>$article->{article_id});
	$out .= "<td align=\"center\">\n<a href=\"$url\"><img src=\"$config{server}{HTMLpath}/_Modules/PhotoForum/tool.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"configure\" title=\"configure\">";
	if ($config{Zine}{icon_labels}) { $out .= "<br>conf"; }
	$out .= "</a></td>\n";
	$ntool++;

	# delete article

	$url = $this->link(zine=>"del",aid=>$article->{article_id});
	$out .= "<td align=\"center\">\n<a href=\"$url\"><img src=\"$config{server}{HTMLpath}/_Modules/PhotoForum/delete.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"delete\" title=\"delete\">"; 
	if ($config{Zine}{icon_labels}) { $out .= "<br>delete"; }
	$out .= "</a></td>\n";
	$ntool++;
    }
    elsif (($config{PhotoForum}{access} eq "public") || 
	   ($config{PhotoForum}{access} eq "members" && $share{DB}->authorize) ||
	   ($article->{owner} == $share{DB}->my_uid)
	   ) {
	# NOT admin mode, but we want to allow new article addition

	if ($share{Page} && ! $share{Page}->publishing) {

	    # new article

	    if ($article->{type} =~ /blog/) {
		my $parent = $article->{article_id};  #### parent_id ???
		my $ref = $article->{reference_id} || $parent;
		$url = $this->link(zine=>"newarticle",aid=>$article->{article_id});
		$out .= "<td align=\"center\">\n<a href=\"$url\"><img src=\"$config{server}{HTMLpath}/_Modules/PhotoForum/new.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"new photo\" title=\"new photo\">";
		if ($config{Zine}{icon_labels}) { $out .= "<br>new photo"; }
		$out .= "</a></td>\n";
		$ntool++;
	    }
	}
    }
    if ($article->{status} &&  $article->{status} ne "active") {
	$out .= "</tr>\n<tr><td colspan=$ntool><span class=error>This $article->{type} is $article->{status}.</span></td><td align=center>";
	if ($article->{status} =~ /inactive|pending|rejected/) {
	    if ($this->moderator) {
		$url = $this->link(zine=>"approve",aid=>$article->{article_id});
		$out .= "<a href=\"$url\"><img src=\"$config{server}{HTMLpath}/_Modules/PhotoForum/approve.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"comment\" title=\"approve\">";
		$out .= "</a>\n";
		$url = $this->link(zine=>"reject",aid=>$article->{article_id});
		$out .= "<a href=\"$url\"><img src=\"$config{server}{HTMLpath}/_Modules/PhotoForum/delete.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"comment\" title=\"reject\">";
		$out .= "</a>\n";
	    }
	}
	$out .= "</td>";
    }

    if ($out) {
	my $align = ($config{Zine}{tool_placement} =~ /right/) ? 
	    " align=\"right\"" : "";
	$out = "<table class=\"ZineTools\"$align><tr>\n$out\n</tr></table>\n";
    }
    return $out;
}

sub label {
    my ($this,$type,$col) = @_;
    return ($type eq "comment") ?
	$Modules::Zine::comment_label{$col->{column}} || $col->{label} || $col->{column} :
	$label{$col->{column}} || $col->{label} || $col->{column};
}

1;
