package Modules::Questionnaire;

#----------------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#-----------------------------------------------------------------------
# Modules::Questionnaire
# This is a general-purpose interface to the Questionnaire framework.
#-----------------------------------------------------------------------

use strict;

use CGI::Carp	qw(fatalsToBrowser);
use ExSite::Config;
use ExSite::Misc;
use ExSite::Page;
use ExSite::Input;
use ExSite::ReportBuilder;

use Modules::Questionnaire::Questionnaire;
use Modules::Questionnaire::QuestionBase;
use Modules::Questionnaire::Question;
use Modules::Questionnaire::Report;
use Modules::Questionnaire::Response;
use Modules::Questionnaire::ResponseFilter;
use Modules::Questionnaire::ResponseMatch;
use Modules::Questionnaire::QuestionnaireHTML;

# inherit basic module behaviours from BaseDCD
use base	qw(Modules::BaseDCD);

# new(), read(): inherited from BaseDCD
sub new($) {
	my $type = shift @_;
	my $this = {};
	my $class = ref($type) || $type || "Modules::Questionnaire";
	bless $this, $class;
	$this = $this->SUPER::new(@_);
	return $this;
}

# For convenience... store an ExSite::Input
# object and return it when requested
sub input($) {
	my $this = shift @_;

	my $input = $this->get('input');
	return $input if $input;

	$input = new ExSite::Input;
	$this->set('input', $input);
	return $input;
}

sub evite($) {
	my $this = shift @_;

	my $evite = $this->get('evite');
	return $evite if $evite;

	require Modules::Questionnaire::Evite;
	$evite = new Modules::Questionnaire::Evite;
	$this->set('evite', $evite);
	return $evite;
}

sub write {
	my ($this, $opt) = @_;
	# $opt is the options parameter passed from the CMS tag
	# Assuming this is formatted using URL-encoding, we can break
	# it out into hard-coded module parameters:
	my %option = &DecodeString($opt);
	my $action = $option{'action'} || $this->{'input'}{'action'};

	# Additionally, we can get our "soft" parameters from the 
	# web page input: $this->{input}{...}

	my $preview = $option{'previewFlag'};
	my $qid = $option{'qid'} || $this->{'input'}{'qid'};
	my $qcmd = $this->{'input'}{'qcmd'};
	my $version = $this->{'input'}{'version'} || $option{'version'};
	my $submit = $this->{'input'}{'q_submit'};
	my $reply = $this->reply($this->{'input'}{'reply'});

	my $out = "";  # our HTML output

	# - question_make is an exception because we're inserting 2 records
	# ... one of which relies on the previous record's ID value
	# it doesn't seem as though form->do can take care of all of this
	# - previewUpdate also works the same way. It needs to update added questions,
	# deleted questions and question ordering. previewUpdate takes care of all 3 of these
	# actions

	$this->form->get_data;

	if($qcmd && $qcmd !~ /approve|question_swap|question_make|preview_changes|submit_questionnaire|update_final|insert_final|add_report_question|insert_report|insert_response_filter|update_report|update_response_filter|unlink_response|evite|copy_questionnaire/) {
		my $form = $this->form;
		return $form->do;
	}

	my $data = $this->form->parse(%{$this->{form}->{data}});

	return $this->copyQuestionnaire($qid, $data->{'q_questionnaire'}->{'_'}) if $qcmd =~ /copy_questionnaire/;
	return $this->swapQuestions($qcmd, $qid, $this->{'input'}{'qqid'}) if $qcmd =~ /question_swap/;
	return $this->approveBaseVersion($this->{'input'}{'qbid'}, $reply) if $qcmd =~ /approve/;
	return $this->makeQuestion($data->{'q_question_base'}->{'_'}, $data->{'q_question'}->{'_'}) if $qcmd =~ /question_make/;
	return $this->insertFinalMessage($qid, $this->{'input'}{'qqid'}, $this->{'input'}, $data->{'q_question_base'}->{'_'}) if $qcmd =~ /insert_final/;
	return $this->previewUpdate($qid, $this->{'input'}) if $qcmd =~ /preview_changes/;
	return $this->addReport($qid, $this->{'input'}) if $qcmd =~ /^insert_report/;
	return $this->updateReport($this->{'input'}{'report_id'}, $this->{'input'}) if $qcmd =~ /^update_report/;
	return $this->addReportQuestion($this->{'input'}{'qqid'}, $this->{'input'}{'report_id'}) if $qcmd =~ /add_report_question/;
	return $this->insertResponseFilter($this->{'input'}{'qid'}, $this->{'input'}) if $qcmd =~ /insert_response_filter/;
	return $this->updateResponseFilter($this->{'input'}{'response_filter_id'}, $this->{'input'}) if $qcmd =~ /update_response_filter/;
	return $this->unlinkResponse($this->{'input'}->{'response_id'}) if $qcmd =~ /^unlink_response$/;
	return $this->eviteForm($this->{'input'}) if $qcmd =~ /evite_form/;
	$this->sendEvites($qid, $this->{'input'}{'evite_recipients'}, $this->{'input'}{'evite_from'}, $this->{'input'}{'evite_subject'}, $this->{'input'}{'evite_message'}) if $qcmd =~ /^evite$/;
	# The allowed actions for an end user are:
	# 1) view a questionnaire
	# 2) submit answers to a questionnaire
	if($submit) {
		$this->processInput($qcmd, $qid, $version, $this->{'input'}{'response_id'}, $this->{'input'});
		return $this->questionnaireThankyou($qid, $version);
	} else {
		my $questionnaire = new Modules::Questionnaire::Questionnaire($qid, $version);
		return $this->viewQuestionnaire($questionnaire, $this->{'input'}{'response_id'});
	}

	return undef;
}

sub ioctl {
    my $this = shift;
    $_ = shift;
    if (/ModuleInfo/) {
		return <<"DONE";
Questionnaire is a general-purpose question/answer processing system.  
You can build arbitrary sets of questions, define allowed answers, collect 
responses, and view reports on those responses.
DONE
    }
    elsif (/ControlPanel/) {
		return \&ctrl_panel;
    }
    elsif (/DynContentParameter/) {
    	return $this->getDynContentParameter(shift);
    }
    return undef;
}

sub ctrl_panel {
	my $this = shift;

	# This is where we generate our administrator screens.
	my $level = $share{'DB'}->authorize();

	return "" unless ($level > 2);

	my $qcmd = $this->{'input'}{'qcmd'};     # administrator action/command
	my $sid = $this->{'input'}{'section_id'};     # administrator action/command
	my $qid =  $this->{'input'}{'qid'};      # questionnaire ID
	my $qqid =  $this->{'input'}{'qqid'};    # question ID
	my $qbid = $this->{'input'}{'qbid'}; # base question ID

	#my $out = "<link rel='stylesheet' type='text/css' href='$config{server}->{HTMLpath}/_Modules/Questionnaire/Questionnaire.css'>".$this->backLink($qcmd, $qid)."<br>\n";
	my $out = "<link rel='stylesheet' type='text/css' href='$config{server}->{HTMLpath}/_Modules/Questionnaire/Questionnaire.css'>\n<script language='javascript' src='$config{server}->{HTMLpath}/_ExSite/js/misc.js'></script>".$this->backLink($qcmd, $qid, $this->{'input'})."<br>\n";
	if($this->{'input'}{'submit_action'}) {
		$out = $this->write();
		if($this->form->{'diagnostics'}->{'error'} && $this->form->{'diagnostics'}->{'error'}->[0]) {
			return $this->show_diagnostics('error', 'HTML').$out;
		} elsif($this->form->{'errors'} && scalar(@{$this->form->{'errors'}})) {
			return $this->show_diagnostics('error', 'HTML').$out;
		}
	} else {
		if($this->isDeleteCommand($qcmd)) {
			my $delResult = $this->performDelete($qcmd, $this->{'input'});
			if($this->{'diagnostics'}->{'error'}) {
				return &Modules::Questionnaire::QuestionnaireHTML::errorScreen('error_msg' => $this->show_diagnostics('error', 'HTML'));
			}
		}
		if(!$qcmd || $qcmd =~ /all_sites_view/) {
			return $out.$this->allSitesView();
		}
		elsif($qcmd =~ /section_view/) {
			return $out.$this->sectionView($sid);
		} elsif($qcmd =~ /^question_base_view$/) {
			return $out.$this->questionBaseView($qbid, $this->{'input'}{'version'});
		} elsif($qcmd =~ /new_question_base_view|new_question_create|edit_master/) {
			return $out.$this->editQuestionBaseView($sid, $qid, $qbid, $qcmd);
		} elsif($qcmd =~ /^new_version$/) {
			return $out.$this->editQuestionVersion($qbid, $this->{'input'}{'version'});
		} elsif($qcmd =~ /^edit_version$/) {
			return $out.$this->editQuestionVersion($qbid, $this->{'input'}{'version'});
		} elsif($qcmd =~ /^questionnaire_view|unlink|link_question$/) {
			return $out.$this->questionnaireView($qid, $this->{'input'}{'version'}, $this->{'input'}{'view_type'}, $this->{'input'}{'report_id'}, $this->{'input'});
		} elsif($qcmd =~ /new_questionnaire_view/) {
			return $out.$this->newQuestionnaireForm($sid);	
		} elsif($qcmd =~ /copy_questionnaire/) {
			return $out.$this->copyQuestionnaireForm($qid);	
		} elsif($qcmd =~ /new_final|new_final_version/) {
			return $out.$this->newFinal($sid, $qid);
		} elsif($qcmd =~ /edit_final/) {
			return $out.$this->editFinal($qid);
		} elsif($qcmd =~ /new_report|edit_report/) {
			return $out.$this->reportForm($qid, $this->{'input'}->{'report_id'}, $this->{'input'});
		} elsif($qcmd =~ /new_response_filter|edit_response_filter/) {
			return $out.$this->newResponseFilterForm($qid, $this->{'input'}->{'response_filter_id'}, $this->{'input'});
		} elsif($qcmd =~ /evite_form/) {
			return $out.$this->eviteForm($this->{'input'});
		}
	}
	if($this->{'input'}{'reply'}) {
		print "Location: ".$this->reply."\n\n";
		return undef;
	}
	return undef;
}

# Returns questionnaire objects for all of the available questionnaires
# on a given site (identified by $sid)
sub fetchSiteQuestionnaires($) {
	my $this = shift @_;
	my $sid = shift @_ || {};

	return undef unless $sid;

	my $questionnaires = $this->get('questionnaires_'.$sid);
	return @{$questionnaires} if $questionnaires;

	my $questRecs = $share{'DB'}->fetch_match('q_questionnaire',
		{'section_id' => $sid}
	) || [];

	$questionnaires = [];
	foreach my $q (@{$questRecs}) {
		my $questionnaire = new Modules::Questionnaire::Questionnaire();
		$questionnaire->set('q_questionnaire', $q);
		$questionnaire->set('version', 'English');
		push @{$questionnaires}, $questionnaire;
	}
	$this->set('questionnaires_'.$sid, $questionnaires);
	return @{$questionnaires};
}

# Fetch and return all q_question_base records for the current site.
# Modules::Questionnaire::QuestionBase objects are returned
sub fetchSiteBaseQuestions($) {
	my $this = shift @_;
	my $sid = shift @_;

	return undef unless $sid;

	my $questions = $this->get('q_question_base_'.$sid);
	return @{$questions} if $questions;

	my $qRecords = $share{'DB'}->fetch_match('q_question_base',
		{'section_id' => $sid}
	);
	$questions = [];
	foreach my $q (@{$qRecords}) {
		my $obj = new Modules::Questionnaire::QuestionBase($q);
		push @{$questions}, $obj;
	}
	$this->set('q_question_base_'.$sid, $questions);
	return @{$questions};
}

# Return an ExSite::Form object
# Method either creates, caches and returns a new object
# or retrieves and returns the cached copy
sub form($) {
	my $this = shift @_;

	my $form = $this->get('form');
	return $form if $form;

	$form = new ExSite::Form;
	$this->addHandlers($form);
	$this->set('form', $form);
	return $form;
}

# Get the stored 'reply' link
# If a $reply option is passed, the stored 'reply' is
# set to $reply
sub reply($) {
	my $this = shift @_;
	my $reply = shift @_;

	if(defined($reply)) {
		$this->set('reply', $reply);
		return $reply;
	}

	$reply = $this->get('reply');
	return $reply if $reply;

	$reply = $this->{'input'}{'reply'};
	$this->set('reply', $reply);
	return $reply;
}

# Links to each questionnaire for the dynContentParameter
sub printQuestionnaireLinks($) {
	my $this = shift @_;
	my @questionnaires = @_;
	my $out = "";
	foreach my $q (@questionnaires) {
		$out .= "<option value='qid=".$q->id."&action=do'>".$q->title."</option>\n";
	}
	return $out;
}

# Loads a questionnaire, prepops all of the important fields in %data
# and returns QuestionnaireHTML::questionnaireForm with the prepopped fields
sub editQuestionnaireForm($) {
	my $this = shift @_;
	my $questionnaire = shift @_;

	return undef unless $questionnaire;

	my %data = (
		'section_id' => $questionnaire->section_id,
		'qid' => $questionnaire->id,
		'qcmd' => 'update_questionnaire',
		'title' => $questionnaire->title,
		'versions' => $questionnaire->versions,
		'buttons' => $this->adminFormButtons(),
	);
	return &Modules::Questionnaire::QuestionnaireHTML::questionnaireForm(%data);
}

# Returns an HTML pull-down menu filled with all of the valid (and answerable)
# questions
sub questionSelect($) {
	my $this = shift @_;
	my $questionnaire = shift @_;
	my $questionId = shift @_;

	return undef unless ref($questionnaire);

	$questionnaire->loadAllQuestions();
	my $out = "<select name='question_id'>\n";

	while((my $q = $questionnaire->getNextQuestion())) {
	# Don't print invalid questions or "questions" that are
	# just labels. We don't want to provide and option to filter labels
		next unless ($q->id && $q->format !~ /label/);
		my $selected = ($q->id == $questionId) ? " selected" : "";
		$out .= "\t<option value='".$q->id."'$selected>".$q->label."</option>\n";
	}

	$out .= "</select>\n";
	return $out;
}

# Simple wrapper used to call Modules::Questionnaire::Questionnaire::copy
sub copyQuestionnaire($) {
	my $this = shift @_;
	my $qid = shift @_;
	my $newQuestionnaire = shift @_;

	return undef unless $qid;

	my $questionnaire = new Modules::Questionnaire::Questionnaire($qid);
	return $questionnaire->copy(%{$newQuestionnaire});
}

# Swap a question (identified by qqid) in the direction specified
# in $qcmd (should contain the string 'up' or 'down')
sub swapQuestions($$$) {
	my $this = shift @_;
	my $qcmd = shift @_;
	my $qid = shift @_;
	my $qqid = shift @_;

	return undef unless $qqid || $qcmd !~ /up|down/;

	my $questionnaire = new Modules::Questionnaire::Questionnaire($qid);
	
	my $direction = ($qcmd =~ /up/) ? -1 : 1;
	my @questions = $questionnaire->getAllQuestions();

	my $pos = -1;
	my $currentQuestion = undef;
	my $swapQuestion = undef;
	for(my $i = 0; $i < @questions; $i++) {
		if($questions[$i]->id == $qqid) {
			$pos = $i;
		}
	}
	if($pos == 0 && $direction < 0) {
		$this->error("Can't swap the first question up.");
		return undef;
	}
	if($pos == ($questionnaire->count-1) && $direction > 0) {
		$this->error("Can't swap the last question down.");
		return undef;
	}

	my $currentHash = $questions[$pos]->get('q_question');
	my $swapHash = $questions[($pos+$direction)]->get('q_question');
	my $tmpSort = $currentHash->{'sort_key'};
	$currentHash->{'sort_key'} = $swapHash->{'sort_key'};
	$swapHash->{'sort_key'} = $tmpSort;
	my $up1 = $questions[$pos]->update(%{$currentHash});
	my $up2 = $questions[($pos+$direction)]->update(%{$swapHash});
	return ($up1 && $up2);
}

# Insert a new questionnaire by using the 
# Modules::Questionnaire::Questionnaire::make method
sub insertQuestionnaire($) {
	my $this = shift @_;
	my $sid = shift @_;
	my $data = shift @_;

	return undef unless $sid;

	my $questionnaire = new Modules::Questionnaire::Questionnaire();

	my %qHash = (
		'section_id' => $sid,
		'title' => $data->{'title'},
		'versions' => $data->{'versions'},
	);
	return $questionnaire->make(%qHash);
}

# Update a questionnaire by using the
# Modules::Questionnaire::Questionnaire::make method
sub updateQuestionnaire {
	my $this = shift @_;
	my $qid = shift @_;
	my $data = shift @_;

	return undef unless $qid;

	my $questionnaire = new Modules::Questionnaire::Questionnaire($qid);

	my %qHash = (
		'questionnaire_id' => $questionnaire->id,
		'section_id' => $questionnaire->section_id,
		'title' => $data->{'title'},
		'versions' => $data->{'versions'},
	);

	return $questionnaire->make(%qHash);
}

# Generates a report to list the questionnaires provided in the $questionnaire
# array ref.
# This is no longer used.
sub questionnaireList($) {
	my $this = shift @_;
	my $questionnaire = shift @_;
	my $section = shift @_ || {};

	return "" unless ref($questionnaire) =~ /ARRAY/;

	my $class = $this->get('div_class');
	my $headers = [];
	my $data = [];

 	my $module = "Questionnaire";
	my @commands = (
		{
			'label' => "Add new $module form to $section->{title}",
			'url' => &relink("qcmd"=>"new", "section_id" => $section->{'section_id'}, "q_submit" => undef),
			'img' => 'new'
		},
	);
	my $links = &Modules::Questionnaire::Questionnaire::newQuestionnaireLink($section->{'section_id'});

	foreach my $q (@{$questionnaire}) {
		my @row = ($q->title);
		my $opts = "";
		$opts = $q->previewLink(); # if scalar(@questions);
		$opts .= $q->reportLink();
		$opts .= $q->detailsLink();
		$opts .= $q->orderQuestionsLink();
		$opts .= $q->deleteLink();
		push @row, $opts;
		push @{$data}, \@row;
	}

	my $out = "<div class='$class'>\n";

	my %report = (
		'title' => $section->{'title'},
		'headers' => ['Questionnaire', 'Options'],
		'tools' => $links,
		'data' => $data,
	);

	$out .= $this->reportbuilder(\%report);
	$out .= "</div>\n<br>";
	return $out ;

}

# Updates a response filter by using the
# Modules::Questionnaire::ReponseFilter make method
sub updateResponseFilter($) {
	my $this = shift @_;
	my $responseFilterId = shift @_;
	my $input = shift @_;

	return 0 unless $input->{'name'} && $responseFilterId;

	my $responseFilter = new Modules::Questionnaire::ResponseFilter($responseFilterId);
	my $data = $responseFilter->get('q_response_filter');
	$data->{'name'} = $input->{'name'};
	my $newId = $responseFilter->make(%{$data});
	return $newId;
}

# Deletes the response identified by $responseId
sub unlinkResponse($) {
	my $this = shift @_;
	my $responseId = shift @_;
	return 0 unless ($responseId > 0);

	my $response = new Modules::Questionnaire::Response($responseId);
	return $response->unlink;
}

# Deletes a response filter identified by $responseFilterId
# (uses Modules::Questionnaire::ReponseFilter::delete to perform the
# actual delete)
sub deleteResponseFilter($) {
	my $this = shift @_;
	my $responseFilterId = shift @_;

	return 0 unless $responseFilterId;

	my $responseFilter = new Modules::Questionnaire::ResponseFilter($responseFilterId);
	return $responseFilter->delete;
}

# Delete (trash) an actual report identified by $reportId
sub deleteReport($) {
	my $this = shift @_;
	my $reportId = shift @_;

	my $report = new Modules::Questionnaire::Report($reportId);

	return 0 unless $report->id;

	return $share{'DB'}->trash_r('q_report', $report->id);
}

# Removes a question frorm a report.
# If no report exists (reportId is undef), the report is created, then the 
# question is removed
# If a report exists, the question is removed from the report in question
sub deleteReportQuestion($) {
	my $this = shift @_;
	my $reportId = shift @_;
	my $questionId = shift @_;
	my $questionnaireId = shift @_;

	return 0 unless $questionId;
	my $report = new Modules::Questionnaire::Report($reportId);
	my $questionnaire = new Modules::Questionnaire::Questionnaire($questionnaireId);
	if(!$reportId) {
		my @questions = $questionnaire->getAllQuestions();
		$report->make('name' => 'test', 'questionnaire_id' => $questionnaireId);
		$report->makeQuestions(@questions);
		my $reply = $this->reply;
		$this->reply($reply.'&report_id='.$report->id) unless $reply =~ /report_id/;
	}
	return $report->removeQuestion($questionId, $questionnaire);
}

# Insert a ResponseMatch record by using
# Modules::Questionnaire::ResponseMatch::make
sub insertResponseMatch($) {
	my $this = shift @_;
	my $responseFilterId = shift @_;
	my $data = shift @_;

	my %record = (
		'response_filter_id' => $responseFilterId,
		'question_id' => $data->{'question_id'},
		'filter' => $data->{'filter'},
	);
	my $responseMatch = new Modules::Questionnaire::ResponseMatch();
	return $responseMatch->make(%record);
}

# Update a response match record by using the 
# Modules::Questionnaire::ResponseMatch::make method
sub updateResponseMatch($) {
	my $this = shift @_;
	my $responseMatchId = shift @_;
	my $data = shift @_;

	return undef unless $responseMatchId;

	my $match = new Modules::Questionnaire::ResponseMatch($responseMatchId);
	# Get the record and update it with $data, then run 'make'
	# again.
	# Make will update as long as the response_match_id is present
	my $record = $match->get('q_response_match');

	$record->{'filter'} = $data->{'filter'};
	$record->{'question_id'} = $data->{'question_id'};
	return $match->make(%{$record});
}

# Delete a response match record matching $responseMatchId
sub deleteResponseMatch($) {
	my $this = shift @_;
	my $responseMatchId = shift @_;
	
	return undef unless $responseMatchId;

	return $share{'DB'}->delete('q_response_match', {'response_match_id' => $responseMatchId});
}

# Creates a drop-down box of all of the questionnaires available to the
# current user
# (currently unused)
sub questionnaireSelect($) {
	my $this = shift @_;

	my @sites = $share{'DB'}->my_real_sites;
	
	my $out = "<h2>Questionnaire List</h2>\n";
	foreach my $s (@sites) {
		my @questionnaires = $this->fetchSiteQuestionnaires($s->{'section_id'});
		$out .= $this->questionnaireList(\@questionnaires, $s);
	}
	return $out;
}

# Returns a list of base questions available to a user
sub questionList($) {
	my $this = shift @_;
	my @sites = $share{'DB'}->my_real_sites();

	my $class = $this->get('div_class');
	my $headers = ['Question', 'Version', 'Questionnaires', 'Options'];
	my $out = "<div class='$class'>\n";
	$out .= "<h2>Site Questions</h2>\n";
	my $links = "";

	foreach my $s (@sites) {
		my $base = new Modules::Questionnaire::QuestionBase;
		my $data = [];
		my $sectionQs = $base->fetchAll($s->{'section_id'});
	
		# Generate a list only of the master questions
		foreach my $q (@{$sectionQs}) {
			next if $q->master_id || $q->isFinal;

			push @{$data}, $this->questionBaseRow($q);
		}

		#$links = $this->NewQuestion($s->{'section_id'});
		$links = &Modules::Questionnaire::QuestionBase::newIcon($s->{'section_id'});

		my %report = (
			'title' => $s->{'title'},
			'headers' => $headers,
			'tools' => $links,
			'data' => $data,
		);
		$out .= $this->reportbuilder(\%report)."<br>\n";
	}
	$out .= "</div>\n";

	return $out;
}

# Reports a list of final messages and their versions
# (a "final message" is just a label question with position set to 
# $Modules::Questionnaire::Questionnaire::FINAL_QUESTION_POSITION)
# this method assumes that the final questions are being passed as the last
# argument
sub reportFinal($) {
	my $this = shift @_;
	my $title = shift @_;
	my $base = shift @_;
	my @questions = @_;
	
	my $data = [];

	my $class=  $this->get('div_class');

	foreach my $q (@questions) {
		push @{$data}, $this->finalRow($q);
	}

	my %report = (
		'title' => $title,
		'headers' => ['Question', 'Version', 'Questionnaires', 'Options'],
		'data' => $data,
		'tools' => $base->newVersionIcon(undef),
	);

	return "<div class='$class'>".$this->reportbuilder(\%report)."</div>\n";
}

# Return an array as a report row for printing the final questions report
sub finalRow($) {
	my $this = shift @_;
	my $questionBase = shift @_;

	return undef unless $questionBase->isFinal;

	# Get the linking questions that use this q_question_base record
	my @questions = $questionBase->fetchQuestions();

	my $questionnaires = [];
	my @allQuestionnaires = ();
	
	foreach my $q (@questions) {
		my $questionnaire = $share{'DB'}->fetch_match('q_questionnaire',
			{'questionnaire_id' => $q->questionnaire_id}
		);
		push @allQuestionnaires, $questionnaire->[0]->{'title'};
	}
	my $allQuestionnaires = join ", ", @allQuestionnaires;
	my $options = "";
	$options = $this->questionnaireQuestionLink($questionBase->id) unless $questionBase->master_id;
	$options .= $questionBase->editFinalIcon();
	$options .= $questionBase->newVersionIcon() unless $questionBase->master_id;
	$options .= $questionBase->deleteIcon($this->reply);

	my $row = [
		$questionBase->label,
		$questionBase->version,
		$allQuestionnaires,
		$options
	];

	return $row;
}

sub questionnaireQuestionLink($$) {
	my $this = shift @_;
	my $qbid = shift @_;

	return "" unless $qbid;
	my $out = "<a href='".&relink("qcmd"=>"new_question", "qbid" => $qbid, "q_submit" => undef)."'><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/list12.gif' border='0'></a>\n";
	return $out;
}

sub deleteQuestionnaire($) {
	my $this = shift @_;
	my $qid = shift @_;

	my $questionnaire = new Modules::Questionnaire::Questionnaire($qid);

	return $questionnaire->delete();
}

sub QuestionnaireReports($) {
	my $this = shift @_;
	my $qid = shift @_;

	return "" unless $qid;

	my $out = "<a href='".&relink("qcmd"=>"reports", "qid" => $qid)."'>Report Questionnaire</a>\n";
	return $out;
}

# No longer used.
sub questionOrderTable($) {
	my $this = shift @_;
	my $qid = shift @_;
	my $version = shift @_;

	return "" unless $qid;

	my $header = ['Question', 'Position'];
	my $data = [];

	my $questionnaire = new Modules::Questionnaire::Questionnaire($qid, $version);
	my $nQuestions = $questionnaire->count;

	for(my $i = 0; $i < $nQuestions; $i++) {
		my $quest = $questionnaire->getQuestionNumber($i);
		push @{$data}, $this->buildOrderingRow($quest);
	}

	my $out = "<form method='get'>\n";
	$out .= "<input type='hidden' name='q_submit' value='1'>\n";
	$out .= "<input type='hidden' name='qcmd' value='sort'>\n";
	my %report = (
		'title' => $questionnaire->title,
		'headers' => ['Question', 'Position'],
		'data' => $data,
	);

	$out .= $this->reportbuilder(\%report);
	$out .= "<br>\n".$this->adminFormButtons()."\n";
	$out .= "</form>\n";
	return $out;
}

# No longer used
sub buildOrderingRow($) {
	my $this = shift @_;
	my $question = shift @_;

	return undef unless $question->id;

	my $rec = $question->get('q_question');
	my $sort = $rec->{'sort_key'};

	my $row = [];
	push @{$row}, $question->label;
	push @{$row}, "<input type='text' value='$sort' name='sort_$rec->{question_id}'>";
	return $row;
}

# Creates an HTML form for creating new q_response_filter objects
sub newResponseFilterForm($) {
	my $this = shift @_;
	my $qid = shift @_;
	my $fid = shift @_;

	return "" unless $qid;

	my $questionnaire = new Modules::Questionnaire::Questionnaire($qid);
	my $filter = new Modules::Questionnaire::ResponseFilter($fid);
	my @questions = $questionnaire->getAllQuestions();
	my @matches = $filter->fetchMatches();
	my %matchHash = ();
	map {$matchHash{$_->question_id} = $_;} @matches;

	my $filterButton = "<input type='button' value='Add Filter' onClick='javascript:addFilter(document.filterForm.questionList.options[document.filterForm.questionList.selectedIndex].value, document.filterForm.match.value)'>\n";

	my $questionList = "<select name='questionList' id='questionList'>\n<option value=''>=== select ===</option>\n";
	my $jsQuestionList = "var questionList = new Array();\n";
	for(my $i = 0; $i < @questions; $i++) {
		next if $questions[$i]->fetchBase->format =~ /label/;
		my $label = $questions[$i]->label;
		$label =~ s/\'/\&\#39\;/g;
		$questionList .= "<option value='".$questions[$i]->id."'>".$label."</option>\n" unless $matchHash{$questions[$i]->id};
		$jsQuestionList .= "questionList[".$questions[$i]->id."] = '".$label."';\n";
	}
	$questionList .= "</select>\n";

	my $filters = "";
	my $filterTables = "";
	if($filter->id) {
		my %qHash = ();
		map {$qHash{$_->id} = $_;} @questions;
		if(scalar(@matches)) {
			$filterTables .= "<table style=\'width: 400px;\'>\n<tr><th style=\'width: 300px\'>Question</th><th style=\'width: 100px;\'>Filter</th></tr></table>\n";
		}
		for(my $i = 0; $i < @matches; $i++) {
			my $m = $matches[$i];
			next unless $qHash{$m->question_id};
			my $question = $qHash{$m->question_id};
			my $deleteLink = "<a onClick='removeFilter(".($i+2).", ".$m->question_id.")'><img src=\'$config{server}->{HTMLpath}/_Modules/Questionnaire/rm12.gif' border='0'></a>\n";
			$filters .= "filters[".$m->question_id."] = new Array(".$m->question_id.", '".$question->label."', '".$m->filter."', ".$m->id.");\n";
			$filterTables .= "<div id='".($i+1)."'><table style=\'width:400px;\'><tr><td style=\'width: 300px;\'>".$deleteLink.$question->label."</td><td style=\'width: 100px\'>".$m->filter."</td></tr></table></div>";
		}
	}

	my %data = (
		'questionnaire_id' => $qid,
		'qcmd' => ($filter->id) ? "update_response_filter" : 'insert_response_filter',
		'reply' => &relink('qcmd' => 'questionnaire_view', 'view_type' => 'report'),
		'name_input' => $this->form->input_html('type' => 'text', 'name' => 'name', 'value' => $filter->name),
		'id_input' => ($filter->id) ? $this->form->input_html('type' => 'hidden', 'name' => 'response_filter_id', 'value' => $filter->id) : "",
		'filter_input' => $this->form->input_html('type' => 'text', 'name' => 'match'),
		'question_list' => $questionList,
		'js_question_list' => $jsQuestionList,
		'add_filter_button' => $filterButton,
		'filters_init' => $filters,
		'filter_tables' => $filterTables,
	);
	return &Modules::Questionnaire::QuestionnaireHTML::responseFilterForm(%data);
}

# returns a form for editing a response filter
sub editResponseFilterForm($) {
	my $this = shift @_;
	my $responseFilterId = shift @_;

	return 0 unless $responseFilterId;

	my $responseFilter = new Modules::Questionnaire::ResponseFilter($responseFilterId);

	my $data = {
		'questionnaire_id' => $responseFilter->questionnaire_id,
		'name' => $responseFilter->name,
		'buttons' => $this->adminFormButtons(),
		'response_filter_id' => $responseFilter->id,
		'action' => 'update_response_filter',
	};

	my $out = $responseFilter->reportLink."<br>".$this->responseFilterForm($data);
	
	my $reportData = [];
	my @matches = $responseFilter->fetchMatches();
	
	my $options = "";

	foreach my $m (@matches) {
		my $q = $m->fetchQuestion;
		$options = $m->editLink;
		$options .= $m->deleteLink;
		push @{$reportData}, [$q->label, $m->filter, $options];
	}
	my %report = (
		'headers' => ['Question', 'Filter Rule', 'Options'],
		'data' => $reportData,
		'title' => 'Filter Rules',
		'tools' => Modules::Questionnaire::ResponseMatch::newLink($responseFilter->id),
	);
	return $out . $this->reportbuilder(\%report);
}

# HTML wrapper for the response filter form.
# This should probably go into Modules::Questionnaire::QuestionnaireHTML
sub responseFilterForm($) {
	my $this = shift @_;
	my $data = shift @_;

	return <<"DONE";
<form method='post'>
	<input type='hidden' name='qid' value='$data->{questionnaire_id}'>
	<input type='hidden' name='qcmd' value='$data->{action}'>
	<input type='hidden' name='response_filter_id' value='$data->{response_filter_id}'>
	Filter name:
	<input type='text' size='30' name='name' value='$data->{name}'>
	<br>
	<br>
	$data->{buttons}
</form>
DONE
}

# Returns a form for creating a response match for
# the questionnaire identified by $questionnaireId. The
# match will be added to the response filter identified by 
# $responseFilterId
sub newResponseMatchForm($) {
	my $this = shift @_;
	my $questionnaireId = shift @_;
	my $responseFilterId = shift @_;

	return undef unless $responseFilterId;

	my $questionnaire = new Modules::Questionnaire::Questionnaire($questionnaireId);
	$questionnaire->loadAllQuestions();

	my %data = (
		'question_select' => $this->questionSelect($questionnaire),
		'response_filter_id' => $responseFilterId,
		'action' => 'insert_response_match',
		'buttons' => $this->adminFormButtons(),
	);

	return &Modules::Questionnaire::QuestionnaireHTML::responseMatchForm(%data);
} 

# Returns a form for editing a response match
sub editResponseMatchForm($) {
	my $this = shift @_;
	my $responseMatchId = shift @_;
	return undef unless $responseMatchId;

	my $match = new Modules::Questionnaire::ResponseMatch($responseMatchId);
	my $filter = new Modules::Questionnaire::ResponseFilter($match->response_filter_id);
	my $questionnaire = new Modules::Questionnaire::Questionnaire($filter->questionnaire_id);

	my %data = (
		'question_select' => $this->questionSelect($questionnaire, $match->question_id),
		'response_filter_id' => $filter->id,
		'filter' => $match->filter,
		'action' => 'update_response_match',
		'buttons' => $this->adminFormButtons(),
	);
	return &Modules::Questionnaire::QuestionnaireHTML::responseMatchForm(%data);	
}

sub getDynContentParameter($) {
	my $this = shift @_;
	my $arg = shift @_;
	# we should return an input tag 
	# that shows all of the questionnaires that can be inserted
	# into a page of the current site.
	my @q = $this->fetchSiteQuestionnaires($arg);
	my $qhtml = $this->printQuestionnaireLinks(@q);
	return <<"DONE";
<table>
	<tr>
		<td>
			<b>Questionnaire:</b>
		</td>
		<td>
			<select name=\"param\">
			$qhtml
			</select>
		</td>
	</tr>
</table>
DONE
}

# a little tool tip used in the questionnaire details
sub quickhelp {
	my $this = shift @_;

	my $img = "$config{server}->{HTMLpath}/_Modules/Questionnaire";

	return <<"DONE";
<hr>
<B>Option Key:</B><br>
<img src='$img/view12.gif' align='middle'> - preview item<br>
<img src='$img/edit12.gif' align='middle'> - item details<br>
<img src='$img/list12.gif' align='middle'> - link question to questionnaire<br>
<img src='$img/rm12.gif' align='middle'> - unlink question from questionnaire<br>
<img src='$img/copy12.gif' align='middle'> - edit question ordering<br>
<img src='$img/new12.gif' align='middle'> - new base question<br>
DONE
}

# A questionnaire version drop down list.
# This is used on the questionnaire preview screen to view the
# questionnaire in different languages. It basically just changes
# the current query string's 'version' value to whichever value
# you choose
sub questionnaireVersionSelect($) {
	my $this = shift @_;
	my $questionnaire = shift @_;
	my $version = shift @_;

	return undef unless $questionnaire;

	my $link = &relink('version'=>undef);
	my $versionsSelect = "<select name='version' onChange='document.location=\"${link}&version=\"+this.value'>\n";
	my @versions = $questionnaire->getVersions;

	foreach my $v (@versions) {
		my $link = &relink('version' => $v);
		my $selected = ($v eq $version) ? " selected" : "";
		my $printVersion = $v;
		$v =~ s/\&/%26/g;
		$v =~ s/\;/%3B/g;
		$v =~ s/\#/%23/g;
		$versionsSelect .= "<option value='$v'$selected>$printVersion</option>\n";
	}
	$versionsSelect .= "</select>\n";

	return "<form method='get'>\n$versionsSelect\n</form>\n";
}

# This method is used for actually printing the questionnaire.
# If $responseId is set, then the questionnaire's input values
# will be pre-populated with response $responseId's values
# If $previewFlag is set, all of the javascript (and few additional
# fields) will be added to the page that prints the questionnaire.
# $previewFlag should never be set when we're printing a 'real'
# questionnaire
sub viewQuestionnaire($) {
	my $this = shift @_;
	my $questionnaire = shift @_;
	my $responseId = shift @_;
	my $previewFlag = shift @_;

	# Don't allow an invalid use to even see the questionnaire.

	my $qid = $questionnaire->id;
	my $version = $questionnaire->version;
	my @q = $questionnaire->getAllQuestions();

	my $out;
	if ($previewFlag) {
	    $out .= $this->iconKey('up' => "Move question up", 
				   'down' => "Move Question Down",
				   'edit' => "Edit question", 
				   'rm' => "Remove question from questionnaire");
	}
	$out .= "<h3>Questionnaire Version</h3>\n".$this->questionnaireVersionSelect($questionnaire, $version) if $previewFlag;
	$out .= $this->openQuestionnaire($questionnaire, $version, $previewFlag, $responseId);
	$out .= $questionnaire->show($responseId, $previewFlag);
	$out .= $this->baseDivs($questionnaire) if $previewFlag;
	$out .= $this->closeQuestionnaire($qid, $previewFlag);
	$out .= $this->previewAddForm($questionnaire) if $previewFlag;
	return $out;
}

# Simply prints the questions that are not being used by $questionnaire
# in a drop-down list.
# The 'add form' lets you include the selected question into the
# questionnaire
sub previewAddForm($) {
	my $this = shift @_;
	my $questionnaire = shift @_;

	my $select = $this->unusedQuestionsSelect($questionnaire, $questionnaire->version);
	my $reply = &relink();
	my $buttons = "<input type='button' name='add' value='Add' onClick='addQuestion()'>&nbsp;<input type='button' name='new_question' value='New Question' onClick='window.location=\"".&relink('qcmd' => 'new_question_create', 'qid' => $questionnaire->id, 'section_id' => $questionnaire->section_id, 'reply' => $reply)."\"'>\n";

	return &Modules::Questionnaire::QuestionnaireHTML::previewAddForm('select' => $select, 'buttons' => $buttons);
}

# TODO: This method should probably just be replaced with processQuestionnaire
sub processInput($) {
	my $this = shift @_;
	my $qcmd = shift @_;
	my $qid = shift @_;
	my $version = shift @_;
	my $responseId = shift @_;
	my $data = shift @_;

	return $this->processQuestionnaire($qid, $version, $responseId, $data);
}

# Process the questionnaire input and do all of the answer/response saving...
sub processQuestionnaire($) {
	my $this = shift @_;
	my $qid = shift @_;
	my $version = shift @_;
	my $responseId = shift @_;
	# Query data
	my $data = shift @_;

	return 0 unless $qid;
	my $questionnaire = new Modules::Questionnaire::Questionnaire($qid, $version);
	my @questions = $questionnaire->getAllQuestions();
	my $r = new Modules::Questionnaire::Response();

	my $insertedAnswers = 0;
	my $newRespId = 0;
	my $msg = "";

	if($responseId) {
		$newRespId = $responseId;
	} else {
		# Create a response
		my %response = (
			'member_id' => $share{'DB'}->my_uid,
			'questionnaire_id' => $questionnaire->id,
			'version' => $version,
		);
		($newRespId, $msg) = $questionnaire->addResponse(\%response);
		if(!$newRespId) {
			$this->error($msg);
			return undef;
		}
	}
	# Save the answers for each question (and have them point to this
	# response)
	
	foreach my $q (@questions) {
		my %answer = (
			'question_id' => $q->id,
			'response_id' => $newRespId,
			'value' => $data->{'question_'.$q->fetchBase->master->{'question_base_id'}},
		);
		my ($newAnswId, $msg) = $q->addAnswer(\%answer) if $newRespId;
		$insertedAnswers++ if $newAnswId;
	}
	return 1;
}

# Retrieve the list of questions stored in $data, figure out their
# (potentially new) sort_key values, then update the database to save
# these values
sub sortQuestions($) {
	my $this = shift @_;
	my $qid = shift @_;
	my $data = shift @_;

	my @questions = grep { /^sort_\d+$/ } keys %{$data};
	foreach my $q (@questions) {
		my $id = $q;
		$id =~ s/^sort_//;
		$share{'DB'}->update('q_question', {'question_id' => $id, 'sort_key' => $data->{$q}});
	}
	return;
}

# This method isn't being used anymore due to changes to the look + feel
# questionnaireDetail's functionality has been replaced by
# sub questionnaireView
sub questionnaireDetails($) {
	my $this = shift @_;
	my $qid = shift @_;
	my $version = shift @_;

	my $questionnaire = new Modules::Questionnaire::Questionnaire($qid, $version);
	my @questions = $questionnaire->getAllQuestions();
	my $finalMessage = $questionnaire->finalMessage($version);
	my $qdata = $questionnaire->get('q_questionnaire');
	my $data = [];

	foreach my $q (@questions) {
		my $opts = $q->editIcon().$q->unlinkIcon();
		my @values = $q->fetchBase->getMasterValues();
		my $values = join ",<br>", @values;
		push @{$data}, [$q->label, $q->sort_key, &Modules::Questionnaire::QuestionBase::formatText($q->format), $values, $q->flags, $opts];
	}

	my %report = (
		'title' => $qdata->{'title'},
		'headers' => ['Question', 'Position', 'Type', 'Possible Values', 'Flags', 'Options'],
		'data' => $data,
	);

	my $out = "<a href='".$this->topLink."'>Back to Questionnaire Control Panel</a>".$this->editQuestionnaireForm($questionnaire);
	$out .= $this->finalMessagePreview($questionnaire, $finalMessage);
	$out .= "<h2>Questions</h2>\n";
	$out .= $this->reportbuilder(\%report);
	$out .= $this->quickhelp;
	return $out;
}

# TODO: This method exists in Modules::Questionnaire::QuestionnaireHTML, but
# this version is still being used. It should be replaced.
sub baseQuestionForm($) {
	my $this = shift @_;
	my $data = shift @_;

	return <<"DONE"
	$data->{js}
	$data->{top_link}
	<form method='post'>
	<input type='hidden' name='qcmd' value='$data->{qcmd}'>
	<input type='hidden' name='q_submit' value='1'>
	<input type='hidden' name='section_id' value='$data->{section_id}'>
	<input type='hidden' name='master_id' value='$data->{master_id}'>
	<input type='hidden' name='questionnaire_id' value='$data->{questionnaire_id}'>
	<input type='hidden' name='qbid' value='$data->{qbid}'>
	<table>
		<tr>
			<td>
				Question Label:
			</td>
			<td>
				<input type='text' name='label' value='$data->{label}' size='30'>
			</td>
		</tr>
		<tr>
			<td>
				Question Version:
			</td>
			<td>
				<input type='text' name='version' value='$data->{version}'>
			</td>
		</tr>
		<tr>
			<td>
				Question Format: 
			</td>
			<td>
				$data->{format}
			</td>
		</tr>
		<tr>
			<td>
				Possible answers (seperated by the pipe [|] character):
			</td>
			<td>
				<textarea name='value' rows='15' cols='30'>$data->{value}</textarea>
			</td>
		</tr>
		<tr>
			<td colspan='2'>
				$data->{buttons}
			</td>
		</tr>
	</table>
	</form>
DONE
}

# Print a drop down box containing all of the available question
# formats.
# &Modules::Questionnaire::QuestionBase::allFormats grabs the formats
# out of the actual datatype, so no hard coding of additional formats
# should be needed.
# &Modules::Questionnaire::QuestionBase::formatText makes the formats
# a little more human-readable, so additional formats may need to be included
# there. By default, a 'readable' format will be returned as the format
# string with the first letter in upper case.
sub baseQuestionFormatSelect($$) {
	my $this = shift @_;
	my $data = shift @_ || {};

	my $id = $data->{'id'};
	my $other = "";

	my @allFormats = &Modules::Questionnaire::QuestionBase::allFormats;

	foreach my $f (@allFormats) {
		if($data->{$f.'_selected'}) {
			$other = &Modules::Questionnaire::QuestionBase::formatInfo($f);
			last;
		}
	}
	if($data->{'select_selected'}) {
		$other .= "<br>Allow &quot;other&quot; input: <input type='checkbox' name='flags' value='other' $data->{other_checked}>";
	}

	my $out = "<select name='q_question_base#$id#format' onChange='javascript:formatOptions(this, \"$id\")'>\n";
	foreach my $f (@allFormats) {
		$out .= "<option value='".$f."' ".$data->{$f.'_selected'}.">".&Modules::Questionnaire::QuestionBase::formatText($f)."</option>\n";
	}
	$out .= "</select>\n";
	$out .= "<div id='formatOther'>\n$other\n</div>\n";
	return $out;
}

# New base question creation form
sub newBaseQuestion($) {
	my $this = shift @_;
	my $sid = shift @_;

	my %data = (
		'buttons' => $this->adminFormButtons(),
		'section_id' => $sid,
		'qcmd' => 'insert_question_base',
		'version' => 'English',
		'format' => $this->baseQuestionFormatSelect,
		'js' => $this->questionnaireJavascript(),
	);
	return $this->baseQuestionForm(\%data);
}

# New Final page creation form
sub newFinal($) {
	my $this = shift @_;
	my $sid = shift @_;
	my $qid = shift @_;

	my $questionnaire = new Modules::Questionnaire::Questionnaire($qid);
	my @versions = ();
	my $final = $questionnaire->finalMessage();
	my $finalBase = $final->fetchBase;
	my @usedVersions = $finalBase->fetchAllVersions();
	my %usedVersionHash = ();
	map {$usedVersionHash{$_->version} = 1;} @usedVersions;
	my $baseId = $finalBase->id;

	foreach my $v ($questionnaire->getVersions()) {
		push @versions, $v unless $usedVersionHash{$v};
	}

	my $reply = $this->reply || &relink();
	my $hiddens = $this->form->input_html('type' => 'hidden', 'name' => 'qcmd', 'value' => 'insert_final').
		$this->form->input_html('type' => 'hidden', 'name' => 'reply', 'value' => $reply).
		$this->form->input_html('type' => 'hidden', 'name' => 'q_submit', 'value' => '1').
		$this->form->input_html('type' => 'hidden', 'name' => 'q_question_base##section_id', 'value' => $sid).
		$this->form->input_html('type' => 'hidden', 'name' => 'q_question_base##master_id', 'value' => $finalBase->master->{'question_base_id'}).
		$this->form->input_html('type' => 'hidden', 'name' => 'q_question#'.$qid.'#question_id', 'value' => $final->id).
		$this->form->input_html('type' => 'hidden', 'name' => 'q_question#'.$qid.'#questionnaire_id', 'value' => $qid).
		$this->form->input_html('type' => 'hidden', 'name' => 'q_question_base##format', 'value' => 'label').
		$this->form->input_html('type' => 'hidden', 'name' => 'q_question#'.$qid.'#sort_key', 'value' => $Modules::Questionnaire::Questionnaire::FINAL_QUESTION_POSITION);
		$this->form->input_html('type' => 'hidden', 'name' => 'qqid', 'value' => $final->id);

	my $versionInput = $this->form->input_html('type' => 'select', 'name' => 'q_question_base##version', 'value' => '', 'options' => \@versions);

	my %data = (
		'hiddens' => $hiddens,
		'label_input' => $this->form->input_column('table' => 'q_question_base', 'column' => 'label', 'value' => ''),
		#'version_input' => $this->form->input_column('table' => 'q_question_base', 'column' => 'version', 'value' => 'English'),
		'version_input' => $versionInput,
		'value_input' => $this->form->input_column('table' => 'q_question_base', 'column' => 'value', 'value' => ''),
#		'buttons' => $this->adminFormButtons(),
		'buttons' => $this->form->make_form_buttons,
		'questionnaire_id' => $qid,
		'js' => $this->questionnaireJavascript(),
	);
	return &Modules::Questionnaire::QuestionnaireHTML::finalScreenForm(%data);
}

# Edit a previously-created final page form
sub editFinal {
	my $this = shift @_;
	my $qid = shift @_;

	my $questionnaire = new Modules::Questionnaire::Questionnaire($qid);
	my $final = $questionnaire->finalMessage();
	my $base = $final->fetchBase();
	my $qbid = $base->id;
	my $reply = $this->reply || &relink();

	my $hiddens = $this->form->input_html(type => 'hidden', name => 'qcmd', value => 'update_final').
		$this->form->input_html(type => 'hidden', name => 'reply', value => $reply).
		$this->form->input_html(type => 'hidden', name => 'q_submit', value => '1').
		$this->form->input_html(type => 'hidden', name => 'q_question_base#'.$qbid.'#section_id', value => $base->section_id).
		$this->form->input_html(type => 'hidden', name => 'q_question_base#'.$qbid.'#question_base_id', 'value' => $base->id);

	my %data = (
		'hiddens' => $hiddens,
		'label_input' => $this->form->input_column('table' => 'q_question_base', 'column' => 'label', 'record' => $qbid, 'value' => $base->label),
		'version_input' => $this->form->input_column('table' => 'q_question_base', 'column' => 'version', 'record' => $qbid, 'value' => $base->version),
		'value_input' => $this->form->input_column('table' => 'q_question_base', 'column' => 'value', 'record' => $qbid, 'value' => $base->value),
#		'label' => $base->label,
		'value' => $base->value,
		'buttons' => $this->adminFormButtons(),
		'section_id' => $base->section_id,
		'qcmd' => 'update_final',
		'js' => $this->questionnaireJavascript(),
	);
	my @version = $base->fetchAllVersions();
	unshift @version, $base unless $base->master_id;
	my $report = "";
	$report = $this->reportFinalScreens($final, @version) unless $base->master_id;
	my $out = &Modules::Questionnaire::QuestionnaireHTML::finalScreenForm(%data);
	$out .= $report;
	return $out;
}

# Edit a question base version
sub editVersion {
	my $this = shift @_;
	my $qbid = shift @_;

	return "" unless $qbid;

	my $base = new Modules::Questionnaire::QuestionBase($qbid);
	if($base->master_id) {
		my $masterId = $base->master_id;
		$base = new Modules::Questionnaire::QuestionBase($base->master_id, $base->version);
	}
	my $format = $base->format;

	my $formatHash = {
		$format.'_selected' => 'selected',
		'other_checked' => ($base->flags =~ /other/) ? "checked" : "",
		'id' => $qbid,
	};
	my $formatString = ($base->master_id) ? $base->formatLabel : $this->baseQuestionFormatSelect($formatHash);
	my $baseHash = {
		'label' => $base->label,
		'version' => $base->version,
		'value' => $base->value,
		'section_id' => $base->section_id,
		'master_id' => $base->master_id,
		'qcmd' => 'update_version',
		'qbid' => $qbid,
		'buttons' => $this->adminFormButtons(),
		'format' => $formatString,
		'flags' => $base->flags,
		'js' => $this->questionnaireJavascript(),
	};

	my @version = $base->fetchAllVersions();
	my $report = "";
	$report = $this->reportBaseQuestions($base, @version) unless $base->master_id;
	my $out = $this->baseQuestionForm($baseHash);
	$out .= $report;
	return $out;
} 

# Edit a master base question
sub editBaseQuestion {
	my $this = shift @_;
	my $qbid = shift @_;

	return "" unless $qbid;

	my $base = new Modules::Questionnaire::QuestionBase($qbid);
	if($base->master_id) {
		my $masterId = $base->master_id;
		$base = new Modules::Questionnaire::QuestionBase($base->master_id, $base->version);
	}
	my $format = $base->format;

	my $formatHash = {
		$format.'_selected' => 'selected',
		'other_checked' => ($base->flags =~ /other/) ? "checked" : "",
		'id' => $qbid,
	};
	my $formatString = ($base->master_id) ? $base->formatLabel : $this->baseQuestionFormatSelect($formatHash);
	my $baseHash = {
		'label' => $base->label,
		'version' => $base->version,
		'value' => $base->value,
		'section_id' => $base->section_id,
		'master_id' => $base->master_id,
		'qcmd' => 'update_question_base',
		'qbid' => $qbid,
		'buttons' => $this->adminFormButtons(),
		'format' => $formatString,
		'flags' => $base->flags,
		'js' => $this->questionnaireJavascript(),
		'top_link' => "<a href='".$this->topLink."'>Back to Questionnaire Control Panel</a>",
	};

	my @version = $base->fetchAllVersions();
	return $this->baseQuestionForm($baseHash);
}

# Form for creating a new question version
sub newVersionQuestion($) {
	my $this = shift @_;
	my $qbid = shift @_;

	return "" unless $qbid;

	my $buttons = $this->adminFormButtons();
	my $base = new Modules::Questionnaire::QuestionBase($qbid);
	my $format = $base->formatLabel;
	my $label = $base->label;
	my $value = $base->value;

	my $data = {
		'qcmd' => 'insert_version',
		'master_id' => $qbid,
		'label' => $base->label,
		'format' => $base->formatLabel,
		'value' => $base->value,
		'buttons' => $this->adminFormButtons(),
		'js' => $this->questionnaireJavascript(),
	};

	return $this->baseQuestionForm($data);
}

# Insert a base question from all of the base question-related
# fields in $input
# This method determines that every $input hash that contains
# a 'master_id' must be a sub-version of some master question
# (identified by master_id)
sub insertBaseQuestion($) {
	my $this = shift @_;
	my $input = shift @_;

	my %baseFields = (
		'section_id' => $input->{'section_id'},
		'master_id' => $input->{'master_id'},
		'label' => $input->{'label'},
		'version' => $input->{'version'},
		'format' => $input->{'format'},
		'value' => $input->{'value'},
		'flags' => $input->{'flags'},
	);
	my $base = new Modules::Questionnaire::QuestionBase();
	my $newId = 0;
	if($baseFields{'master_id'} && $baseFields{'master_id'} > 0) {
		$base->fetch($baseFields{'master_id'});
		$newId = $base->makeVersion(%baseFields);
	} else {
		$newId = $base->make(%baseFields);
	}
	return $newId;
}

# Method that takes care of insertion of the question_base
# and question records for the qusetionnaire final screen
sub insertFinalMessage {
	my $this = shift @_;
	my $qid = shift @_;
	my $qqid = shift @_;
	my $input = shift @_;
	my $baseInfo = shift @_;

	my $base = $this->insertBaseQuestion($baseInfo);
	my $q = undef;

	if(!$baseInfo->{'master_id'} || $baseInfo->{'master_id'} < 0) {
		my %questionInfo = (
			'question_base_id' => $base,
			'questionnaire_id' => $qid,
			'sort_key' => $Modules::Questionnaire::Questionnaire::FINAL_QUESTION_POSITION,
			'flags' => $input->{'flags'}
		);
		my $q = $this->linkQuestion($base, \%questionInfo);
	} 

	return $q;
}

# Does a simple database update. 
sub updateBaseQuestion($) {
	my $this = shift @_;
	my $qbid = shift @_;
	my $input = shift @_;

	my %baseFields = (
		'question_base_id' => $qbid,
		'section_id' => $input->{'section_id'},
		'master_id' => $input->{'master_id'},
		'label' => $input->{'label'},
		'version' => $input->{'version'},
		'format' => $input->{'format'},
		'flags' => $input->{'flags'},
		'value' => $input->{'value'},
	);
	my $upId = $share{'DB'}->update('q_question_base', \%baseFields);
}

# Insert a versioned entry for a base question
sub insertVersionQuestion($) {
	my $this = shift @_;
	my $qbid = shift @_;
	my $input = shift @_;

	return 0 unless $qbid;

	my $base = new Modules::Questionnaire::QuestionBase($qbid);

	my $sectionId = $base->get('master')->{'section_id'};

	my %baseFields = (
		'section_id' => $sectionId,
		'master_id' => $qbid,
		'label' => $input->{'label'},
		'version' => $input->{'version'},
		'format' => $base->format,
		'value' => $input->{'value'},
	);
	return $base->makeVersion(%baseFields);
}

# Javascript functionality related to adding questions to questionnaires
# To be honest, I'm not sure how this works or if it's still important
# It's being used in sub linkQuestionForm
sub linkQuestionJavascript($) {
	my $this = shift @_;
	my $questionnaires = shift @_ || [];

	my $allTotals = "var allTotals = new Array();\n";
	foreach my $q (@{$questionnaires}) {
		$allTotals .= "allTotals[".$q->id."] = ".($q->count+1).";\n";
	}
	return <<"DONE";
	function setDefaultPosition(form, id) {
$allTotals
		if(allTotals[id]) {
			form.sort_key.value=allTotals[id];
		} else {
			form.sort_key.value=1;
		}
	}
DONE
}

# Select box to be used on the questionnaire view/preview page.
sub linkQuestionQuestionnaireSelect($) {
	my $this = shift @_;
	my $base = shift @_;
	my $selectedId = shift @_;

	return "" unless ref($base) =~ /QuestionBase/;

	my $sid = $base->section_id;
	my $questionnaires = $this->fetchAllQuestionnaires($sid);
	my @questions = $base->fetchQuestions;
	my %usedQuestionnaires = ();
	map {$usedQuestionnaires{$_->questionnaire_id} = 1;} @questions;
	my $qList = "<select name='questionnaire_id' onChange='javascript:setDefaultPosition(this.form, this.value)'>\n<option value=''></option>\n";
	foreach my $q (@{$questionnaires}) {
		next if $q->id != $selectedId && $usedQuestionnaires{$q->id};
		my $selected = (defined($selectedId) && ($selectedId == $q->id)) ? " selected" : "";
		my $numQuestions = $q->count;
		$qList .= "<option value='".$q->id."'$selected>".$q->title."</option>\n";
	}
	$qList .= "</select>\n";
	return $qList;
}

sub editQuestion($) {
	my $this = shift @_;
	my $qqid = shift @_;
	my $version = shift @_;

	return "" unless $qqid;

	my $question = new Modules::Questionnaire::Question($qqid);
	my $mandatory_selected = ($question->isFlagged('mandatory')) ? " selected" : "";
	my %data = (
		'qcmd' => 'update_question',
		'label' => $question->label,
		'questionnaire_list' => $this->linkQuestionQuestionnaireSelect($question->fetchBase, $question->questionnaire_id),
		'sort_key' => $question->sort_key,
		'mandatory_selected' => $mandatory_selected,
		'buttons' => $this->adminFormButtons,
	);

	return $this->questionForm(\%data);
}

sub linkQuestionForm($) {
	my $this = shift @_;
	my $sid = shift @_;
	my $qbid = shift @_;
	my $version = shift @_;

	return "" unless $qbid;

	my $base = new Modules::Questionnaire::QuestionBase($qbid, $version);

	my $qList = $this->linkQuestionQuestionnaireSelect($base);
	my @questions = $base->fetchQuestions;
	my $sid = $base->get('current')->{'section_id'};
	my $label = $base->get('current')->{'label'};
	my $questionnaires = $this->fetchAllQuestionnaires($sid);
	my $js = $this->linkQuestionJavascript($questionnaires);

	my %data = (
		'qcmd' => 'link_action',
		'question_base_id' => $base->id,
		'questionnaire_list' => $qList,
		'js' => $js,
		'label' => $label,
		'section_id' => $base->section_id,
		'sort_key' => '',
		'regular_selected' => '',
		'mandatory_selected' => '',
		'buttons' => $this->adminFormButtons(),
	);
	return $this->questionForm(\%data);
}

sub questionForm($) {
	my $this = shift @_;
	my $data = shift @_;

	return <<"DONE";
<script language='javascript'>
<!--
$data->{js}
-->
</script>
	Link question <b>$data->{label}</b> to questionnaire:
	<form method='post'>
		<input type='hidden' name='qcmd' value='$data->{qcmd}'>
		<input type='hidden' name='q_submit' value='1'>
		<input type='hidden' name='section_id' value='$data->{section_id}'>
		<input type='hidden' name='qbid' value='$data->{question_base_id}'>
		<table>
			<tr>
				<td>
					Select Questionnaire:
				</td>
				<td>
					$data->{questionnaire_list}
				</td>
			</tr>
			<tr>
				<td>
					Position:
				</td>
				<td>
					<input type='text' value='$data->{sort_key}' name='sort_key'>
				</td>
			</tr>
			<tr>
				<td>
					Question type:
				</td>
				<td>
					<select name='flags'>
						<option value='' $data->{regular_selected}>Regular</option>
						<option value='mandatory' $data->{mandatory_selected}>Mandatory</option>
					</select>
				</td>
			</tr>
			<tr>
				<td colspan='2'>
					$data->{buttons}
				</td>
			</tr>
		</table>
	</form>
DONE
}

# Actually links a base question to a questionnaire by calling
# Modules::Questionnaire::Question::link
sub linkQuestion($$) {
	my $this = shift @_;
	my $qbid = shift @_;
	my $data = shift @_;
	my $base = new Modules::Questionnaire::QuestionBase($qbid);
	my $question = {
		'question_base_id' => $qbid,
		'questionnaire_id' => $data->{'questionnaire_id'},
		'sort_key' => $data->{'sort_key'},
		'flags' => $data->{'flags'}
	};
	my $q = new Modules::Questionnaire::Question();
	my $id = $q->link($question, $base->get('current'), $data->{'flags'});
	return $id;
}

# Simple question update
sub updateQuestion($$) {
	my $this = shift @_;
	my $qqid = shift @_;
	my $input = shift @_;

	my $question = new Modules::Questionnaire::Question($qqid);
	return undef unless $question->get('q_question');
	return $question->update(%{$input});
}

# Fetch all of the questionnaires for the site identified by
# $sid 
sub fetchAllQuestionnaires($) {
	my $this = shift @_;
	my $sid = shift @_;

	my $questionnaires = $this->get('allQuestionnaires');
	return $questionnaires if $questionnaires;

	$questionnaires = $share{'DB'}->fetch_match('q_questionnaire',
		{'section_id' => $sid}
	);

	my $qObjs = [];
	foreach my $q (@{$questionnaires}) {
		push @{$qObjs}, new Modules::Questionnaire::Questionnaire($q);
	}

	$this->set('allQuestionnaires', $qObjs);
	return $qObjs;
}

# Deletes a base question identified by $qbid
# Uses Modules::Questionnaire::QuestionBase::delete to do the
# actual deletion
sub deleteQuestionBase($) {
	my $this = shift @_;
	my $qbid = shift @_;

	return undef unless $qbid;
	my $base = new Modules::Questionnaire::QuestionBase($qbid);
	my @deleted = $base->delete();
	return @deleted;
}

# Unlink question (identified by qqid) from a questionnaire
sub unlinkQuestion($) {
	my $this = shift @_;
	my $qqid = shift @_;
	my $question = new Modules::Questionnaire::Question($qqid);
	return $question->unlink;
}

# Unlink a final question (ie. thank you screen) from a questionnaire.
# In this case (and only this case), unlinking the question from the questionnaire
# also causes the base question to be deleted
sub unlinkFinal($) {
	my $this = shift @_;
	my $qqid = shift @_;

	my $question = new Modules::Questionnaire::Question($qqid);
	
	return undef unless $question->id && $question->fetchBase;

	my $base = new Modules::Questionnaire::QuestionBase($question->fetchBase->id);
	$question->unlink;
	$base->delete;
	return;
}

# Simple boolean to help us control the flow
sub isDeleteCommand($) {
	my $this = shift @_;
	my $qcmd = shift @_;

	if($qcmd =~ /^(delete|unlink)/) {
		return 1;
	}

	return 0;
}

# This method holds all of the individual deletes and should know
# what to do for each case.
sub performDelete($) {
	my $this = shift @_;
	my $qcmd = shift @_;
	my $input = shift @_;

	my $qid = $input->{'qid'};
	my $qqid = $input->{'qqid'};
	my $qbid = $input->{'qbid'};
	my $sid = $input->{'section_id'};

	my $delResult = 0;
	my $msg = "";
	if($qcmd eq "unlink") {
		($delResult, $msg) = $this->unlinkQuestion($qqid);
	} elsif($qcmd eq "unlink_final") {
		($delResult, $msg) = $this->unlinkFinal($qqid);
	} elsif($qcmd eq "delete_questionnaire") {
		($delResult, $msg) = $this->deleteQuestionnaire($qid);
	} elsif($qcmd =~ /delete_question_base|delete_final_version|delete_version/) {
		($delResult, $msg) = $this->deleteQuestionBase($qbid);
	} elsif($qcmd eq "delete_response_filter") {
		($delResult, $msg) = $this->deleteResponseFilter($input->{'response_filter_id'});
	} elsif($qcmd eq "delete_response_match") {
		($delResult, $msg) = $this->deleteResponseMatch($input->{'response_match_id'});
	} elsif($qcmd eq "delete_report") {
		($delResult, $msg) = $this->deleteReport($input->{'report_id'});
	} elsif($qcmd eq "delete_report_question") {
		($delResult, $msg) = $this->deleteReportQuestion($input->{'report_id'}, $input->{'qqid'}, $input->{'qid'});
	}

	$this->error($msg) if $msg;

	return $delResult;
}

# Generic buttons for administrative pages
sub adminFormButtons($) {
	my $this = shift @_;

	my $link = $this->cancelButtonLink();

	return <<"DONE";
<input type='submit' value='Submit'>&nbsp;<input type='reset' value='Reset'>&nbsp;<input type='button' value='Cancel' onclick='$link'>
DONE
}

# Buttons to be printed for the questionnaire.
# The submit requires the use of 'validateSubmit' in order to
# properly validate an input form
sub questionnaireButtons($) {
	my $this = shift @_;

	my $link = $this->cancelButtonLink();

	return <<"DONE";
<input type='button' value='Submit' onClick='javascript:validateSubmit(this.form)'>&nbsp;<input type='reset' value='Reset'>&nbsp;<input type='button' value='Cancel' onclick='$link'>
DONE
}

# For committing changes to the questionnaire layout:
# question ordering, questions used, etc.
sub adminPreviewButtons($) {
	my $this = shift @_;

	my $link = $this->cancelButtonLink();
	my $reply = &relink();
	return <<"DONE";
<input type='hidden' name='submit_action' value='Submit'>
<input type='hidden' name='reply' value='$reply'
><input type='button' name='Submit' value='Save' onClick='adminPreviewSubmit(this.form)'>
DONE

}

sub previewQuestionnaireButtons($) {
	my $this = shift @_;
	
	my $link = &relink('qcmd' => undef, 'qid' => undef, 'qid' => undef, 'questionnaire_id' => undef);
	return <<"DONE";
<input type='button' value='Back' onClick='document.location=\"$link\"'>
DONE
}

# Was intended to be used everywhere for all 'cancel' buttons.
# I'd be surprised if it's being used at all.
sub cancelButtonLink($) {
	my $this = shift @_;

	return "history.back();";
}

# Prints the top wrapper for a questionnaire.
# This mostly includesa any special preview-specific functionality.
sub openQuestionnaire($) {
	my $this = shift @_;
	my $questionnaire = shift @_;
	my $version = shift @_;
	my $previewFlag = shift @_;
	my $responseId = shift @_;

	my $buttons = $this->adminPreviewButtons() if $previewFlag;
	my $reply = &relink();

	my %data = (
		'qid' => $questionnaire->id,
		'js' => $this->questionnaireJavascript($questionnaire, $previewFlag),
		'hiddens' => $this->previewHiddens($questionnaire, $previewFlag),
		'previewStyle' => ($previewFlag) ? "style='width: 100%; border-width: 2px; border-color: black; border-style: solid'" : "",
		'version' => $version,
		'buttons' => $buttons,
		'eviteLink' => $this->eviteLink,
		'eviteForm' => &Modules::Questionnaire::QuestionnaireHTML::hiddenEviteForm('reply' => $reply),
		'response_id' => $responseId,
		'target' => "action='".$this->link()."'",
	);

	return &Modules::Questionnaire::QuestionnaireHTML::openQuestionnaire(%data);
}

# Closes a questionnaire by closing the wrapper table,
# printing the submission buttons and closing the form
sub closeQuestionnaire($) {
	my $this = shift @_;
	my $qid = shift @_;
	my $previewFlag = shift @_;

	my $buttons = ($previewFlag) ? $this->previewQuestionnaireButtons($qid) : $this->questionnaireButtons($qid);
	my $buttons = $this->questionnaireButtons($qid);
	return "\t<tr>\n\t\t<td>\n\t\t\t".$buttons."\n\t\t</td>\n\t</tr>\n</form>\n</table>\n";
}

# Any javascript specific to a questionnaire is printed here.
# The amount of output depends on whether or not $previewFlag
# is set.
sub questionnaireJavascript($) {
	my $this = shift @_;
	my $questionnaire = shift @_;
	my $previewFlag = shift @_;

	my $mandatories = "";
	my $slider = "";
	my $previewFunctions = "";
	
	my %formatInfo = ();
	my @formats = &Modules::Questionnaire::QuestionBase::allFormats();

	foreach my $f (@formats) {
		$formatInfo{$f} = &Modules::Questionnaire::QuestionBase::formatInfo($f);
	}

	if($questionnaire) {
		$mandatories = $questionnaire->show_mandatory_js;
		$slider = $questionnaire->show_slider_js;
	}
	if($previewFlag) {
		$previewFunctions = $this->previewJavascript($questionnaire);
	} else {
		$previewFunctions = $this->dummyJavascript;
	}

	return <<"DONE";
	$slider
	<script language='javascript'>
	<!--
	$previewFunctions

	var mandatory = new Array();
	function makeMandatory(id, name) {
		mandatory[id] = name;
	}

	function validateSubmit(form) {
		if(form) {
			for (id in mandatory) {
				for(var i = 0; i < form.elements.length; i++) {
					if(form.elements[i].name == id) {
						if(!form.elements[i].value) {
							alert(mandatory[id]+' value is missing');
							return false;
						}
						break;
					}
				}
			}
			return form.submit();
		}
	}

	function formatOptions(element, id) {
		if(!element) {
			return;
		}
		if(!id) {
			id = "";
		}
		var value = element.value;
		var div = document.getElementById('formatOther');
		var inputName = 'q_question_base#'+id+'#flags';

		if(div && value == 'select') {
			div.innerHTML = "$formatInfo{select}";
			div.innerHTML += "<br>Allow \\\"other\\\" input: <input type='checkbox' name='"+inputName+"' value='other'>";
		} 
		else if(value == 'radio') {
			div.innerHTML = "$formatInfo{radio}";
		}
		else if(value == 'multiselect') {
			div.innerHTML = "$formatInfo{multiselect}";
		}
		else if(div && value == 'slider') {
			div.innerHTML = "$formatInfo{slider}";
		}
		else if(div && value == 'label') {
			div.innerHTML = "$formatInfo{label}";
		}
		else if(div) {
			div.innerHTML = '';	
		}
	}
	
	$mandatories
	-->
	</script>
DONE
}

sub previewJavascript($) {
	my $this = shift @_;
	my $questionnaire = shift @_;

	my @questionList = $questionnaire->getAllQuestions();
	my @unused = $questionnaire->getUnusedQuestions($questionnaire->version);

	my $swapArrows = $this->swapArrows();
	my $reply = &relink();
	my $editQuestionLink = &relink('qcmd' => 'edit_master', 'reply' => $reply);
	my $questionArray = "var questionArray = new Array();\n";
	my $baseArray = "var baseArray = new Array();\n";
	my $javascript = "";
	if($questionList[0]) {
		my $lastPosition = $questionList[0]->sort_key;
		for(my $i = 0; $i < @questionList; $i++) {
			my $baseId = $questionList[$i]->fetchBase->master->{'question_base_id'};
			my $position = $questionList[$i]->sort_key || $lastPosition + 1;
			my $arrayKey = 'q'.($i+1);
			my $label = $questionList[$i]->label;
			$label =~ s/\n/\\n/g;
			$label =~ s/\r/\\r/g;
			$label =~ s/\"/\&\#35\;/g;
			$label =~ s/\'/\&quot\;/g;
			$questionArray .= "questionArray['$arrayKey'] = new Array($baseId, ".$questionList[$i]->id.", ".$position.")\n";
			$lastPosition = $questionList[$i]->sort_key || $position;
			$baseArray .= "baseArray[$baseId] = new Array(\"$label\", \"\", ".$questionList[$i]->id.");\n";
		}
	}
	for(my $i = 0; $i < @unused; $i++) {
		my $id = $unused[$i]->master->{'question_base_id'};
		my $label = $unused[$i]->label;
		my $html = $unused[$i]->show('u_'.$id);
		$html =~ s/\n/\\n/g;
		$html =~ s/\r/\\r/g;
		$html =~ s/\"/\\"/g;
		$html =~ s/\'/\\'/g;
		$label =~ s/\"/\&\#35\;/g;
		$label =~ s/\'/\&quot\;/g;
		my $js = $unused[$i]->hasJavascript;
		$baseArray .= "baseArray[$id] = new Array(\"$label\", \"$html\", $js);\n";
		$javascript .= "case '$id':\n".$unused[$i]->show_js.";\nbreak;" if $js;
	}

	my $nextDivId = "q".($questionnaire->count+1);

	return <<"DONE";
	var highlighted = null;
	var unusedHighlighted = null;
	var origHtml = null;
	var currentQuestion = null;
	var arrowsNode = null;
	var rmList = new Array();
	var addList = new Array();
	var sortList = new Array();
	var nextDiv = '$nextDivId';
	var dirty = false;

	$questionArray

	$baseArray

	function runQuestionJavascript(id) {
		switch(id) {
			$javascript 
			default:
				break;
		}
		return id;
	}

	function addUnused(id) {
		if(!id) {
			alert('addUnused: ID missing');
			return null;
		}
		if(addList[id]) {
			alert('addUnused: invalid ID: '+id);
			return null;
		}
		var select = document.getElementById('questionsSelect');

		if(!select) {
			alert('addUnused: unable to find question select list');
			return null;
		}
		var option = new Option(baseArray[id][0], id);
		select.options[select.options.length] = option; 
		return select;
	}

	function highlightUnused(id) {
		if(!id) {
			alert('highlightUnused: no ID supplied');
			return null;
		}
		if(addList[id]) {
			alert('highlightUnused: invalid ID: '+id);
			return null;
		}
		var div = document.getElementById(id);
		if(!div) {
			alert('highlightUnused: unable to locate question DIV');
			return null;
		}
		if(unusedHighlighted) {
			unusedHighlighted.innerHTML = origHtml;
			unusedHighlighted.style.backgroundColor = this.parent.backgroundColor || this.parent.parent.backgroundColor || "white";
		}
		origHtml = div.innerHTML;
		div.style.backgroundColor = 'red';
		var numId = parseInt(id.substr(2));
		div.innerHTML = "<a onClick=addQuestion\('"+numId+"'\)><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/new12.gif' border='0'></a> "+baseArray[numId][0];
		unusedHighlighted = div;
		return unusedHighlighted;
	}

	function highlight(id) {
		if(!id) {
			alert('highlight: no ID supplied');
			return null;
		}
		var div = document.getElementById(id);
		if(!div || !div.innerHTML) {
			alert('highlight: invalid ID supplied: ('+id+')');
			return null;
		}
		if(!highlighted) {
			origHtml = div.innerHTML;
			div.className = 'highlight';
			arrowsNode = div.id;
			div.innerHTML += "<div style='text-align: right'>$swapArrows<a onClick='javascript:editQuestion()'><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/edit12.gif' border='0'></a><a onClick='javascript:deleteQuestion()'><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/rm12.gif' border='0'></a></div>";
		}
		else if(highlighted && highlighted != div) {
			highlighted.className = '';
			highlighted.innerHTML = origHtml;
			origHtml = div.innerHTML;
			div.className = 'highlight';
			if(arrowsNode != div.id) {
				div.innerHTML += "<div style='text-align: right'>$swapArrows<a onClick='javascript:editQuestion()'><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/edit12.gif' border='0'></a><a onClick='javascript:deleteQuestion()'><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/rm12.gif' border='0'></a></div>";
				arrowsNode = div.id
			}
		}
		highlighted = div;
		return highlighted;
	}

	function swapDivs(direction) {
		if(!highlighted) {
			alert('No question selected. Please select a question and try again');
		} 
		var div1 = highlighted;
		var div2 = null;
		var div2Key = null;
		var div1Pos = parseInt(div1.id.substr(1));
		var div2Pos = div1Pos + parseInt(direction);
		div2Key = 'q'+div2Pos;
		div2 = document.getElementById(div2Key);

		if(!div2) {
			alert('Unable to find question for swapping!');
		}
		dirty = true;
		var tmpHtml = div1.innerHTML;
		var tmpBorder = new Array(div1.style.borderColor, div1.style.borderWidth, div1.style.borderStyle);
		div1.style.borderColor = div2.style.borderColor;
		div1.style.borderWidth = div2.style.borderWidth;
		div1.style.borderStyle = div2.style.borderStyle;
		div2.style.borderColor = tmpBorder[0];
		div2.style.borderWidth = tmpBorder[1];
		div2.style.borderStyle = tmpBorder[2];
		div1.innerHTML = div2.innerHTML;
		div2.innerHTML = tmpHtml;
		div2.className = 'highlight';
		div1.className = '';
		highlighted = div2;
		swapDivArrayPositions(div1.id, div2.id);
		arrowsNode = div2.id
		return null;
	}

	function swapDivArrayPositions(key1, key2) {
		var e1 = questionArray[key1];
		var e2 = questionArray[key2];

		var tmpQId = e1[0];
		var tmpBId = e1[1];
		e1[0] = e2[0];
		e1[1] = e2[1];
		e2[0] = tmpQId
		e2[1] = tmpBId

		// probably not required, but what the heck
		questionArray[key1] = e1;
		questionArray[key2] = e2;
		return null;
	}

	function alertQuestions() {
		var output = '';
		for (var key in questionArray) {
			output += key + ': ' + questionArray[key] + "\\n";
		}
		alert(output);
		return null;
	}

	function adminPreviewSubmit(form) {
		// Update all position values
		if(dirty) {
			form.qcmd.value = 'preview_changes';
			var sortElement = document.getElementById('sortList');
			for (var key in questionArray) {
				if(questionArray[key]) {
					if(questionArray[key][2]) {
						sortElement.value += questionArray[key][0]+':'+questionArray[key][2]+','
					}
				}
			}
			var rmKey = document.getElementById('rmList');
			for (var key in rmList) {
				if(rmList[key]) {
					rmKey.value += key+',';
				}
			}
			var addKey = document.getElementById('addList');
			for (var key in addList) {
				if(addList[key]) {
					addKey.value += key+',';
				}
			}
			return form.submit();
		}
		alert('No changes made.');
		return false;
	}

	function deleteQuestion() {
		if(!highlighted) {
			alert('Unable to delete question; none selected.');
			return null;
		}
		dirty = true;
		highlighted.className = 'Question';
		baseArray[questionArray[highlighted.id][0]][1] = origHtml;
		highlighted.innerHTML = '';
		origHtml = '';
		var qElement = questionArray[highlighted.id];
		rmList[qElement[1]] = 1;
		addList[qElement[0]] = null;
		addUnused(qElement[0]);
		shiftDivs(highlighted.id.substr(1));
		questionArray[nextDiv] = null;
		highlighted = null;
		return null;
	}

	function shiftDivs(start) {
		var i = parseInt(start);
		var id = 'q'+i;
		var div = null;
		var nextFree = null;
		while(document.getElementById(id)) {
			div = document.getElementById('q'+i);
			var next = 'q'+(i+1);
			if(!next) {
				div.innerHTML = '';
				nextDiv = div.id;
				break;
			} else if (!next.innerHTML || next.innerHTML.length == 0) {
				nextDiv = div.id;
				div.innerHTML = '';
				break;
			}
			alert('qusetionArray['+id+'] = questionArray['+next+'])');
			questionArray[id] = questionArray[next];
			div.innerHTML = next.innerHTML;
			i++;
			id = 'q'+i;
		}
		return (i - start);
	}

	function listQuestions() {
		var questionsHtml = '';
		for(var i in questionArray) {
			questionsHtml += questionArray[i]+"\\n";
		}
		alert(questionsHtml);
		return questionsHtml;
	}

	function addQuestion() {
		dirty = true;
		var select = document.getElementById('questionsSelect');
		var newDiv = document.getElementById(getNextDiv());

		if(!select || !newDiv) {
			return null;
		}
		var id = select.options[select.selectedIndex].value;
		var sortKey = newDiv.id.substr(1);
	
		if(!baseArray[id] || !baseArray[id][1]) {
			alert('Error: Question is either missing, or has been setup improperly.');
			return null;
		}

		addList[id] = sortKey;
		rmList[id] = null;
		newDiv.innerHTML = baseArray[id][1];
		questionArray[newDiv.id] = new Array(id, null, sortKey);
		select.options[select.selectedIndex] = null;
		highlight(newDiv.id);

		// baseArray[id][2] means there's a javascript element
		if(baseArray[id][2]) {
			runQuestionJavascript(id);
		}

		return newDiv;
	}

	function addQuestionOLD(id) {
		dirty = true;
		var newDiv = document.getElementById(getNextDiv());
		if(!newDiv) {
			return null;
		}
		var baseId = id;
		var sortKey = newDiv.id.substr(1);
		var origDiv = document.getElementById('u_'+id);
		addList[id] = sortKey;
		rmList[id] = null;
		newDiv.innerHTML = baseArray[id][1];
		newDiv.style.borderWidth = 1;
		newDiv.style.borderColor = 'black';
		newDiv.style.borderStyle = 'solid';
		origDiv.innerHTML = '';
		questionArray[newDiv.id] = new Array(baseId, null, sortKey);
		unusedHighlighted = null;
		return newDiv;
	}

	function getNextDiv() {
		var returnDiv = nextDiv;
		var id = parseInt(nextDiv.substr(1));
		id++;
		nextDiv = 'q'+id;
		return returnDiv;
	}

	// take us to the 'edit question' page for the highlighted
	// question
	function editQuestion() {
		if(!highlighted) {
			return null;
		}
		var id = highlighted.id;
		var baseId = questionArray[id][0];
		window.location='$editQuestionLink&qbid='+baseId;
		return baseId;
	}
DONE
}

sub dummyJavascript($) {
	my $this = shift @_;

	return <<"DONE";
	function highlight() {
		return null;
	}
DONE
}

sub swapArrows($) {
	my $this = shift @_;

	return "<a onClick='javascript:swapDivs(-1)'><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/up12.gif' border='0'></a><a onClick='javascript:swapDivs(1)'><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/down12.gif' border='0'></a>";
}

sub divDeleteButton($) {
	my $this = shift @_;
	my $divId = shift @_;

	return "<a onClick='javascript:deleteQuestion($divId)'><img src='$config{server}->{HTMLpatH}/_Modules/Questionnaire/rm12.gif' border='0'></a>";
}

sub populateFormJavascript($) {
	my $this = shift @_;
	my $questionnaire = shift @_;

	my @questions = $questionnaire->getAllQuestions;
	my $answers = $this->get('userAnswers');
	my $answerHash = { };
	map { $answerHash->{$_->{'question_id'}} = $_; }

	my $out = "";
	foreach my $q (@questions) {
		my $answer = $answerHash->{$q->id};
		$answer->{'name'} = $q->label;
		$out .= $this->setFormValueJavascript("document.questionnaireForm", $answer);
	}
	return $out;
}

sub setFormValueJavascript($) {
	my $this = shift @_;
	my $form = shift @_;
	my $q = shift @_;

	return <<"DONE";
${form}.$q->{name}.value = $q->{value};
DONE
}

sub questionnaireSummary($) {
	my $this = shift @_;
	my $qid = shift @_;

	return "" unless $qid;
	my $out = "";
	my $questionnaire = new Modules::Questionnaire::Questionnaire($qid);

	my @questions = $questionnaire->getAllQuestions();
	
	foreach my $q (@questions) {
		my @values = $q->getValues() ;
		@values = $q->getAnswerValues() if !scalar(@values);
		my $data = [];
		if(@values) { 
			foreach my $v (@values) {
				my $percentile = sprintf("%2.0f%", ($q->answerCount($v)/$q->totalAnswerCount()) * 100);
				push @{$data}, [$v, $q->answerCount($v), $percentile];
			}
		}
		my %report = (
			'title' => $q->label,
			'data' => $data,
			'headers' => ['Choice', 'Count', 'Percent'],
		);
		$out .= $this->reportbuilder(\%report)."<br>\n";
	}
	
	return $out;
}

sub previewHiddens($) {
	my $this = shift @_;

	my $out = <<"DONE";
	<input type='hidden' id='sortList' value='' name='sortList'>
	<input type='hidden' id='rmList' value='' name='rmList'>
	<input type='hidden' id='addList' value='' name='addList'>
DONE
	return $out;
}

sub questionnaireThankyou($$$) {
	my $this = shift @_;
	my $qid = shift @_;
	my $version = shift @_;

	my $questionnaire = new Modules::Questionnaire::Questionnaire($qid, $version);

	my $final = $questionnaire->finalMessage();

	my $out = "";
	if($final->format =~ /label/) {
		my $question = $final->show(0, undef, $version);
		$out =<<"DONE";
	<table>
		$question
	</table>
DONE
	} 
	return $out;
}

sub previewUpdate($$) {
	my $this = shift @_;
	my $qid = shift @_;
	my $data = shift @_;

	my $questionnaire = new Modules::Questionnaire::Questionnaire($qid, $data->{'version'});
	my @questions = $questionnaire->getAllQuestions();
	my %qHash = ();
	map {$qHash{$_->id} = $_;} @questions;
	my %baseQuestionHash = ();
	my @unlinkList = split /\s*,\s*/, $data->{'rmList'};
	my @addList = split /\s*,\s*/, $data->{'addList'};
	my @sortQuestions = split /\s*,\s*/, $data->{'sortList'};

	foreach my $base (@addList) {
		my %questionInfo = (
			'questionnaire_id' => $qid,
			'question_base_id' => $base,
			'flags' => '',
		);
		$questionnaire->addQuestion(\%questionInfo, \%questionInfo);
	}
	my @questions = $questionnaire->getAllQuestions();

	foreach my $q (@unlinkList) {
		my $question = $qHash{$q};
		$questionnaire->delQuestion($question);
		delete $qHash{$q};
	}

	# Grab all of the questions from the questionnaire object again
	# since some of the questions have been updated (added and/or deleted)
	# and the Modules::Questionnaire::Questionnaire object should be keeping
	# track (without requiring additional DB fetches)
	map {$baseQuestionHash{$_->fetchBase->id} = $_;} $questionnaire->getAllQuestions();
	foreach my $s (@sortQuestions) {
		my ($baseId, $sortKey) = split /\s*:\s*/, $s;
		my $q = $baseQuestionHash{$baseId};	
		if($q) {
			$share{'DB'}->update('q_question', {'question_id' => $q->id, 'sort_key' => $sortKey}) if $q;
		}
	}

	return;
}

sub finalMessagePreview($) {
	my $this = shift @_;
	my $questionnaire = shift @_;
	my $finalMessage = shift @_;

	my $label = $finalMessage->label;
	my $text = $finalMessage->value;
	my $finalMessageAction = ($finalMessage->fetchBase->id >= 0) ? "edit_final" : "new_final";
	my $deleteLink = "";
	my $reply = &relink();
	my $editLink = "<a href='".&relink('qcmd' => $finalMessageAction, 'qid' => $questionnaire->id, 'section_id' => $questionnaire->section_id, 'qbid' => $finalMessage->fetchBase->id, 'reply' => $reply)."'><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/edit12.gif' border='0'></a>\n";

	if($finalMessage->id > 0) {
		$deleteLink = "<a href='".&relink('qcmd' => 'unlink_final', 'qid' => $questionnaire->id, 'section_id' => $questionnaire->section_id, 'qqid' => $finalMessage->id, 'reply' => $reply)."'><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/rm12.gif' border='0'></a>\n";
	}

	return <<"DONE";
	<h2>Form Final Message</h2>
	<div id='finalMessage' style='border-width: 1px; border-style: solid;'>
	$deleteLink
	$editLink
	<table>
		<tr>
			<td>
				$label
			</td>
		</tr>
		<tr>
			<td>
				$text
			</td>
		</tr>
	</table>
	</div>
DONE
}

sub baseDivs($) {
	my $this = shift @_;
	my $questionnaire = shift @_;
	my $version = shift @_;

	return "" unless ref($questionnaire) =~ /Questionnaire/;

	my $count = $questionnaire->count + 1;
	my @unused = $questionnaire->getUnusedQuestions($version);

	return "" if (scalar(@unused) == 0);

	my $out = "";
	for(my $i = $count; $i < (scalar(@unused) + $count); $i++) {
		$out .= "<tr><td><div id='q$i' onClick='javascript:highlight(\"q$i\");'></div></td></tr>\n";
	}
	return $out;
}

sub topLink($) {
	my $this = shift @_;

	my %query = &DecodeString($ENV{'QUERY_STRING'});
	foreach my $k (%query) {
		$query{$k} = undef;
	}
	return &relink(%query);
}

sub backLink($) {
	my $this = shift @_;
	my $qcmd = shift @_;
	my $qid = shift @_;
	my $input = shift @_ || {};

	my %emptyLinks = ();

	foreach my $k (keys %{$input}) {
		$emptyLinks{$k} = undef if($k =~ /^report_/);
	}

	my %link = (
		'questionnaire_view' => {
			'label' => 'Section View',
			'reply' => undef,
			'response_details' => undef, 
			'view_type' => undef,
			'version' => undef,
			'qid' => undef, 
			'qbid' => undef, 
			'qqid' => undef,
			'qcmd' => 'section_view',
			%emptyLinks,
		},
		'section_view' => {
			'label' => 'All Sites View',
			'view_type' => undef,
			'qid' => undef, 
			'qbid' => undef, 
			'qqid' => undef, 
			'qcmd' => '',
			'version' => undef,
			%emptyLinks,
		},
		'question_base_view' => {
			'label' => 'Section View', 
			'view_type' => undef,
			'qid' => undef, 
			'qbid' => undef, 
			'qqid' => undef, 
			'qcmd' => 'section_view',
			%emptyLinks,
		},
	);

	my $label = $link{$qcmd}->{'label'};
	delete $link{$qcmd}->{'label'};
	if($qcmd =~ /question_base_view/ && $qid) {
		$link{$qcmd}->{'qid'} = $qid;
		$link{$qcmd}->{'qcmd'} = 'questionnaire_view';
		$link{$qcmd}->{'version'} = undef;
		return "<a href='".&relink(%{$link{$qcmd}})."'>Questionnaire View</a>\n";
	}

	return "<a href='".&relink(%{$link{$qcmd}})."'>$label</a>\n" if defined($link{$qcmd});
}

sub approveBaseVersion($$$) {
	my $this = shift @_;
	my $qbid = shift @_;
	my $reply = shift @_;

	return undef unless $qbid;

	my $base = new Modules::Questionnaire::QuestionBase($qbid);
	my $master = $base->master;
	my $approve = $base->approve();
	if($reply) {
		print "Location: $reply\n\n";
	}
	return $approve;
}

################# CTRL PANEL PAGES
sub allSitesView($) {
	my $this = shift @_;

	my @sites = $share{'DB'}->my_real_sites;
	if(scalar(@sites) == 1) {
		return $this->sectionView($sites[0]->{'section_id'});
	}

	my $data = [];

	foreach my $s (@sites) {
		my @questionnaires = $this->fetchSiteQuestionnaires($s->{'section_id'});
		my @baseQuestions = $this->fetchSiteBaseQuestions($s->{'section_id'});
		my $title = "<a href='".&relink('qcmd' => 'section_view', 'section_id' => $s->{'section_id'})."'>$s->{title}</a>";
		push @{$data}, [$title, scalar(@baseQuestions), scalar(@questionnaires)];
	}

	my %report = (
		'title' => "All Sites",
		'headers' => ['Site', 'Questions', 'Questionnaires'],
		'data' => $data,
	);

	return &Modules::Questionnaire::Report::reportbuilder(%report);
}

sub sectionView($) {
	my $this = shift @_;
	my $sid = shift @_;

	return undef unless $sid;

	my $questionnaires = $this->listQuestionnaires($sid);
	my @questions = $this->fetchSiteBaseQuestions($sid);
	my @formats = &Modules::Questionnaire::QuestionBase::allFormats;
	my %formatHash = ();
	my %data = ();
	my $questionIcon = "<img src=\"$config{server}->{HTMLpath}/_Modules/Questionnaire/add.gif\" border=\"0\">";
	my $jsIcons = "SetIcons(\"<img src=\\\"$config{server}->{HTMLpath}/_Modules/Questionnaire/add.gif\\\" border=\\\"0\\\">\", \"<img src=\\\"$config{server}->{HTMLpath}/_Modules/Questionnaire/minus.gif\\\" border=\\\"0\\\">\");\n";
	my $questionInit = "<script language=\"javascript\">\n$jsIcons";
	my $newQuestionnaireLink = "<br>".$this->newQuestionnaireLink($sid)."<br>\n";
	my $questions = "<h2>Questions</h2>";
	foreach my $q (@questions) {
		next if $q->master_id || $q->isFinal;
		my $qLink = $this->questionBaseLink($q);
		$qLink =~ s/\"/&quot;/g;
		$qLink =~ s/\n/\\n/g;
		$questionInit .= "AddQuestion(\"".$q->format."\", \"$qLink\");\n";
		$formatHash{$q->format}++;
#		$out .= $this->questionBaseLink($q)."<br>\n";
	}
	foreach my $f (@formats) {
		my $printIcon = ($formatHash{$f}) ? $questionIcon : "";
		$questions .= "<a onClick=\"javascript:ToggleExpand('$f','$f')\">".&Modules::Questionnaire::QuestionBase::formatText($f)."</a>\n<span id=\"".$f."_icon\">$printIcon</span><div id=\"$f\"></div><br>\n";
	}
	$questionInit .= "</script>\n";
	my $newQuestionLink = "<br>".$this->newQuestionBaseLink($sid)."<br>\n";
	$data{'questionnaires'} = $questionnaires;
	$data{'new_questionnaire'} = $newQuestionnaireLink;
	$data{'questions'} = $questions;
	$data{'js'} = "<script language=\"javascript\" src=\"$config{server}->{HTMLpath}/_Modules/Questionnaire/js/Questionnaire.js\"></script>\n";
	$data{'js'} .= $questionInit;
	$data{'css'} = "";
	$data{'new_question'} = $newQuestionLink;
	return &Modules::Questionnaire::QuestionnaireHTML::sectionView(%data);
}

sub listQuestionnaires($) {
	my $this = shift @_;
	my $sid = shift @_;

	my @questionnaires = $this->fetchSiteQuestionnaires($sid);
	my $data = [];
	my $questionnaires = "<h2>Questionnaires</h2>";
	foreach my $q (@questionnaires) {
		#$questionnaires .= $this->questionnaireLink($q)."<br>\n";
		push @{$data}, [$this->questionnaireLink($q), $q->copyIcon];
	}
	my %report = (
		'title' => "Questionnaires",
		'headers' => ['Questionnaire Name', 'Options'],
		'data' => $data,
	);
	return &Modules::Questionnaire::Report::reportbuilder(%report);
#	return $questionnaires;
}

sub editQuestionBaseView($) {
	my $this = shift @_;
	my $sid = shift @_;
	my $qid = shift @_;
	my $qbid = shift @_;
	my $qcmd = shift @_;

	return undef unless $sid;

	my $record = {};
	my $format = "";
	my $base = undef;
	if($qbid) {
		$base = new Modules::Questionnaire::QuestionBase($qbid);
		# Get the master, even though the 'master' and 'current'
		# records should be the same
		$record = $base->master;
		$format = $base->format;
	}
	
	my $id = $record->{'question_base_id'};
	my $form = $this->form;
	my $action = ($qcmd =~ /new_question_create/) ? "question_make" : "insert_question_base";
	my $hiddens = $form->input_html('type' => 'hidden', 'name' => "q_question_base#$id#section_id", 'value' => $sid).
		$form->input_html('type' => 'hidden', 'name' => "q_question_base#$id#master_id", 'value' => '0').
		$form->input_html('type' => 'hidden', 'name' => "q_question_base#$id#question_base_id", 'value' => "$id").
		$form->input_html('type' => 'hidden', 'name' => 'reply', 'value' => $this->reply).
		$form->input_html('type' => 'hidden', 'name' => 'qcmd', 'value' => $action).
		$form->input_html('type' => 'hidden', 'name' => "q_question_base#$id#version", 'value' => 'master');

	my $formatHash = {
		$format.'_selected' => 'selected',
		'other_checked' => ($base && $base->flags =~ /other/) ? "checked" : "",
		'id' => $qbid,
	};

	my %data = (
		'hiddens' => $hiddens,
		'label_input' => $form->input_column('table' => 'q_question_base', 'column' => 'label', 'record' => $id, 'value' => $record->{'label'}),
#		'version_input' => $form->input_column('table' => 'q_question_base', 'column' => 'version', 'record' => $id, 'value' => $record->{'version'}),
		'version_input' => 'master',
		'format_input' => $this->baseQuestionFormatSelect($formatHash),
#		'format_input' => $form->input_column('table' => 'q_question_base', 'column' => 'format', 'record' => $id, 'value' => $record->{'format'}),
		'value_input' => $form->input_column('table' => 'q_question_base', 'column' => 'value', 'record' => $id, 'value' => $record->{'value'}),
		'buttons' => $form->make_form_buttons,
		'js' => $this->questionnaireJavascript(),
	);
	$data{'link_hiddens'} = $this->linkQuestionHiddens($qid) if $qcmd =~ /new_question_create/;
	return &Modules::Questionnaire::QuestionnaireHTML::editQuestionBase(%data);
}

sub linkQuestionHiddens($) {
	my $this = shift @_;
	my $qid = shift @_;
	my $flags = shift @_;

	my $questionnaire = new Modules::Questionnaire::Questionnaire($qid);
	
	my $form = $this->form;

	$questionnaire->loadAllQuestions();
	my $nextPos = ($questionnaire->count)+1;
	my $out = $form->input_html('type' => 'hidden', 'name' => 'q_question##questionnaire_id', 'value' => $questionnaire->id);
	$out .= $form->input_html('type' => 'hidden', 'name' => 'q_question##flags', 'value' => $flags);
	$out .= $form->input_html('type' => 'hidden', 'name' => 'q_question##sort_key', 'value' => $nextPos);
	return $out;
}

sub questionBaseView($) {
	my $this = shift @_;
	my $qbid = shift @_;
	my $version = shift @_;

	return undef unless $qbid;

	my $question = new Modules::Questionnaire::QuestionBase($qbid, $version);
	my $masterQuestion = ($question->master_id) ? new Modules::Questionnaire::QuestionBase($question->master_id) : $question;
	my @version = $question->fetchAllVersions();
	my $out = $this->questionBasePreview($question);
	unshift @version, $masterQuestion ;
	$out .= "<br>".$this->reportBaseQuestions($question, @version);
	return $out;
}

sub questionnaireView($) {
	my $this = shift @_;
	my $qid = shift @_;
	my $version = shift @_;
	my $viewType = shift @_;
	my $rid = shift @_;
	my $input = shift @_;

	return undef unless $qid;

	my $questionnaire = new Modules::Questionnaire::Questionnaire($qid, $version);
	my $uid = $share{'DB'}->my_uid;
	my $out = $this->questionnaireViewTabs($questionnaire, $input);
	my $responseId = $input->{'response_id'};

	if($viewType =~ /overview/ || !$viewType) {
		$out .= $this->questionnaireOverview($questionnaire);
	} elsif($viewType =~ /preview/) {
		my $response = undef;
		if($uid) {
			$response = new Modules::Questionnaire::Response;
			$response->fetchByUid($uid, $qid);
			$responseId = $response->id;
		}

		$out .= $this->viewQuestionnaire($questionnaire, $responseId, 1);
	} elsif($viewType =~ /report/) {
		if($input->{'excel'}) {
			return $this->questionnaireReportExcel($questionnaire, $rid, $input);
		} elsif($input->{'response_id'} && $input->{'response_details'}) {
			$out .= $this->responseDetailsReport($questionnaire, $input->{'response_id'}, $input);	
		} elsif($input->{'response_details'}) {
			$out .= $this->questionnaireResponsesReport($questionnaire, $input);
		} else {
			$out .= $this->questionnaireReports($questionnaire, $rid, $input);
		}
	} elsif($viewType =~ /evite/) {
		$out .= $this->eviteForm($questionnaire);
	}
	$out .= "</td></tr></table>\n";
	return $out;
}

sub questionnaireOverview($) {
	my $this = shift @_;
	my $questionnaire = shift @_;

	return undef unless $questionnaire;
	my @unused = $questionnaire->getUnusedQuestions($questionnaire->version);
	my $finalMessage = $questionnaire->finalMessage;

	my $form = $this->form;
	my $reply = &relink();

	my $iconKey = $this->iconKey(
		'star' => "Make question mandatory",
		'up' => "Move question up",
		'down' => "Move question down",
		'rm' => "Remove question from questionnaire",
	);

	my %data = (
		'icon_key' => $iconKey,
		'title' => "<h2>".$questionnaire->title."</h2>",
		'linked_questions' => $this->reportLinkedQuestions($questionnaire),
		'final_message' => $this->finalMessagePreview($questionnaire, $finalMessage),
	);


	if(scalar(@unused)) {
		my $availableQuestions = "<h2>Available Questions</h2>\n".
			"<form method='post'>\n".
			$form->input_html('type' => 'hidden', 'name' => 'qcmd', 'value' => 'link_question').
			$form->input_html('type' => 'hidden', 'name' => 'reply', 'value' => $reply).
			$this->unusedQuestionsSelect($questionnaire, $questionnaire->get('version')).
			$form->input_html('type' => 'submit', 'name' => 'submit_action', 'value' => 'Add').
			"&nbsp;\n".
			"<input type='button' name='new_question' value='New Question' onClick='window.location=\"".&relink('qcmd' => 'new_question_create', 'qid' => $questionnaire->id, 'section_id' => $questionnaire->section_id, 'reply' => $reply)."\">\n".
			"\n</form>\n";
		$data{'unused_questions'} = $availableQuestions;
	}
	return &Modules::Questionnaire::QuestionnaireHTML::questionnaireOverview(%data);
}

sub questionnaireReports($) {
	my $this = shift @_;
	my $questionnaire = shift @_;
	my $rid = shift @_;
	my $input = shift @_;

	return undef unless $questionnaire;

	my $reply = &relink('reply' => undef);
	my $report = new Modules::Questionnaire::Report($rid);
	my @allReports = $questionnaire->getAllReports();
	my @allFilters = $questionnaire->getAllResponseFilters();
	my @allQuestions = $questionnaire->getAllInputQuestions();
	my %excelAll = ('excel' => 1);
	my @reportFilters = ();

	if(!$report->id) {
		my @filters = $questionnaire->getFilters($input);
		$report->addFilters(@filters);

	}
	@reportFilters = $report->getQueryFilters($input);
	my %resetHash = ('report_id' => undef, 'reply' => $reply);

	foreach my $r (@reportFilters) {
		$resetHash{'apply_filter_'.$r->id} = undef;
	}
	foreach my $q (@allQuestions) {
		my $key = 'report_question_'.$q->id;
		$resetHash{$key} = undef;
		$excelAll{$key} = 1;
	}

	my $resetLink = &relink(%resetHash);
	
	my @usedQuestions = $report->displayQuestions($questionnaire, $input);
	my $questionsReport = $report->report($questionnaire, @usedQuestions);
	my %availableQuestions = $report->reportQuestionsHash(@usedQuestions);
	my $questionHiddens = "";

	foreach my $q (@usedQuestions) {
		$questionHiddens .= $this->form->input_html('type' => 'hidden', 'name' => 'report_question_'.$q->id, 'value' => '1')."\n";
	}

	my %usedQuestions = ();
	map {$usedQuestions{$_->id} = $_;} @usedQuestions;

	my %data = ( 
		'js_include' => "<script language='javascript' src='$config{server}->{HTMLpath}/_ExSite/js/misc.js'></script>",
		'question_hiddens' => $questionHiddens,
		'save_report_link' => "<a href=\"#\" onClick=\"popup_medium('".&relink('qcmd' => 'new_report', %availableQuestions)."', 'mywindow1')\">save report</a>",
		'new_filter_link' => "<a href=\"#\" onClick=\"popup_medium('".&relink('qcmd' => 'new_response_filter')."')\">new filter</a>\n",
		'export_report_link' => "<a href='".&relink('excel' => '1')."'>Export Current Report to Excel</a> : <a href='".&relink(%excelAll)."'>Export All</a>",
	);
	
	my $savedReports = "";
	my $reportFilters = "";
	my $currentReport = "<b>Current Report</b>: ";
	if(scalar(@allReports)) {
		for(my $i = 0; $i < @allReports; $i++) {
			$savedReports .= $this->reportRow($allReports[$i], $rid);
		}
	}
	if(scalar(@allFilters)) {
		for(my $i = 0; $i < @allFilters; $i++) {
			$reportFilters .= $this->filterRow($allFilters[$i], $report);
		}
	}
	if(scalar(@reportFilters)) {
		$currentReport .= $reportFilters[0]->name if $reportFilters[0]->id;
		for(my $i = 1; $i < @reportFilters; $i++) {
			$currentReport .= ", ".$reportFilters[$i]->name;
		}
	}

	my $availableQuestions = "";
	foreach my $q (@allQuestions) {
		next if $usedQuestions{$q->id};
		$availableQuestions{'report_question_'.$q->id} = 1;
		$availableQuestions .= $q->label."&nbsp;<a href='".&relink('qqid' => $q->id, 'report_id' => undef, 'reply' => $reply, %availableQuestions)."'>[+]</a><br>\n";
		delete $availableQuestions{'report_question_'.$q->id};
	}

	$data{'saved_reports'} = $savedReports;
	$data{'response_filters'} = $reportFilters;
	$data{'questions_report'} = $questionsReport;
	$data{'available_questions'} = $availableQuestions;
	$data{'current_report'} = $currentReport;
	$data{'reset_link'} = "<a href='".$resetLink."'>reset</a>\n";
	$data{'summary_link'} = "<a href=\"".&relink('view' => 'questionnaire_view', 'response_details' => 1)."\">Full Summary</a>";
	$data{'icon_key'} = $this->iconKey(
		'view' => "See individual response summary",
		'rm' => "Delete response from questionnaire",
	);
	$data{'hidden_form'} = &Modules::Questionnaire::QuestionnaireHTML::reportForm(
		'reply' => $reply,
		'qcmd' => $this->form->input_html('type' => 'hidden', 'name' => 'qcmd', 'value' => 'insert_report'),
		'report_name_input' => $this->form->input_html('type' => 'hidden', 
			'name' => 'name', 'value' => ''),
		'report_questionnaire_input' => $this->form->input_html('type' => 'hidden', 
			'name' => 'questionnaire_id', 'value' => $questionnaire->id),
		'submit_input' => $this->form->input_html('type' => 'hidden', 
			'name' => 'submit_action', 'value' => 'Submit'),
		'question_hiddens' => $questionHiddens,
	);
	return &Modules::Questionnaire::QuestionnaireHTML::reportView(%data);
}

sub questionnaireReportExcel($) {
	my $this = shift @_;
	my $questionnaire = shift @_;
	my $reportId = shift @_;
	my $input = shift @_;

	my $report = new Modules::Questionnaire::Report($reportId);
	my @usedQuestions = $report->displayQuestions($questionnaire, $input);
	my @responses = $report->getFilteredResponses($questionnaire, $input);
	return $report->exportToExcel($questionnaire, \@usedQuestions, \@responses);
}

sub responseDetailsReport($) {
	my $this = shift @_;
	my $questionnaire = shift @_;
	my $responseId = shift @_;
	my $input = shift @_;

	return undef unless $responseId;

	my $report = new Modules::Questionnaire::Report();
	my $response = new Modules::Questionnaire::Response($responseId);
	return $report->reportResponse($questionnaire, $response);
}

sub questionnaireResponsesReport($) {
	my $this = shift @_;
	my $questionnaire = shift @_;
	my $input = shift @_;

	my $report = new Modules::Questionnaire::Report();
	my %data = (
		'report' => $report->reportAllResponses($questionnaire, 1, $input),
		'return_link' => "<a href=\"javascript:history.back()\">Back</a>\n",
	);
	return &Modules::Questionnaire::QuestionnaireHTML::responseSummaryView(%data);
}

sub reportForm($) {
	my $this = shift @_;
	my $questionnaireId = shift @_;
	my $rid = shift @_;
	my $input = shift @_;

	my $questionnaire = new Modules::Questionnaire::Questionnaire($questionnaireId);
	my $report = new Modules::Questionnaire::Report($rid);
	my @filters = $questionnaire->getFilters($input);
	my %filtersHash = ();

	foreach my $f (@filters) {
		my @matches = $f->fetchMatches;
		map {$filtersHash{$_->question_id} .= "<li>".$_->filter."<br>\n";} @matches;
	}
	my @questions = $report->displayQuestions($questionnaire, $input);
	my @filterKeys = grep { /^apply_filter_/ && $input->{$_} } keys %{$input};
	my $data = [];
	foreach my $q (@questions) {
		my $filter = ($filtersHash{$q->id}) ? $filtersHash{$q->id} : "";
		push @{$data}, [$q->label, $filter];
	}

	my $questionFilterReport = &Modules::Questionnaire::Report::reportbuilder(
		'headers' => ['Questions', 'Filters'],
		'data' => $data
	);
	my $name = $this->form->input_column('table' => 'q_report', 'column' => 'name', 'value' => $report->name);
	my $hiddens = "";
	foreach my $q (@questions) {
		$hiddens .= $this->form->input_html('type' => 'hidden', 'name' => 'report_question_'.$q->id, 'value' => '1')."\n";
	}

	foreach my $f (@filterKeys) {
		my $key = $f;
		$key =~ s/^apply_filter_//;
		$hiddens .= $this->form->input_html('type' => 'hidden', 'name' => 'report_filter_'.$key, 'value' => $key)."\n";
	}
	$hiddens .= $this->form->input_html('type' => 'hidden', 'name' => 'q_report_filter##report_id', 'value' => $rid)."\n";
	$hiddens .= $this->form->input_html('type' => 'hidden', 'name' => 'q_report#'.$report->id.'#report_id', 'value' => $report->id)."\n";
	$hiddens .= $this->form->input_html('type' => 'hidden', 'name' => 'report_id', 'value' => $report->id)."\n";
	my %data = (
		'hiddens' => $hiddens,
		'name' => $report->name,
		'name_input' => $name,
		'question_filter_report' => $questionFilterReport,
		'qcmd' => ($report->id) ? 'update_report' : 'insert_report',
		'reply' => &relink('qcmd' => 'questionnaire_view', 'view_type' => 'report'),
	);
	return &Modules::Questionnaire::QuestionnaireHTML::reportEditForm(%data);
}

sub reportRow($) {
	my $this = shift @_;
	my $report = shift @_;
	my $queryReportId = shift @_;

	my $reply = &relink('reply' => undef);
	my $reportLink = $report->name;
	my $loadLink = ($report->id == $queryReportId) ? "<a href='".$report->unloadLink."'>unload</a>" : "<a href='".$report->loadLink."'>load</a>";
	my $editLink = "<a href=\"javascript:popup_medium('".$report->editLink."')\"'>edit</a>";
	my $deleteLink = "<a href='".&relink('report_id' => $report->id, 'qcmd' => 'delete_report', 'reply' => $reply)."'>delete</a>";
	my $out = "<tr><td style='width: 300px;'>".$reportLink."</td><td style='width: 100px'>$loadLink</td><td style='100px;'>$editLink</td><td>$deleteLink</td></tr>\n";

	return $out;
}

sub filterRow($) {
	my $this = shift @_;
	my $filter = shift @_;
	my $report = shift @_;

	my $reply = &relink('reply' => undef, 'apply_filter_'.$filter->id => undef);
	my $applyRemove = ($filter->isApplied() || $report->filterEnabled($filter)) ? "<a href='".&relink('apply_filter_'.$filter->id => undef, 'report_id' => undef)."'>remove</a>" : "<a href='".&relink('apply_filter_'.$filter->id => 1)."'>apply</a>";
		'new_filter_link' => "<a href=\"#\" onClick=\"popup_medium('".&relink('qcmd' => 'new_response_filter')."')\">new filter</a>\n",
	my $editLink = "<a href=\"javascript:popup_medium('".&relink('qcmd' => 'edit_response_filter', 'response_filter_id' => $filter->id)."');\">edit</a>";
	my $deleteLink = "<a href='".&relink('qcmd' => 'delete_response_filter', 'response_filter_id' => $filter->id, 'reply' => $reply)."'>delete</a>";
	my $out = "<tr><td style='width: 300px;'>".$filter->name."</td><td style='width: 100px;'>$applyRemove</td><td style='100px;'>$editLink</td><td>$deleteLink</td></tr>\n";
	return $out;
}

sub eviteForm($) {
	my $this = shift @_;
	# In some cases, we will have supplied all of the fields through
	# the query string (or some other form of input). In this case, we'll
	# be passing these values to the eviteForm method. These values take
	# precedence over any auto-generated fields
	my $input = shift @_;
	my $notes = "";
	my $js = "<script language='javascript'>\n<!--\nfunction submitForm() { document.eviteForm.submit(); }\n-->\n</script>";
	# The evite_form command happens in a popup. We want to include all
	# of the javascript neccessarry for proper submission to the opening
	# form
	if($input->{'qcmd'} eq "evite_form") {
		$js = $this->eviteFormJavascript;
	}

	my $evite = $this->evite;
	if(!$input->{'evite_url'}) {
		$input->{'evite_url'} = $evite->urlToken;
		$notes = "<div class='help'>Before sending your evite, please replace the \"$input->{evite_url}\" token(s) with the proper URL to the page of your choice.</div><br>";
	}
	my %data = (
		'from_email' => $input->{'from_email'} || $share{'DB'}->my_email || $config{'site'}{'administrator'},
		'evite_subject' => $input->{'evite_subject'} || $evite->defaultSubject(),
		'evite_message' => $input->{'evite_message'} || $evite->defaultMessage('url' => $input->{'evite_url'}),
		'reply' => $input->{'reply'} || &relink(),
		'buttons' => $this->form->make_form_buttons,
		'js' => $js,
		'submit_function' => 'submitForm(this)',
		'notes' => $notes,
	);
	return &Modules::Questionnaire::QuestionnaireHTML::eviteForm(%data);
}

sub eviteFormJavascript($) {
	my $this = shift @_;

	return <<"DONE";
<script language='javascript'>
<!--
	function submitForm() {
		var form = document.eviteForm

		var evite_recipients = opener.document.eviteForm.evite_recipients;
		var evite_from = opener.document.eviteForm.evite_from;
		var evite_subject = opener.document.eviteForm.evite_subject;
		var evite_message = opener.document.eviteForm.evite_message;

		evite_recipients.setAttribute('type', 'hidden');
		evite_recipients.setAttribute('name', 'evite_recipients');
		evite_recipients.setAttribute('value', document.eviteForm.evite_recipients.value);
		evite_from.setAttribute('type', 'hidden');
		evite_from.setAttribute('name', 'evite_from');
		evite_from.setAttribute('value', document.eviteForm.evite_from.value);
		evite_subject.setAttribute('type', 'hidden');
		evite_subject.setAttribute('name', 'evite_subject');
		evite_subject.setAttribute('value', document.eviteForm.evite_subject.value);
		evite_message.setAttribute('type', 'hidden');
		evite_message.setAttribute('name', 'evite_message');
		evite_message.setAttribute('value', document.eviteForm.evite_message.value);

		opener.document.eviteForm.submit();
		window.close();
	}
-->
</script>
DONE
}

#################### CTRL PANEL CONSTRUCTION METHODS
sub questionnaireLink($) {
	my $this = shift @_;
	my $questionnaire = shift @_;

	return undef unless $questionnaire;

	my $reply = &relink();
	#return "<li> <a href='".&relink('qcmd' => 'questionnaire_view', 'qid' => $questionnaire->id, 'section_id' => $questionnaire->section_id, 'reply' => $reply)."'>".$questionnaire->title."</a>";
	return "<a href='".&relink('qcmd' => 'questionnaire_view', 'qid' => $questionnaire->id, 'section_id' => $questionnaire->section_id, 'reply' => $reply)."'>".$questionnaire->title."</a>";
}

sub newQuestionnaireLink($) {
	my $this = shift @_;
	my $sid = shift @_;

	return undef unless $sid;

	my $reply = &relink();
	return "<a href='".&relink('qcmd' => 'new_questionnaire_view', 'section_id' => $sid, 'reply' => $reply)."'><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/add.gif' border='0'> new questionnaire</a>";
}

sub questionBaseLink($) {
	my $this = shift @_;
	my $questionBase = shift @_;
	my $version = shift @_;

	return undef unless $questionBase;

	my $reply = &relink();
	my $linkLabel = &ExSite::Misc::html_to_plaintext($questionBase->label);
	return "<li> <a href='".&relink('qcmd' => 'question_base_view', 'qbid' => $questionBase->id, 'section_id' => $questionBase->section_id, 'version' => $version, 'reply' => $reply)."'>".$linkLabel."</a>";
}

sub questionVersionLink($) {
	my $this = shift @_;
	my $questionBase = shift @_;

	return undef unless $questionBase;

	my $version = $questionBase->version;
	my $versionLabel = ($questionBase->master_id) ? $version : "master";

	my $reply = &relink();
	return "<a href='".&relink('qcmd' => 'question_base_view', 'qbid' => $questionBase->id, 'section_id' => $questionBase->section_id, 'version' => $version, 'reply' => $reply)."'>$versionLabel</a>";
}

sub newQuestionBaseLink($) {
	my $this = shift @_;
	my $sid = shift @_;

	return undef unless $sid;

	my $reply = &relink();
	return "<a href='".&relink('qcmd' => 'new_question_base_view', 'section_id' => $sid, 'reply' => $reply)."'><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/add.gif' border='0'> new question</a>";
}

sub questionnaireViewTabs($) {
	my $this = shift @_;
	my $questionnaire = shift @_;
	my $input = shift @_;

	return undef unless $questionnaire;

	# Some defaults and constants that apply to all links
	my %defHash = ('qcmd' => 'questionnaire_view');
	foreach my $k (keys %{$input}) {
		if($k =~ /^(apply_filter_|response_id|report_question_|version)/) {
			$defHash{$k} = undef;
		} 
	}

	my $overviewLink = "<a href='".&relink('view_type' => 'overview', 'reply' => undef, %defHash)."'>Overview</a>";
	my $previewLink = "<a href='".&relink('view_type' => 'preview', 'reply' => undef, %defHash)."'>Preview</a>";
	my $reportLink = "<a href='".&relink('view_type' => 'report', 'report_id' => undef, 'response_details' => undef, 'reply' => undef, %defHash)."'>Report</a>";
	my $eviteLink = "<a href='".&relink('view_type' => 'evite', 'reply' => undef, %defHash)."'>Evites</a>";
	my $deleteIcon = $questionnaire->deleteIcon(&relink());

	return &Modules::Questionnaire::QuestionnaireHTML::questionnaireViewTabs(
		'overviewLink' => $overviewLink,
		'previewLink' => $previewLink,
		'reportLink' => $reportLink,
		'eviteLink' => $eviteLink,
		'deleteIcon' => $deleteIcon,
	);
}

######################### REPORT METHODS
sub questionBasePreview($) {
	my $this = shift @_;
	my $question = shift @_;

	return undef unless ref($question) =~ /QuestionBase/;

	my @versions = $question->fetchAllVersions();
	#my $preview = $question->show('q'.$question->id);
	my $preview = $question->show_input(1, undef, undef);
	my $links = "";
	my $js = &Modules::Questionnaire::QuestionnaireHTML::sliderJavascript() if $question->format =~ /slider/;
	foreach my $v (@versions) {
		$links .= $this->questionVersionLink($v)."&nbsp;";
	}
	my %data = (
		'data' => $js.$preview.$links,
		'title' => 'Preview',
	);
	return &Modules::Questionnaire::Report::reportbuilder(%data);
}

sub reportBaseQuestions($) {
	my $this = shift @_;
	my $base = shift @_;
	my @questions = @_;
	
	my $data = [];

	my $class=  $this->get('div_class');

	foreach my $q (@questions) {
		push @{$data}, $this->questionBaseRow($q);
	}

	my %report = (
		'title' => 'All Versions',
		'headers' => ['Version', 'Question', 'Answers', 'Options'],
		'data' => $data,
		'tools' => $base->newVersionIcon(),
	);

	return "<div class='$class'>".&Modules::Questionnaire::Report::reportbuilder(%report)."</div>\n";
}

# Very similar to reportBase questions, but each final screen is automatically
# associated with a questionnaire, and requires different labels on their
# 'edit' pages. This requires links to different pages.
# Most of the functionality for changing a questionnaire's final screen
# is exactly the same as a base question, however
sub reportFinalScreens($) {
	my $this = shift @_;
	my $base = shift @_;
	my @questions = @_;

	my $data = [];

	my $class = $this->get('div_class');
	foreach my $q (@questions) {
		push @{$data}, $this->questionBaseRow($q);
	}

	my %report = (
		'title' => 'All Versions',
		'headers' => ['Version', 'Question', 'Answers', 'Options'],
		'data' => $data,
		'tools' => $base->newFinalIcon(),
	);

	return "<div class='$class'>".&Modules::Questionnaire::Report::reportbuilder(%report)."</div>\n";
}

sub questionBaseRow($) {
	my $this = shift @_;
	my $questionBase = shift @_;

	return undef unless ref($questionBase) =~ /QuestionBase/ || $questionBase->isFinal;

	my $options = "";
	my $id = 'q'.$questionBase->id;
	if($questionBase->isApproved) {
		$options = "<div id='$id'>";
	} else {
		$options = "<div id='$id' style='background-color: red; width: 100%; height: 100%'>";
	}
	$options .= $questionBase->approveIcon() if !$questionBase->isApproved;
	$options .= $questionBase->editIcon($questionBase->version);
	$options .= $questionBase->deleteIcon($this->reply);
	$options .= "</div>";
	my $version = ($questionBase->master_id) ? $questionBase->version : "master";
	my @values = $questionBase->getValues();
	my $valueStr = "";
	if($questionBase->format !~ /slider/) {
		$valueStr = join ",<br>", @values;
	} else {
		$valueStr = $values[0]." - ".$values[$#values];
	}
	my $row = [
		$version,
		$questionBase->label,
		$valueStr,
		$options
	];

	return $row;
}

sub reportLinkedQuestions($) {
	my $this = shift @_;
	my $questionnaire = shift @_;

	return undef unless $questionnaire;
	
	my @versions = $questionnaire->getVersions();
	my @questions = $questionnaire->getAllQuestions();

	my $data = [];
	splice @versions, 0, 1;

	foreach my $q (@questions) {
		push @{$data}, $this->linkedQuestionsRow($q, @versions);
	}

	my $headers = ['Question', @versions, 'Options'];
	my %report = (
		'headers' => $headers,
		'data' => $data,
	);
	my $out = "<div class='label'>* indicates a mandatory question.</div>";
	return $out.&Modules::Questionnaire::Report::reportbuilder(%report);
}

sub linkedQuestionsRow($) {
	my $this = shift @_;
	my $question = shift @_;
	my @versions = @_;

	return undef unless $question;

	my @qVersions = $question->fetchBase->fetchAllVersions();
	my %availableVersions = ();
	map { $availableVersions{$_->version} = $_; } @qVersions;

	my @versionLabels = ();
	my $masterId = $question->fetchBase->master->{'question_base_id'};
	my $reply = &relink();
	foreach my $v (@versions) {
		if($availableVersions{$v}) {
			push @versionLabels, "<div style='width: 100%'>".$availableVersions{$v}->viewLink($reply)."</div>";
		} else {
			push @versionLabels, "<div style='position: relative; width: 100%; height: 100%; background-color: red;'><a href='".&relink('qcmd' => 'new_version', 'version' => $v, 'qbid' => $masterId, 'reply' => $reply)."'>MISSING</a></div>";
		}
	}

	my $options = $question->moveUpIcon.$question->moveDownIcon.$question->mandatoryIcon.$question->unlinkIcon;
	my $row = [
		$question->viewBaseLink,
		@versionLabels,
		$options
	];
	return $row;
}

#################### CTRL PANEL FORMS
sub editQuestionVersion($) {
	my $this = shift @_;
	my $qbid = shift @_;
	my $version = shift @_;
	
	return undef unless $qbid;

	my $form = $this->form;
	my $base = new Modules::Questionnaire::QuestionBase($qbid, $version);
	my $master = $base->master;
	my $current = {};

	# If the base question provided is a version, then we know we want to edit
	# the current question version.
	#
	# If it's a master, then that indicates that we're creating a new base
	# question version
	$current = $base->current if $base->master_id;

	my %data = (
		'version' => $version,
		'master_label' => $master->{'label'},
		'version_label' => $form->input_column(
				'table' => 'q_question_base',
				'column' => 'label',
				'record' => $current->{'question_base_id'},
				'value' => $current->{'label'},
			),
	);

	# We need the user to enter the desired version
	if(!$version) {
		$data{'version_input'} = "<br>\n<br>\nVersion: <br>\n".
			$form->input_column(
				'table' => 'q_question_base',
				'column' => 'version',
				'record' => $current->{'question_base_id'},
				'value' => $version
			);
	}

	if($base->allowsValues()) {
		my $masterValue = $master->{'value'};
		$masterValue =~ s/\s*\|\s*/|/g;
		if($base->isFinal()) {
			$data{'version_values'} = "<br>\n<br>\n<b>Please enter a translation of the \"Thank You\" screen below.</b><br>\n".$masterValue;
		} else {
			$data{'version_values'} = "<br>\n<br>\n<b>Please enter a translation of the question's answer values below. Each entry should be delimited by a pipe (\"|\") character.</b><br>\n".$masterValue;
		}
		$data{'version_values'} .= "<br>".$form->input_column(
				'table' => 'q_question_base',
				'column' => 'value',
				'record' => $current->{'question_base_id'},
				'value' => $current->{'value'}
			);
	}
	my $hiddens = 
		$form->input_html('type' => 'hidden', 'name' => "q_question_base#$current->{question_base_id}#section_id", 
			'value' => $base->section_id).
		$form->input_html('type' => 'hidden', 'name' => "q_question_base#$current->{question_base_id}#master_id", 
			'value' => $base->master->{'question_base_id'}).
		$form->input_html('type' => 'hidden', 'name' => "q_question_base#$current->{question_base_id}#format", 
			'value' => $base->format).
		$form->input_html('type' => 'hidden', 'name' => "q_question_base#$current->{question_base_id}#question_base_id", 
			'value' => $current->{'question_base_id'}).
		$form->input_html('type' => 'hidden', 'name' => 'reply', 'value' => $this->reply);

	$hiddens .= $form->input_html('type' => 'hidden', 'name' => "q_question_base#$current->{question_base_id}#version", 'value' => $version) if $version;
	$data{'hiddens'} = $hiddens;
	$data{'buttons'} = $form->make_form_buttons;
	return &Modules::Questionnaire::QuestionnaireHTML::editQuestionVersion(%data);
}

sub unusedQuestionsSelect($) {
	my $this = shift @_;
	my $questionnaire = shift @_;

	return undef unless ref($questionnaire);

	my $version = $questionnaire->version;

	my @unused = $questionnaire->getUnusedQuestions($version);

	my $form = $this->form;
	my $value = [];
	#my $out = $form->input_html('type' => 'select', 'name' => 'q_question##question_base_id', 'id' => 'questionsSelect', 'options' => $value, 'value' => '');
	my $out = $form->input_html('type' => 'hidden', 'name' => 'q_question##questionnaire_id', 'value' => $questionnaire->id);
	$out .= $form->input_html('type' => 'hidden', 'name' => 'q_question##sort_key', 'value' => ($questionnaire->count+1));
	$out .= $form->input_html('type' => 'hidden', 'name' => 'q_question##flags', 'value' => '');
	$out .= "<select name='q_question##question_base_id' id='questionsSelect'>\n";
	foreach my $q (@unused) {
		my $label = &ExSite::Misc::html_to_plaintext($q->label);
		$out .= "<option value=".$q->master->{'question_base_id'}.">$label</option>\n";
	}
	$out .= "</select>\n";	
	return $out;
}

sub newQuestionnaireForm($) {
	my $this = shift @_;
	my $sid = shift @_;

	return undef unless $sid;

	my $form = $this->form;
	my $out = "<h2>New Questionnaire</h2>";
	#$out .= "<form method='post'>\n";
	$out .= "<form method='post' enctype='application/x-www-form-urlencoded'>\n";
	$out .= $form->input_html('type' => 'hidden', 'name' => 'q_questionnaire##section_id', 'value' => $sid);
	$out .= $form->input_html('type' => 'hidden', 'name' => 'reply', 'value' => $this->reply);
	$out .= "<table>\n";
	$out .= "<tr><td>Title:</td><td>".$form->input_column('table' => 'q_questionnaire','column' => 'title')."</td></tr>\n";
	$out .= "<tr><td>Versions:</td><td>".$form->input_column('table' => 'q_questionnaire','column' => 'versions')."</td></tr>\n";
	$out .= "<tr><td colspan='2'>".$form->make_form_buttons."</td></tr>\n";
	$out .= "</table>\n";
	$out .= "</form>\n";
	return $out;
}

sub copyQuestionnaireForm($) {
	my $this = shift @_;
	my $qid = shift @_;

	return undef unless $qid;

	my $questionnaire = new Modules::Questionnaire::Questionnaire($qid);
	my $form = $this->form;
	my $out = "<h2>Copy Questionnaire</h2>";
	$out .= "<form method='post' enctype='application/x-www-form-urlencoded'>\n";
	$out .= $form->input_html('type' => 'hidden', 'name' => 'q_questionnaire##section_id', 'value' => $questionnaire->section_id);
	$out .= $form->input_html('type' => 'hidden', 'name' => 'reply', 'value' => $this->reply);
	$out .= "<table>\n";
	$out .= "<tr><td>Title:</td><td>".$form->input_column('table' => 'q_questionnaire','column' => 'title', 'value' => $questionnaire->title)."</td></tr>\n";
	$out .= "<tr><td>Versions:</td><td>".$form->input_column('table' => 'q_questionnaire','column' => 'versions', 'value' => $questionnaire->versions)."</td></tr>\n";
	$out .= "<tr><td colspan='2'>".$form->make_form_buttons."</td></tr>\n";
	$out .= "</table>\n";
	$out .= "</form>\n";
	return $out;
}

################## A few DB modifying methods
sub makeQuestion($) {
	my $this = shift @_;
	my $baseData = shift @_;
	my $questionData = shift @_;

	return undef unless $baseData && $questionData;

	my $question = new Modules::Questionnaire::Question();

	return $question->make($questionData, $baseData);
}

sub addReport($) {
	my $this = shift @_;
	my $questionnaireId = shift @_;
	my $input = shift @_;

	return undef unless $questionnaireId;

	my $report = new Modules::Questionnaire::Report();
	my @questionIds = grep { /^report_question_/ && $input->{$_} } keys %{$input};
	my @filterIds = grep { /^apply_filter_/ && $input->{$_} } keys %{$input};

	my $result = $report->make('questionnaire_id' => $questionnaireId, 'name' => $input->{'name'});
	return undef unless $result;

	foreach my $q (@questionIds) {
		$q =~ s/report_question_//g;
		my $question = new Modules::Questionnaire::Question($q);
		$report->addQuestion($question);
	}
	foreach my $f (@filterIds) {
		$f =~ s/^apply_filter_//;
		my $filter = new Modules::Questionnaire::ResponseFilter($f);
		$report->enableFilter($filter);
	}
	return $report->id;
}

sub updateReport($) {
	my $this = shift @_;
	my $rid = shift @_;
	my $input = shift @_;

	return undef unless $rid;

	my $report = new Modules::Questionnaire::Report($rid);
	my @questionIds = grep { /^report_question_/ && $input->{$_} } keys %{$input};
	my @filterIds = grep { /^apply_filter_/ && $input->{$_} } keys %{$input};

	my %questionsHash = ();
	my @realQuestions = $report->getAllQuestions();
	map {$questionsHash{$_->id} = $_;} @realQuestions;
	my %filtersHash = ();
	my @realFilters = $report->getAllFilters();
	map {$filtersHash{$_->id} = $_;} @realFilters;

	$report->make('name' => $input->{'name'});

	foreach my $q (@questionIds) {
		$q =~ s/report_question_//;
		if($questionsHash{$q}) {
			delete $questionsHash{$q};
		} else {
			my $question = new Modules::Questionnaire::Question($q);
			$report->addQuestion($question);
		}
	}

	foreach my $q (keys %questionsHash) {
		$report->removeQuestion($questionsHash{$q});
	}

	foreach my $f (@filterIds) {
		$f =~ s/^apply_filter_//;
		if($filtersHash{$f}) {
			delete $filtersHash{$f};
		} else {
			my $filter = new Modules::Questionnaire::ResponseFilter($f);
			$report->enableFilter($filter);
		}
	}

	foreach my $f (keys %filtersHash) {
		$report->removeFilter($filtersHash{$f});
	}

	return $report->id;
}

sub addReportQuestion($) {
	my $this = shift @_;
	my $questionId = shift @_;
	my $reportId = shift @_;

	return undef unless $questionId;
	my $report = new Modules::Questionnaire::Report($reportId);
	my $question = new Modules::Questionnaire::Question($questionId);
	if(!$report->id) {
		$report->make('name' => 'test', 'questionnaire_id' => $question->questionnaire_id);
		$report->makeQuestions();
	}
	return $report->addQuestion($question);
}

# Insert a response filter + its matches from the response filter form
sub insertResponseFilter($) {
	my $this = shift @_;
	my $questionnaireId = shift @_;
	my $input = shift @_;

	return undef unless $questionnaireId;

	my %filter = ('name' => $input->{'name'}, 'questionnaire_id' => $questionnaireId);
	my $newFilter = new Modules::Questionnaire::ResponseFilter();

	my $filterId = $newFilter->make(%filter);
	my @matchKeys = grep { /^response_match/ } keys %{$input};

	foreach my $m (@matchKeys) {
		my $field = $input->{$m};
		my $matchId = $field;
		my $questionId = $field;
		my $filter = $field;
		$matchId =~ s/\:.*//;
		$questionId =~ s/^[-]\d+:(\d+):.*/$1/;
		$filter =~ s/^.*:.*://;
		$newFilter->addMatchCriteria($questionId, $filter);
	}
	#return $filterId;
	return undef;
}

sub updateResponseFilter($) {
	my $this = shift @_;
	my $filterId = shift @_;
	my $input = shift @_;
	my $filter = new Modules::Questionnaire::ResponseFilter($filterId);
	return undef unless $filter->id;

	my $name = $input->{'name'};
	if($name ne $filter->name) {
		$filter->update('name' => $name);
	}

	my @filterMatches = $filter->fetchMatches();
	my %matchHash = ();
	map {$matchHash{$_->id} = $_;} @filterMatches;
	my @matchKeys = grep { /^response_match/ } keys %{$input};

	foreach my $m (@matchKeys) {
		my $val = $input->{$m};
		my $matchId = $val;
		my $questionId = $val;
		my $regex = $val;
		$matchId =~ s/\:.*//;
		$questionId =~ s/^[-]\d+:(\d+):.*/$1/;
		$regex =~ s/^.*:.*://;
		if($matchId && $matchHash{$matchId}) {
			$matchHash{$matchId}->update('filter' => $regex, 'question_id' => $questionId);
			delete $matchHash{$matchId};
		} elsif(($matchId < 0) || (length($matchId) == 0)) {
			$filter->addMatchCriteria($questionId, $regex);
		}
	}

	# All of the remaining matches must've been removed.
	# We'll do that now.
	foreach my $k (keys %matchHash) {
		$matchHash{$k}->delete;	
	}
	return 1;
}

sub sendEvites($) {
	my $this = shift @_;
	my $questionnaireId = shift @_;
	my $recipients = shift @_;
	my $from = shift @_;
	my $subject = shift @_;
	my $message = shift @_;

	if(!$recipients) {
		$this->error("Error: No valid email recipients defined.");
		return undef;
	}
	if(!$subject) {
		$this->error("Error: E-vite missing subject.");
		return undef;
	}
	if(!$message) {
		$this->error("Error: E-vite missing body.");
		return undef;
	}

	my $evite = $this->evite;

	$evite->setRecipients($recipients);
	$evite->setSubject($subject);
	$evite->setMessage($message);
	$evite->setFrom($from);
	if(!$evite->validateMessage()) {
		$this->error("Error: Please replace the token \"".$evite->urlToken."\" with a valid URL.");
		return undef;
	}

	return $evite->send;
}

# Returns the link to be used in an evite email.
# The logic here exists to make sure that $share{'Page'}
# is only used on public pages. Non-public pages get an
# empty string, and the admin is expected to enter their
# own URL manually.
sub eviteLink() {
	my $this = shift @_;
	my $showUrl = shift @_;

	return "" unless $share{'DB'}->is_manager();

	return "" if $share{'Page'} && $share{'Page'}->is_publishing();

	if($share{'Page'}) {
		my $reply = $share{'Page'}->get_url;
		my $eviteUrl = $share{Page}->get_url;
		return "<a href='javascript:popup_large(\"".&relink('qcmd' => 'evite_form', 'evite_url' => $eviteUrl, 'reply' => $reply)."\");'>send evite</a><br>\n";
	}
	#return "<a href='javascript:popup_large(\"".&relink('qcmd' => 'evite_form', 'evite_url' => '', 'reply' => &relink())."\");'>send evite</a><br>\n";
	return "<a href='javascript:popup_large(\"".&relink('qcmd' => 'evite_form', 'evite_url' => '')."\");'>send evite</a><br>\n";
}

sub addHandlers($) {
	my $this = shift @_;
	my $obj = shift @_;

	$obj->handler('validate', \&Modules::Questionnaire::QuestionBase::validate);
}

sub iconKey($) {
	my $this = shift @_;
	my %selectedIcons = @_;

	my $out = "<table class=\"iconKey\">\n";
	$out .= "\t<tr>\n\t\t<th>Icon Key</th>\n\t</tr>\n";

	# Order matters in certain cases (such as placing "up" and "down" arrows together)
	if($selectedIcons{'up'}) {
		$out .= "\t<tr>\n\t\t<td><img src=\"$config{server}->{HTMLpath}/_Modules/Questionnaire/up12.gif\" border=\"0\"> - $selectedIcons{up}</td>\n\t</tr>\n";
		delete $selectedIcons{'up'};
	}
	if($selectedIcons{'down'}) {
		$out .= "\t<tr>\n\t\t<td><img src=\"$config{server}->{HTMLpath}/_Modules/Questionnaire/down12.gif\" border=\"0\"> - $selectedIcons{down}</td>\n\t</tr>\n";
		delete $selectedIcons{'down'};
	}
	foreach my $k (keys %selectedIcons) {
		$out .= "\t<tr>\n\t\t<td><img src=\"$config{server}->{HTMLpath}/_Modules/Questionnaire/${k}12.gif\" border=\"0\"> - $selectedIcons{$k}</td>\n\t</tr>\n";
	} 
	$out .= "</table>\n";
	return $out;
}

1;
__END__
