package Modules::Questionnaire::Evite;

use strict;
use ExSite::Misc;
use ExSite::Config;
use ExSite::Mail;

use base	qw(ExSite::Base);

our $DEFAULT_EVITE_SUBJECT = "Your input is requested.";
our $DEFAULT_URL_TOKEN = "[[EVITE_URL]]";

sub new($) {
	my $type = shift @_;
	my $recipients = shift @_;
	my $message = shift @_;

	my $this = {};
	my $class = ref($type) || $type || "Modules::Questionnaire::Evite";
	bless $this, $class;

	$this->setRecipients($recipients);
	$this->setMessage($message);

	return $this;
}

# Retrieve the default subject line
sub defaultSubject($) {
	my $this = shift @_;

	return $DEFAULT_EVITE_SUBJECT;
}

# Token to represent a valid URL in the E-vite message. We use this in place
# of the URL if no URL can be determined on the current page.
# This is probably only useful for unpublished pages
sub urlToken($) {
	my $this = shift @_;

	return $DEFAULT_URL_TOKEN;
}

# The default message to be printed/emailed. This should be some
# editable field.
sub defaultMessage($) {
	my $this = shift @_;
	my %data = @_;
#	my $url = shift @_;

	$data{'first_name'} |= 'Friend';
	my $printableUrl = $data{'url'};
	$printableUrl =~ s/\&/\&amp\;/g;
	return<<"DONE";
Dear $data{first_name},

We request that you fill out this questionnaire, yadda, bladda, bladda.

Please click here to <a href='$data{url}'>link</a> to the questionnaire.

If the link doesn't display properly, please copy the following link into
your browser's address bar:

$printableUrl

Sincerely,

$share{identity}{name}
DONE
}

# Disable actual emailing, in case you want to prevent
# emails from actually being sent (especially for debugging
# purposes)
sub disable($) {
	my $this = shift @_;

	$this->set('disabled', 1);
	return 0;
}

# Enable emailing. This will probably be the default.
sub enable($) {
	my $this = shift @_;

	$this->set('disabled', 0);
}

# Tells us whether actual emailing has been enabled or
# disabled.
sub isEnabled($) {
	my $this = shift @_;
	
	return !($this->get('disabled'));
}

# Get the list of recipients for the object
sub recipients($) {
	my $this = shift @_;

	my $recipients = $this->get('recipients') || "";
	return $recipients;
}

# Set the object's recipients list.
sub setRecipients($) {
	my $this = shift @_;
	my $recipients = shift @_;

	my @tmpRecipientList = split /\s*,\s*/, $recipients;
	my @recipientList = ();

	foreach my $recip (@tmpRecipientList) {
		next unless $recip;
		if($recip =~ /\w+\@\w+\.\w+/) {
			push @recipientList, $recip;
		} else {
			$recipients =~ s/$recip//;
		}
	}
	$this->set('recipients', $recipients);
	$this->set('recipientsList', \@recipientList);
	return @recipientList;
}

# Retrieve the subject of the Evite email
sub subject($) {
	my $this = shift @_;

	my $subject = $this->get('subject') || "";
	return $subject;
}

# Set the subject of the Evite email
sub setSubject($) {
	my $this = shift @_;
	my $subject = shift @_;

	$this->set('subject', $subject);
	return $subject;
}

# Returns the Evite message currently stored in the object
sub message($) {
	my $this = shift @_;

	my $message = $this->get('message') || "";
	return $message;
}

# Set the object's message
sub setMessage($) {
	my $this = shift @_;
	my $message = shift @_;

	# FORMAT MESSAGE
	# VALIDATE MESSAGE?
	$this->set('message', $message);
	return $message;
}

# Returns the 'from' address for the current Evite
sub from($) {
	my $this = shift @_;

	return $this->get('from');
}

# Set the from address in the object
sub setFrom($) {
	my $this = shift @_;
	my $fromAddress = shift @_;

	if($fromAddress !~ /\w+\@\w+\.\w+/) {
		return undef;
	}
	$this->set('from', $fromAddress);
	return $fromAddress;
}

# Tells us whether or not we should show an e-vite link
# for the current questionnaire. The logic is that the 
# e-vite should only actually appear on real pages for
# real administrators
sub showLink($) {
	my $this = shift @_;

	return 0 unless $share{'DB'}->is_manager();
# CHECK FOR PRIVILEGES
	return 0 unless $share{'Page'}->is_publishing();
# CHECK FOR PUBLISHED PAGE
}

# Returns a list of regular expressions to use to check a
# questionnaire link (to ensure it has all of the required components)
sub availableLinks($) {
	my $this = shift @_;
	my @questionnaires = @_;

	my @links = ();
	# QCMD=do?
	my $action = qr/action=do/;
	foreach my $q (@questionnaires) {
		my $questionnaireId = $q->id;
		my $qIdMatch = qr/qid=$questionnaireId/;
		push @links, [$action, $questionnaireId];
	}
	return @links;
}

# Check the message text to see if a (proper) link has
# been entered. We should have some kind of kick-back if
# a good URL doesn't exist in the message.
sub validateMessage($) {
	my $this = shift @_;

	my $message = $this->message;
	if(!$message) {
		$this->error("Error: E-vite message is missing.");
		return 0;
	}
	my $urlToken = $this->urlToken;
	if($message =~ /$urlToken/ || $message =~ /href=(>|''|"")/) {
		$this->error("Error: invalid or missing questionnaire link.");
		return 0;
	}
	return 1;
}

# Send the stored email text to the list of stored recipients.
# Should take no arguments (the object should already be
# set up with all that it needs).
sub send($) {
	my $this = shift @_;

	my $recipients = $this->recipients;
	my $from = $this->from || $config{site}{administrator};
	my $subject = $this->subject;
	my $message = $this->message;

	return undef unless $recipients && $subject && $message;

	my @recipients = split /\s*,\s*/, $recipients;
	foreach my $to (@recipients) {
	    &ExSite::Mail::SendMail($to, $from, $subject, $message) if $this->isEnabled;
	}
	#$this->sendmail($recipients, $from, $subject, $message) if $this->isEnabled;
}

# Use MIME::Lite to send an email to the recipients in string $recipients
sub sendmail($$$$) {
	my $this = shift @_;
	my $recipients = shift @_;
	my $from = shift @_;
	my $subject = shift @_;
	my $message = shift @_;

	require MIME::Lite;

	my $recipients = $this->get('recipientsList') || [];

	my $htmlMessage = $message;
	$htmlMessage =~ s/\n/<br>\n/g;
	my $sent = 0;
	foreach my $r (@{$recipients}) {
		my $msg = new MIME::Lite('Type' => 'multipart/alternative', 'To' => $r, 'From' => $from, 'subject' => $subject);
		#$msg->attach('Type' => 'text/plain', 'Data' => $message);
		#$msg->attach('Type' => 'text/html', 'Data' => $htmlMessage);
		$msg->attach('Type' => 'text/plain', 'Data' => $message);
		$msg->attach('Type' => 'text/html', 'Data' => $htmlMessage);
		$sent++ if $msg->send;
	}
	return $sent;
}

1;

__END__
