package Modules::Questionnaire::Question;

use strict;
use ExSite::Config;
use ExSite::Report;
use ExSite::Misc;
use ExSite::Util;
use ExSite::Mail;
use Modules::Questionnaire::Response;
use Modules::Questionnaire::Answer;

use base	qw(Modules::Questionnaire::QuestionBase);

=pod

=head1 Question.pm

A Question is an instance of a question in the context of a questionnaire.
This context gives the question a few extra properties:

- a sortkey to set the position in the questionnaire
- the resulting position in the questionnaire
- a flag to determine whether a response is required or optional
- flow-control logic for how subsequent questions will be handled

=cut

sub new {
	my ($this,$id,$version) = @_;
	my $obj = {};
	my $class = ref($this) || $this;
	bless $obj, $class;
	if ($id =~ /^\d+$/) { 
		$obj->fetch($id,$version);
	} elsif(ref($id) =~ /HASH/ && $id->{'question_id'}) {
		$obj->setQuestion($id, $version);	
	} elsif(ref($id) eq $class) {
		$obj->setQuestion($id->get('q_question'), $version);
	}
	if($obj->isFlagged('mandatory')) {
		$obj->fetchBase->set('label_class', 'reqd_field');
	}
	return $obj;
}

# Fetch a q_question record and store it
sub fetch($) {
	my $this = shift @_;
	my $id = shift @_;
	my $version = shift @_;

	my $question = $this->get('q_question');
	return $question if $question;

	if($id =~ /^\d+$/) {
		$question = $share{'DB'}->fetch('q_question', $id);
	} else {
		$question = $id;
	}
	$this->set('version', $version);
	$this->set('q_question', $question);
	$this->fetchBase;
	return $question;
}

# Get the questionnaire that's associated with this question
sub fetchQuestionnaire($) {
	my $this = shift @_;
	my $id = shift @_;
	my $version = shift @_;

	my $questionnaire = $this->get('questionnaire');
	return $questionnaire if $questionnaire;

	require ExSite::Input;
	my $in = new ExSite::Input;

	my $query = $in->post_or_query;
	my $questionnaire = undef;

	require Modules::Questionnaire::Questionnaire;
	if(!$id && $query->{'questionnaire_id'} || $query->{'qid'}) {
		$id = $query->{'questionnaire_id'} || $query->{'qid'};
		$version = "";
		$questionnaire = new Modules::Questionnaire::Questionnaire($id, $version);
	} else {
		$questionnaire = new Modules::Questionnaire::Questionnaire($id, $version);
	}
	$this->set('questionnaire', $questionnaire);
	return $questionnaire;
}

# Fetch the question that matches the question_base_id and questionnaire_id
sub fetchMatch($) {
	my $this = shift @_;
	my $baseId = shift @_;
	my $qid = shift @_;
	my $version = shift @_;

	my $question = $this->get('q_question');
	return $question if $question;

	my $matchingQuestions = $share{'DB'}->fetch_match(
		'q_question', 
		{'question_base_id' => $baseId, 'questionnaire_id' => $qid}
	);

	if(scalar(@{$matchingQuestions}) > 1) {
		$this->error("fetchMatch: too many questions found: ".scalar(@{$matchingQuestions})." (should only be one or 0)");
	}

	$question = $matchingQuestions->[0];
	$this->set('q_question', $question);
	$this->set('version', $version);

	return $question;
}

# Fetch the q_question_base record for this question
# Return and store a Modules::Questionnaire::QuestionBase object
sub fetchBase($) {
	my $this = shift @_;

	my $base = $this->get('q_question_base');
	return $base if $base && $base->id;

	my $q = $this->get('q_question');
	return undef unless $q;

	$base = new Modules::Questionnaire::QuestionBase($q->{'question_base_id'}, $this->get('version'));
	$this->set('q_question_base', $base);
	return $base;
}

# Fetch the master q_question_base record for this question
# Return and store a Modules::Questionnaire::QuestionBase object
sub fetchBaseMaster($) {
	my $this = shift @_;

	my $base = $this->get('master_q_question_base');
	return $base if $base && $base->id;

	my $vBase = $this->fetchBase;
	if(!$vBase->master_id) {
		$this->set('master_q_question_base', $vBase);
		return $vBase;
	}
	$base = new Modules::Questionnaire::QuestionBase($vBase->master);
	$this->set('master_q_question_base', $base);
	return $base;
}

# Return an array of existing versions for this question.
# eg ("English", "Francais")
sub getVersions {
	my ($this) = @_;

	my $base = $this->getAllBase() || [];
	my @versions = ();

	foreach my $b (@{$base}) {
		push @versions, $b;
	}
	return @versions;
}

# Set the current question's version
sub setVersion {
	my ($this,$version) = @_;

	my @allVer = $this->getVersions();
	foreach my $v (@allVer) {
		if($v eq $version) {
			$this->set('version', $v);
		}
	}
	return $this->get('version');
}

# Fetches all base responses associated with this question
sub getAllBase {
	my $this = shift @_;

	my $base = $this->get('allBase');
	return $base if $base;

	my $quest = $this->get('master');

	$base = $this->fetch_match(
		'q_question_base', {'master_id' => $quest->{'id'}}
	);

	$this->set('allBase', $base);
	return $base;
}

# Current object's record ID
sub id($) {
	my $this = shift @_;

	my $q = $this->get('q_question') || {};
	return $q->{'question_id'};
}

sub question_base_id($) {
	my $this = shift @_;
	my $q = $this->get('q_question') || {};
	return $q->{'question_base_id'};
}

# Current question's flags (only handles mandatory
# questions right now)
sub flags($) {
	my $this = shift @_;
	my $question = $this->get('q_question');
	if($question) {
		return $question->{'flags'};
	}
	return "";
}

# Tells us if the question flag matches $flag
sub isFlagged($$) {
	my $this = shift @_;
	my $flag = shift @_;

	return ($this->flags =~ /$flag/);
}

# The current object's sort_key
sub sort_key($) {
	my $this = shift @_;

	return $this->get('q_question')->{'sort_key'};
}

# The current object's label
sub label($) {
	my $this = shift @_;

	my $base = $this->fetchBase();
	if($base) {
		return $base->label;
	}
	return "";
}

# The current object's questionnaire ID
sub questionnaire_id($) {
	my $this = shift @_;

	return $this->get('q_question')->{'questionnaire_id'};
}

# Returns a printable label 
# Printing style should be based on question internals
# such as the question flag value
sub printLabel($) {
	my $this = shift @_;

	my $label = $this->label;
	if($this->isFlagged('mandatory')) {
		return $label." *";
	}
	return $label;
}

# The question's base question format
# equivalent to $this->fetchBase->format
sub format($) {
	my $this = shift @_;

	my $base = $this->fetchBase();
	if($base) {
		return $base->format;
	}
	return "";
}

# The current object's base question answer values
# equivalent to $this->fetchBase->value
sub value($) {
	my $this = shift @_;

	my $base = $this->fetchBase();
	if($base) {
		return $base->value;
	}
	return "";
}

# Set the question values to the contents of the
# passed hash
sub setQuestion($) {
	my $this = shift @_;
	my $hash = shift @_;
	my $version = shift @_;

	return undef unless ref($hash) =~ /HASH/;
	$this->set('q_question', $hash);
	$this->set('version', $version);
	$this->SUPER::fetch($hash->{'question_base_id'}, $version);
	return;
}

# Compare one Question object against another
# To be used for sorting question objects
sub compare($$) {
	my $this = shift @_;
	my $question = shift @_;

	return 1 unless $this->id;
	return -1 unless $question->id;

	# If both this and $question's sort_key values are entirely composed of
	# digits, we'll compare the values numerically.
	# If one of them has a non-digit character, the sort_key values are compared
	# alphanumerically
	# If either of these comparisons turns up nothing, we sort by the 
	# object IDs (keys) numerically
	return ($this->sort_key =~ /^\d+$/ && $question->sort_key =~ /^\d+$/) ?
		$this->sort_key <=> $question->sort_key : 
			$this->sort_key cmp $question->sort_key || $this->id <=> $question->id;
}

=pod

=head2 Displaying a Question

Overloads Questionnaire::QuestionBase::show.
acts more like a wrapper to Questionnaire::QuestionBase::show, adding
question-specific logic to printing
=cut
sub show {
	my $this = shift @_;
	my $id = shift @_;
	my $value = shift @_;
	my $version = shift @_;
	my $flags = shift @_;
	my $previewFlag = shift @_;

	my $question = $this->get('q_question');
	$this->set('version', $version) if $version;

	return "" unless $question && $question->{'question_id'};

	my $baseQ = $this->fetchBase();

	# use QuestionBase::show to generate the basic HTML
	$value =~ s/\'/&quot;/g;
 	my $out = $baseQ->show("q".$id, $value, $flags, $previewFlag);
	# add some validation JS to check for required questions, if needed (?)

	return $out;
}

# Returns the javascript command to make the current question mandatory
# (if the current question is, in fact mandatory)
sub show_mandatory_js {
	my $this = shift @_;

	my $question = $this->get('q_question');
	return "" unless $question;

	my $id = $this->fetchBase->id;
	my $name = $this->fetchBase->label;
	# escape single quotes so they don't bork the javascript
	$name =~ s/\'/\&\#39;/g;

	if($question->{'flags'} =~ /mandatory/) {
		return "makeMandatory('question_$id', '$name');\n";
	}
	return "";
}

=pod

=head2 Building a Questionnaire

make() - install a new question AND tie it to a questionnaire

link() - tie a pre-existing questin to a questionnaire

=cut

sub make {
	my ($this,$question,$questionBase, $flags) = @_;

	my $b = new Modules::Questionnaire::QuestionBase;
	my $newId = 0;

	if($questionBase->{'master_id'}) {
		$b->fetch($questionBase->{'master_id'});
		$b->set('current', $questionBase);
		$newId = $b->makeVersion(%{$questionBase});
	} else {
		$b->set('master', $questionBase);
		$newId = $b->make(%{$questionBase});
	}
	$questionBase->{'question_base_id'} = $newId;
	return $this->link($question, $questionBase, $flags);
}

sub copy($) {
	my $this = shift @_;
	my $questionnaire = shift @_;

	return undef unless ($this->id && $questionnaire->id && $this->questionnaire_id != $questionnaire->id);
	# Don't copy questionnaires from different sections
	return undef if ($this->section_id != $questionnaire->section_id);

	my %question = %{$this->get('q_question')};
	delete $question{'question_id'};
	$question{'questionnaire_id'} = $questionnaire->id;
	my $newId = $share{'DB'}->insert('q_question', \%question);
	my $newQ = new Modules::Questionnaire::Question;
	$newQ->set('q_question', \%question);
	return $newQ;
}

sub link {
	my ($this, $question, $questionBase, $flags) = @_;

	return undef unless $questionBase->{'question_base_id'} && defined($question->{'questionnaire_id'});

	$question->{'question_base_id'} = $questionBase->{'question_base_id'};
	my $newId = $share{'DB'}->insert('q_question', $question);
	if($newId) {
		$question->{'question_id'} = $newId;
		$this->set('q_question', $question);
		$this->set('q_question_base', undef);
	}

	return $newId;
};

# Basically deletes a question. The question base is preserved, but nothing
# links it to the current questionnaire anymore
sub unlink {
	my ($this) = @_;

	return undef unless $this->id;

	my $delResult = $share{'DB'}->trash_r('q_question', $this->id);
	$this->set('q_question', undef) if $delResult;
	return $delResult;
}

# Update the q_question object with %vals
sub update($) {
	my $this = shift @_;
	my %vals = @_;
	
	return undef unless $this->id;

	my $questionHash = $this->get('q_question');
	return unless $questionHash;

	foreach my $k (keys %{$questionHash}) {
		$questionHash->{$k} = $vals{$k} if defined($vals{$k});
	}

	my $upResult = $share{'DB'}->update('q_question', $questionHash);
	$this->set('q_question', $questionHash) if $upResult;
	return $upResult;
}

# Tell us how many times a given question has been answered
# with the value provided
sub answerCount($) {
	my $this = shift @_;
	my $answerValue = shift @_;

	$this->loadAnswers();
	my $answers = $this->get('answerLists');
	my $allAnswers = $answers->{$answerValue} || [];
	return scalar(@{$allAnswers});
}

# Tells us the total number of answers provided for this
# question
sub totalAnswerCount($) {
	my $this = shift @_;

	my $allAnswers = $this->get('totalAnswerCount');
	return $allAnswers if defined($allAnswers);

	my @answers = $this->loadAnswers();

	my $answerTotal = 0;

	foreach my $a (@answers) {
		my $aValue = $a->value();
		$answerTotal += ($aValue) ? 1 : 0;
	}

	$this->set('totalAnswerCount', $answerTotal);
	return $this->get('totalAnswerCount');
}

# Get all of the values stored for this question.
# This only retrieves the stored answers, and might not
# cover all of the options available as defined in the
# q_question_base record.
sub getAnswerValues($) {
	my $this = shift @_;

	$this->loadAnswers();
	# Use the hash whose keys are the answer values in order
	# to avoid printing the same answer value multiple times
	my $answerHash = $this->get('answerLists') || {};

	my @valueList = ();
	foreach my $k (keys %{$answerHash}) {
		push @valueList, $k;
	}
	return @valueList;
}

# Returns the list of all possible values for the question and
# adds any other answers that may have been entered through the
# form (such as entries to an 'other' input for a select)
sub getValues($) {
	my $this = shift @_;

	my $values = $this->get('allValues');
	return @{$values} if $values;

	my @values = $this->fetchBase->getValues();
	my @aValues = $this->getAnswerValues();
	my %seenValues = ();
	map { $seenValues{$_} = 1; } @values;
	my @totalValues = @values;
	foreach my $v (@aValues) {
		push @totalValues, $v unless (!$v || $seenValues{$v}++);
	}
	$this->set('allValues', \@totalValues);
	return @totalValues;
}

# Load all of the answers for the current question
# (for all Questionnaires)
sub loadAnswers($) {
	my $this = shift @_;
	
	my $answers = $this->get('allAnswers');
	return @{$answers} if $answers;

	$answers = $share{'DB'}->fetch_match('q_answer', 
		{'question_id' => $this->id}
	);

	my $answerLists = {};
	my $answerObjs = [];
	foreach my $a (@{$answers}) {
		my $obj = new Modules::Questionnaire::Answer($a);
		push @{$answerObjs}, $obj;
		push @{$answerLists->{$obj->value}}, $obj;
	}
	$this->set('answerLists', $answerLists);
	$this->set('allAnswers', $answerObjs);
	return @{$answerObjs};
}

# Load all of the answers for the current question in the supplied
# questionnaire
sub loadQuestionnaireAnswers($) {
	my $this = shift @_;
	my $qid = shift @_;
	
	return undef unless $qid;

	my $answers = $this->get('allQuestionnaireAnswers');
	return $answers if $answers;

	$answers = $share{'DB'}->custom_query("SELECT q_answer.* FROM q_answer, q_response WHERE q_answer.response_id = q_response.response_id AND q_response.questionnaire_id=$qid");

	my $objs = [];
	foreach my $a (@{$answers}) {
		push @{$objs}, new Modules::Questionnaire::Answer($a);
	}

	$this->set('allQuestionnaireAnswers', $objs);
	return $objs;
}

# Load the answers for the current question, in the current questionnaire
# matching the current question
sub loadMemberQuestionnaireAnswer($) {
	my $this = shift @_;
	my $qid = shift @_;
	my $memberId = shift @_;

	return undef unless $qid && $memberId;

	my $answers = $this->get('allMemberQuestionnaireAnswers');
	return $answers if $answers;

	$answers = $share{'DB'}->custom_query("SELECT q_answer.* FROM q_answer, q_response WHERE q_answer.response_id=q_response.response_id AND q_response.questionnaire_id='$qid' AND q_response.member_id='$memberId'");

	my $objs = [];
	foreach my $a (@{$answers}) {
		push @{$objs}, new Modules::Questionnaire::Answer($a);
	}

	$this->set('allMemberQuestionnaireAnswers', $objs);

	return $objs;
}

# Load a specific answer to a specific question on a specific questionnaire
sub loadResponseAnswer {
	my $this = shift @_;
	my $qid = shift @_;
	my $responseId = shift @_;

	return undef unless $qid && $responseId;

	my $answers = $this->get('answer');
	return $answers if $answers;

	my $id = $this->id;

	$answers = $share{'DB'}->custom_query("SELECT q_answer.* FROM q_answer, q_response WHERE q_answer.response_id=q_response.response_id AND q_answer.question_id='$id' AND q_response.response_id='$responseId'");

	my $answerObj = new Modules::Questionnaire::Answer($answers->[0]);
	$this->set('answer', $answerObj);

	return $answerObj;
}

# In the same way that questionnaire has addResponse, a question
# has addAnswer
sub addAnswer($) {
	my $this = shift @_;
	my $data = shift @_;

	if(!$data) {
		require ExSite::Input;
		my $input = new ExSite::Input;
		$data = $input->post_or_query;
	}
	my $msg = "";
	my $newId = 0;

	# If there's a response, fetch the answer associated with the current
	# response, otherwise, we get a new answer object
	my $answer = $this->loadResponseAnswer($this->questionnaire_id, $data->{'response_id'}) || new Modules::Questionnaire::Answer;

	my %record = (
		'answer_id' => $answer->id,
		'question_id' => $this->id,
		'response_id' => $data->{'response_id'},
		'value' => $data->{'value'},
	);
	if(!$record{'answer_id'} || ($answer->value ne $record{'value'})) {
		$newId = $answer->make(%record);
	}
	return ($newId, $msg);
}

sub viewBaseLink($) {
	my $this = shift @_;
	my $reply = shift @_;

	return "" unless $this->id && $this->fetchBase->id;

	my $version = ($this->fetchBase->master_id) ? $this->version : undef;
	
	$reply = $reply || &relink();
	return "<a href='".&relink('qcmd' => 'question_base_view', 'qbid' => $this->fetchBase->id, 'version' => $version, 'reply' => $reply)."'>".$this->printLabel."</a>";

}

# Question-based icons:
sub editIcon($) {
	my $this = shift @_;

	return "" unless $this->id;
	my $out = "<a href='".&relink('qcmd'=>'edit_question', 'qqid' => $this->id, 'q_submit' => undef)."'><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/new12.gif' border='0'></a>\n";
	return $out;
}

sub moveUpIcon($) {
	my $this = shift @_;

	return "" unless $this->id;

	my $reply = &relink();
	my $out = "<a href='".&relink('qcmd' => 'question_swap_up', 'qqid' => $this->id, 'qid' => $this->questionnaire_id, 'reply' => $reply, 'submit_action' => 'Submit')."'><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/up12.gif' border='0'></a>";
	return $out;
}

sub moveDownIcon($) {
	my $this = shift @_;

	return "" unless $this->id;

	my $reply = &relink('reply' => undef);
	my $out = "<a href='".&relink('qcmd' => 'question_swap_down', 'qqid' => $this->id, 'qid' => $this->questionnaire_id, 'reply' => $reply, 'submit_action' => 'Submit')."'><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/down12.gif' border='0'></a>";
	return $out;
}

sub unlinkIcon($) {
	my $this = shift @_;

	return "" unless $this->id;

	my $reply = &relink('reply' => undef);
	my $out = "<a href='".&relink("qcmd"=>"unlink", "qqid" => $this->id, 'reply' => $reply, "q_submit" => undef)."'><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/rm12.gif' border='0'></a>\n";
	return $out;
}

sub newFinalIcon($) {
	my $this = shift @_;

	return "" unless $this->id;

	my $reply = &relink('reply' => undef);
	my $out = "<a href='".&relink('qcmd'=>'new_final_version', 'version' => undef, 'qid' => $this->questionnaire_id, 'qbid' => $this->id, 'qqid' => $this->id, 'section_id' => $this->fetchBase->section_id)."'><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/edit12.gif' border='0'></a>";
	return $out;
}

sub mandatoryIcon($) {
	my $this = shift @_;

	return "" unless $this->id;

	my $id = $this->id;
	my $reply = &relink('reply' => undef);

	if($this->isFlagged('mandatory')) {
		return "<a href='".&relink('q_question#'.$id.'#flags' => '', 'q_question#'.$id.'#question_id' => $id, 'submit_action' => 'Submit', 'reply' => $reply)."'><img alt='Toggle Mandatory Flag' src='$config{server}->{HTMLpath}/_Modules/Questionnaire/star12.gif' border='0'></a>";
	}
	return "<a href='".&relink('q_question#'.$id.'#flags' => 'mandatory', 'q_question#'.$id.'#question_id' => $id, 'submit_action' => 'Submit', 'reply' => $reply)."'><img alt='Toggle Mandatory Flag' src='$config{server}->{HTMLpath}/_Modules/Questionnaire/star12.gif' border='0'></a>";
}

1;
__END__
