package Modules::Questionnaire::QuestionBase;

use strict;
use ExSite::Config;
use ExSite::Report;
use ExSite::Misc;
use ExSite::Util;
use ExSite::Mail;

use base	qw(ExSite::Base);

our $QUESTION_BASE_VALUES_DELIM = "\\|";

our %QUESTION_BASE_INFO = (
	'radio' => "Please enter the selection of answer values below. Each entry should be delimited by a &quot;[[DELIM]]&quot; character.",
	'select' => "Please enter the selection of answer values below. Each entry should be delimited by a &quot;[[DELIM]]&quot; character.",
	'multiselect' => "Please enter the selection of answer values below. Each entry should be delimited by a &quot;[[DELIM]]&quot; character.",
	'slider' => "Please set the minimum and maximum values for the slider. Separate the values by a &quot;[[DELIM]]&quot; character.",
	'label' => "Any information entered into the &quot;answers&quot; box will be displayed verbatim underneath the label."
);

=pod

=head1 QuestionBase.pm

A QuestionBase object references a single question from the question database.
This question can be a master, or a secondary version.

Some methods select (or create) the question that the object references.
Other methods 


=cut

# new: define a QuestionBase object
sub new {
	my ($this,$id,$version) = @_;
	my $obj = {};
	my $class = ref($this) || $this;
	bless $obj, $class;

	if($id =~ /^\d+$/) {
		$obj->fetch($id,$version);
	} elsif(ref($id) =~ /HASH/ && $id->{'question_base_id'}) {
		if($id->{'master_id'}) {
			$obj->set('current', $id);
			my $master = $share{'DB'}->fetch('q_question_base', $id->{'master_id'});
			$obj->set('master', $master);
		} else {
			$obj->set('current', $id);
			$obj->set('master', $id);
		}
	}
	return $obj;
}

# fetch: fetch the question data from the database
# $id can be either the question_base_id value (for fetching from the DB)
sub fetch {
	my ($this,$id,$version) = @_;

	# case 0: id is not defined, and we use the currently-loaded question
	# return cached copy
	if(!$id) {
		my $question = $this->current;
		return $question if $question && $question->{'question_base_id'};
	}
	
	my $question = undef;
	# case 1: id is a numeric record ID

	# fetchById: ($id, $version)
	if($id =~ /^\d+$/) {
		$question = $this->fetchById($id, $version);
	}

	# case 2: id is the question label
	# 
	if(!$question) {
		$question = $this->fetchByLabel($id, $version);
	}


	# look for a master question that matches this id

	if(!$question) {
		$this->error("Unable to find question matching $id");
	}
	# authenticate ownership of question
	# DAN: How is authentication done?
	# We shouldn't have to authenticate the ownership of question_base. We should
	# only have to ensure that it's being loaded for the correct site

	# Determine the site for our question and the allowable sites for the user
	# and check each

	if(!$share{'DB'}->is_member({'section_id' => $question->{'section_id'}})) {
		$share{'DB'}->error("Attempt to fetch question from invalid section");
		return 0;
	}

	# fetchById and fetchByLabel grab the master
	# and the version record (if it exists). If a version has been found,
	# it'll cache the master and version then return the version, otherwise
	# it'll return the master.
	# This means any lookups for alternate versions isn't really required
	# here (along with the caching)
	# look for alternate version of the master that matches the version
	# select the alternate version, if found;  the master otherwise
	# store the selected version as the current question
	return $question;
}

# Fetch the q_question_base record whose question_base_id value is $id
# if the fetched record is the 'master' version, both the 'current' and 'master'
# hashes will represetn the master version
# if the fetched record is a sub-version of the master, then 'current'
# contains the versioned record and 'master' will contain the master to
# the current version
sub fetchById() {
	my $this = shift @_;
	my $id = shift @_;
	my $version = shift @_;

	my $record = $share{'DB'}->fetch('q_question_base', $id);
	my $master = undef;
	my $current = undef;

	# If the ID provided is for a version, we'll take the info
	# from our retrieved record to get the master record
	if($record->{'master_id'}) {
		$master = $share{'DB'}->fetch('q_question_base', $record->{'master_id'});
		$current = $record;
	} elsif($version) {
		# If the ID provided was for a master version and a version was provided,
		# we fetch that version and set it as the current (used) version
		$current = $share{'DB'}->fetch_match('q_question_base', {'master_id' => $id, 'version' => $version})->[0];
		$master = $record;
	} else {
		# If the ID provided was for the master and no version was provided,
		# then this is simply a master record, so 'current' and 'master'
		# should be the same
		$master = $current = $record;
	}

	$this->set('master', $master);
	$this->set('current', $current || $master);
	return $this->current;
}

# Fetch a question based on its label
sub fetchByLabel() {
	my $this = shift @_;
	my $label = shift @_;
	my $version = shift @_;

	my $master = $share{'DB'}->fetch_match('q_question_base', 
		{'label' => $label, 'version' => 'null'})->[0];
	my $recordVersion = $share{'DB'}->fetch_match('q_question_base', 
		{'label' => $label, 'version' => $version})->[0];

	$this->set('master', $master);
	$this->set('current', $recordVersion || $master);
	return $this->current;	
}

# Allow a master record to fetch all of the versions
# that refer to the current base question
sub fetchAllVersions($) {
	my $this = shift @_;

	my $versions = $this->get('allVersions');
	return @{$versions} if $versions;

	my $id = $this->master_id || $this->id;
	my $versionRecs = $share{'DB'}->fetch_match(
		'q_question_base', {'master_id' => $id}
	);

#	# Start by putting the master record in the list
	$versions = [];
	# Continue on with all of the versions
	for(my $i = 0; $i < @{$versionRecs}; $i++) {
		my $v = new Modules::Questionnaire::QuestionBase();
		$v->set('master', $this->master);
		$v->set('current', $versionRecs->[$i]);
		push @{$versions}, $v;
	}

	$this->set('allVersions', $versions);
	return @{$versions};
}

# Fetch all of the base questions for a given section/site
sub fetchAll() {
	my $this = shift @_;
	my $siteId = shift @_;

	return undef unless $siteId;

	my $questions = $this->get('allQuestions');
	return $questions if $questions;

	$questions = $share{'DB'}->fetch_match(
		'q_question_base', {'section_id' => $siteId}
	);

	my $qObjs = [];

	foreach my $q (@{$questions}) {
		push @{$qObjs}, new Modules::Questionnaire::QuestionBase($q);
	}

	$this->set('allQuestions', $qObjs);
	return $qObjs;
}

# Fetch all question links that are using this base question
# Takes: q_question_base ID (question_base_id)
sub fetchQuestions($) {
	my $this = shift @_;

	return undef unless $this->id ;

	my $questions = $this->get('questions');
	return @{$questions} if $questions;

	$questions = $share{'DB'}->fetch_match('q_question', 
		{'question_base_id' => $this->master->{'question_base_id'}}
	);

	my @qObj = ();
	foreach my $q (@{$questions}) {
		my $questionObject = new Modules::Questionnaire::Question($q);
		$questionObject->set('q_question_base', $this);
		push @qObj, $questionObject;
	}

	$this->set('questions', \@qObj);
	return @qObj;
}

# isFinal is meant to tell us if this QuestionBase record represents the 
# "final screen" for a questionnaire. The final screen is saved as a label
# 'question', but isn't actually shown on the questionnaire or any of the
# questions listings.
sub isFinal($) {
	my $this = shift @_;

	return undef unless $this->id;
	my $isFinal = $this->get('isFinal');

	return $isFinal if defined($isFinal);

	my @questions = $this->fetchQuestions;
	if(scalar(@questions)) {
		if($questions[0]->sort_key eq $Modules::Questionnaire::Questionnaire::FINAL_QUESTION_POSITION) {
			$this->set('isFinal', 1);
			return 1;
		}
	}
	$this->set('isFinal', 0);
	return 0;
}

# If it's a master record, it's always going to be up-to-date
# If it's not a master record, its timestamp has to be the same
# or more recent than the master. If this isn't true, then it
# indicates that the master version has been re-edited since
# the creation of the version, and the version should be checked
# to ensure that it accurately represents the master version
sub isApproved($) {
	my $this = shift @_;

	return 1 if !$this->master_id;

	return ($this->master->{'timestamp'} <= $this->current->{'timestamp'});
}

# Certain questions (multiselect, select, radio and label) require
# an additional "values" field to tell us what options to present
# the end-user. This method tells us if the current question is of
# that type.
sub requiresValues($) {
	my $this = shift @_;
	return 0 if !$this->id;
	return &Modules::Questionnaire::QuestionBase::formatRequiresValues($this->format);
}

# We check here to see if the question can use values (but doesn't neccessarily
# require them)
sub allowsValues($) {
	my $this = shift @_;
	my $format = $this->format;
	return 1 if ($format =~ /multiselect|select|radio|label/);
	return 0;
}

# We check here to see if the question NEEDS values in order to make sense.
sub formatRequiresValues($) {
	my $format = shift @_;

	return 1 if ($format =~ /multiselect|select|radio/);
	return 0;
}

# The question_base_id for the current question.
sub id($) {
	my $this = shift @_;

	my $question = $this->current || {};
	return $question->{'question_base_id'};
}

# clear: unset the current question
sub clear {
	my $this = shift @_;

	$this->set('master', undef);
	$this->set('current', undef);
	return;
}

# Return the current question's label
sub label($) {
	my $this = shift @_;

	return $this->current->{'label'};
}

sub printableLabel($) {
	my $this = shift @_;

	my $l = $this->label;
	$l =~ s/\"/&quot;/g;
	$l =~ s/<br>/\r/g;
	$l =~ s/<[^>]*>//g;
	return $l;
}

# Returns the master question's label.
sub masterLabel($) {
	my $this = shift @_;

	return $this->master->{'label'};
}

# Label for the "other" field:
sub otherLabel($) {
	my $this = shift @_;

	my $delim = $this->delim;
	my @values = split /\s*$delim\s*/, $this->value;
	my $lastVal = $values[$#values];
	if($lastVal =~ /^!/) {
		return substr($lastVal, 1);
	}
	return "Other (please specify)\n";
}

# Get the base question format (version shouldn't matter here)
sub format($) {
	my $this = shift @_;
	return $this->master->{'format'};
}

# Get the version of the question
sub version($) {
	my $this = shift @_;
	return $this->current->{'version'};
}

# Get the section_id value for the question record
sub section_id($) {
	my $this = shift @_;

	return $this->current->{'section_id'};
}

# Get the current version's master_id value. This means
# that if we're using a master version, the master_id will
# be NULL/undef
sub master_id($) {
	my $this = shift @_;
	return $this->current->{'master_id'};
}

# Return the object's flags. The master and current (version) record's
# flags should be the same, so we're going to always use the master
sub flags($) {
	my $this = shift @_;

	return $this->master->{'flags'};
}

# Return the hash representing the master record
sub master($) {
	my $this = shift @_;
	return $this->get('master');
}

# return the hash representing the current record.
# If we're using a version, current holds the current version's information
# if it's a master, then the record is the same as $this->master
sub current($) {
	my $this = shift @_;
	return $this->get('current');
}

# Easy-to-read version of the question format
# I don't think this is used anymore.
sub formatText($) {
	my $format = shift @_;

	if($format eq "string") {
		return "Text input";
	} elsif($format eq "textarea") {
		return "Large text input";
	} elsif($format eq "select") {
		return "Pull-down list";
	} elsif($format eq "multiselect") {
		return "Multiple checkboxes";
	} elsif($format eq "radio") {
		return "Radio buttons";
	} elsif($format eq "checkbox") {
		return "Single checkbox";
	} elsif($format eq "label") {
		return "Label";
	} elsif($format eq "slider") {
		return "Slider";
	}

	return ucfirst(lc($format));
}

# Returns an easy-to-read version of the question format...
# doesn't look like it's being used anymore
sub formatLabel($) {
	my $this = shift @_;

	my $format = $this->master->{'format'};
	return &formatText($format);
}

# Get the 'value' for the current question
sub value($) {
	my $this = shift @_;
	return $this->current->{'value'};
}

# Return a list of all of the values entered as possible answers
# for the question
sub getValues($) {
	my $this = shift @_;

	my $delim = $this->delim;
	#my @answers = split /\s*$delim\s*/, $this->current->{'value'};
	# need to trim trailing whitespace and linebreaks
	my $value = $this->current->{'value'};
	$value =~ s/[\s\r\n]*$//;
	my @answers = split /\s*$delim\s*/, $value;
	if($answers[$#answers] =~ /^!/) {
		pop @answers;
	}

	if($this->format =~ /slider/) {
		my ($min, $max) = @answers;
		@answers = ();
		for(my $i = $min; $i <= $max; $i++) {
			push @answers, $i;
		}
	}

	return @answers;
}

# Return a list of all of the values entered as possible answers
# for the question master version
sub getMasterValues($) {
	my $this = shift @_;

	my $delim = $this->delim;
	# need to trim trailing whitespace and linebreaks
	my $value = $this->master->{'value'};
	$value =~ s/[\s\r\n]*$//;
	my @answers = split /\s*$delim\s*/, $value;
	if($answers[$#answers] =~ /^!/) {
		pop @answers;
	}
	return @answers;
}

# Return a count of all of the possible answers for this question. 0 is returned if there's
# no values, or if the values for the current and master records seem to be out of sync
sub valueCount($) {
	my $this = shift @_;

	my $current = $this->current;
	my $master = $this->master;

	my @currentValues = $this->getValues();
	my @masterValues = $this->getMasterValues();

	# The current (versioned) and master base questions should never have
	# a different list of answers
	if(scalar(@currentValues) != scalar(@masterValues)) {
		return 0;
	}
	return scalar(@currentValues);
}

# Retrieves all of the matching question_base records and lists their versions
sub getVersions {
	my ($this) = @_;

	my $allVersions = $this->get('versionList');
	return @{$allVersions} if $allVersions;

	my @versions = $this->fetchAllVersions();
	$allVersions = [];
	foreach my $v (@versions) {
		push @{$allVersions}, $v;
	}
	$this->set('versionList', $allVersions);
	return @{$allVersions};
}

# Sets the preferred version for the object. This only takes effect
# if the questionnaire supports the version
sub setVersion {
	my ($this,$version) = @_;

	my @versions = $this->getVersions();
	my $currentVersion = $this->get('version');

	foreach my $v (@versions) {
		if($v eq $version) {
			$this->set('version', $v);
			$currentVersion = $v;
			last;
		}
	}
	return $currentVersion;
}

=pod

=head2 Displaying a Questionnaire

=cut

# return HTML to display the question.
# You need to provide an ID to get proper question processing; otherwise
# this is little more than a preview function.  The ID is used to name
# the question DIV, and the INPUT.
sub show {
	my ($this, $id, $value, $flags, $previewFlag) = @_;
	my $out = "";
	# compose input field in $out
	if($out = $this->show_input($id, $value, $flags, $previewFlag)) {
		# validation javascript?
		#return "<div class=\"Question\" id=\"$id\">\n$out\n</div>\n";
		return "\t<tr>\n\t\t<td>\n\t\t\t$out\n\t\t</td>\n\t</tr>\n";
	}
	return "";
}


### private methods to compose the input fields for particular 
### question styles, eg. 
### 
### show_string()
### show_textarea()
### show_select()
### show_radio()
### show_multiselect()
### show_checkbox()

# show_input is basically just a switch statement to select the
# correct input field for the given question
sub show_input() {
	my $this = shift @_;
	my $id = shift @_;
	my $value = shift @_;
	my $flags = shift @_;
	my $previewFlag = shift @_;

	my $masterQuestion = $this->master;
	my $versionQuestion = $this->current;
	return "" unless $masterQuestion;

	my $format = $masterQuestion->{'format'};
	my $label = $this->label;
	my $out = "";
	my $inputId = "question_".$this->master->{'question_base_id'};

	my $count = ($flags =~ /grid/) ? $this->valueCount() : 1;

	if($format eq "string") {
		$out = $this->show_string($inputId, $value);
	} elsif($format eq "textarea") {
		$out = $this->show_textarea($inputId, $value);
	} elsif($format eq "select") {
		$out = $this->show_select($inputId, $value);
	} elsif($format eq "radio") {
		$out = $this->show_radio($inputId, $value, $count, $flags);
	} elsif($format eq "multiselect") {
		$out = $this->show_multiselect($inputId, $value, $count, $flags);
	} elsif($format eq "checkbox") {
		$out = $this->show_checkbox($inputId, $value);
	} elsif($format eq "label") {
		$out = $this->show_label($inputId, $value);
	} elsif($format eq "slider") {
		$out = $this->show_slider($inputId, $value);
	}
	if($out || ($label && $format eq "label")) {
		if($flags =~ /grid/) {
			my $values = $this->valuesList;
			return $this->gridQuestionWrapper($id, $label, $out, $values, $flags, $previewFlag);
		}
		return $this->questionWrapper($id, $label, $out, $flags, $previewFlag); 
	}
	return "";
}

# Return the ID value for an individual input element
sub input_id() {
	my $this = shift @_;

	my $id = $this->get('input_id');
	
	if(!$id) {
		my $question = $this->current;
		# Use the 'master id' unless it's null (which means we already
		# have the master). In this case, we'll just use the main ID
		# for the question
		$id = $question->{'master_id'} || $question->{'question_base_id'};
	}

	return "" unless $id;
	return "q".$id;
}

# Prints a wrapping div and table for each each question
sub questionWrapper($) {
	my $this = shift @_;
	my $id = shift @_;
	my $label = shift @_;
	my $input = shift @_;
	my $flags = shift @_;
	my $previewFlag = shift @_;

	return "" unless $id && $label;

	if(!$input) {
		return $this->labelWrapper($id, $label, $flags, $previewFlag);
	}

	return <<"DONE";
<div class="Question" id='$id' onClick='javascript:highlight("$id")'>
	<table class='QuestionTable'>
		<tr>
			<td>
				<span class='QuestionLabel'>$label</span>
			</td>
		</tr>
		<tr>
			<td>
				$input
			</td>
		</tr>
	</table>
</div>
DONE
}

sub labelWrapper($) {
	my $this = shift @_;
	my $id = shift @_;
	my $label = shift @_;
	my $flags = shift @_;
	my $previewFlag = shift @_;

	return "" unless $id && $label;

	return <<"DONE";
<div class="Question" id='$id' onClick='javascript:highlight("$id")'>
	<table class='QuestionTable'>
		<tr>
			<td>
				<span class='QuestionLabel'>$label</span>
			</td>
		</tr>
	</table>
</div>
DONE
}

# Wrapper div + table for a 'gridded' question (either a radio or multiselect
# question whose values are listed left-right instead of up-down)
sub gridQuestionWrapper($) {
	my $this = shift @_;
	my $id = shift @_;
	my $label = shift @_;
	my $input = shift @_;
	my $values = shift @_;
	my $flags = shift @_;
	my $previewFlag = shift @_;

	return "" unless $id && $label && $input;

	my $out = "";
	if($flags =~ /header/) {
		$out = "<!-- GRID START -->\n<table class=\"RadioQuestionInput\">\n";
		$out .= "\t<tr>\n\t\t<td></td>\n\t\t$values</tr>\n";
	}
	
	$out .= <<"DONE";
<div id='$id' class=\"GridQuestionInput\">
	<tr>
		<td>
			$label
		</td>
		$input
	</tr>
</div>
DONE
	

	if($flags =~ /footer/ ){
		$out .= "</table>\n";
	}
	return $out;
}

# Short string input (input='text')
sub show_string() {
	my $this = shift @_;
	my $id = shift @_ || $this->input_id;
	my $value = shift @_;

	return "<input type='text' id='$id' name='$id' size='40' value='$value'>";
}

# Textarea input (<textarea>blah</textarea>
sub show_textarea() {
	my $this = shift @_;
	my $id = shift @_ || $this->input_id;
	my $value = shift @_;

	return "<textarea rows='10' cols='40' id='$id' name='$id'>$value</textarea>";
}

# Select drop-down list (<select>possible values</select>)
sub show_select() {
	my $this = shift @_;
	my $id = shift @_ || $this->input_id;
	my $value = shift @_;

	my @currentValues = $this->getValues();
	my @masterValues = $this->getMasterValues();

	if(scalar(@currentValues) == 0 || scalar(@masterValues) != scalar(@currentValues)) {
		return "";
	}

	my ($selectValue, $otherValue) = split /\;/, $value;
	$otherValue =~ s/^\s*//;

	my $selectHash = { $selectValue.'_selected' => 'selected' };
	my $otherSelected = ($otherValue && !$selectValue) ? " selected" : "";

	my $out = "<select name='$id' id='$id'>\n";
	$out .= "\t<option value=''>== select ==</option>\n";
	for(my $i = 0; $i < @currentValues; $i++) {
		$out .= "\t<option value='$masterValues[$i]' ".$selectHash->{$masterValues[$i].'_selected'}.">$currentValues[$i]</option>\n";
	}
	$out .= "</select>\n";
	$out .= "<br>".$this->otherLabel.": <input type='text' cols='30' name='$id' value='$otherValue'>" if $this->flags =~ /other/;
	return $out;
}

# Radio buttons (<input type='radio'>)
sub show_radio() {
	my $this = shift @_;
	my $id = shift @_ || $this->input_id;
	my $value = shift @_;
	my $columns = shift @_ || 1;
	my $flag = shift @_;

	return $this->show_grid($id, $value, $columns, $flag, 'radio') if $flag =~ /grid/;

	my @values = $this->getMasterValues();
	my @pValues = $this->getValues();

	my $checkHash = { $value.'_checked' => 'checked' };

	my $out = "";

	$out .= "\t<tr>\n\t\t<td></td>\n\t\t<td>\n".$this->show_values()."</td>\n\t</tr>\n" if $flag =~ /header/;
	for(my $i = 0; $i < @values; $i += $columns) {
		$out .= "\t<tr>";
		for(my $j = $i; $j < ($i+$columns) && $j < @values; $j++) {
			my $label = ($columns == 1) ? $pValues[$j] : "";
			$out .= "<td><input name='$id' id='$id' type='radio' value='$values[$j]' ".$checkHash->{$values[$j].'_checked'}.">$label&nbsp;</td>";
		}
		$out .= "</tr>\n";
	}
	return $out;
}

# Multiselect (multiple items-selectable list ... 
# will have to look this up)
sub show_multiselect() {
	my $this = shift @_;
	my $id = shift @_ || $this->input_id;
	my $value = shift @_;
	my $columns = shift @_ || 1;
	my $flag = shift @_;

	return $this->show_grid($id, $value, $columns, $flag, 'checkbox') if $flag =~ /grid/;

	my @masterValues = $this->getMasterValues();
	my @currentValues = $this->getValues();

	if(scalar(@currentValues) == 0 || scalar(@masterValues) != scalar(@currentValues)) {
		return "";
	}

	my @selectedValues = split /\s*;\s*/, $value;
	my $selectedHash = {};
	map { $selectedHash->{$_.'_checked'} = 'checked'; } @selectedValues;

	my $out = "";
	for(my $i = 0; $i < scalar(@masterValues); $i += $columns) {
		$out .= "\t<tr>\n";
		for(my $j = $i; $j < ($i+$columns) && $j < scalar(@masterValues); $j++) {
			my $label = ($columns == 1) ? $currentValues[$j] : "";
			$out .= "\t\t<td><input type='checkbox' name='$id' id='$id' value='".$masterValues[$j]."' ".$selectedHash->{$masterValues[$j].'_checked'}.">&nbsp;".$label."</td>\n";
		}
		$out .= "\t</tr>\n";
	}
	return $out;
}

sub show_grid($) {
	my $this = shift @_;
	my $id = shift @_ || $this->input_id;
	my $value = shift @_;
	my $columns = shift @_ || 1;
	my $flag = shift @_;
	my $type = shift @_;

	my @masterValues = $this->getMasterValues();
	my @currentValues = $this->getValues();

	if(scalar(@currentValues) == 0 || scalar(@masterValues) != scalar(@currentValues)) {
		return "";
	}

	my @selectedValues = split /\s*;\s*/, $value;
	my $selectedHash = {};
	map { $selectedHash->{$_.'_checked'} = 'checked'; } @selectedValues;

	my $out = "";
	for(my $i = 0; $i < scalar(@masterValues); $i += $columns) {
		for(my $j = $i; $j < ($i+$columns) && $j < scalar(@masterValues); $j++) {
			my $label = ($columns == 1) ? $currentValues[$j] : "";
			$out .= "\t\t<td><input type='$type' name='$id' id='$id' value='".$masterValues[$j]."' ".$selectedHash->{$masterValues[$j].'_checked'}.">&nbsp;".$label."</td>\n";
		}
	}
	return $out;
}

# Checkbox (<input type='checkbox')
sub show_checkbox() {
	my $this = shift @_;
	my $id = shift @_ || $this->input_id;
	my $value = shift @_;
	
	my $checked = ($value) ? "checked" : "";
	my $out = "";
	$out .= "<input type='checkbox' name='$id' id='$id' $checked>&nbsp;";
	return $out;
}

sub show_label($) {
	my $this = shift @_;
	my $value = $this->value;

	return "" unless $value;
	return <<"DONE";
	<tr>
		<td>
			$value
		</td>
	</tr>
DONE
}

sub show_slider($) {
	my $this = shift @_;
	my $id = shift @_ || $this->input_id;
	my $value = shift @_;

	if($this->get("slider_input_$id")) {
		return $this->get("slider_input_$id");
	}

	my $js = $this->insertJavascript($id, $value);

	my $out = <<"DONE";
	<span id=\"rating_$id\"></span>
	<div class=\"slider\" id=\"slider_$id\" tabIndex=\"1\">
		<input class=\"slider-input\" name=\"$id\" id=\"$id\">
	</div>
	<script language=\"javascript\">
		$js
	</script>
DONE
	$this->set("slider_input_$id", $out);
	return $out;
}

sub show_js($) {
	my $this = shift @_;
	my $id = shift @_ || $this->input_id;
	my $value = shift @_;

	return $this->insertJavascript($id,$value);
}

sub insertJavascript($) {
	my $this = shift @_;
	my $id = shift @_;
	my $value = shift @_;

	my $format = $this->format;
	if($format =~ /slider/) {
		return $this->insertSliderJavascript($id, $value);
	}
	return "";
}

sub insertSliderJavascript($) {
	my $this = shift @_;
	my $id = shift @_ || $this->input_id;
	my $value = shift @_ || "0";

	if($this->get('slider_javascript')) {
		return $this->get('slider_javascript');
	}
	my ($min, $max) = $this->getMasterValues();

	if(!defined($min) || !defined($max)) {
		$min = "0";
		$max = "100";
	}

	my $javascript = <<"DONE";
	// Create a slider object
	var slider_$id = new Slider(document.getElementById(\"slider_$id\"), document.getElementById(\"$id\"));
	// Set an onchange handler that updates the input field
	slider_$id.onchange = GetSliderValue;
	// Set a default value for the slider
	slider_$id.setValue($value);
	document.getElementById('rating_$id').innerHTML = '$value';
	slider_$id.input.value = '$value';
	slider_$id.setMinimum($min);
	slider_$id.setMaximum($max);
DONE
	$this->set('slider_javascript', $javascript);
	return $javascript;
}

sub valuesList($) {
	my $this = shift @_;

	my @values = $this->getValues();
	my $out = "";
	for(my $i = 0; $i < @values; $i++) {
		$out .= "\t\t<td style='text-align: center'>$values[$i]</td>\n";
	}
	return $out;
}

# Show a list of the possible answer values.
# This is mainly used to printing a header over a number of
# multiselect questions with the same options
sub show_values($) {
	my $this = shift @_;
	
	my $out = "\t<tr>\n".$this->valuesList()."\t</tr>\n";
	return $out;
}

# Returns true if the current object's values are the same
# as the passed object's values
sub compareValues($) {
	my $this = shift @_;
	my $question = shift @_;

	return 0 unless $question;

	my @thisValues = $this->getValues();
	my @qValues = $question->getValues();

	return 0 unless scalar(@thisValues) == scalar(@qValues);
	return 0 if (scalar(@thisValues) == 0);

	for(my $i = 0; $i < @thisValues; $i++) {
		if($thisValues[$i] ne $qValues[$i]) {
			return 0;
		}
	}
	return 1;
}

=pod

=head2 Create a Question

C<$q-E<gt>make(%q)> - create a new master question, 
and insert it into the database.  C<%q> gives the basic data values 
for the question.  The newly created question becomes the current
question, if successfully inserted.

C<$q-E<gt>makeVersion(%q)> - create a new question version, 
and insert it into the database.  C<%q> gives the basic data values 
for the version.  The version is attached to the master of the current
question, and becomes the current question, if successfully inserted.

=cut

sub make {
	my $this = shift @_;
	my %q = @_;

	if(!($q{'section_id'} || $q{'label'})) {
		# Missing required values
		return 0;
	} elsif($q{'version'} !~ /English/ || ($q{'master_id'} && $q{'master_id'} > 0)) {
		# ensure that the version is NOT defined (ie. make() only creates masters)
		# Not a master record
		return 0;	
	}
	# validate $q:
	# ensure that the section number of the question is properly defined
	#
	return 0 unless $this->validate_section(%q);

	delete $q{'master_id'};
	my $newId = $share{'DB'}->insert('q_question_base', \%q);
	if($newId) {
		$q{'question_base_id'} = $newId;
		$this->set('current', \%q);
		$this->set('master', \%q);
		return $newId;
	}
	return 0;
}

# Tells us whether or not we're actually supposed to be editing
# the current base question based on the sectionId value
sub validate_section($) {
	my $this = shift @_;
	my %q = @_;

	my $sectionId = $q{'section_id'};
	return 0 unless $sectionId;
	foreach my $s ($share{'DB'}->my_sites) {
		# Compare section ID:
		# if found return 1
		return 1 if $s->{'section_id'} == $sectionId;
	}
	return 0;
}

# Used for 'approving' changes to a version
# This basically consists of updating the timestamp value.
# Once the timestamp for a question version has been set to
# a more recent value than the master, it's been 'approved'
sub approve($) {
	my $this = shift @_;

	return undef unless $this->id && $this->master_id;
	# Don't do anything if the current record is already up to date
	return undef if $this->isApproved;
	my $upResult = $share{'DB'}->update('q_question_base', 
		{'question_base_id' => $this->id, 'timestamp' => undef}
	);
	my $result = $share{'DB'}->fetch('q_question_base', $this->id);
	$this->set('current', $result);
	return $result->{'timestamp'};
}

# Makes a new 'versioned' question_base record 
sub makeVersion {
	my ($this, %q) = @_;

	# ensure that we are already working with a defined question
	my $master = $this->master;

	# ensure that the passed version has a defined version name
	# ... and that the defined version isn't the default
	return 0 unless $q{'version'} && $master->{'question_base_id'} &&
		($q{'version'} ne $master->{'version'});
	# if the defined question is not a master version, switch to the
	# master version using 
	return 0 unless $this->validate_section(%q);

	$q{'master_id'} = $master->{'question_base_id'};
	# attach the new question to the master version
	my $newId = $share{'DB'}->insert('q_question_base', \%q);
	return $newId;
}

# Delete a base question and all sub-questions
sub delete {
	my ($this) = @_;

	my $current = $this->current;
	
	my $msg = "";
	if(!$current->{'question_base_id'}) {
		$msg = "Can't find valid q_question_base record to delete (no ID provided).";
		return (0, $msg);
	}

	# Are we using a master record?
	if(!$current->{'master_id'}) {
		# TODO: Only delete the master when there are no more versions abailable

		my @v = $this->fetchAllVersions();
		if(scalar(@v)) {
			$msg = "Error: Can't delete base question when question versions still exist.";
			$this->error($msg);
			return (0, $msg);
		}

		# check for real questions that reference these questions. 
		# Do not proceed if real questions exist.
		my $realQuestions = $share{'DB'}->fetch_match(
			'q_question', {'question_base_id' => $current->{'question_base_id'}}
		);
		if($realQuestions->[0]->{'question_id'}) {
			$msg = "This base question is still in use.";
			$this->error($msg);
			return (0, $msg);
		} else {
			# delete all references to the question from the database.
			# if we are working with a master, delete all versions as well.
			#$share{'DB'}->custom_query("DELETE FROM q_question_base WHERE master_id=$current->{question_base_id}");
			$share{'DB'}->custom_query("DELETE FROM q_question_base WHERE master_id=$current->{question_base_id}");
		}
		# if we are working with a version, change the current question 
		# back to the master after we delete.
	}

	# Then clear the current question.
	$share{'DB'}->delete('q_question_base', {'question_base_id' => $current->{'question_base_id'}});

	$this->set('current', $this->master);
	return (1);
}

# Icon outputting:
sub newIcon($) {
	my $sid = shift @_;

	return "" unless $sid;

	my $out = "<a href='".&relink("qcmd"=>"new_base_question", "section_id" => $sid)."'><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/new12.gif' border='0'></a>\n";
	return $out;
}

sub newVersionIcon($) {
	my $this = shift @_;
	my $version = shift @_;

	return "" unless $this->id;

	my $reply = &relink();
	my $out = "<a href='".&relink("qcmd" => "new_version", 'version' => $version, 'section_id' => $this->section_id, "qbid" => $this->id, 'reply' => $reply, "q_submit" => undef)."'><img alt='new version' src='$config{server}->{HTMLpath}/_Modules/Questionnaire/new12.gif' border='0'></a>\n";
	return $out;
}

sub approveIcon($) {
	my $this = shift @_;

	return "" unless $this->id;
	my $reply = &relink();
	my $id = $this->id;
	my $out = "<a href='".&relink('qcmd' => 'approve', 'submit_action' => 'Submit', 'section_id' => $this->section_id, "qbid" => $id, 'reply' => $reply, 'version' => $this->version)."'><img alt='approve' src='$config{server}->{HTMLpath}/_Modules/Questionnaire/approve12.gif' border='0'></a>\n";
}

sub editIcon($) {
	my $this = shift @_;

	return "" unless $this->id;

	my $qcmd = "edit_master";

	if($this->master_id) {
		$qcmd = "edit_version";
	}
	
	my $reply = &relink();
	my $out = "<a href='".&relink("qcmd"=>$qcmd, 'section_id' => $this->section_id, "qbid" => $this->id, 'master_id' => $this->master_id, "reply" => $reply, "q_submit" => undef, 'version' => $this->version)."'><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/edit12.gif' border='0'></a>\n";
	return $out;
}

sub editFinalIcon {
	my $this = shift @_;

	return "" unless $this->id;
	my $out = "<a href='".&relink("qcmd"=>"edit_final_message", 'section_id' => $this->section_id, "qbid" => $this->id, "q_submit" => undef)."'><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/edit12.gif' border='0'></a>\n";
	return $out;
}

sub deleteIcon($) {
	my $this = shift @_;
	my $reply = shift @_;

	return "" unless $this->id;

	my $qcmd = "delete_question_base";
	my $reply = $reply || &relink('qcmd' => 'section_view', 'section_id' => $this->section_id);
	if($this->master_id) {
		$qcmd = "delete_version";
		$reply = &relink('version' => undef, 'qbid' => $this->master_id);
	}
	my $label = $this->printableLabel;
	my $deleteLink = &relink('qcmd' => $qcmd, 'section_id' => $this->section_id, 'qbid' => $this->id, 'master_id' => $this->master_id, 'reply' => $reply);
	my $out = "<a href='#' onClick=\"javascript:confirm_custom('Delete question ($label)?', '$deleteLink');\"><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/rm12.gif' border='0'></a>\n";
	return $out;
}

sub viewLink($) {
	my $this = shift @_;
	my $reply = shift @_;

	return "" unless $this->id;

	my $version = ($this->master_id) ? $this->version : undef;
	
	$reply = $reply || &relink();
	return "<a href='".&relink('qcmd' => 'question_base_view', 'qbid' => $this->id, 'version' => $version, 'reply' => $reply)."'>".$this->label."</a>";

}

# build reports using ExSite::ReportBuilder
sub reportbuilder{
	my ($this,$opt) = @_;
	my $rb = $this->{rb} || new ExSite::ReportBuilder();
	$rb->init(
		title => $opt->{'title'},
		data => $opt->{'data'},
		tools => $opt->{'tools'},
		headers => $opt->{'headers'},
		foot => $opt->{'foot'},
		width => $opt->{'width'},
	);
	return $rb->make();
}

# Returns a list of all possible format values for a q_question_base
# record
#
# Now grabs these values from the actual cgitype record
sub allFormats($) {
	my $this = shift @_;

	if($share{'ALL_QUESTION_BASE_FORMATS'}) {
		return @{$share{'ALL_QUESTION_BASE_FORMATS'}};
	}
	my @formats = split /\s*\|\s*/, $share{'DB'}->{'map'}->{'datatype'}->{'list:question_base_format'}->{'regexp'};
	$share{'ALL_QUESTION_BASE_FORMATS'} = \@formats;
	return @formats;
}

# Custom base question validation.
# Our main focus in this method is making sure that the q_question_base object gets
# a 'value' field if the question requires it
sub validate($) {
	my $this = shift @_;
	my $action = shift @_;
	my $table = shift @_;
	my $column = shift @_;
	my $data = shift @_;

	return undef unless $table =~ /q_question_base/;

	my @errors = ();
	
	push @errors, $this->error("Error inserting to q_question_base. No data provided.") if !$data;

	if(!$data->{'section_id'}) {
		push @errors, $this->error("Error: Can't insert to q_question_base without a section_id value.");
	}
	if(!$data->{'label'}) {
		push @errors, $this->error("Error: Can't insert to q_question_base without a question label.");
	}
	if($data->{'format'}) {
		if(&Modules::Questionnaire::QuestionBase::formatRequiresValues($data->{'format'}) && !$data->{'value'}) {
			push @errors, $this->error("Error: You can't select type $data->{format} without providing some information in the 'answers' section.");
		}
	}
	return undef;
}

# Tell us if there's a javascript component to this question
sub hasJavascript($) {
	my $this = shift @_;

	if($this->format =~ /slider/) {
		return 1;
	}
	return 0;
}

sub delim($) {
	my $this = shift @_;

	my $delim = $config{'Questionnaire'}{'question_value_delim'} || $QUESTION_BASE_VALUES_DELIM;

	if($delim =~ /|/) {
		return "\\$delim";
	}

	return $delim;
}

sub formatInfo($) {
	my $format = shift @_;

	my $delim = &delim;
	my $string = $QUESTION_BASE_INFO{$format};
	$delim =~ s/\\//;
	$string =~ s/\[\[DELIM\]\]/$delim/g;
	return $string;
}

1;
__END__
