package Modules::Questionnaire::Questionnaire;

use strict;
use ExSite::Config;
use ExSite::Report;
use ExSite::Misc;
use ExSite::Util;
use ExSite::Mail;
use Modules::Questionnaire::Report;
use Modules::Questionnaire::Response;
use Modules::Questionnaire::ResponseFilter;

use base	qw(Modules::Questionnaire::Question);

our $DEFAULT_MASTER_VERSION = "English";

# For setting the final message.
# When sorted alphanumerically, this should be the very last possible value
our $FINAL_QUESTION_POSITION = "ZZZZZZZZZZ";
# too many Z's messes up report formatting
#our $FINAL_QUESTION_POSITION = "ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ";

our @EXPORT_OK = qw($FINAL_QUESTION_POSITION);

=pod

=head1 Questionnaire.pm

A questionnaire is an ordered grouping of questions.  

The questionnaire object can have a "current question", which is
simply the default question to operate on, when question-specific
functions are invoked.  That allows us to inherit question-specific
methods from the Question class.

C<my $q = new Modules::Questionnaire::Questionnaire($id,$vers);>

defines a new Questionnaire object, based on questionnaire # $id in
the database.  If we specify a version name, that means we prefer to
work with that version (typically a language translation).  Both 
parameters are optional, 

C<$q-E<gt>fetch($id,$vers) = new Modules::Questionnaire::Questionnaire($id,$vers);>

defines a new Questionnaire object, based on questionnaire # $id in
the database.  If we specify a version name, that means we prefer to
work with that version (typically a language translation).

=cut

sub new {
    my ($this,$id,$version) = @_;
    my $obj = {};
    my $class = ref($this) || $this;
    bless $obj, $class;

    if ($id) { 
	$obj->fetch($id,$version);
    }

    return $obj;
}

# Fetch the questionnaire matching $id
# if $id is a hash, then we assume that it's the questionnaire
# record information we were looking for.
sub fetch {
	my ($this,$id,$version) = @_;

	my $q = undef;
	if(ref($id) =~ /HASH/) {
		$q = $id;	
	} else {
		$q = $share{'DB'}->fetch("q_questionnaire", $id);
	}
	if ($q) {
		# FIXME: authenticate ownership of this questionnaire
		#
		# approved - set the questionnaire attributes
		$this->set('q_questionnaire', $q);
		$this->setVersion($version);
		return $q;
	}
	$this->warn("q_questionnaire $id not found");
	return undef;
}

# Return the questionnaire ID
sub id {
	my ($this) = @_;
	return undef unless $this->get('q_questionnaire');

	return $this->get('q_questionnaire')->{'questionnaire_id'};
}

# return the questionnaire title
sub title {
	my ($this) = @_;

	return undef unless $this->get('q_questionnaire');
	return $this->get('q_questionnaire')->{'title'};
}

# Returns the section_id for the questionnaire
sub section_id {
	my $this = shift @_;
	return undef unless $this->get('q_questionnaire');
	return $this->get('q_questionnaire')->{'section_id'};
}

# fetch all questions for the questionnaire from database
sub loadAllQuestions {
	my ($this) = @_;

	return if ($this->get('loaded'));

	my $version = $this->version;
	# fetch all question links attached to this questionnaire
	my @qlink = $share{'DB'}->fetch_child(
		"q_question", "q_questionnaire", $this->id
	);

	# order them by sortkey and id
	my @q = sort { 
		($a->{'sort_key'} =~ /^\d+$/ && $b->{'sort_key'} =~ /^\d+$/) ? 
			$a->{'sort_key'} <=> $b->{'sort_key'} : $a->{'sort_key'} cmp $b->{'sort_key'} ||
			$a->{'question_id'} <=> $b->{'question_id'} 
	} @qlink;

	# normalize the order, in case we have duplicate sortkeys
	my ($q,$lastq);
	foreach $q (@q) {
	    if ($lastq && $q->{sort_key} =~/^\d+$/ &&
		($q->{sort_key} <=> $lastq->{sort_key}) <= 0) {
		# Oops, our sortkeys are not incrementing correctly.  
		# Revise the sortkey so that it increments correctly.
		$q->{sort_key} = $lastq->{sort_key} + 1;
		# Update the DB so our sortkey fixes will stick.
		$share{DB}->update("q_question",$q);
	    }
	    $lastq = $q;
	}

	if(scalar(@q) > 0) {
		my $lastItem = new Modules::Questionnaire::Question($q[$#q], $version);
		if($lastItem->fetchBase->format =~ /label/ && $lastItem->sort_key eq $FINAL_QUESTION_POSITION) {
			$this->set('finalMessage', $lastItem);
			pop @q;
		}
	}
	# save the order for future reference
	$this->set('qorder', \@q);
	$this->set('qcount', scalar(@q));
	$this->set('qorder_pos', 0);    # index of first question
	$this->set('loaded', 1);
	return;
}

# Return the number of questions used by the questionnaire
sub count {
	my ($this) = @_;
	$this->loadAllQuestions;
	return $this->get('qcount');
}

# Get all of the reports that are using the current questionnaire
sub loadAllReports {
	my ($this) = @_;

	return if ($this->get('reports_loaded'));
	my @rlink = $share{'DB'}->fetch_child(
		'q_report', 'q_questionnaire', $this->id
	);
	my @rObjs = ();
	foreach my $r (@rlink) {
		push @rObjs, new Modules::Questionnaire::Report($r);
	}
	$this->set('reports', \@rObjs) if scalar(@rlink) && $rlink[0];
	$this->set('reports_count', scalar(@rObjs));
	$this->set('reports_loaded', 1);
	return;
}


=pod

=head2 Version Handling

We only keep track of the preferred and available versions.  All
real version handling occurs down in the QuestionBase class.

Available versions is technically just a "wish list" of versions we
presume to support.  There is no requirement that those actual
versions are available.

=cut

# return array of defined versions for this questionnaire
sub getVersions {
	my ($this) = @_;
	my $questionnaire = $this->get('q_questionnaire') || {};
	return split /\s*,\s*/, $questionnaire->{'versions'};
}

# return the current version being used
sub version($) {
	my ($this) = @_;

	return $this->get('version');
}

# Return the versions as a simple string (as it exists in the DB)
sub versions($) {
	my ($this) = @_;

	return $this->get('q_questionnaire')->{'versions'};
}

# set preferred version of the questionnaire
sub setVersion {
	my ($this,$version) = @_;

	my @versions = $this->getVersions();
	my $currentVersion = $this->get('version');
	foreach my $v (@versions) {
		if($v eq $version) {
			$this->set('version', $v);
			$currentVersion = $v;
			last;
		}
	}
	return $currentVersion;
}

=pod

=head2 Questions




=cut

# set/return a question by id
# $id can be one of:
#   - question label
#   - question name
#   - question number

sub getQuestion {
	my ($this,$id) = @_;
	if (! $this->{qorder} || ref $this->{qorder} ne "ARRAY") {
		$this->loadAllQuestions;
	}
	my $question = undef;

	if ($id =~ /^\d+$/) {
		$question = $this->getQuestionNumber($id);
	} 

	# Weren't able to fetch the question by ID. Try the
	# question by name and/or label
	if(!$question) {
		$question = $this->getQuestionByName($id);
	}
	if(!$question) {
		$question = $this->getQuestionByLabel($id);
	}

	# Return whatever we've found ($question == undef if we haven't found
	# anything)
	return $question;
}

# Return the question number $num from the ordered list
sub getQuestionNumber($) {
	my $this = shift @_;
	my $num = shift @_;
	$this->loadAllQuestions;
	if($num < $this->count) {
		$this->set('qorder_post', $num);
		my $q = $this->get('qorder');
		return new Modules::Questionnaire::Question($q->[$num]);
	}
	return undef;
}

sub getByQuestionBase($) {
	my $this = shift @_;
	my $num = shift @_;

	$this->loadAllQuestions;
	my $base = $this->get('q_base_questions');
	if(!$base) {
		$base = {};
		my $questions = $this->get('qorder') || [];
		map { $base->{$_->{'question_base_id'}} = $_; } @{$questions};
		$this->set('q_base_questions', $base);
	}
	return new Modules::Questionnaire::Question($base->{$num});
}

# Search and try to match the question name with $name
sub getQuestionByName($) {
	my $this = shift @_;
	my $name = shift @_;;

	$this->loadAllQuestions;
	
	my @qList = @{$this->get('qorder')};
	for(my $i = 0; $i < @qList; $i++) {
		if($qList[$i]->{'name'} eq $name) {
			$this->set('qorder_pos', $i);
			return new Questionnaire::Question($qList[$i]);
		}
	}
	return undef;
}

# Search and try to match a question label with $label
sub getQuestionByLabel($) {
	my $this = shift @_;
	my $label = shift @_;

	$this->loadAllQuestions;

	my @qList = @{$this->get('qorder')};
	for(my $i = 0; $i < @qList; $i++) {
		if($qList[$i]->{'label'} eq $label) {
			$this->set('qorder_pos', $i);
			return new Modules::Questionnaire::Question($qList[$i]);
		}
	}
	return undef;
}

# return array of ordered question objects
sub getAllQuestions {
	my ($this) = @_;

	$this->loadAllQuestions;

	# convert each to an object
	my @question = ();
	my $version = $this->version;

	my @qlist = @{$this->get('qorder')};

	for(my $i = 0; $i < @qlist; $i++) {
		next unless $qlist[$i];
		my $q = $qlist[$i];
		push @question, new Modules::Questionnaire::Question($q, $version);
	}
	return @question;
}

# Return an array of ordered question objects with the 'label' questions removed.
sub getAllInputQuestions($) {
	my $this = shift @_;
	
	my $inputQs = $this->get('input_questions');
	return @{$inputQs} if $inputQs;

	my @q = $this->getAllQuestions;

	my @nonLabels = ();
	foreach my $q (@q) {
		push @nonLabels, $q unless $q->fetchBase->format =~ /label/;
	}
	$this->set('input_questions', \@nonLabels);
	return @nonLabels;
}

# set/return first question object
sub getFirstQuestion {
	my ($this) = @_;
	$this->loadAllQuestions;
	$this->set('qorder_pos', 0);
	my $pos = 0;    # point index to first question
	if ($this->count > 0) {
		my $q = $this->get('qorder')->[0];
		return new Modules::Questionnaire::Question($q);
	}
	return undef;
}

# set/return next question object
sub getNextQuestion {
	my ($this) = @_;
	$this->loadAllQuestions;
	my $pos = $this->get('qorder_pos'); # get question pointer 
	$this->set('qorder_pos', $pos+1);
	if ($pos < $this->count) {
		my $q = $this->get('qorder')->[$pos];
		return new Modules::Questionnaire::Question($q);
	}
	return undef;
}

# set/return last question object
sub getLastQuestion {
	my ($this) = @_;
	$this->loadAllQuestions;
	# Move the position to the end
	my $pos = ($this->count-1);
	$this->set('qorder_pos', $pos);
	if($pos >= 0) {
		my $q = $this->get('qorder')->[$pos];
		return new Modules::Questionnaire::Question($q);
	}
	return undef;
}


# return a list of known questions that can be included in this questionnaire
sub getUnusedQuestions {
	my $this = shift @_;

	return $this->getMyQuestions(@_);
}

# Return a list of all of the questionnaire's reports
sub getAllReports {
	my $this = shift @_;

	$this->loadAllReports;
	my $reports = $this->get('reports') || [];
	return ($reports) ? @{$reports} : ();
}

# Return the final message question base object
sub finalMessage($) {
	my $this = shift @_;

	$this->getAllQuestions;
	my $final = $this->get('finalMessage');

	if(!$final) {
		my $defaultLabel = "<b>Thank You</b>\n";
		my $defaultValue = "Your answers have been stored.<br>\n";
		# Construct a fake question object, and leave it
		# as our default final screen.
		my $finalMessageInfo = {
			'question_id' => -1,
			'questionnaire_id' => $this->id,
			'question_base_id' => -1,
			'sort_key' => $FINAL_QUESTION_POSITION,
		};
		my $finalMessageBaseInfo = {
			'question_base_id' => -1,
			'section_id' => $this->section_id,
			'master_id' => undef,
			'label' => $defaultLabel,
			'version' => $config{'Questionnaire'}{'default_version'} || $DEFAULT_MASTER_VERSION,
			'format' => 'label',
			'flags' => '',
			'value' => $defaultValue,
		};
		$final = new Modules::Questionnaire::Question($finalMessageInfo);
		my $base = new Modules::Questionnaire::QuestionBase;
		$base->set('current', $finalMessageBaseInfo);
		$base->set('master', $finalMessageBaseInfo);
		$final->set('q_question_base', $base);
		$this->set('finalMessage', $final);
	}
	return $final;
}

# Get all of the final messages in the database for the site
sub getAllFinalMessages($) {
	my $this = shift @_;

	my $allFinals = $this->get('allFinalMessages');
	return @{$allFinals} if $allFinals;

	my @sites = $share{'DB'}->my_sites;
	my %siteHash = ();
	map {$siteHash{$_->{'section_id'}} = 1;} @sites;

	my $finalMessages = $share{'DB'}->fetch_match('q_question', {'sort_key' => $FINAL_QUESTION_POSITION});
	my @finalObjs = ();
	foreach my $f (@{$finalMessages}) {
		next unless $f->{'question_id'};
		my $final = new Modules::Questionnaire::Question($f);
		push @finalObjs, $final if $siteHash{$final->fetchBase->section_id};
	}
	$this->set('allFinalMessages', \@finalObjs);
	return @finalObjs;
}

# Load the questions, create Question objects with the record information,
# sort the questions and return them
sub getMyQuestions($) {
	my $this = shift @_;
	my $version = shift @_;
	
	my $unused = $this->get('unusedQuestions_'.$version);
	return @{$unused} if $unused;

	my $sid = $this->section_id;
	$this->setVersion($version) if $version;
	my @questions = $this->getAllQuestions();
	my @finals = $this->getAllFinalMessages();

	my %preExisting = ( );
	map {$preExisting{$_->fetchBase->master->{'question_base_id'}} = 1;} @questions;
	map {$preExisting{$_->fetchBase->master->{'question_base_id'}} = 1;} @finals;

	# Get all of the 'version' base questions if we're requesting a
	# version
	#print "LIST: ".scalar(@{$list})."<br>\n";
	my @finalList = ();
	if($version) {
		my $list = $share{'DB'}->fetch_match(
			'q_question_base', {'section_id' => $sid, 'version' => $version}
		);
		foreach my $f (@{$list}) {
			my $base = new Modules::Questionnaire::QuestionBase($f, $version);
			my $baseId = $base->master_id || $base->id;
			push @finalList, $base unless $preExisting{$baseId};
			$preExisting{$baseId} = 1;
		}
	}

	my $list = $share{'DB'}->fetch_match(
		'q_question_base', {'section_id' => $sid}
	);

	foreach my $f (@{$list}) {
		next if $f->{'master_id'};
		push @finalList, new Modules::Questionnaire::QuestionBase($f, $version) unless $preExisting{$f->{'question_base_id'}};
	}
	@finalList = sort { $a->master->{'question_base_id'} <=> $b->master->{'question_base_id'} } @finalList;
	$this->set('unusedQuestions_'.$version, \@finalList);
	return @finalList;
}
=pod

=head2 Responses



=cut

# Get all of the responses for the current questionnaire.
sub getAllResponses {
	my $this = shift @_;

	my $responses = $this->get('responses');
	return @{$responses} if $responses;

	my $questionnaireId = $this->get('q_questionnaire')->{'questionnaire_id'};
	$responses = $share{'DB'}->fetch_child("q_response", "q_questionnaire", $questionnaireId);

	my $respObj = [];

	foreach my $r (@{$responses}) {
		my $obj = new Modules::Questionnaire::Response;
		$obj->set('q_response', $r);
		$obj->loadAllAnswers();
		$obj->fetch($r->{'response_id'});
		push @{$respObj}, $obj;
	}

	$this->set('responses', $respObj);
	$this->set('no_responses', scalar(@{$respObj}));
	$this->set('response_pos', 0);
	return @{$respObj};
}

sub getFilteredResponses($) {
	my $this = shift @_;
	my $responseFilterId = shift @_;

	my @responses = $this->getAllResponses();
	return () unless scalar(@responses);
	return @responses if !$responseFilterId;

	my $filter = new Modules::Questionnaire::ResponseFilter($responseFilterId);
	$filter->setResponses(@responses);
	my @filteredResponses = $filter->filteredResponses();
	$this->set('responses', \@filteredResponses);
	$this->set('no_responses', scalar(@filteredResponses));
	$this->set('response_pos', 0);
	return @filteredResponses;
}
sub getFirstResponse {
	my $this = shift @_;

	$this->getAllResponses();

	$this->set('response_pos', 0);
	return $this->get('responses')->[0];
}


sub getNextResponse {
	my $this = shift @_;

	my $responsePos = $this->get('response_pos')+1;
	if($responsePos >= $this->get('no_responses')) {
		$this->set('response_pos', $this->get('no_responses'));
		return undef;
	}
	$this->set('no_responses', $responsePos);
	return $this->get('responses')->[$responsePos];
}

# create and enter a complete reponse to the database
sub addResponse {
	my $this = shift @_;
	my $data = shift @_;

	if(!$data) {
		require ExSite::Input;
		my $in = new ExSite::Input;
		$data = $in->post_or_query;
	}

	my $msg = "";
	my $newId = 0;

	my $rec = $share{'DB'}->my_user_record;
	my %record = (
		'questionnaire_id' => $data->{'questionnaire_id'},
		'version' => $data->{'version'},
		'member_id' => $rec->{'member_id'},
	);
	my $response = new Modules::Questionnaire::Response();
	$newId = $response->make(%record);
	$msg = "Response added to questionnaire $data->{questionnaire_id}";

	return ($newId, $msg);
} 

# Return all filters for the current questionnaire
sub getAllResponseFilters($) {
	my $this = shift @_;

	my $id = $this->id;
	return undef unless $id;

	my $responseFilters = $this->get('q_response_filters');
	return @{$responseFilters} if $responseFilters;

	$responseFilters = $share{'DB'}->fetch_match("q_response_filter",
		{'questionnaire_id' => $id}	
	);

	my @objects = ();
	foreach my $r (@{$responseFilters}) {
		push @objects, new Modules::Questionnaire::ResponseFilter($r);
	}

	$this->set('q_response_filters', \@objects) if scalar(@objects);

	return @objects;
}


# Fetch custom reports associated with this questionnaire

=pod

=head2 Displaying a Questionnaire



=cut

# return the questionnaire inputs, without the form tags or submit
# buttons, including javascript for branches

sub show {
	my ($this, $responseId, $previewFlag) = @_;
	
	my $nq = 1;
	my $version = $this->version;
	my $questionnaire = $this->get('q_questionnaire');
	my $out = "";
	my @q = $this->getAllQuestions;
	for(my $i = 0; $i < @q; $i++) {
		my $q = $q[$i];
		my $value = ($responseId) ? $q->loadResponseAnswer($this->id, $responseId)->rawValue : "";
		my $flags = ($this->showGrid(\@q, $i)) ? "grid" : "";
		$flags .= ", header" if $this->firstGrid(\@q, $i);
		$flags .= ", footer" if $this->lastGrid(\@q, $i);
		$flags .= $q->flags;
		my $question = $q->show($nq, $value, $version, $flags);
		if($question) {
			$nq++;
			$out .= $question;
		}
	}
	return $out;
}

# Print all of the mandatory javascript function calls
sub show_mandatory_js {
	my $this = shift @_;

	my @q = $this->getAllQuestions;
	my $allMandatory = "";
	for(my $i = 0; $i < @q; $i++) {
		my $q = $q[$i];
		$allMandatory .= $q->show_mandatory_js;
	}
	return $allMandatory;
}

# Print a general set of include statements for slider questions if they
# exist in our list of questions.
sub show_slider_js {
	my $this = shift @_;

	my $sliderInc = &Modules::Questionnaire::QuestionnaireHTML::sliderJavascript();
	return $sliderInc;
}

# Returns true if the question at $pos is the first in a line
# of gridded questions. This is good to know so that we can
# print a header of possible values before moving on with
# the gridded values
sub firstGrid($) {
	my $this = shift @_;
	my $questions = shift @_;
	my $pos = shift @_;

	return 0 unless $questions;

	my $currentQuestion = $questions->[$pos];
	if(!$this->isGridType($currentQuestion)) {
		return 0;
	}
	# The previous question can't be a griddable type
	if(($pos > 0) && $questions->[$pos-1] && 
		($questions->[$pos-1]->format eq $currentQuestion->format &&
		 $currentQuestion->compareValues($questions->[$pos-1]))
	) {
		return 0;
	}
	return $this->showGrid($questions, $pos);
}

# Returns true if the question at $pos is the first in a line
# of gridded questions. This is good to know so that we can
# print a header of possible values before moving on with
# the gridded values
sub lastGrid($) {
	my $this = shift @_;
	my $questions = shift @_;
	my $pos = shift @_;

	return 0 unless $questions;

	my $currentQuestion = $questions->[$pos];

	if(!$this->isGridType($currentQuestion)) {
		return 0;
	}
	# The next question can't be a griddable type
	if($questions->[$pos+1] &&
		($questions->[$pos+1]->format eq $currentQuestion->format &&
			$currentQuestion->compareValues($questions->[$pos+1]))
	) {
		return 0;
	}
	return $this->showGrid($questions, $pos);
}

# Given the current position in the questions list, should we be showing
# the multiselect option as a 'grid'?
sub showGrid($) {
	my $this = shift @_;
	my $questions = shift @_;
	my $pos = shift @_;

	return 0 unless $questions;

	my $currentQuestion = $questions->[$pos];
	if(!$this->isGridType($currentQuestion)) {
		return 0;
	}
	
	my $numValues = $currentQuestion->valueCount();
	return 0 if ($numValues == 0);
	if($questions->[$pos+1] && $questions->[$pos+1]->format eq $currentQuestion->format
		&& $currentQuestion->fetchBase->compareValues($questions->[$pos+1]->fetchBase)) {
		return 1;
	}
	if(($pos > 0) && $questions->[$pos-1] && $questions->[$pos-1]->format eq $currentQuestion->format
		&& $currentQuestion->fetchBase->compareValues($questions->[$pos-1]->fetchBase)) {
		return 1;
	}
	return 0;
}

# Returns true if the current question's format should
# be gridded if there are a lot of the same type in a
# row (radio buttons, multi-select right now)
sub isGridType($) {
	my $this = shift @_;
	my $question = shift @_;

	return ($question->format =~ /multiselect|radio/);
}

=pod

=head2 Building a Questionnaire



=cut

# returns 0 if insert failed.
# returns the new record ID value on success
sub make {
	my $this = shift @_;
	my %questionnaire = @_;

		#print STDERR "UPDATE (q_questionnaire)\n";
		#foreach my $k (keys %questionnaire) {
		#	print STDERR "<b>$k</b>: $questionnaire{$k}\n";
		#}
	return 0 unless $questionnaire{'section_id'} ;

	if(!$this->validate_section(%questionnaire)) {
		# Print an error. Invalid section
		return 0;
	}

	my $newId = 0;
	if($questionnaire{'questionnaire_id'}) {
		$newId = $share{'DB'}->update('q_questionnaire', \%questionnaire);
	} else {
		$newId = $share{'DB'}->insert('q_questionnaire', \%questionnaire);
		$questionnaire{'questionnaire_id'} = $newId;
		$this->setVersion("English");
	}
	$this->set('q_questionnaire', \%questionnaire);
	return $newId;
}

sub delete {
	my $this = shift @_;

	return 0 unless $this->id;

	my @questions = $this->getAllQuestions;

	my %questionnaire = %{$this->get('q_questionnaire')};
	if(!$this->validate_section(%questionnaire)) {
		# Print an error. Invalid section
		$this->error("Error. Questionnaire belongs to the wrong session.");
		return 0;
	}

	my $delId = $share{'DB'}->trash_r('q_questionnaire', $this->id);
	$this->set('q_questionnaire', undef);
	return $delId;
}

sub copy($) {
	my $this = shift @_;
	my %questionnaire = @_;

	return undef unless $this->id;

	# If no questionnaire hash was supplied, we'll use all of the 
	# values contained in the current questionnaire
	if(!scalar(keys %questionnaire) && $this->get('q_questionnaire')) {
		%questionnaire = %{$this->get('q_questionnaire')};
	}
	delete $questionnaire{'questionnaire_id'};

	my $newQuestionnaire = new Modules::Questionnaire::Questionnaire();
	$newQuestionnaire->make(%questionnaire);
	my @questions = $this->getAllQuestions;
	my @newQuestions = ();
	foreach my $q (@questions) {
		push @newQuestions, $q->copy($newQuestionnaire);
	}
	$newQuestionnaire->set('qorder', \@newQuestions);
	$newQuestionnaire->set('qcount', scalar(@newQuestions));
	$newQuestionnaire->set('qorder_pos', 0);    # index of first question

	my @reports = $this->getAllReports;

	my @newReports = ();
	foreach my $r (@reports) {
		push @newReports, $r->copy($newQuestionnaire);
	}
	$newQuestionnaire->set('reports', \@newReports) if scalar(@newReports) && $newReports[0];
	$newQuestionnaire->set('reports_count', scalar(@newReports));
	$newQuestionnaire->set('reports_loaded', 1);
	return $newQuestionnaire;
}

# Returns 0 if insert failed. Returns the new question
# ID on success
sub addQuestion {
	my $this = shift @_;
	my $qlink = shift @_;
	my $qbase = shift @_;

	# If question_base_id exists, we're going to completely ignore
	# whatever was passed as $qbase
	# 
	# Make sure all of the questions are loaded beforehand
	# This way we know that we always have to include the newly
	# added question into the list of cached entries
	$this->loadAllQuestions();

	my $baseId = $qlink->{'question_base_id'};
	my $newId = undef;
	my $question = undef;
	my $nextQuestionNum = $qlink->{'sort_key'} || $this->count;
	my %qHash = (
		'questionnaire_id' => $qlink->{'questionnaire_id'},
		'sort_key' => $nextQuestionNum,
		'flags' => $qlink->{'flags'}
	);

	if(!$qlink->{'question_base_id'}) {
		# Insert qBase
		$question = new Modules::Questionnaire::Question;
		$newId = $question->make(\%qHash, $qbase);
	} else {
		$question = new Modules::Questionnaire::Question();
		$newId = $question->link(\%qHash, $qbase);
	}
	if($newId) {
		my @q = $this->getAllQuestions();
		push @q, $question->get('q_question');
		$this->set('qorder', \@q);
		$this->set('qcount', scalar(@q));
		$this->set('qorder_pos', $nextQuestionNum);    # index of first question
	}

	return $newId;
}

# Returns the DB delete call value on success (number of
# rows affected, should be 1) or 0 on failure
#
# This method actually uses the Modules::Questionnaire::Question objects'
# delete method to do the actual deletion. It also takes care of internal caching of
# previously fetched values
sub delQuestion($) {
	my $this = shift @_;
	my $question = shift @_;

	return 0 unless ref($question) =~ /Question/ && $question->id;

	my @q = $this->getAllQuestions();
	my $id = $question->id;
	my $delValue = $question->unlink;
	if(!$delValue) {
		return 0;
	}
	for(my $i = 0; $i < @q; $i++) {
		if($q[$i]->id == $id) {
			splice @q, $i, 1;
			last;
		}
	}
	$this->set('qorder', \@q);
	$this->set('qcount', scalar(@q));
	if($this->get('qorder_pos' > $this->get('qcount'))) {
		$this->set('qorder_pos', scalar(@q)-1);
	}
	return $delValue;
}

# Buttons used on the actual questionnaire form
sub questionnaireButtons($) {
	my $this = shift @_;
	my $qid = shift @_;

	return "" unless $qid;

	my $link = &relink('qcmd' => undef, 'qid' => undef, 'q_submit' => undef);
	return "<input type='button' value='Submit' onClick='javascript:validateSubmit()'>&nbsp;<input type='reset' value='Reset'>&nbsp;<input type='button' value='Cancel' onClick='document.location=\"$link\"'>";
}

# Used for the questionnaire preview.
# The arrows links call javascript methods that move the questions
# around
sub swapArrows($) {
	my $this = shift @_;
	my $position = shift @_;
	my @questions = @_;

	return "" unless $questions[$position];

	my $up = "";
	my $down = "";
	my $current = "q$position";
	if($position < (scalar(@questions) - 1)) {
		my $next = "q".($position+1);
		$down = "<a onClick='javascript:swapDivs(\"$current\", \"$next\")'><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/down.gif' border='0'></a>\n";
	}
	if($position > 0) {
		my $prev = "q".($position-1);
		$up = "<a onClick='javascript:swapDivs(\"$current\", \"$prev\")'><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/up.gif' border='0'></a>\n";
	}
	return $up.$down;
}

sub previewWrapper($) {
	my $this = shift @_;
	my $data = shift @_;

	return <<"DONE";
<table style='width: 100%'>
	<tr>
		<td style='width: 30px'>
			$data->{question_opts}
		</td>
		<td>
			$data->{question}
		</td>
	</tr>
</table>
DONE
}

# Link/icon methods:
sub previewLink($) {
	my $this = shift @_;

	return "" unless $this->id;

	my $out = "<a href='".&relink("qcmd"=>"preview","qid" => $this->id, "q_submit" => undef)."'><img alt='Preview questionnaire' src='$config{server}->{HTMLpath}/_Modules/Questionnaire/view12.gif' border='0'></a>\n";
	return $out;
}

sub detailsLink($) {
	my $this = shift @_;

	return "" unless $this->id;

	my $out = "<a href='".&relink("qcmd"=>"details", "qid" => $this->id, "q_submit" => undef)."'><img alt='Questionnaire Details' border='0' src='$config{server}->{HTMLpath}/_Modules/Questionnaire/edit12.gif'></a>\n";
	return $out;
}

sub orderQuestionsLink($) {
	my $this = shift @_;

	return "" unless $this->id;

	my $out = "<a href='".&relink("qcmd"=>"reorder", "qid" => $this->id, "q_submit" => undef)."'><img alt='Re-order questions' src='$config{server}->{HTMLpath}/_Modules/Questionnaire/copy12.gif' border='0'></a>\n";
	return $out;
}

sub deleteLink($) {
	my $this = shift @_;

	return "" unless $this->id;

	my $link = &relink('qcmd' => 'delete_questionnaire', 'qid' => $this->id);

	return "<a href='#' onClick=\"javascript:confirm_custom('Are you sure you wish to delete the entire questionnaire?', '$link')\"><img alt='Delete Questionnaire' src='$config{server}->{HTMLpath}/_Modules/Questionnaire/rm12.gif' border='0'></a>\n";
}

sub reportLink($) {
	my $this = shift @_;

	return "" unless $this->id;

	return "<a href='".&relink("qcmd" => "report_questionnaire", "qid" => $this->id, 'response_match_id' => undef, 'response_filter_id' => undef)."'><img alt='Questionnaire Reports' src='$config{server}->{HTMLpath}/_Modules/Questionnaire/list12.gif' border='0'></a>\n";
}

sub newQuestionnaireLink($) {
	my $sectionId = shift @_;

	return "" unless $sectionId;

	return "<a href='".&relink('section_id' => $sectionId, 'qcmd' => 'new_questionnaire', 'q_submit' => undef)."'><img border='0' src='$config{server}->{HTMLpath}/_Modules/Questionnaire/new12.gif'></a>";
}

=pod

=head2 Reporting


=cut
sub report {
	my $this = shift @_;
	my $responseFilterId = shift @_;

	my $class = $this->get('div_class');
	my @participants = $this->fetchParticipants();
	my $data = [];

	foreach my $p (@participants) {
		push @{$data}, 
			[$p->{'member_id'}, $p->{'first'}." ".$p->{'last'}, $p->{'email'}, $p->{'title'}];
	}

	my %report = (
		'title' => $this->label,
		'headers' => ['Total Participants', 'Total Questions'],
		'data' => [$this->numParticipants, $this->numQuestions],
	);
	my $summary = $this->reportbuilder(\%report);

	%report = (
		'title' => $this->label." Participants",
		'headers' => ['ID', 'Name', 'Email', 'Title'],
		'data' => $data,
	);
	my $participants = "<br>".$this->reportbuilder(\%report);

	my @filters = $this->getAllResponseFilters();

	$data = [];
	foreach my $f (@filters) {
		my $options = $f->reportLinkIcon();
		$options .= $f->reportViewLink();
		$options .= $f->deleteResponseFilterLink();
		push @{$data}, [$f->name, $options];
	}

	%report = (
		'title' => "Filtered reports",
		'headers' => ['Name', 'Options'],
		'data' => $data,
		'tools' => &Modules::Questionnaire::ResponseFilter::newResponseFilterLink($this->id),
	);
	
	my $filters = $this->reportbuilder(\%report)."\n";
	my $out = "<div class='$class'>\n";
	$out .= $this->reportbuilder(
		{
			'data' => $summary,
			'foot' => $participants."<br>".$filters
		}
	);
	$out .= "<br>".$this->questionnaireSummary($responseFilterId);
	$out .= "</div>\n";
	return $out;
}

# Get the summary for each question, but instead of listing through each
# question and fetching the responses along the way, we're going to fetch
# all responses, filter the responses then fetch the questions for each
# response as we go
sub questionnaireSummary($) {
	my $this = shift @_;
	my $responseFilterId = shift @_;

	return "" unless $this->id;

	my $out = "<table>\n";

	my @responses = $this->getAllValidResponses();
	my $filter = new Modules::Questionnaire::ResponseFilter($responseFilterId);
	$filter->setResponses(@responses);
	@responses = $filter->filteredResponses();

	my @questions = grep { $_->format !~ /label/; } $this->getAllQuestions();

#	foreach my $q (@questions) {

	for(my $i = 0; $i < @questions; $i += 4) {
		$out .= "\t<tr align='top' valign='top'>\n";
		for(my $j = $i; $j < (@questions) && $j < $i+4; $j++) {
			my $q = $questions[$j];
			$out .= "\t\t<td align='top' style='width: 25%; border-width: 1px; border-color: black; border-style: solid;'>\n";
			$out .= $this->reportQuestion($q, $filter)."<br>\n";
			$out .= "\t\t</td>\n";
		}
		$out .= "\t</tr>\n";
	}

	$out .= "</table>\n";
	return $out;
}

sub reportQuestion($) {
	my $this = shift @_;
	my $q = shift @_;
	my $filter = shift @_;

	return "" unless ref($q) =~ /Modules::Questionnaire::Question/ && ref($filter);

	my $data = [];
	my @values = $q->getValues();
	my $totalAnswers = $filter->totalAnswerCount($q->id);
	foreach my $v (@values) {
		my $responseCount = $filter->responseCount($q->id, $v) || "0";
		my $percentile = ($totalAnswers) ? sprintf("%2.0f", (($responseCount/$totalAnswers) * 100)) : "0%";
		push @{$data}, [$v, $responseCount, $percentile] if $responseCount || $this->alwaysShowValue($q);
	}
	my %report = (
		'title' => $q->label,
		'data' => $data,
		'headers' => ['Choice', 'Count', 'Percent'],
		'width' => '100%',
	);
	return $this->reportbuilder(\%report);
}

sub alwaysShowValue($) {
	my $this = shift @_;
	my $question = shift @_;

	return 1 if $question->format =~ /select|radio|multiselect|checkbox/;
	return 0;
}

sub questionSummaryOLD($) {
	my $this = shift @_;
	my $responseFilterId = shift @_;

	return "" unless $this->id;

	my $out = "";

	my @questions = $this->getAllQuestions();

	my $filter = undef;
	$filter = new Modules::Questionnaire::ResponseFilter($responseFilterId) if $responseFilterId;

	foreach my $q (@questions) {
		my @values = $q->getValues();
		@values = $q->getAnswerValues() if !scalar(@values);
		my $data = [];
		if(@values) { 
			foreach my $v (@values) {
				my $percentile = sprintf("%2.0f%", ($q->answerCount($v)/$q->totalAnswerCount()) * 100);
				push @{$data}, [$v, $q->answerCount($v), $percentile];
			}
		}
		my %report = (
			'title' => $q->label,
			'data' => $data,
			'headers' => ['Choice', 'Count', 'Percent'],
		);
		$out .= $this->reportbuilder(\%report)."<br>\n";
	}
	
	return $out;
}

# Get the number of participants for the current questionnaire
sub numParticipants($) {
	my $this = shift @_;

	my @participants = $this->fetchParticipants();
	return scalar(@participants);
}

sub numQuestions($) {
	my $this = shift @_;

	my @qs = $this->getAllQuestions();
	return scalar(@qs);
}

sub fetchParticipants($) {
	my $this = shift @_;

	my $p = $this->get('participants');
	return @{$p} if $p;

	my $qid = $this->id;
	$p = $share{'DB'}->custom_query("SELECT DISTINCT q_response.member_id, member.* FROM q_response LEFT JOIN member ON member.member_id=q_response.member_id WHERE q_response.questionnaire_id='$qid'");
	if($p && $p->[0]) {
		$this->set('participants', $p);
	}
	return @{$p};
}

# Get all filters as indicated in the $input hash
sub getFilters($) {
	my $this = shift @_;
	my $input = shift @_ || {};

	my $queryFilters = $this->get('query_filters');
	return @{$queryFilters} if $queryFilters;

	$queryFilters = [];
	my @filterKeys = grep { /^apply_filter_/ } keys %{$input};
	foreach my $k (@filterKeys) {
		my $filterId = $k;
		$filterId =~ s/^apply_filter_//;
		push @{$queryFilters}, new Modules::Questionnaire::ResponseFilter($filterId);
	}
	$this->set('query_filters', $queryFilters) if $queryFilters;
	return @{$queryFilters};
}

sub deleteIcon($) {
	my $this = shift @_;
	my $reply = shift @_;

	return "" unless $this->id;

	my $reply = &relink('qcmd' => 'section_view', 'section_id' => $this->section_id);

	my $deleteLink = &relink('qcmd' => 'delete_questionnaire', 'qid' => $this->id, 'reply' => $reply);
	return "<a href='#' onClick=\"javascript:confirm_custom('Are you sure you wish to delete the entire questionnaire?', '$deleteLink');\"><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/rm12.gif' border='0'></a>\n";
}

sub copyIcon($) {
	my $this = shift @_;

	return "" unless $this->id;

	my $reply = &relink();
	my $copyLink = &relink('qcmd' => 'copy_questionnaire', 'qid' => $this->id, 'reply' => $reply);
	return "<a href=\"$copyLink\"><img src=\"$config{server}->{HTMLpath}/_Modules/Questionnaire/copy12.gif\" border=\"0\"></a>";
}

1;

__END__

