package Modules::Questionnaire::Report;

use strict;
use POSIX	qw(ceil);
use base	qw(ExSite::Report);
use ExSite::Misc;
use ExSite::Config;
use ExSite::Excel;
use Modules::Questionnaire::Question;
use Modules::Questionnaire::ResponseFilter;

our $REPORT_BUILDER = new ExSite::ReportBuilder();
our $DEFAULT_NUMBER_OF_QUESTIONS = 5;
our $MAXIMUM_NUMBER_OF_QUESTIONS = 8;
our $DEFAULT_EXCEL_FILENAME = "questionnaire_report.xls";

sub new($) {
	my $type = shift @_;
	my $id = shift @_;

	my $this = {};
	my $class = ref($type) || $type || "";
	bless $this, $class;

	if(ref($id)) {
		$this->set('q_report', $id);
	} elsif($id =~ /^\d+$/) {
		$this->fetch($id);
	}

	$this->set('excel_filename', ($config{'Questionnaire'}{'excel_filename'} || $DEFAULT_EXCEL_FILENAME));

	return $this;
}

sub fetch($) {
	my $this = shift @_;
	my $id = shift @_;

	return undef unless $id;

	my $report = $share{'DB'}->fetch('q_report', $id);
	$this->set('q_report', $report) if $report;
	return $report;
}

sub id($) {
	my $this = shift @_;
	return undef unless $this->get('q_report');
	return $this->get('q_report')->{'report_id'};
}

sub name($) {
	my $this = shift @_;
	return undef unless $this->get('q_report');
	return $this->get('q_report')->{'name'};
}

sub questionnaire_id($) {
	my $this = shift @_;
	return undef unless $this->get('q_report');
	return $this->get('q_report')->{'questionnaire_id'};
}

# Get all of the filters saved for the report.
# If there is an $input field passed, the response_filters
# indicated in the $input (query string) override the filters
# requested by the report. This means if $input tells us to
# create an extra filter or remove one of the report's pre-defined
# filters, we will do that before returning the array of filters.
sub getAllFilters($) {
	my $this = shift @_;
	my $input = shift @_ || {};

	my $filters = $this->get('q_response_filters');

	return @{$filters} if $filters;
	return () unless $this->id;

	my $id = $this->id;
	$filters = [];

	my $filterList = $share{'DB'}->custom_query("SELECT q_response_filter.* FROM q_response_filter, q_report_filter WHERE q_report_filter.report_id='$id' AND q_response_filter.response_filter_id=q_report_filter.response_filter_id");
	for(my $i = 0; $i < @{$filterList}; $i++) {
		next unless $filterList->[$i];
		push @{$filters}, new Modules::Questionnaire::ResponseFilter($filterList->[$i]);
	}
	$this->set('q_response_filters', $filters) if $filters;
	return @{$filters};
}

sub getQueryFilters($) {
	my $this = shift @_;
	my $input = shift @_ || {};

	my $queryFilters = $this->get('q_response_query_filters');
	return @{$queryFilters} if $queryFilters;

	$queryFilters = [];
	my @filterKeys = grep { /^apply_filter_/ } keys %{$input};
	foreach my $f (@filterKeys) {
		$f =~ s/^apply_filter_//;
		push @{$queryFilters}, new Modules::Questionnaire::ResponseFilter($f);
	}
	$this->set('q_response_query_filters', $queryFilters);
	return @{$queryFilters};
}

sub addFilters($) {
	my $this = shift @_;
	my @addFilters = @_;

	my $filters = $this->get('q_response_filters') || [];
	foreach my $f (@addFilters) {
		push @{$filters}, $f if $f && $f->id;
	}
	$this->set('q_response_filters', $filters);
	return @{$filters};
}

sub getAllQuestions($) {
	my $this = shift @_;

	return () unless $this->id;

	my $questions = $this->get('q_questions');
	return @{$questions} if $questions;

	my $id = $this->id;
	my $questionList = $share{'DB'}->custom_query("SELECT q_question.* FROM q_question, q_report_question WHERE q_report_question.question_id=q_question.question_id AND q_report_question.report_id=$id");
	$questions = [];
	for(my $i = 0; $i < @{$questionList}; $i++) {
		next unless $questionList->[$i];
		push @{$questions}, new Modules::Questionnaire::Question($questionList->[$i]);
	}
	@{$questions} = sort { $a->compare($b) } @{$questions};
	$this->set('q_questions', $questions) if $questions;
	return @{$questions};
}

sub getQueryQuestions($) {
	my $this = shift @_;
	my $input = shift @_ || {};

	my @questionKeys = grep { /^report_question_/ } keys %{$input};

}

sub getFilteredResponses($) {
	my $this = shift @_;
	my $questionnaire = shift @_;
	my $input = shift @_;

	return () unless $questionnaire && $questionnaire->id;

	if(!$this->id) {
		my @filters = $questionnaire->getFilters($input);
		$this->addFilters(@filters);
	}
	my @responses = $questionnaire->getAllResponses();
	@responses = $this->filterResponses($input, @responses);
	return @responses;
}

# Print a report of questions based on the report information stored in the
# database
sub report($) {
	my $this = shift @_;
	my $questionnaire = shift @_;
	my @questions = @_;

	my $limit = $config{'Questionnaire'}{'default_reported_questions'} || $DEFAULT_NUMBER_OF_QUESTIONS;
	my $max = $config{'Questionnaire'}{'maximum_reported_questions'} || $MAXIMUM_NUMBER_OF_QUESTIONS;
	my $id = $this->id;
	if(!$id && !@questions) {
		my @questions = $questionnaire->getAllInputQuestions();
		@questions = splice @questions, 0, $limit;
		return $this->reportQuestions($questionnaire, @questions);
	} elsif(!@questions) {
		my @questions = $this->getAllQuestions();
		splice @questions, 0, $limit;
	} 
	my $out = "";
	while(@questions) {
		my @tmpQuestions = splice @questions, 0, $max;
		$out .= $this->reportQuestions($questionnaire, @tmpQuestions)."<br>\n";
	}
	return $out;
}

sub reportQuestions($) {
	my $this = shift @_;
	my $questionnaire = shift @_;
	my @questions = @_;

	return undef unless scalar(@questions);

	my $headers = [];
	my $data = [];
	my $reply = &relink();

	my $input = $share{'input'}->{'get'}->{'data'};
	my @responses = $this->getFilteredResponses($questionnaire, $input);
	my $numQuestions = scalar(@questions);
	my $removeLink = 1;

	if(scalar(@questions) == 1) {
		$removeLink = 0;
	}
	for(my $i = 0; $i < @questions; $i++) {
		my %tmpDisplay = $this->reportQuestionsHash(@questions);
		$tmpDisplay{'report_question_'.$questions[$i]->id} = undef;
		my $link = ($removeLink) ? "&nbsp;<a href='".&relink('qcmd' => 'questionnaire_view', 'report_id' => undef, 'qid' => $questionnaire->id, 'reply' => $reply, %tmpDisplay)."'>[-]</a>" : "";
		#push @{$headers}, $questions[$i]->label."&nbsp;<a href='".&relink('qcmd' => 'questionnaire_view', 'report_id' => undef, 'qid' => $questionnaire->id, 'reply' => $reply, %tmpDisplay)."'>[-]</a>";
		push @{$headers}, $questions[$i]->label.$link;
		for(my $j = 0; $j < @responses; $j++) {
			my $answer = $responses[$j]->questionAnswer($questions[$i]->id);
			my $value = ($answer) ? $answer->printValues: "&nbsp;";

			$data->[$j]->[$i] = $value;
		}
	}
	for(my $i = 0; $i < @responses; $i++) {
		$data->[$i]->[$numQuestions] = $responses[$i]->unlinkLink."&nbsp;".$responses[$i]->detailsLink;
	}
	push @{$headers}, 'Options';
	my %data = (
		'title' => '',
		'headers' => $headers,
		'data' => $data,
	);
	return &Modules::Questionnaire::Report::reportbuilder(%data);
}

sub reportResponse($) {
	my $this = shift @_;
	my $questionnaire = shift @_;
	my $response = shift @_;

	my $class = $this->get('div_class');
	my @participants = $questionnaire->getAllResponses();
	my @questions = $questionnaire->getAllQuestions();
	my $nParticipants = scalar(@participants);
	my $nQuestions = scalar(@questions);

	my $out = "<div class='$class'>\n";
	$out .= $this->questionnaireSummary($questionnaire, $response);
	$out .= "</div>\n";
	return $out;
}

sub reportAllResponses($) {
	my $this = shift @_;
	my $questionnaire = shift @_;
	my $questionsPerRow = shift @_ || 1;
	my $input = shift @_;

	my $class = $this->get('div_class');
	my @participants = $this->getFilteredResponses($questionnaire, $input);
	# We only want to show 'questions' that can take some kind of
	# input

	my @questionKeys = grep { /^report_question_/ } keys %{$input} if $input;
	my @questions = ();
	if(scalar(@questionKeys)) {
		@questions = $this->displayQuestions($questionnaire, $input);
	} else {
	    # this version removes labels from the output
	    #@questions = $questionnaire->getAllInputQuestions();

	    # but having the labels is nice because it helps with formatting
	    @questions = $questionnaire->getAllQuestions();
	}

	my $out = "<div class='$class'>\n<table class=\"reportTable\">\n";
	for(my $i = 0; $i < @questions; $i += $questionsPerRow) {
		$out .= "\t<tr align='top' valign='top'>\n";
		for(my $j = $i; $j < @questions && $j < ($i+$questionsPerRow); $j++) {
			$out .= "\t\t<td class=\"reportTableQuestion\">\n";
			$out .= $this->questionSummary($questions[$j], \@participants);
			$out .= "\t\t</td>\n";
		}
		$out .= "\t</tr>\n";
	}
	$out .= "</div>\n";
	return $out;
}

# Get the summary for each question, but instead of listing through each
# question and fetching the responses along the way, we're going to fetch
# all responses, filter the responses then fetch the questions for each
# response as we go
sub questionnaireSummary($) {
	my $this = shift @_;
	my $questionnaire = shift @_;
	my $response = shift @_;

	return "" unless $questionnaire->id;
	my $out = "<table>\n";
	my @questions = grep { $_->format !~ /label/; } $questionnaire->getAllQuestions();

	for(my $i = 0; $i < @questions; $i += 4) {
		$out .= "\t<tr align='top' valign='top'>\n";
		for(my $j = $i; $j < scalar(@questions) && $j < $i+4; $j++) {
			my $q = $questions[$j];
			$out .= "\t\t<td align='top' style='width: 25%; border-width: 1px; border-color: black; border-style: solid;'>\n";
			$out .= $this->reportQuestion($q, $response->questionAnswer($q->id))."<br>\n";
			$out .= "\t\t</td>\n";
		}
		$out .= "\t</tr>\n";
	}

	$out .= "</table>\n";
	return $out;
}

sub questionSummary($) {
	my $this = shift @_;
	my $question = shift @_;
	my $responses = shift @_ || [];

	return "" unless $question->id;

	my $data = [];
	my @allAnswers = $question->getValues();
	my @answerValues = ();
	my $printedAnswers = [];
	my $totalAnswers = scalar(@{$responses});
	my $allAnswers = 0;
	my %answerHash = ();

	foreach my $r (@{$responses}) {
		my $answer = $r->questionAnswer($question->id);
		my $v = ($answer) ? $answer->value : "";
		push @answerValues, $v unless $answerHash{$v};
		$allAnswers++ if $v;
		$answerHash{$v}++ if $v;
	}

	my %report = (
		'title' => $question->label,
		'width' => '100%',
		'cwidth' => ['','100','100','100'],
	);

	if($question->format =~ /slider/) {
		my $sections = $config{'Questionnaire'}{'slider_report_sections'} || 5;
		my $interVal = POSIX::ceil((@allAnswers / $sections));
		for(my $i = 0; $i < @allAnswers; $i += $interVal) {
			my $responseCount = 0;
			for(my $j = $allAnswers[$i]; $j < ($i+$interVal) && $j < scalar(@allAnswers); $j++) {
				
				$responseCount += $answerHash{$allAnswers[$j]};
			}
			my $pct = ($totalAnswers) ? ($responseCount/$totalAnswers) * 100 : 0;
			my $percentile = sprintf("%2.0f%%", $pct);
			my $chart = $pct ? "<img src=\"$config{server}{HTMLpath}/_Modules/Questionnaire/bluedot.gif\" height=\"10\" width=\"$pct\">" : "";
			my $upper = (($i+$interVal-1) < @allAnswers) ? ($i+$interVal-1) : $#allAnswers;
			push @{$data}, ["$allAnswers[$i] - ".$allAnswers[$upper], $responseCount, $percentile, $chart];
		}
		$report{data} = $data;
		$report{headers} = ['Choice', 'Count', 'Percent', 'Chart'];
	} elsif ($question->format =~ /label/) {
	        return "<h3>$report{title}</h3>\n";
		#$report{data} = " ";
	} elsif($this->alwaysShowValue($question)) {
		foreach my $v (@allAnswers) {
			my $responseCount = $answerHash{$v} || "0";
			my $pct = ($totalAnswers) ? ($responseCount/$totalAnswers) * 100 : 0;
			my $percentile = sprintf("%2.0f%%", $pct);
			my $chart = $pct ? "<img src=\"$config{server}{HTMLpath}/_Modules/Questionnaire/bluedot.gif\" height=\"10\" width=\"$pct\">" : "";
			push @{$data}, [$v, $responseCount, $percentile, $chart];
		}
		$report{data} = $data;
		$report{headers} = ['Choice', 'Count', 'Percent', 'Chart'];
	} else {
		my $pct = ($allAnswers) ? ($allAnswers/$totalAnswers) * 100 : 0;
		my $percentile = sprintf("%2.0f%%", $pct);
		my $chart = $pct ? "<img src=\"$config{server}{HTMLpath}/_Modules/Questionnaire/bluedot.gif\" height=\"10\" width=\"$pct\">" : "";
		push @{$data}, ["Answered Questions", $allAnswers, $percentile, $chart];
		$printedAnswers = \@answerValues;
		$report{data} = $data;
		$report{headers} = ['', 'Count', 'Percent', 'Chart'];
	}
	return &Modules::Questionnaire::Report::reportbuilder(%report);
}

sub reportQuestion($) {
	my $this = shift @_;
	my $q = shift @_;
	my $a = shift @_;

	my $data = [];
	my @values = $q->getValues();
	my @answerValues = ($a) ? $a->allValues() : ();
	my %answersHash = ();
	map {$answersHash{$_}++} @answerValues;
	# TODO: Fix this
	my $totalAnswers = $q->totalAnswerCount();
	if($this->alwaysShowValue($q)) {
		foreach my $v (@values) {
			my $responseCount = $answersHash{$v} || "0";
			my $percentile = ($totalAnswers) ? sprintf("%2.0f%%", (($responseCount/$totalAnswers) * 100)) : "0%";
			push @{$data}, [$v, $responseCount, $percentile];
		}
	} elsif($a && $answersHash{$a->value}) {
		my $responseCount = $answersHash{$a->value};
		my $percentile = ($totalAnswers) ? sprintf("%2.0f", (($responseCount/$totalAnswers) * 100)) : "0%";
		push @{$data}, [$a->value, $responseCount, $percentile];
	}
	my %report = (
		      'title' => $q->label,
		      'data' => $data,
		      'headers' => ['Choice', 'Count', 'Percent'],
		      'width' => '100%',
		      );
	return &Modules::Questionnaire::Report::reportbuilder(%report);
}

sub alwaysShowValue($) {
	my $this = shift @_;
	my $question = shift @_;

	return 1 if $question->format =~ /select|radio|multiselect|checkbox/;
	return 0;
}

sub exportToExcel($) {
	my $this = shift @_;
	my $questionnaire = shift @_;
	my $questions = shift @_ || [];
	my $responses = shift @_ || [];

	my $filename = $this->get('excel_filename');
	my $data = [];
	my %heading = ();
	my @columns = ();

	for(my $i = 0; $i < @{$questions}; $i++) {
		my $col = $questions->[$i]->id;
		push @columns, $col;
		$heading{$col} = $questions->[$i]->fetchBase->label;
	}

	for(my $i = 0; $i < @{$responses}; $i++) {
		my $r = $responses->[$i];
		$r->loadAllAnswers;
		for(my $j = 0; $j < @{$questions}; $j++) {
			my $answer = $r->questionAnswer($questions->[$j]->id) || new Modules::Questionnaire::Answer();
			my $qHash = $questions->[$j]->get('q_question') || {};
			my $col = $questions->[$j]->id;
			$data->[$i]{$col} = $answer->printValues;
		}
	}
	print "Content-type: application/vnd.ms-excel\n";
	print "Content-disposition: attachment; filename=$filename\n\n";
	print $share{'DB'}->export(
		{
			'table' => 'q_questionnaire',
			'format' => 'excel',
			'data' => $data,
			'column' => \@columns,
			'heading'=>\%heading
		}
	);
	return;
}

# Create a new 'report' entry in the database with the default report values
sub make(%) {
	my $this = shift @_;
	my %reportInputs = @_;

	return undef unless ($reportInputs{'name'} && $this->id) || ($reportInputs{'name'} && $reportInputs{'questionnaire_id'});

	my %reportRecord = (
		'name' => $reportInputs{'name'},	
	);
	if($this->id) {
		$reportRecord{'report_id'} = $this->id;
		$reportRecord{'questionnaire_id'} = $this->questionnaire_id;
		$share{'DB'}->update('q_report', \%reportRecord);
	} else {
		$reportRecord{'questionnaire_id'} = $reportInputs{'questionnaire_id'};
		my $reportId = $share{'DB'}->insert('q_report', \%reportRecord);
		$reportRecord{'report_id'} = $reportId;
	}
	$this->set('q_report', \%reportRecord);
	return $reportRecord{'report_id'};
}

# Copy the report information over to $questionnaire
# it's assumed that all of $questionnaire's information is correct
# so all filters will be copied over to point to $quetionnaire's
# questions
sub copy($$) {
	my $this = shift @_;
	my $questionnaire = shift @_;

	return undef unless $this->id && $questionnaire->id;

	my %newReport = %{$this->get('q_report')};
	delete $newReport{'report_id'};
	$newReport{'questionnaire_id'} = $questionnaire->id;
	my $newReport = new Modules::Questionnaire::Report;
	my $newRepId = $newReport->make(%newReport);
	my @questions = $this->getAllQuestions();
	my @filters = $this->getAllFilters();
	# Keep track of the new filters by holding them in their
	# old filter position
	my %newFilters = ();

	foreach my $q (@questions) {
		my $qQuestion = $questionnaire->getByQuestionBase($q->question_base_id);
		$newReport->addQuestion($qQuestion) if ($qQuestion && $qQuestion->id);
	}

	foreach my $f (@filters) {
		$newFilters{$f->id} = $f->copy($questionnaire);
		$newReport->enableFilter($newFilters{$f->id});
	}

	return $newReport;
}

# Make questions automatically adds the first $DEFAULT_NUMBER_OF_QUESTIONS
# questions to the report.
sub makeQuestions() {
	my $this = shift @_;
	my @questions = @_;

	return undef unless $this->id;

	my $numAdded = 0;
	my @qRecords = ();
	my $limit = $config{'Questionnaire'}{'default_reported_questions'} || $DEFAULT_NUMBER_OF_QUESTIONS;
	for(my $i = 0; $i < @questions && $i < $limit; $i++) {
		my $qRecordId = $this->addQuestion($questions[$i]);
		$numAdded++ if $qRecordId;
	}
	$this->set('q_questions', \@questions);
	return $numAdded;

}

# Remove a question from the report.
# If a report entry exists in the database, we simply remove the
# q_report_question entry for the report.
#
# If a report entry doesn't exist, we set up a new report
# with the default values and then remove the question
sub removeQuestion($) {
	my $this = shift @_;
	my $qqid = shift @_;
	my $questionnaire = shift @_;

	# Don't try to delete anything unless a valid 
	# question ID has been provided
	return undef unless $qqid =~ /^\d+$/;

	# No ID means this is a report setup
	if(!$this->id) {
		$this->make('name' => 'test', 'questionnaire_id' => $questionnaire->id);
		my @questions = $questionnaire->getAllQuestions();
		my $nQuestions = $this->makeQuestions(@questions);
	}

	if(!$this->id) {
		$this->error("Error: can't find report that needs the question removed.");
		return undef;
	}
	
	my $match = $share{'DB'}->fetch_match('q_report_question', {'report_id' => $this->id, 'question_id' => $qqid})->[0];
	my @questions = $this->getAllQuestions();
	my $delRes = 0;
	if($match && $match->{'report_question_id'}) {
		$delRes = $share{'DB'}->delete('q_report_question', {'report_question_id' => $match->{'report_question_id'}});
		for(my $i = 0; $i < @questions; $i++) {
			if($questions[$i]->id == $qqid) {
				splice @questions, $i, 1;
				$this->set('q_questions', \@questions);
				last;
			}
		}
	}

	return $delRes;
}

sub addQuestion($) {
	my $this = shift @_;
	my $question = shift @_;

	return undef unless $this->id && $question && $question->id;

	my @questions = $this->getAllQuestions();
	my $record = {'report_id' => $this->id, 'question_id' => $question->id};
	my $qRecordId = $share{'DB'}->insert('q_report_question', $record);
	push @questions, $question;
	$this->set('q_questions', \@questions);
	return $qRecordId;
}

sub removeQuestion($) {
	my $this = shift @_;
	my $question = shift @_;

	return undef unless $this->id && $question && $question->id;

	my @questions = $this->getAllQuestions();
	for(my $i = 0; $i < @questions; $i++) {
		if($questions[$i]->id == $question->id) {
			splice @questions, $i, 1 ;
			last;
		}
	}
	$this->set('q_questions', \@questions);
	return $share{'DB'}->delete('q_report_question', {'question_id' => $question->id, 'report_id' => $this->id});
}

sub enableFilter($) {
	my $this = shift @_;
	my $filter = shift @_;

	return undef unless $this->id && $filter->id;

	my $filterHash = {'response_filter_id' => $filter->id, 'report_id' => $this->id};
	my $newId = $share{'DB'}->insert('q_report_filter', $filterHash);

	my @filters = $this->get('q_response_filters');
	push @filters, $filter;
	
	return $newId;
}

sub removeFilter($) {
	my $this = shift @_;
	my $filter = shift @_;

	return undef unless $this->id && $filter->id;

	my @filters = $this->getAllFilters();
	for(my $i = 0; $i < @filters; $i++) {
		if($filters[$i]->id == $filter->id) {
			splice @filters, $i, 1;
			last;
		}
	}
	$this->set('q_response_filter', \@filters);
	return $share{'DB'}->delete('q_report_filter', {'response_filter_id' => $filter->id, 'report_id' => $this->id});
}

sub filterEnabled($) {
	my $this = shift @_;
	my $filter = shift @_;

	return undef unless $this->id && $filter->id;

	my $filtersHash = $this->get('q_response_filter_hash');

	if(!$filtersHash) {
		$filtersHash = {};
		my @filters = $this->getAllFilters();
		map {$filtersHash->{$_->id} = $_;} @filters;
		$this->set('q_response_filter_hash', $filtersHash);
	}
	return $filtersHash->{$filter->id};
}

sub filterResponses($) {
	my $this = shift @_;
	my $input = shift @_;
	my @responses = @_;

	my @validResponses = ();
	foreach my $r (@responses) {
		push @validResponses, $r if $this->filterResponse($input, $r);
	}
	return @validResponses;
}

sub filterResponse($) {
	my $this = shift @_;
	my $input = shift @_;
	my $response = shift @_;

	return undef unless $response && $response->id;

	# Apply only the requested filters...
	my @filters = $this->getQueryFilters($input);
	foreach my $f (@filters) {
		if(!$f->filterResponse($response)) {
			return 0;
		}
	}
	return 1;
}

sub filterResponsesByMember($) {
	my $this = shift @_;
	my @responses = @_;

	return undef unless scalar(@responses);

	my @filters = $this->getAllFilters();
	foreach my $f (@filters) {
		@responses = $f->filterByMember(@responses);
	}
	return @responses;
}

sub displayQuestions($) {
	my $this = shift @_;
	my $questionnaire = shift @_;
	my $input = shift @_;

	my $questions = $this->get('display_questions');
	return @{$questions} if $questions;

	my @questionKeys = grep { /^report_question_/ } keys %{$input} if $input;
	my @questions = ();
	if($this->id && !@questionKeys) {
		@questions = $this->getAllQuestions();
	} elsif(!$this->id && $questionnaire && !@questionKeys) {
		@questions = $questionnaire->getAllInputQuestions();
		@questions = splice @questions, 0, ($config{'Questionnaire'}{'default_reported_questions'} || $DEFAULT_NUMBER_OF_QUESTIONS);
	} elsif(@questionKeys) {
		foreach my $q (@questionKeys) {
			$q =~ s/^report_question_//;
			push @questions, new Modules::Questionnaire::Question($q);
		}
	}
	@questions = sort {$a->compare($b)} @questions;
	$this->set('display_questions', \@questions);
	return @questions;
}

# Returns a hash containing the proper key/value pairs
# for telling the report which questions to show
sub reportQuestionsHash($) {
	my $this = shift @_;
	my @questions = @_;

	return () unless scalar(@questions);

	my %qHash = ();
	map {$qHash{'report_question_'.$_->id} = 1;} @questions;
	return %qHash;
}

sub loadLink($) {
	my $this = shift @_;
	
	my @filters = $this->getAllFilters();
	my @questions = $this->getAllQuestions();
	my %linkHash = (
		'report_id' => $this->id
	);
	foreach my $f (@filters) {
		$linkHash{'apply_filter_'.$f->id} = 1;
	}
	foreach my $q (@questions) {
		$linkHash{'report_question_'.$q->id} = 1;
	}
	return &relink(%linkHash);
}

sub unloadLink($) {
	my $this = shift @_;

	my @filters = $this->getAllFilters();
	my @questions = $this->getAllQuestions();
	my %unlinkHash = (
		'report_id' => undef
	);
	foreach my $f (@filters) {
		$unlinkHash{'apply_filter_'.$f->id} = undef;
	}
	foreach my $q (@questions) {
		$unlinkHash{'report_question_'.$q->id} = undef;
	}

	return &relink(%unlinkHash);
}

sub editLink($) {
	my $this = shift @_;
	my $input = shift @_;

	my @filters = $this->getAllFilters();
	my @queryFilters = $this->getQueryFilters($input);
	my @questions = $this->getAllQuestions();
	my @queryQuestions = $this->getQueryQuestions($input);

	my %editLink = (
		'qcmd' => 'edit_report',
		'report_id' => $this->id,
	);

	foreach my $f ((@queryFilters, @filters)) {
		$editLink{'apply_filter_'.$f->id} = 1;
	}
	foreach my $q ((@queryQuestions, @questions)) {
		$editLink{'report_question_'.$q->id} = 1;
	}
	return &relink(%editLink);
}

# STATIC REPORT BUILDER FUNCTION -
# doesn't reply on a Modules::Questionnaire::Report
# object to be built, so it should be available to all
sub reportbuilder {
	my (%opt) = @_;
	$REPORT_BUILDER->init(
		title => $opt{'title'},
		data => $opt{'data'},
		tools => $opt{'tools'},
		headers => $opt{'headers'},
		foot => $opt{'foot'},
		width => $opt{'width'},
		cwidth => $opt{'cwidth'},
	);
	return $REPORT_BUILDER->make();
}

1;
__END__
