package Modules::Questionnaire::Response;

use strict;
use ExSite::Config;
use ExSite::Misc;
use Modules::Questionnaire::Question;
use Modules::Questionnaire::Answer;

use base	qw(ExSite::Base);

sub new($) {
	my $type = shift @_;
	my $id = shift @_;

	my $this = {};	
	my $class = ref($type) || $type || "Modules::Questionnaire::Response";
	bless $this, $class;

	if(ref($id) =~ /HASH/) {
		$this->set('q_response', $id);
		$this->loadAllAnswers();
	} elsif($id =~ /^\d+$/) {
		$this->fetch($id);
	}

	return $this;
}

# Fetch the response based on the ID value $id
sub fetch($$) {
	my $this = shift @_;
	my $id = shift @_;

	my $response = $this->get('q_response');

	if(!$response) {
		$response = $share{'DB'}->fetch('q_response', $id);
	}

	if($response) {
		if(!$this->get('q_answer')) {
			$this->loadAllAnswers();
		}
		$this->set('q_response', $response);
	}
	return $response;
}

# Fetch the response based on the user ID
# if there are multiple responses, the last response found will be
# used
#
# Addition: we have to add the questionnaire id, otherwise this will
# return any response for the user. This may mean a response for another
# questionnaire
sub fetchByUid($) {
	my $this = shift @_;
	my $uid = shift @_;
	my $qid = shift @_;

	return undef unless $uid && $qid;

	my $responses = $share{'DB'}->fetch_match('q_response', 
		{'member_id' => $uid, 'questionnaire_id' => $qid});

	my $responseHash = undef;
	if(scalar(@{$responses}) && $responses->[0]) {
		$responseHash = $responses->[$#{@$responses}];
	}

	if($responseHash) {
		$this->set('q_response', $responseHash);
		$this->set('q_answer', undef);
		$this->loadAllAnswers();
	}
	return $responseHash;
}

sub id($) {
	my $this = shift @_;

	return undef unless $this->get('q_response');
	return $this->get('q_response')->{'response_id'};
}

sub member_id($) {
	my $this = shift @_;
	return undef unless $this->get('q_response');
	return $this->get('q_response')->{'member_id'};
}

# Return the current value of the response
# (the value of the answer associated with this response)
sub value($) {
	my $this = shift @_;

	my $answer = $this->answer->{'value'};
	return $answer->{'value'};
}

# Return the specific answer for this reponse and the given
# user and questionnaire (there should only be one answer associated
# with a particular user, questionnaire and question)
sub questionAnswer($) {
	my $this = shift @_;
	my $questionId = shift @_;

	return undef unless $questionId;

	my $answersHash = $this->get('allAnswersHash');
	return $answersHash->{$questionId} if $answersHash;
	$this->loadAllAnswers();
	$answersHash = {};
	my $answers = $this->get('allAnswers');
	map {$answersHash->{$_->question_id} = $_;} @{$answers};
	$this->set('allAnswersHash', $answersHash);
	return $answersHash->{$questionId};
}

# Load all of the answers for this response.
sub loadAllAnswers($) {
	my $this = shift @_;

	my $answers = $this->get('allAnswers');
	return $answers if $answers;

	my $id = $this->id;
	return undef unless $id;

	$answers = $share{'DB'}->fetch_match('q_answer',
		{'response_id' => $id}
	);
	my $answerObjs = [];
	foreach my $a (@{$answers}) {
		push @{$answerObjs}, new Modules::Questionnaire::Answer($a);
	}
	$this->set('allAnswers', $answerObjs);
	return $answerObjs;
}

# Create a new response record for the current questionnaire
# and member
sub make($) {
	my $this = shift @_;
	my %response = @_;

	return undef unless $response{'questionnaire_id'};

	$response{'version'} = "English" unless $response{'version'};

	# Insert the response record.
	my $newId = $share{'DB'}->insert('q_response', \%response);
	$response{'response_id'} = $newId;

	$this->set('q_response', \%response);
	return $newId;
}

sub unlink($) {
	my $this = shift @_;

	return unless $this->id;

	my $delResult = $share{'DB'}->trash_r('q_response', $this->id);
	$this->set('q_response', undef);
	return $delResult;
}

sub unlinkLink($) {
	my $this = shift @_;

	return "" unless $this->id;

	my $reply = &relink();
	my $unlinkUrl = &relink('qcmd' => 'unlink_response', 'response_id' => $this->id, 'reply' => $reply, 'submit_action' => 'Submit');
	return "<a alt='delete response' href='#' onClick=\"javascript:confirm_custom('Are you sure you want to delete this user\\\'s response?', '$unlinkUrl')\"><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/rm12.gif' border='0'></a>";
}

sub detailsLink($) {
	my $this = shift @_;

	return "" unless $this->id;

	my $reply = &relink();
	my $detailsUrl = &relink('response_details' => 1, 'response_id' => $this->id);

	return "<a alt='view details' href='$detailsUrl'><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/view12.gif' border='0'></a>\n";

}

sub copyLink($) {
	my $this = shift @_;

	return "" unless $this->id;

	my $reply = &relink();
	my $copyUrl = &relink('qcmd' => 'edit_questionnaire');

	return "<a alt='copy questionnaire' href='$copyUrl'><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/copy12.gif' border='0'></a>\n";
}


1;
__END__

