package Modules::Questionnaire::ResponseFilter;

use strict;
use ExSite::Config;
use ExSite::Misc;
use Modules::Questionnaire::ResponseMatch;
use Modules::Questionnaire::Answer;

use base	qw(ExSite::Base);

# Creates a response_filter object.  $id = the record ID of the
# response_filter in the DB.  Can be left off, if fetch() or
# make() is used to set the ID manually.
sub new($$) {
	my $type = shift @_;
	my $id = shift @_;

	my $this = {};
	my $class = ref($type) || $type || "Modules::Questionnaire::ResponseFilter";
	bless $this, $class;

	if(ref($id) =~ /HASH/) {
		$this->set('q_response_filter', $id);
	} else {
		$this->fetch($id) if $id;
	}
	return $this;
}

# Fetch q_response_filter entry in the database
# Fetches the basic data for the response_filter record, and
# then loads the match criteria for that response_filter.
# Invoked automatically by new(), but can be called explicitly
# to change response_filters.  Parameters are same as new().
sub fetch($) {
	my $this = shift @_;
	my $id = shift @_;

	return undef unless $id;

	my $filter = $this->get('q_response_filter');
	return $filter if $filter;

	$filter = $share{'DB'}->fetch('q_response_filter', $id);

	$this->set('q_response_filter', $filter) if scalar(keys %{$filter});
	return $filter;
}

# Get all of the q_response_match records associated with
# the current q_response_filter object
sub fetchMatches($) {
	my $this = shift @_;

	my $matches = $this->get('q_response_matches');
	return @{$matches} if $matches;

	my $id = $this->id;

	$matches = $share{'DB'}->fetch_match('q_response_match',
		{'response_filter_id' => $id}
	);

	my $matchObjs = [];
	foreach my $m (@{$matches}) {
		push @{$matchObjs}, new Modules::Questionnaire::ResponseMatch($m);
	}

	$this->set('q_response_matches', $matchObjs) if scalar(@{$matchObjs});
	return @{$matchObjs};
}

# Removing the q_response_match records from this object seems
# to happen on a semi-regular basis, so there's a method here
# to do the job for us:
sub clearMatches($) {
	my $this = shift @_;

	return $this->set('q_response_matches', undef);
}

################# ACCESSORS
# Accessors used for convenience:
sub id($) {
	my $this = shift @_;

	return undef unless $this->get('q_response_filter');
	return $this->get('q_response_filter')->{'response_filter_id'};
}

sub name($) {
	my $this = shift @_;

	return undef unless $this->get('q_response_filter');
	return $this->get('q_response_filter')->{'name'};
}

sub filter($) {
	my $this = shift @_;
	return undef unless $this->get('q_response_filter');
	return $this->get('q_response_filter')->{'filter'};
}

sub questionnaire_id($) {
	my $this = shift @_;

	return undef unless $this->get('q_response_filter');
	return $this->get('q_response_filter')->{'questionnaire_id'};
}

################# FILTERING
# Given an array of Response objects (such as is returned by
# Questionnaire::getAllResponses()), filterList() returns an array
# containing the subset of the responses that match the filter
# match criteria.
sub filterList($@) {
	my $this = shift @_;
	my @allResponses = shift @_;

	my @matchedResponses = ();
	foreach my $r (@allResponses) {
		push @matchedResponses, $r if $this->filterResponse($r);
	}
	return @matchedResponses;
}

# Given an array of response objects, filterByMember returns
# an array the responses for each member that matches the
# criteria. 
# eg. If user A answered "Jim" for his first name, and "Jim" was
# being filtered out, then NONE of Jim's responses will be returned
sub filterByMember($) {
	my $this = shift @_;
	my @allResponses = @_;

	my @matchedResponses = ();
	my %members = ();

	return @allResponses unless $this->id;

	# Gather all of the member IDs that need to be removed
	foreach my $r (@allResponses) {
		$members{$r->member_id}++ unless $this->filterResponse($r) && $r->answer;
	}
	# Only save those responses whose member_ids weren't
	# filtered in the previous search
	foreach my $r (@allResponses) {
		push @matchedResponses, $r unless $members{$r->member_id};
	}
	return @matchedResponses;
}

# Filter a single response record.
# Return boolean value to indicate whether or not the
# response matched our criteria
# (true = match, false = no match)
sub filterResponse($) {
	my $this = shift @_;
	my $response = shift @_;

	return 0 unless ref($response) =~ /Response/ ;

	my @matches = $this->fetchMatches();
	# Match is successful if no criteria exist:
	return 1 unless scalar(@matches);
	foreach my $m (@matches) {
		next unless $m->filter;
		my $answer = $response->questionAnswer($m->question_id);
		if($answer) {
			my $filter = $m->filter;
			return 0 if $answer->rawValue !~ /$filter/;
		}
	}
	return 1;	
}

############### CREATION
# Creates the database record for a new response_filter.
# %response_filter is a hash containing the base response_filter
# data.  Match criteria are added using the addMatchCriteria()
# method.  0 is returned on failure, 1 on success.  On success,
# the Response_Filter object is switched over to the new
# response_filter.
sub make($%) {
	my $this = shift @_;
	my %filter = @_; 

	return 0 unless $filter{'questionnaire_id'} && defined($filter{'name'});

	my $resId = 0;
	# If a filter exists, we'll update
	if($filter{'response_filter_id'}) {
		$resId = $share{'DB'}->update('q_response_filter', \%filter);
	} else {
		$resId = $share{'DB'}->insert('q_response_filter', \%filter);
	}
	if($resId) { 
		$filter{'response_filter_id'} = $resId;
		$this->set('q_response_filter', \%filter);
		# Matches will have changed along with the new object, so
		# we reset this for the next fetchMatches call
		$this->set('q_response_matches', undef);
	}
	return $resId;
}

sub copy($) {
	my $this = shift @_;
	my $questionnaire = shift @_;

	return 0 unless $this->id && $questionnaire->id;

	my %filter = %{$this->get('q_response_filter')};
	delete $filter{'response_filter_id'};
	$filter{'questionnaire_id'} = $questionnaire->id;
	my $newFilter = new Modules::Questionnaire::ResponseFilter;
	my $newId = $newFilter->make(%filter);

	my @matches = $this->fetchMatches();
	my @newMatches = ();
	
	foreach my $m (@matches) {
		my $q = $m->fetchQuestion;
		my $newQ = $questionnaire->getByQuestionBase($q->question_base_id);
		if($newQ && $newQ->id) {
			$m->make('response_filter_id' => $newFilter->id, 'question_id' => $newQ->id,
				'filter' => $m->filter);
			push @newMatches, $m;
		}
	}
	$newFilter->set('q_response_matches', \@newMatches);
	return $newFilter;
}

sub update($%) {
	my $this = shift @_;
	my %values = @_;

	my %filter = %{$this->get('q_response_filter')};
	foreach my $k (keys %values) {
		if(exists $filter{$k}) {
			$filter{$k} = $values{$k};
		}
	}
	my $resId = $share{'DB'}->update('q_response_filter', \%filter);
	$this->set('q_response_filter', \%filter);
	return $resId;
}

# Adds a match criterion to the current response_filter.  $qid
# is the id of the question whose answers should be selected;
# $regexp is the match regular expression (typically a simple
# string) to match to.  0 is returned on failure, 1 on success.
sub addMatchCriteria($) {
	my $this = shift @_;
	my $qqid = shift @_; # Question ID
	my $regexp = shift @_; # Regex match

	# Don't allow a match to be inserted without a valid question
	# ID or regular expression match
	return 0 unless $qqid && $regexp;

	# First, lets make sure that a question exists matching
	# ID $qqid
	require Modules::Questionnaire::Question;
	my $question = new Modules::Questionnaire::Question($qqid);
	return 0 unless $question->id;

	my $matchHash = {
		'question_id' => $qqid,
		'filter' => $regexp,
		'response_filter_id' => $this->id,
	};

	my $newId = $share{'DB'}->insert('q_response_match', $matchHash);
	# For the time being: reset the match list and re-fetch so that it matches
	# our change.
	# We could simply add the match to the list, but we don't know if the current
	# list really represents the DB (it could be empty for example, and adding
	# the current match to the stored list would prevent a future fetch from gathering
	# all of the other match records specific to this record)
	$this->set('q_response_matches', undef);
	return $newId;
}

# Set and store responses to make determining certain statistics faster:
sub setResponses($) {
	my $this = shift @_;
	my @responses = @_;

	my @filteredResponses = $this->filterByMember(@responses);

	$this->set('all_responses', \@filteredResponses);
	my %responsesByQuestion = {};
	foreach my $r (@filteredResponses) {
		push @{$responsesByQuestion{$r->answer->question_id}}, $r;
	}
	$this->set('responsesByQuestion', \%responsesByQuestion);
	return;
}

sub getResponsesByQuestion($) {
	my $this = shift @_;
	my $questionId = shift @_;

	return undef unless $questionId;

	my $hash = $this->get('responsesByQuestion');
	my $array = $hash->{$questionId} || [];
	return @{$array};
}

sub filteredResponses($) {
	my $this = shift @_;
	
	my $responses = $this->get('all_responses') || [];
	return @{$responses};
}

# Methods that are mostly useful for preparing data for reports...
#
# Return a count of the responses in the array provided that
# have value $value AND pass our filters
sub responseCount($) {
	my $this = shift @_;
	my $questionId = shift @_;
	my $value = shift @_;

	my @responses = $this->getResponsesByQuestion($questionId);

	my %answerCounts = ();

	foreach my $r (@responses) {
		my @values = $r->answer->allValues();
		foreach my $v (@values) {
			$answerCounts{$v}++;
		}
		#$answerCounts{$r->answer->{'value'}}++;
	}
	return $answerCounts{$value};
}

sub totalAnswerCount($) {
	my $this = shift @_;
	my $questionId = shift @_;

	my @responses = $this->getResponsesByQuestion($questionId);
	my $total = 0;
	foreach my $r (@responses) {
		$total += $r->answer->count;
	}
#	return scalar(@responses);
	return $total;
}

# Remove the filter record from the database.
# Delete the current q_response_filter record and all of its
# associated q_response_match records.
sub delete($) {
	my $this = shift @_;

	my $id = $this->id;
	$this->clearMatches();
	my @matches = $this->fetchMatches();

	my $delRes = 0;
	foreach my $m (@matches) {
		$m->delete();
	}
	$delRes = $share{'DB'}->delete('q_response_filter', {'response_filter_id' => $id});

	$this->clearMatches();
	$this->set('q_response_filter', undef);
	return $delRes;
}

sub isApplied($) {
	my $this = shift @_;
	return 0 unless $this->id;

	if($share{'input'}->{'get'}->{'data'}->{'apply_filter_'.$this->id}) {
		return 1;
	}
	return 0;
}

# Icon link to the current custom report.
sub reportLinkIcon($) {
	my $this = shift @_;

	return "" unless $this->id;

	return "<a href='".&relink('qcmd' => 'edit_response_filter', 'response_filter_id' => $this->id)."'><img alt='".$this->name." report' src='$config{server}->{HTMLpath}/_Modules/Questionnaire/view12.gif' border='0'></a>\n";
}

sub reportLink($) {
	my $this = shift @_;

	return "" unless $this->id;

	return "<a href='".&relink('qcmd' => 'report_questionnaire', 'response_filter_id' => $this->id, 'questionnaire_id' => $this->questionnaire_id)."'>View Report</a>\n";
}

sub reportViewLink($) {
	my $this = shift @_;

	return "" unless $this->id;

	return "<a href='".&relink('qcmd' => 'report_questionnaire', 'response_filter_id' => $this->id, 'question_id' => $this->questionnaire_id)."'><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/list12.gif' border='0'></a>\n";
}

sub newResponseFilterLink($) {
	my $questionnaireId = shift @_;

	return "" unless $questionnaireId;

	return "<a href='".&relink('qcmd' => 'new_response_filter', 'qid' => $questionnaireId)."'><img alt='New Filtered Report' src='$config{server}->{HTMLpath}/_Modules/Questionnaire/new12.gif' border='0'></a>\n";
}

sub deleteResponseFilterLink($) {
	my $this = shift @_;

	return "" unless $this->id;

	return "<a href='".&relink('qcmd' => 'delete_response_filter', 'response_filter_id' => $this->id)."'><img alt='New Response Filter' src='$config{server}->{HTMLpath}/_Modules/Questionnaire/rm12.gif' border='0'></a>\n";
}


1;

__END__
