package Modules::StripGallery;

#----------------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#-----------------------------------------------------------------------
# StripGallery : tool to browse an image library
#
# We present a set of thumbnails in a strip (horizontal or vertical).
# Selecting a thumbnail brings up the full-sized image.
# By default, the images will load into a special DIV on the page.  If 
# you set the popup parameter, they they will popup in their own windows
# instead.
#
#
# Options:
#
# id=N         numeric ID of library
# name=ABC     name of library
# format=horizontal|vertical|free,popup|hover|click,strip|display|full
# thumbnail_size=n|x,y	thumbnail size
#
#
#
# To enabel image caption display, you need to add caption paramter to your module call.
#
# There are four options for caption display:
#	l: left
#	r: right
#	t: top
#	b: bottom
#
# e.g. <!--&StripGallery(format=free,hover,full,caption=r)-->
#
# To display captions for thumbnail images, add 'caption_thumbnail=n' or 'caption_thumbnail=d' to the format options
# 	n: name of the content object
#	d: the description of the content object
#
# e.g. <!--&StripGallery(format=free,hover,full,caption=r,caption_thumbnail=n)-->
#
#-----------------------------------------------------------------------
# Examples:
#
# <!--&StripGallery()-->
#      - horizontal thumbnail strip, with images inserted above the strip
#
# <!--&StripGallery(format=popup)-->
#      - horizontal thumbnail strip with popup images
#
# <!--&StripGallery(format=vertical)-->
#      - vertical thumbnail strip, with images inserted left of the strip
#
# <!--&StripGallery(format=vertical,popup)-->
#      - vertical thumbnail strip with popup images
#
# <!--&StripGallery(format=free,hover,full)-->
#	- free form thumbnail grid with rollver images.
#
# To make the thumbnail strip scroll, you need to emply some CSS, eg:
#
# /* horizontal scrolling strip with 500px wide viewport */
# div.StripGallery {
#    width:500px;
#    overflow:auto;
# }
#
# /* vertical scrolling strip with 500px tall viewport */
# div.StripGallery {
#    height:500px;
#    width:130px;
#    overflow:auto;
# }
#
# /* 640x480 image viewing area, with scrollbars if overflow */
# div.StripGalleryDisplay {
#    width:640px;
#    height:480px;
#    overflow:auto;
#    text-align:center;
# }
#
# /* 640x480 image viewing area, image centered, no overflow handling */
# td.StripGalleryDisplay {
#    width:640px;
#    height:480px;
# }
#
#-----------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Input;
use ExSite::Misc;
use ExSite::Content;
use Modules::BaseDCD;

# inherit basic behaviours from BaseDCD

use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

# provide our own write() method

sub write {
    my ($this,$options) = @_;
    my $in = new ExSite::Input;
    my %opt = &DecodeString($options);
    if (! $opt{format}) { $opt{format} = "full,horizontal"; }
    $this->{db} = $share{Page} || $share{DB};

    # Set display caption flag
    $this->set_caption($opt{format});
    
    my ($next, $curr, $prev);
    my ($album,$album_id);
    if ($opt{id}) {
	# album passed by id#
	$album_id = $opt{id};
	$album = $this->{db}->fetch("page",$album_id);
    }
    elsif ($opt{name}) {
	# album passed by name
#        my @lib = $this->{db}->fetch_match("page",{section_id=>$share{Page}->id("section"),
#						   filename=>$opt{name}});
#	$album = $lib[0];
#	$album_id = $album->{page_id};
        my $lib = $this->{db}->fetch("page",$opt{name});
	$album = $lib;
	$album_id = $lib->{page_id};
    }
    if (!$album) {
	# look for "images" library in current site
        my @lib = $this->{db}->fetch_match("page",{section_id=>$share{Page}->id("section"),
                                           filename=>"images"});
	$album = $lib[0];
	$album_id = $album->{page_id};
    }
    return $this->error("Gallery $opt{name} $opt{id} not found.") 
	if (! $album_id);
    $this->{album_id} = $album_id;
    $this->{album} = $album;

    my @pics = $this->{db}->fetch_child("content","page",$this->{album_id},"name");

    my $out;
    if ($opt{format} =~ /popup/) {
	$out .= "<script langauge=\"JavaScript\" src=\"$config{server}{HTMLpath}/_ExSite/js/misc.js\"></script>\n";
    }
    else {
	$out .= $this->javascript_image_swap();
	
	if ($opt{format} !~ /popup|strip/) {
	    $out .= "<table><tr><td align=\"center\" class=\"StripGalleryDisplay\">\n";
	}
	if ($opt{format} =~ /display|full/) {	    
	    my $c = new ExSite::Content(content=>$pics[0]);	    
	    $c->set_context("child");
	    my $main_image;
	    if ($c->content_type() =~ /^image\/(jpe?g|gif|png)$/) {
		my $url = $c->get_url;
		my $attr = $c->get("content_data","attributes");
		$main_image = "<img src=\"$url\"$attr>\n";
	    }
	    my $main_display = <<END;
<div class="StripGalleryDisplay" id="SG_Display">
$main_image
</div>
END
	    # Display image captions
	    if ($this->use_caption()){
	    	# Get caption position
		$opt{format} =~ /caption=(.{1})/i;
		my $caption_pos = $1;
		# Get description
		my $descr = $c->get("content","description");
		# This div encloses the caption
		my $caption = <<END;
<div class="StripGalleryCaption" id="SG_Caption">
$descr
</div>
END
		# Captions on the top
		if ($caption_pos =~ /t/i){
		    $out .= <<END;
$caption
$main_display
END
		# Captions on the bottom
		}elsif ($caption_pos =~ /b/i){
		    $out .= <<END;
$main_display
$caption
END
		# Captions on the left
		}elsif ($caption_pos =~ /l/i){
		    $out .= <<END;
<table>
<tr>
	<td class="StripGalleryCaption">
	$caption
	</td>
	<td class="StripGalleryDisplay">
	$main_display
	</td>
</tr>
</table>
END
		# Captions on the right
		}elsif ($caption_pos =~ /r/i){
		    $out .= <<END;
<table>
<tr>
	<td class="StripGalleryDisplay">
	$main_display
	</td>
	<td class="StripGalleryCaption">
	$caption
	</td>
</tr>
</table>
END
		}
	    }else{
	    	# Caption option wasn't used
	    	$out .= $main_display;
	    }
	}
	if ($opt{format} =~ /full/) {
	    if ($opt{format} =~ /vertical/) {
		$out .= "</td><td>\n";
	    }
	    else {
		$out .= "</td></tr>\n<tr><td>\n";
	    }
	}
    }

    # display the thumb grid

    if ($opt{format} =~ /full|strip/) {
    	# Free mode flag
    	my $free_mode = $opt{format} =~ /free/ ? 1 : 0;
	# Display thumbnails in a div in free mode
    	if ($free_mode){
		$out .= <<END
<div class="StripGallery">
END
	}else{
		$out .= "<div class=\"StripGallery\"><table class=\"StripGallery\"><tr>\n";
	}
	my $n = 0;
	foreach my $p (@pics) {
	    my $c = new ExSite::Content(content=>$p);
	    my $descr = $c->get("content","description");
	    $c->set_context("child");
	    next if ($c->content_type() !~ /^image\/(jpe?g|gif|png)$/);	    
	    my $thumb = $c->get_thumb_html;
	    my $url = $c->get_url;
	    
	    # Set default thumbnail size
 	    my $attr = $c->get("content_data","attributes");
	    $attr =~ /width=\"?(\d+)\"?/i;
	    my $width = $1+20 || 600;
	    $attr =~ /height=\"?(\d+)\"?/i;
	    my $height = $1+20 || 600;
	    
	    # Set thumbnail dimension
	    if ($opt{thumbnail_size}){
	    	$thumb =~ /width=\"?(\d+)\"?/i;
		my $t_width = $1 ||  100;
		$thumb =~ /height=\"?(\d+)\"?/i;
		my $t_height = $1 || 100;	    	
	    	($t_width,$t_height) = $this->set_thumbnail_dimension($t_width,$t_height,$opt{thumbnail_size});
		my $thumb_url = $c->get_thumb_url;
		# Display thumbnail and preload images
		$thumb = <<END;
<img src="$thumb_url" width="$t_width" height="$t_height" border="0">
<div style="display:none;"><img src="$url"></div>
END
	    }

	    # Construct thumbnail image
	    my $image_link;
	    # Popup image
	    if ($opt{format} =~ /popup/) {
		$image_link .= "<a href=\"javascript:popup_custom($width,$height,'$url')\">$thumb</a>";
	    }
	    else {
	    	my $js_call = $this->javascript_image_swap_call($url,$descr);
		my $href = "javascript:$js_call";
		my $event_handler;
		# Set link properties. i.e. href and onMouseOver
		if ($opt{format} =~ /hover/){
			$href= "#";
			$event_handler = " onMouseOver=\"$js_call\"";
		}						
		$image_link .= "<a href=\"${href}\"${event_handler}>$thumb</a>";
	    }
	    
	    
	    if ($free_mode){
	    	$out .= "<div class=\"StripGalleryThumb\" style=\"float:left;\">";
	    }else{
	    	$out .= "<td>";
	    }
	    
	    $out .= $image_link;

	    # Get thumbnail caption
	    $out .= $this->get_thumbnail_caption($c);
	    
	    if ($free_mode){
	    	$out .= "</div>";
	    }else{
	    	if ($opt{format} =~ /vertical/){
			$out .= "</td></tr>\n<tr>";
		}else{
			$out .= "</td>\n";
		}
	    }
	    $n++;
	}
	if (!$free_mode){
		$out .= "</tr></table>\n";
	}
	$out .= "</div>\n";
    }
    if ($opt{format} !~ /popup|strip/) {
	$out .= "</td></tr></table>\n";
    }
    return $out;
}
# Get the thumbnail captions
sub get_thumbnail_caption{
	my ($this,$content) = @_;
	if ($this->use_thumbnail_caption()){
		if ($this->get_thumbnail_caption_type()){
			my $t_caption = $content->get("content",$this->get_thumbnail_caption_type());
			return "<br><span class=\"StripGalleryThumbCaption\">$t_caption</span>";
		}		
	}
	return;
}
# Get thumbnail caption type (content name or description)
sub get_thumbnail_caption_type(){
	my $this = shift;
	my %type = (
		n=>"name",
		d=>"description",
	);
	# Check for valid thumbnail caption type
	if (exists($type{$this->{thumbnail_caption_type}})){
		return $type{$this->{thumbnail_caption_type}};
	}
	return;
}
# Escape caption text
sub clean_caption{
	my ($this,$text) = @_;
	my $img_caption = $text;
	# Escape \n
	$img_caption =~ s/(\r)\n/<br>/g;
	# Escape double quotes
	$img_caption =~ s/"/&quot;/g;
	# Escape single quotes
	$img_caption =~ s/'/\\\'/g;
	return $img_caption;
}
# Get javascript image swap call
sub javascript_image_swap_call{
	my ($this,$url,$description) = @_;
	my $out = "subImage('SG_Display','$url',";
	if ($this->use_caption()){
		my $caption = $this->get_javascript_caption_param($description);
		$out .= "'SG_Caption','$caption'";
	}else{
		$out .= "'',''";
	}
	$out .= ")";
	return $out;

}
# Set display caption flag
sub set_caption{
	my ($this,$option) = @_;
	# Get caption info
	my $use_caption = ($option =~ /caption=.{1}/i && $option !~ /popup/) || 0;
	$this->{use_caption} = $use_caption || 0;
	# Get thumbnail caption info
	if ($option=~ /caption_thumbnail=(.{1})/i){
		my $type = $1;
		if ($type){
			$this->{use_thumbnail_caption} = 1;
			$this->{thumbnail_caption_type} = $type;
		}
	}
	return;
}
# Display caption for thumbnails?
sub use_thumbnail_caption{
	my $this = shift;
	return $this->{use_thumbnail_caption};
}
# Display caption?
sub use_caption{
	my $this = shift;
	return $this->{use_caption} || 0;
}
# Get caption paramter for passing into function subImage
sub get_javascript_caption_param{
	my ($this,$caption) = @_;
	return if !$caption;
	# Image caption paramter
	my $caption_param;
	# Check to see if we need to display image captions
	if ($this->use_caption()){
		return $this->clean_caption($caption);
	}
	return $caption_param;
}
# Javascript subImage function
sub javascript_image_swap{
	my $this = shift;
	return <<END;
<script langauge="JavaScript">
<!--
function subImage(id,imgurl,caption_id,caption){
	if (caption_id){
		subCaption(caption_id,caption);
	}
	var data = '<img src="' + imgurl + '">';
	var loc = document.getElementById(id);
	loc.innerHTML = data;

}
function subCaption(id,caption){
	var caption_div = document.getElementById(id);
	caption_div.innerHTML = caption;
}
//-->
</script>
END
}

sub set_thumbnail_dimension{
	my ($this,$t_width,$t_height,$dim) = @_;
	return ($t_width,$t_height) if !$dim;
	if ($dim =~ /,/){
		$dim =~ s/\s//g;
		($t_width,$t_height) = split(/,/,$dim,2);
	}else{
		if ($t_width != $t_height){
			my $ratio;
			if ($t_width > $t_height){
				$ratio = $t_width / $dim;
				$t_width = $dim;
				$t_height = sprintf("%i",($t_height / $ratio));
			}else{
				$ratio = $t_height / $dim;
				$t_height = $dim;
				$t_width = sprintf("%i",($t_width / $ratio));
			}
		}else{
			$t_width = $dim;
			$t_height = $dim;
		}
	}
	return ($t_width,$t_height);
}

sub show_1st_image {
    my ($this) = @_;
    my @pics = $this->{db}->fetch_child("content","page",
				$this->{album_id},"name",1);

    my $c = new ExSite::Content(content=>$pics[0]);
    my $descr = $c->get("content","description");
    $c->set_context("child");

    return "<div id=\"SG_Display\" class=\"StripGalleryDisplay\"></div>" 
	if ($c->content_type() !~ /^image\/(jpe?g|gif|png)$/);
    
    my $url = $c->get_url;
    my $attr = $c->get("content_data","attributes");
    return "<div id=\"StripGalleryTarget\">
<img src=\"$url\"$attr><br>
<span class=\"caption\">$descr</span>
</div>\n";
}

sub show_gallery {
    my ($this,$target) = @_;

    my $javascript_function = $this->javascript_image_swap();
    my $out  = <<END;
<div class="StripGallery">
$javascript_function
END

    $out .= "<table class=\"StripGallery\"><tr>\n";

    my @pics = $this->{db}->fetch_child("content","page",
				       $this->{album_id},"name");

    # display the thumb grid

    my $n = 0;
    foreach my $p (@pics) {
	my $c = new ExSite::Content(content=>$p);
	my $descr = $c->get("content","description");
	$c->set_context("child");
	next if ($c->content_type() !~ /^image\/(jpe?g|gif|png)$/);
	$out .= "<td align=\"center\" class=\"StripGallery\" valign=\"top\">";
	my $thumb = $c->get_thumb_html;
	my $url = $c->get_url;
	my $attr = $c->get("content_data","attributes");
	$attr =~ /width=\"?(\d+)\"?/i;
	my $width = $1+20 || 600;
	$attr =~ /height=\"?(\d+)\"?/i;
	my $height = $1+20 || 600;
	if ($target) {
	    my $js_call = $this->javascript_image_swap_call($url,$c->get("content","description"));
	    $out .= "<a href=\"javascript:$js_call\">$thumb</a>";
	}
	else {
	    $out .= "<a href=\"javascript:popup_custom($width,$height,'$url')\">$thumb</a>";
	}
	$out .= "</td>\n";
	$n++;
    }
    $out .= "</tr></table>\n</div>\n";
    return $out;
}

sub ioctl {
    my $this = shift;
    $_ = shift;
    if (/ModuleName/) {
	return "Strip Gallery";
    }
    elsif (/ModuleInfo/) {
	return "The Strip Gallery tool displays a library of image thumbnails
in a strip.  Clicking on a thumbnail either pops up the image, or places the
image into a another part of the page.";
    }
    elsif (/DynContentParam/) {
    	return $this->getDynContentParameter;
    }
}

sub getDynContentParameter{
	my ($this,$section_id) = @_;
	my (@libs,$out,$val);
	my $out = "<select name=\"param\">\n";
	if (! $section_id) {
	    my $in = new ExSite::Input;
	    my $q = $in->query;
	    $section_id = $q->{section_id};
	}
	if ($section_id) {
	    require ExSite::Section;
	    my $s = new ExSite::Section(id=>$section_id);
	    @libs = $s->fetch_libraries();
	}
	do {
	    my $lib = shift @libs;
	    $val = $lib ? "id=$lib->{page_id}&" : undef;
#format=horizontal|vertical,popup,strip|display|full
	    $out .= "<option value=\"${val}\">$lib->{title} full gallery, horizontal strip</option>
<option value=\"${val}format=vertical,click,full\">$lib->{title} full gallery, vertical strip</option>
<option value=\"${val}format=vertical,popup,strip\">$lib->{title} vertical strip, popup images</option>
<option value=\"${val}format=horizontal,popup,strip\">$lib->{title} horizontal strip, popup images</option>
<option value=\"${val}format=horizontal,strip\">$lib->{title} horizontal strip only</option>
<option value=\"${val}format=vertical,strip\">$lib->{title} vertical strip only</option>
<option value=\"${val}format=free,popup,strip\">$lib->{title} grid strip, popup images</option>
<option value=\"${val}format=free,click,full\">$lib->{title} full gallery, grid strip with clickable images</option>
<option value=\"${val}format=free,hover,full\">$lib->{title} full gallery, grid strip with rollover images</option>
<option value=\"${val}format=display\">$lib->{title} display only</option>
";
	} until (@libs <= 0);
	$out .= "</select>\n";
	return $out;
}

1;
