package Modules::Upload;
#----------------------------------------------------------------------------
#   The Upload module is a simple utility for uploading files into 
#   content libraries.
#-----------------------------------------------------------------------
#
#   Copyright 2001-2006 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#-----------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Page;
use ExSite::Input;
use ExSite::Misc;

# inherit from BaseDCD class
use Modules::BaseDCD;
use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

# The Upload tool has no public output

sub write {
    my ($this,$options) = @_;
    return;
}

sub ioctl {
    my $this = shift;
    $_ = shift;   # $_ is the ioctl request
    if (/ModuleInfo/) {
	return "The Upload plug-in provides a simple utility for uploading 
files into content libraries.  It is especially useful for large files that
are difficult to manage using other content management tools.";
    }
    elsif (/ControlPanel/) {
	return \&ControlPanel;
    }
    elsif (/isRestricted/) {
	# restrict this to make it harder to fill the disk with junk
	return 1;
    }
}

sub get_library {
    my ($this) = @_;
    my $id = $this->{input}{lib};
    if ($id) {
	my $lib = new ExSite::Page(id=>$id);
	if ($lib && $lib->owns()) {
	    $this->{library} = $lib;
	    return $lib;
	}
    }
    return undef;
}

sub get_all_libraries {
    my ($this) = @_;
    my %library;
    my @sites = $share{DB}->my_real_sites();
    foreach my $site (@sites) {
	$library{$site->{title}} = 
	    $share{DB}->fetch_match("page",{
		type=>"library",
		status=>"active",
		section_id=>$site->{section_id}});
    }
    return %library;
}

sub ControlPanel {
    my $this = shift;
    my $db = $share{DB};
    my $url = $this->link;
    my $out = "<h1>File Upload Utility</h1>
<form ACTION=\"$url\" METHOD=\"POST\" enctype=\"multipart/form-data\">\n";

    my %libs = $this->get_all_libraries();
    my $lib = $this->get_library();

    if ($this->{input}{file}) {
	$out .= $this->do_upload($lib->id());
    }

    if (!$lib) {
	$out .= "
<p>Select a library to upload to:<br>
<select name=\"lib\">\n";
	foreach my $site (keys %libs) {
	    foreach my $lib (@{$libs{$site}}) {
		$out .= "<option value=\"$lib->{page_id}\">$site $lib->{title}</options>\n";
	    }
	}
	$out .= "</select>\n</p>\n";
    }
    else {
	my $libid = $lib->id();
	$out .= "
<input type=\"hidden\" name=\"lib\" value=\"$libid\">
<p>Select a destination object to upload to:<br>
<select name=\"cobj\">
<option value=\"0\">(new file)</option>\n";
	my @c = $lib->fetch_all_content_objs();
	foreach my $content (@c) {
	    my $title = $content->get("content","name");
	    my $id = $content->id();
	    $out .= "<option value=\"$id\">$title</options>\n";
	}
	$out .= "</select>\n</p>\n";

	$out .= "<p>Upload file:<br>
<input name=\"file\" type=\"file\" size=\"60\"></p>\n";
    }
	$out .= "<p><input type=\"submit\" value=\"Submit\">
<input type=\"button\" value=\"Cancel\" onclick=\"javascript:history.back()\"></p>
</form>\n";

    return $out;
}


##### FIXME : check the content attributes to determine 
##### where to upload the file.

sub do_upload {
    my ($this,$library) = (@_);
    my $out;
    my $in = new ExSite::Input;
    my $data = $in->post();
    my ($cobj,$lib);

    # fetch file

    my $f = $in->fetch_file("file");
    my ($filename,undef) = split /$config{form}{sepchar}/, $f, 2;

    # find content object to hold this file

    if ($data->{cobj}) {
	$cobj = new ExSite::Content(id=>$data->{cobj});
	if (! $cobj->owns()) { 
	    return $this->error("permission denied");
	}
    }
    if ($data->{lib}) {
	if ($cobj && $cobj->get("content","page_id") != $data->{lib}) {
	    return $this->error("permission denied: invalid library");
	}
	$lib = $this->get_library();
	if (! $lib) {
	    return $this->error("nowhere to upload to!");
	}
	if (! $lib->owns()) {
	    return $this->error("permission denied");
	}
    }
    else {
	return $this->error("nowhere to upload to!");
    }
    $this->info("Uploading to library $data->{lib}, content object $data->{cobj}");

    $this->info("Filename: $filename");
    my $stat;
    if ($lib && $cobj) {
	# updating an existing content object
	$stat = $cobj->update({
	    content_id=>$data->{cobj},
	    fdata=>$f,
	    mime_type=>&MimeType($filename),
	    note=>"installed by Modules::Upload",
	});
	if (! $stat) {
	    $this->error($cobj->fetch_diagnostics("error"));
	}
    }
    elsif ($lib) {
	# updating an existing content object
	$cobj = new ExSite::Content();
	$stat = $cobj->make({
	    page_id=>$lib->id(),
	    name=>$filename,
	    type=>"editorial",
	},
	{
	    fdata=>$f,
	    mime_type=>&MimeType($filename),
	    note=>"installed by Modules::Upload",
	});
	if (! $stat) {
	    $this->error($cobj->fetch_diagnostics("error"));
	}
    }
    if ($stat) { 
	return "<span class=\"help\">Successfully installed $filename $stat.</span>";
    }
    else {
	return $this->show_diagnostics("info","html").
	    $this->show_diagnostics("warn","html").
	    $this->show_diagnostics("error","html");
    }
}

1;

