#!/usr/bin/perl

package Modules::Zine;

#==========================================================================
# Zine : an e-Zine management system.
#
# For our purposes, an e-Zine is simply a collection of articles,
# along with the tools to link them together into indexes, search, sort, 
# and add comments/feedback.
#
# This package performs general management of article collections, including:
#    - formatting for display in HTML or plain text formats, 
#    - generating indexes for browsing large numbers of articles
#    - generating forms for inserting/updating articles
#    - processing article forms
#    - handling article attachments
#
#==========================================================================

=pod

Features:

- online books with unlimited chapters, with optional comments
- e-Zines with unlimited articles, with optional comments
- Discussion Forums
- Blogs, with optional comments
- PhotoBlogs, with optional comments
- Wikis, with cross-linked articles and optional comments
- comments can be for members only, or for the public
- flexible indexing (threaded, sorted, paged, by date, gallery-style, and more)
- automatic aging of articles
- add images/attachments to any article, with captions & automatic thumbnailing
- flexible CSS-driven formatting
- easy article handling through website front-end
- advanced e-Zine management through control panel back-end
- tamper-proof web forms


TODO

brief format archive indexes
webservice enabling
zine security - config flags to test site ownership of any article
zine security - secure zines do not publish
status flags, archiving
versioning??
moderating
notifications and subscriptions
pagination for large limited indexes
top nav, bottom nav
allow bare ID, bare zine type, or bare article name to be passed to write
view switcher (eg. to gallery or photoindex) on paged indexes
wiki QA

search, search in zine
built-in login/auth



=cut

use strict;
use ExSite::Config;
use ExSite::Report;
use ExSite::Misc;
use ExSite::Util;
use ExSite::Mail;
use ExSite::HTML;
use ExSite::Crypt;
use ExSite::Image;
use ExSite::Time;
use ExSite::Tree;
use Modules::BaseDCD;

use vars qw(@ISA %formtype %required %comment_label %helplabel %article_type);
@ISA = qw(Modules::BaseDCD);

# fields to prompt for on article forms

%formtype = (
	     'e-zine' => ["title","subtitle","summary","about"],
	     book => ["author","title","subtitle","summary","body","content_type","about","sortkey","picture","caption"],
	     forum =>  ["title","subtitle","summary","sortkey"],
	     comment =>  ["author","title","body"],
	     blog =>  ["title","subtitle","summary","about","picture","caption"],
	     wiki =>  ["title","subtitle","summary","about","picture","caption"],
	     photoblog =>  ["title","subtitle","summary","about"],
	     article =>  ["author","title","subtitle","summary","body","content_type","about","sortkey","picture","caption"],
	     article_wiki =>  ["name","author","title","body","content_type","picture","caption"],
	     article_blog =>  ["title","body","about","picture","caption","status"],
	     article_photoblog =>  ["author","title","picture","caption"],
	     );

# required fields on article forms

%required = (
	     'e-zine' => ["title"],
	     book => ["title"],
	     forum =>  ["title"],
	     comment =>  ["title","body"],
	     blog =>  ["title"],
	     wiki =>  ["title"],
	     photoblog =>  ["title"],
	     article =>  ["title"],
	     article_wiki =>  ["name","title","body"],
	     article_blog =>  ["title"],
	     article_photoblog =>  ["picture"],
	     );

%article_type = (
		 'e-zine' => "e-Zine",
		 book => "Book",
		 forum =>  "Forum",
		 comment =>  "Comment",
		 blog =>  "Blog",
		 wiki =>  "Wiki",
		 photoblog => "Photo Blog",
		 article =>  "Article",
		 article_wiki =>  "Wiki Article",
		 article_blog =>  "Blog Article",
		 article_photoblog =>  "Image",
		 );

%comment_label = (
		  author => "From",
		  body => "Comment",
		  title => "Subject",
		  );

sub write {
    my ($this,$options) = @_;
    my (%opt,$in,$id,$cmd,$n);
    
    # allow bare ID, bare zine type, or bare article name to be passed

    %opt = &DecodeString($options);
    $in = $this->{input};
    $cmd = $in->{zine} || $opt{zine};
    $id = $in->{aid} || $opt{aid};
    # variable n = number of articles to show for new articles, latest comments
    $n = $in->{n} || $opt{n};
    if ($id) { $this->fetch($id); }
	
    # modified to pass id and n / moved to top of write function
    if ($cmd eq "newarticles") {
	return $this->index_latest_articles($id,$n);
    }
    elsif ($cmd eq "newcomments") {
	return $this->index_latest_comments($id,$n);
    }	
    elsif ($cmd eq "toggle") {
	return $this->index_toggle("article",$id,$n);
    }	
	
    if ($cmd eq "submit") {
	my @err = $this->doform();
	if (@err) {
	    return "<h3>Problem:</h3>\n<p>The form could not be processed because:<br>".join("<br>\n",@err)."</p><p>[<a href=\"javascript:history.back()\">Back</a>]</p>\n";
	}
	if ($id) {
	    return $this->show;
	}
	else {
	    return $this->topview_admin;
	}
    }
    elsif ($id) {
	if ($cmd eq "index") {
	    return $this->index;
	}
	elsif ($cmd eq "new") {
	    return $this->newmsg;
	}
	elsif ($cmd eq "reply") {
	    return $this->reply;
	}
	elsif ($cmd eq "send") {
	    return $this->send;
	}
	elsif ($cmd eq "post") {
	    return $this->post;
	}
	elsif ($cmd =~ /approve|reject/) {
	    return $this->status($cmd);
	}
	elsif ($cmd eq "newarticle") {
#	    return "<h2>New ".$this->reftype." article</h2>\n".
#		$this->makeform({type=>"article"},$this->{article});
	    return $this->makeform({type=>"article"},$this->{article});
	}
	elsif ($cmd eq "edit") {
	    return "<h2>Edit ".$this->reftype." article</h2>\n".
		$this->makeform($this->{article});
	}
	elsif ($cmd eq "conf") {
	    return $this->confform();
	}
	elsif ($cmd eq "del") {
	    return $this->delete();
	}
	else {
	    return $this->show;
	}
    }
	
    return $this->topview($options);
}

sub ioctl {
    my ($this) = shift;
    $_ = shift;

    if (/ControlPanel/) {
	return \&ctrl_panel;
    }
    elsif (/isRestricted/) {
	return $config{Zine}{isRestricted};
    }
    elsif (/isService/) {
	return $config{Zine}{isService};
    }
    elsif (/Publish/) {
	return \&publish;
    }
}

sub ctrl_panel {
    my $this = shift;
    my $out;
    # insert some style rules for better formatting in control panels
    $out .= "<link REL=\"stylesheet\" TYPE=\"text/css\" HREF=\"$config{server}{HTMLpath}/_Modules/Zine/zine.css\">\n";
    my $in = $this->{input};
    $_ = $in->{zadm};
    $this->{admin} = 1;  # set admin flag to add admin tools


    # setup tabs

    my $url = "$config{server}{CGIpath}/ctrl-panel.cgi/Zine";

    my %tab = (
	       "" =>          {url=>$url,label=>"All Zines"},
	       "liste-zine" =>      {url=>"$url?zadm=liste-zine",label=>"E-Zines"},
	       "listbook" =>      {url=>"$url?zadm=listbook",label=>"Books"},
	       "listblog" =>      {url=>"$url?zadm=listblog",label=>"Blogs"},
	       "listforum" =>     {url=>"$url?zadm=listforum",label=>"Forums"},
	       "listwiki" =>      {url=>"$url?zadm=listwiki",label=>"Wikis"},
	       "listphotoblog" => {url=>"$url?zadm=listphotoblog",label=>"Photoblogs"},
	       "publish" =>   {url=>"$url?zadm=publish",label=>"Publish Images"},

	       );
    my @taborder = ("", "liste-zine", "listbook", "listblog", "listforum", 
		    "listwiki", "listphotoblog", "publish" );
    my @tabs;
    foreach my $t (@taborder) { push @tabs, $tab{$t}; }

    my $thistab = 0;
    foreach my $tabname (@taborder) { 
	last if ($tabname eq $_);
	$thistab++;
    }
#e-zine|book|forum|news|blog|wiki|photoblog|article|comment
    # build pane content

    if ($in->{zine}) { 
	$out .= $this->write; 
    }
    elsif (/list/) {
	/^list(.*)$/;
	my $type = $1;
	$out .= $this->topview_admin($type);
	my $url = $this->link(zadm=>"new$type");
	$out .= &ExSite::HTML::Button(url=>$url,label=>"Create a new $type");
    }
    elsif (/new/) {
	/^new(.*)$/;
	my $type = $1;
	if ($type eq "raw") { $type = "article"; }
	$out .= $this->makeform({type=>$type});
    }
    elsif (/publish/) {
	$out .= "<h3>Publishing Images</h3>
<iframe height=\"300\" width=\"600\" src=\"$config{server}{CGIpath}/publish.cgi/Zine?module=Zine\">
<a href=\"$config{server}{CGIpath}/publish.cgi/Zine?module=Zine\" target=\"publish\">Click here to publish images.</a>
</iframe>\n";
    }
    elsif (/search/) {
	$out .= $share{DB}->make(table=>"article",search=>1);
    }
    else {
        $out .= $this->topview_admin;
	#my $url = $this->link(zadm=>"newraw");
	my $url = "$config{server}{CGIpath}/webdb.cgi?tab=article&action=new";
	$out .= &ExSite::HTML::Button(url=>$url,label=>"Create a custom Zine");
    }

    return &ExSite::HTML::TabBox(
				 tabs=>\@tabs,
				 thistab=>$thistab,
				 fill_tabbar=>1,
				 pane=>$out,
				 );
}

#### publishing tools

sub publish {
    my $this = shift;
    my $db = $share{DB};
    my ($out,@zines);
    if ($this->{input}{aid}) {
	$this->fetch($this->{input}{aid});
	if ($this->{article}{owner} == $db->my_uid) {
	    if ($this->{article}{type} !~ /article|comment/) {
		return $this->publish_zine($this->{article});
	    }
	    else {
		return $this->publish_article($this->{article});
	    }
	}
    }
    else {
	$out = "<h2>Publishing Zines</h2>\n";
	if ($db->is_admin) {
	    @zines = $db->fetch_match("article",{status=>"active",
						 reference=>0,
						 owner=>0});
	    foreach my $z (@zines) { $out .= $this->publish_zine($z); }
	}
	@zines = $db->fetch_match("article",{status=>"active",
					     reference=>0,
					     owner=>$db->my_uid});
	foreach my $z (@zines) { $out .= $this->publish_zine($z); }
    }
    return $out;
}

sub publish_zine {
    my ($this,$zine) = @_;
    if ($zine->{privacy} =~ /members/) {
	return "<p>$zine->{title} is private, and cannot be published.</p>\n";
    }
    my $db = $share{DB};
    my $out = "<h4>Publishing $zine->{title}</h4>\n";
    $out .= $this->publish_article($zine);
    my @articles = $db->fetch_match("article",{status=>"active",
					    reference=>$zine->{article_id}} );
    foreach my $art (@articles) { 
	$out .= $this->publish_article($art);
    }
    return $out;
}

sub publish_article {
    my ($this,$art) = @_;
    return if ($art->{type} eq "comment");
    my $out;
    my $db = $share{DB};
    if ($art->{picture}) {
	$this->set($art);
	my $zpath = $this->diskpath;
	my $mkdir = $this->mkdir($zpath);
	if (! $mkdir) {
	    return $this->show_diagnostics("error","html");
	}
	$out .= $mkdir;
	my $img = new ExSite::Image($art->{picture});
	my $picpath = "$zpath/$art->{article_id}_$img->{filename}";
	$out .= $this->publish_file("picture",$picpath,$img);
	if ($art->{thumb}) {
	    $img->setup($art->{thumb});
	    $picpath = "$zpath/$art->{article_id}T_$img->{filename}";
	    $out .= $this->publish_file("picture",$picpath,$img);
	}
    }
    my @articles = $db->fetch_match("article",{status=>"active",
					       parent_id=>$art->{article_id}} );
    foreach my $article (@articles) { 
	$out .= $this->publish_article($article);
    }
    return $out;
}

sub publish_file {
    my ($this,$col,$file,$img) = @_;
    my $out;
    #return "<p>Zine::publish: permission denied</p>\n" if (! $this->{admin});
    $file =~ /^(((\/[\w-]+)+)\/([\w-]+\.[\w]+))$/;
    my $secure_filename = $1;
    $out .= "Writing $secure_filename... ";
    if (open OUTFILE, ">$secure_filename") {
	print OUTFILE $img->get;
	close OUTFILE;
	$out .= "succeeded.<br>\n";
    }
    else {
	$this->error("Zine::publish: publish $secure_filename failed: $!");
	$out .= "<SPAN CLASS=error>FAILED: $!</SPAN><br>\n";
    }
    return $out;
}

sub mkdir {
    my ($this,$dir) = @_;
    if (! $this->{dircache}) { $this->{dircache} = {}; }
    return $this->{dircache}{$dir} if (exists $this->{dircache}{$dir});
    my $out = " ";  # dummy return value
    $dir =~ /(\/[\w-\/]+)/;
    my $secure_dir = $1;
    if (! $secure_dir) { 
	$this->error("mkdir: invalid directory: $dir");
	return undef;
    }
    my @path = split(/\//,$secure_dir);
    shift @path; # first one is a dummy
    my $path;
    foreach my $dir (@path) {
	$path .= "/$dir";
	next if (-e "$path");
	$out .= "<P>Creating Zine directory $dir... ";
	if (! mkdir "$path") {
	    $this->error("mkdir $dir: failed: $!");
	    #$this->error("mkdir $path: failed: $!");
	    $this->{dircache}{$dir} = undef;
	    return undef;
	}
	else {
	    $this->info("created site directory $dir");
	    $out .= "succeeded.</P>\n";
	    # add blank index file, to prevent snooping
	    my $file = "$secure_dir/index.html";
	    if (open OUTFILE, ">$file") {
		print OUTFILE "<html></html>\n";
		close OUTFILE;
	    }
	    $this->{dircache}{$dir} = " ";
	}
    }
    return $out;
}

# publish methods, above, require special priveleges that only the publish
# program possesses.  If we are not in the publish program, we have to call
# these methods indirectly as follows.

sub run_publisher {
    my ($this,$article) = @_;
    my $out;
    if ($article->{picture}) {
	my $query_save = $ENV{QUERY_STRING};
	$ENV{QUERY_STRING} = "module=PhotoForum&aid=$article->{article_id}";
	$out = `$config{server}{CGIroot}/publish.cgi`;
	$ENV{QUERY_STRING} = $query_save;
    }
    return $out;
}

# find an article attribute
# looks in the current article, 
#    then the reference article(s)
#    then the configuration file

sub find { 
   my ($this,$attribute,$class) = @_;

    # look in the current article
    
    if ($this->{article}{$attribute}) {
	return $this->{article}{$attribute};
    }

    # look through the reference articles

    my $refcol = "reference";
    my $ref = $this->{article}{$refcol};
    while ($ref) {
	my $rart = $share{DB}->fetch("article",$ref);
	if ($rart->{$attribute}) {
	    return $rart->{$attribute};
	}
	$ref = $rart->{$refcol};
    }

    # check the configuration file

    my $reftype = $this->reftype;
    if (! $class) { $class = $this->typeclass; }
    if ($config{Zine}{$reftype}{$class}{$attribute}) {
	return $config{Zine}{$reftype}{$class}{$attribute};
    }

    return undef;
}

# indexlimit: the maximum number of items that will be displayed in an index

sub indexlimit {
    my ($this,$class) = @_;
    # custom article setting
    my $limit = $this->find("indexlimit",$class);
    return $limit if ($limit);
    if (! $class) { $class = $this->typeclass; }
    return 10 if ($class eq "article");
    return 25; # comments
}

# indexage: the maximum age of items that will be displayed in an index
#           NB: indexage is ignored if 0; ie. age is unlimited.

sub indexage {
    my ($this,$class) = @_;
    # custom article setting
    my $age = $this->find("indexage",$class);
    return $age if ($age);
    if (! $class) { $class = $this->typeclass; }
    return 14 if ($class eq "article");
    return 30; # comments
}

# indextype: the rules for sorting the index

sub indextype {
    my ($this,$class) = @_;
    # custom article setting
    my $type;
    if ($class eq "article" || ($this->{article}{type} eq "forum")) {
	$type = $this->find("indextype",$class);
    }
    return $type if ($type);
    if (! $class) { $class = $this->typeclass; }
    return "threaded" if ($class eq "comment");
    my $reftype= $this->reftype;
    #return "reverse-date" if ($reftype eq "blog");
    return "date" if ($reftype eq "blog");
    return "paged" if ($reftype eq "photoblog");
    return "wiki" if ($reftype eq "wiki");
    return "date" if ($this->{article}{type} eq "e-zine");
    return "sorted"; # if ($class eq "article");
}

# indexformat: the rules for formatting the index

sub indexformat {
    my ($this,$class) = @_;
    # custom article setting
    my $fmt = $this->find("indexformat",$class);
    return $fmt if ($fmt);
    if (! $class) { $class = $this->typeclass; }
    return "full" if ($class eq "comment");
    return "teaser" if ($this->{article}{type} eq "e-zine");
    return "photoindex" if ($this->reftype eq "photoblog" && $class eq "article");
    return "full" if ($this->reftype =~ "blog");
    return "sorted" if ($class eq "article");
    return "full"; # comments
}

# comment_policy: whether or not comments are accepted

sub comment_policy {
    my ($this,$class) = @_;
    # custom article setting
    my $com = $this->find("comment_policy",$class);
    return $com if ($com);
    # configuration setting
    my $ref = $this->reftype;
    my $access = $share{DB}->authorize;
    if ($access) {
	$com = $config{Zine}{$ref}{comment_policy}{member};
    }
    else {
	$com = $config{Zine}{$ref}{comment_policy}{public};
    }
    return $com if ($com);
    # comments are forbidden by default, unless this is a forum or comment
    if ($access && $this->{article}{type} =~ /forum|comment/) {
	return "allowed";
    }
    return "forbidden";  
}

# build links to articles

# index:  list articles from a certain point down
# There are two modes:
# - article: indexed articles identified using parent_id field
# - comment: indexed articles identitfied using reference field
#
# There are 3 main formats:
# - threaded (related articles grouped using indentation)
# - list (articles listed in some sort order)
# - paged (next/previous links)

sub index {
    my ($this,$mode,%opt) = @_;

    # save current article
    my $currart = $this->{article};

    my $out;
    my $indextype = $opt{indextype} || $this->indextype($mode);
    
    if ($opt{archive}) {
	# simple list of articles
	$out = $this->index_list($mode,"date",%opt);
    }
    elsif ($indextype eq "threaded") {
	my %match = ( reference=>$this->{article_id}, 
		      type=>$mode,
		      status=>"active" );
	if ($opt{type}) { $match{type} = $opt{type}; }

	my @art = $share{DB}->fetch_match("article",\%match,"ctime");
	$out = $this->index_threaded($mode,\%opt,@art);
    }
    elsif ($indextype eq "wiki") {
	return $this->index_wiki(\%opt);
    }
    elsif ($indextype eq "paged" && $mode eq "article") {
	my @art = $share{DB}->fetch_match("article",
					  {reference=> ($this->{article}{reference} || $this->{article}{article_id}) },
					  {"ctime"=>"desc"},
					  );
	if (@art > 0) {
	    my ($next,$data);
	    if ($this->{article}{type} =~ /article|comment/) {
		# find this article in the list
		for (my $iart = 0; $iart <= $#art; $iart++) {
		    if ($this->{article_id} eq $art[$iart]{article_id}) {
			$data = $art[$iart];
			$iart++;
			if ($iart <= $#art) {
			    $next = $art[$iart]{article_id};
			}
			last;
		    }
		}
	    }
	    else {
		$this->set($art[0]);
		$out .= $this->show_article;
		$out .= $this->tools($art[0]);
		$data = $art[0];
		if ($#art > 0) { 
		    $next = $art[1]{article_id};
		}
	    }
	    # link to next
	    $this->fetch($data->{parent_id});
	    if ($next) {
		my $type = $data->{type};
		if ($type eq "article" && $this->type eq "blog") {
		    $type .= "_blog";
		}
		elsif ($type eq "article" && $this->type eq "photoblog") {
		    $type .= "_photoblog";
		}
		elsif ($type eq "article" && $this->type eq "wiki") {
		    $type .= "_wiki";
		}
		my $url = $this->link(zine=>"show",aid=>$next);
		$out .= "<p><a href=\"$url\">Next $article_type{$type}</a>&nbsp;&rArr;</p>\n";
	    }
	}
	else {
	    $out .= "<p>(End of ".$this->title.".)</p>\n";
	}
    }
    else {
	# simple list of articles
	$out = $this->index_list($mode,$indextype,%opt);
    }

    # restore current article
    $this->set($currart);

    return $out;
}

# A list index shows the articles sequentially.  
#
# There are a few formats:
# - photoindex : displays in a table, using a thumbnailed photo
# - gallery    : checkerboard of thumbnails
# - table      : simple table of title, author, date
# - list       : a sequence of paragraphs

sub index_list {
    my ($this,$mode,$itype,%opt) = @_;

    my $out;
    my $indextype = $itype || $this->indextype($mode);
    my $indexlimit = $opt{indexlimit} || $this->indexlimit($mode);
    my $indexage = $opt{indexage} || $this->indexage($mode);
    my $indexformat = $opt{archive} ? "list" : 
	($opt{indexformat} || $this->indexformat($mode));
    my $comment_policy = $opt{comment_policy} || $this->comment_policy($mode);
	# display non-active and active articles if is_manager
    my %match;
    if ($mode eq "article") {
	%match = ( parent_id=>$this->{article_id} );
    }
    else {
	# comment
	%match = ( reference=>$this->{article_id} );
    }
    if (! $this->{admin} ) { $match{status} = "active"; }
    if ($opt{type}) { $match{type} = $opt{type}; }

    # tabular list of articles

    my @art;
    if ($indextype eq "thread list") {
	$match{thread_id} = 0;
	@art = $share{DB}->fetch_match("article",
				       \%match,
				       {"ctime"=>"desc"},
				       );
    }
    elsif ($indextype eq "sorted") {
	@art = $share{DB}->fetch_match("article",
				       \%match,
#				       {"sortkey"=>"asc"},
#					   {"sortkey"=>"asc","ctime"=>"desc"},
				       ["sortkey",{"ctime"=>"desc"}],
				       );
    }
    elsif ($indextype eq "reverse-date") {
	@art = $share{DB}->fetch_match("article",
				       \%match,
				       {"ctime"=>"asc"},
				       );
    }
    else { #if ($indextype eq "date") {
	@art = $share{DB}->fetch_match("article",
				       \%match,
				       {"ctime"=>"desc"},
				       );
    }

    my $archive;  # if set, means there are archived articles
    my $n = 0;
    if (@art > 0) {
	my $article_save = $this->{article};
	foreach my $art (@art) {
	    next if ($art->{type} eq "comment" and $mode eq "article");
	    next if ($art->{type} ne "comment" and $mode ne "article");
	    $this->set($art);
	    if ($indexage && ($this->age > $indexage) && !$opt{archive}) {
		$archive = 1;
		next;
		#last;
	    }
	    if ($indexformat eq "photoindex") {
		my $thumb = $this->thumb || "<img src=\"$config{server}{HTMLpath}/_Modules/Zine/nophoto.gif\" border=\"0\">";
		my $summary = $this->summarize("teaser");
		my $url = $this->link(zine=>"show",aid=>$this->{article_id});
		$out .= "<tr><td width=\"100\" align=\"center\" class=\"ZineThumb\"><a href=\"$url\">$thumb</a></td><td>$summary</td></tr>\n";
	    }
	    elsif ($indexformat eq "gallery") {
		my $thumb = $this->thumb || "<img src=\"$config{server}{HTMLpath}/_Modules/Zine/nophoto.gif\" height=\"50\" width=\"50\" border=\"0\">"; 
		my $url = $this->link(zine=>"show",aid=>$this->{article_id});
		$out .= "<td align=\"center\" class=\"ZineThumb\"><a href=\"$url\">$thumb</a></td>\n";
	    }
	    elsif ($indexformat eq "table") {
		my $title = $this->title || "(no title)";
		my $author = $this->author;
		my $date = $this->timestamp;
		my $url = $this->link(zine=>"show",aid=>$this->{article_id});
		$out .= "<tr><td><a href=\"$url\">$title</a></td><td>$author</td><td>$date</td></tr>\n";
	    }
	    else {  # simple list
		$out .= "<p>\n";
		$out .= $this->summarize($indexformat);
		if ($indexformat eq "full" || $this->{admin}) {
		    #$out .= $this->index_summary(); # already called in summarize()
		    $out .= $this->tools($art);
		}
		$out .= "</p>\n";
	    }
	    $n++;
	    if ($n == $indexlimit && ! $opt{archive}) {
		$archive = 1;
		last;
	    }
	    if ($indexformat eq "gallery") {
		my $gwidth = $config{Zine}{gallery_width} || 4;
		if (($n % $gwidth) == 0) {
		    $out .= "</tr><tr>\n";
		}
	    }
	}
	$this->set($article_save);
    }
    if ($out) {
	if ($indexformat eq "photoindex") {
	    $out = "<table class=\"ZineIndex\">\n$out";
	}
	elsif ($indexformat eq "gallery") {
	    $out = "<table class=\"ZineIndex\"><tr>\n$out</tr>";
	}
	elsif ($indexformat eq "table") {
	    $out = "<table class=\"ZineIndex\">\n<tr><th colspan=\"3\">Archives</th></tr>\n<tr><th>Title</th><th>Author</th><th>Date Posted</th></tr>\n$out";
	}
	if ($indexformat =~ /photoindex|gallery|table/) {
	    $out .= "</table>\n";
	}
    }
    if ($archive) {
	my $archive_url = $this->link(archive=>1,zine=>"show");
	my $archive_anchor = $config{Zine}{archive_anchor} || "Show archived articles";
	$out .= "<p><a href=\"$archive_url\">$archive_anchor</a>.</p>\n";
    }
    elsif ($opt{archive} && $n > 0) {
	my $archive_url = $this->link(archive=>undef);
	my $current_anchor = $config{Zine}{current_anchor} || "Show current articles";
	$out .= "<p><a href=\"$archive_url\">Show current articles</a>.</p>\n";
    }
    return $out;
}

sub index_threaded_old {
    my ($this,$mode,$opt,@art) = @_;
    my $indexlimit = $opt->{indexlimit} || $this->indexlimit($mode);  ###########recursive
    my $indexage = $opt->{indexage} || $this->indexage($mode);
    my $indexformat = $opt->{indexformat} || $this->indexformat($mode);
    my $comment_policy = $opt->{comment_policy} || $this->comment_policy($mode);
    my $out;
    #foreach my $art (@art) {       # date order            ###### CONFIG THIS
    if (! exists $this->{thread_count}) { $this->{thread_count} = 0; }
    my $archive;
    while (my $art = pop @art) {    # reverse date order
	next if ($art->{type} eq "comment" and $mode eq "article");
	next if ($art->{type} ne "comment" and $mode ne "article");
	$this->set($art);
	my $fmt = $indexformat;
	if ($indexage && ($this->age > $indexage)) {
	    if (! $this->{article}{thread} && 
		$this->{thread_count} > $indexlimit) {
		# do not show old threads if we're past the limit
		$archive = 1;
		next;
	    }
	    # otherwise, demote format to brief
	    $fmt = "brief";
	}

	if ($config{Zine}{shrink_anonymous_comments} && ! $art->{owner}) {
	    $fmt = "brief";
	}

	$this->{thread_count}++;
	if ($this->{thread_count} > $indexlimit) {
	    $fmt = "brief";
	}
	$out .= "<p>".$this->summarize($fmt)."</p>\n";
	if ($fmt eq "full") {
	    $out .= $this->tools($art);
	}
	my @children = $share{DB}->fetch_match("article",
					       {parent_id=>$art->{article_id}},
					       ### status
					       ["ctime"]);
	if (@children) {
	    $out .= "<ul>\n";
	    $out .= $this->index_threaded($mode,$opt,@children);
	    $out .= "</ul>\n";
	}
    }
    if ($archive) {
	my $url = $this->link(archive=>1);
	$out .= "<p><a href=\"$url\">Show archives</a>.</p>\n";
    }
    return $out;
}

sub index_threaded {
    my ($this,$mode,$opt,@art) = @_;
    my $indexlimit = $opt->{indexlimit} || $this->indexlimit($mode);  ###########recursive
    my $indexage = $opt->{indexage} || $this->indexage($mode);
    my $indexformat = $opt->{indexformat} || $this->indexformat($mode);
    my $comment_policy = $opt->{comment_policy} || $this->comment_policy($mode);
    my $out;

    # build a tree from the articles
    my $tree = new ExSite::Tree;
    $tree->addnodes(\@art,"article_id","parent_id",[$this->{article_id}]);

    my $archive;
    $this->{articlecount} = 0;

    # prune uninteresting articles from the thread tree
    foreach my $art ($tree->get_topnodes_data()) {
	# drop comments in article indexes
	if ($art->{type} eq "comment" and $mode eq "article") {
	    $tree->delnode($art->{article_id});
	}
	# drop articles in comment indexes
	if ($art->{type} ne "comment" and $mode ne "article") {
	    $tree->delnode($art->{article_id});
	}
	# prune old threads
	if ($this->age_of_thread($tree,$art) > $indexage) {
	    $tree->delnode($art->{article_id});
	    $archive = 1;
	}
    }

    # display remaining threads
    my @articles = reverse($tree->get_topnodes_data());

    $out .= $this->index_thread($indexformat,$indexlimit,$indexage,
				$tree,@articles);

    if ($archive) {
	# there appear to be some threads that are turned off due to age
	my $url = $this->link(archive=>1);
	$out .= "<p><a href=\"$url\">Show archives</a>.</p>\n";
    }
    return $out;
}

sub index_thread {
    my ($this,$format,$limit,$age,$tree,@articles) = @_;
    my $out;
    foreach my $art (@articles) {
	my $fmt = $format;
	$this->{articlecount}++;
	$this->set($art);
	if ($fmt ne "brief") {
	    if ($this->{articlecount} > $limit) {
		$fmt = "brief";
	    }
	    elsif ($age && ($this->age > $age)) {
		$fmt = "brief";
	    }
	    elsif ($config{Zine}{shrink_anonymous_comments} && ! $art->{owner}) {
		$fmt = "brief";
	    }
	}
	$out .= "<p>".$this->summarize($fmt)."</p>\n";
	if ($fmt eq "full") {
	    $out .= $this->tools($art);
	}
	my @children = $tree->get_child_data($art->{article_id});
	if (@children > 0) {
	    $out .= "<ul>\n";
	    $out .= $this->index_thread($format,$limit,$age,$tree,@children);
	    $out .= "</ul>\n";
	}
    }
    return $out;
}

# the age of a thread is the age of the youngest article in the thread

sub age_of_thread {
    my ($this,$tree,$article) = @_;
    my @nodelist = ($tree->getnode_data($article->{article_id}));
    my $age = $nodelist[0]{mtime};
    while (@nodelist > 0) {
	my $art = shift @nodelist;
	$age = $art->{mtime} < $age ? $art->{mtime} : $age;
	push @nodelist,$tree->get_child_data($art->{article_id});
    }
    # convert to a day count
    my $time = new ExSite::Time;
    my $t1 = $time->write("unix_timestamp"); # current time
    $time->set($age,"sql_timestamp");
    my $t2 = $time->write("unix_timestamp"); # thread time
    return ($t1-$t2)/86400;
}

sub index_wiki {
    my ($this,$opt) = @_;
    my $body = $this->body;
    my @refs;
    while ($body =~ /\[([^\]])\]/) {
	push @refs,$1;
	$body =~ s/\[([^\]])\]//g;
    }
    my ($index,$out);
    foreach my $ref (sort @refs) {
	my $url = $this->link(goto=>$ref);
	$index .= "<li><a href=\"$url\">$ref</a></li>\n";
    }
    if ($index) { 
	$out = "<h4>Related articles:</h4>\n<ul>\n$index\n</ul>\n";
    }

    # TODO: should we also show the articles that link _to_ us?

    return $out;
}

sub summarize {
    my ($this,$format) = @_;
    my $out;
    my $url = $this->link(aid=>$this->{article_id},zine=>"show");
    my $title = $this->title || "(no title)";
    if ($format eq "teaser") {
	my $subtitle = $this->show_data("subtitle");
	my $author = $this->author;
	my $body = $this->body;
	#---
	# disable oysters in teasers
	$body =~ s/<!--\&/<!-- \&/g;
	#---
	my $head = "<div class=\"ZineTeaserHeader\"><a href=\"$url\">$title</a>";
	if ($this->{article}{subtitle}) {
	    $head .= ", <em>$subtitle</em>";
	}
	$head .= "<br>";
	if ($author && $this->{article}{type} !~ /blog/ && $this->reftype() !~ /blog/) {
	    # don't show individual article authors in blogs
	    $head .= "<em>by $author</em> ";
	}
	if ($this->{article}{type} =~ /comment/ || $this->reftype() =~ /blog|e-zine/) {
	    # show article posting dates in comments, blogs, and e-zines only
	    $head .= $this->date;
	}
	$head .= "</div>\n";

	my $teaserlen = $config{Zine}{teaser_length} || 200;
	my $index_summary = $this->index_summary;
	my $summary = $this->{article}{summary};
	if (! $summary) {
	    # no summary - use an abbreviated form of the body
	    $summary = $body;
	    if (length($body) > $teaserlen) {
		my $pos = $teaserlen; 
		my $char;
		while (($pos < length($summary)) && 
		       ($char = substr($summary,$pos,1))) {
		    last if ($char =~ /\s/);
		    $pos++;
		}
		substr($summary,$pos) = "...";
		$summary =~ s/<.+?>//ig;
		if (! $index_summary) { 
		    $summary .= " (&rArr;&nbsp;<a href=\"$url\">more</a>)"; 
		}
	    }
	}
	if ($index_summary) { $index_summary = "($index_summary)"; }
	$out .= "$head\n<div class=\"ZineTeaser\">$summary<br>$index_summary</div>\n";
    }
    elsif ($format eq "full") {
	$_ = $this->type;
	if (/comment/) { $out .= $this->show_comment; }
	else { $out .= $this->show_article; }
    }
    else { #if ($format eq "brief") {
	$out .= "<div class=\"ZineTeaser\"><a href=\"$url\">$title</a>";
	if ($this->{article}{type} eq "comment") {
	    my $date = $this->show_data("ctime");
	    my $author = $this->author;
	    $out .= " (posted on $date by $author)";
	}
	else {
	    my $summ = $this->show_data("summary");
	    my $author = $this->author;
	    if ($author) { $out .= ", <i>by $author</i>" }
	    if ($summ) { $out .= "<br><font size=-1>$summ</font>"; }
	}
	$out .= "</div>";
    }
    return $out;
}

sub show_teaser {
    my $this = shift;
    my $author = $this->author;
    my $body = $this->body;
    my $out = "<p><div class=\"Zine\"><b>".$this->title."</b>";
    if ($this->{subtitle}) {
	$out .= ", <em>".$this->show_data("subtitle")."</em>";
    }
    $out .= "<br>";
    if ($this->{article}{type} !~ /blog/) {
	$out .= "<em>by $author</em> ";
    }
    $out .= $this->date;
    my $teaserlen = $config{Zine}{teaser_length} || 200;
    if (length($body) > $teaserlen) {
	my $pos = $teaserlen; 
	my $char;
	while (($pos < length($body)) && ($char = substr($body,$pos,1))) {
	    last if ($char =~ /\s/);
	    $pos++;
	}
	substr($body,$pos) = "...";
    }
    $out .= "</div><br>\n$body</p>\n";
    return $out;
}

# not a real index, since we don't link to other articles, simply
# to ourself

sub index_summary {
    my $this = shift;
    my $nart = $share{DB}->count("article",{ parent_id=>$this->{article_id},
					     type=>"article",
					     status=>"active" } );
    my $ncom = $share{DB}->count("article",{ reference=>$this->{article_id},
					     type=>"comment",
					     status=>"active" } );
    my @counts;
    if ($nart) { push @counts, "$nart article".($nart > 1 ? "s" : ""); }
    if ($ncom) { push @counts, "$ncom comment".($ncom > 1 ? "s" : ""); }
    my $out = join ("; ",@counts);
    if ($out) { 
	my $url = $this->link(zine=>"show",aid=>$this->{article_id});
	return "<a href=\"$url\">$out</a>";
    }
    return "";
}

# index_latest: summarize recently added articles

sub index_latest {
    my ($this,$type,$id,$number) = @_;
    return undef if ($type !~ /^(article|comment)$/);
	# set $number if it was not passed in  
    if ( !defined $number ) {
		$number |= $config{Zine}{"new_".$type."s"} || 4;
	}
	#
    my ($out,@aid);
    if ($id) {
	@aid = ($id);
    }
    else {
	my @art = $this->our_zines();
	@aid = map { $_->{article_id} } @art;
    }
    my @latest = $share{DB}->custom_query("select * from article where reference in (".join(",",@aid).") and type=\"$type\" and status=\"active\" order by article_id desc limit $number");
    if (@latest > 0) {
	# conf file option to turn off header
	unless ( $config{Zine}{no_header} == 1 ) { 
		$out = "<div class=\"Zine\">\n<h1>Recent ".ucfirst($type)."s</h1>\n";
	}
	foreach my $a (@latest) {
	    $this->set($a);
	    my $title = $this->title();
	    my $auth = $this->author();
	    my $author = $auth ? "by $auth" : "";
	    my $reftitle = $this->reftitle();
	    my $date = $this->date;
	    my $url = $this->link(zine=>"show",aid=>$this->{article}{article_id});
	    my $rurl = $this->link(zine=>"show",aid=>$this->{reference}{article_id});
	    $out .= "<p class=\"ZineTeaser\"><a href=\"$url\" class=\"ZineTitle\">$title</a> $author $date in <a href=\"$rurl\" class=\"ZineRef\">$reftitle</a></p>\n";
	}
	$out .= "</div>\n";
    }
    return $out;
}

sub index_toggle {
    my ($this,$type,$id) = @_;
    return undef if ($type !~ /^(article|comment)$/);
    my ($out,@aid);
    if ($id) {
	@aid = ($id);
    }
    else {
	my @art = $this->our_zines();
	@aid = map { $_->{article_id} } @art;
    }
    my @latest = $share{DB}->custom_query("select * from article where reference in (".join(",",@aid).") and type=\"$type\" and status=\"active\" order by article_id desc");
    if (@latest > 0) {
	# conf file option to turn off header
		$out = <<"DONE";

<script language="JavaScript">
function toggle(link, div1id) {
	var div1 = document.getElementById(div1id);
	if (div1.style.display == 'none') {
		div1.style.display = 'block';
	} 
	else {
		div1.style.display = 'none';
	}
}

</script>
DONE
	foreach my $a (@latest) {
	    $this->set($a);
	    my $title = $this->title();
		my $body = $this->body;
	    my $auth = $this->author();
	    my $author = $auth ? "by $auth" : "";
	    my $reftitle = $this->reftitle();
	    my $time = $this->timestamp;
	    my $url = $this->link(zine=>"show",aid=>$this->{article}{article_id});
	    my $rurl = $this->link(zine=>"show",aid=>$this->{reference}{article_id});
	    $out .= "<p><div class=\"ZineTeaser\"><a onclick=\"toggle(this,$this->{article}{article_id});\" href=\"javascript:void(0);\">$title</a></div></p>\n";
		
		$out .= "<div class=\"ZineBody\" style=\"display: none;\" id=$this->{article}{article_id}><div id=msg>$body</div></div>";
	}
	$out .= "</div>\n";
    }
    return $out;
}



sub index_latest_articles {
    my ($this,$id,$number) = @_;
    return $this->index_latest("article",$id,$number);
}

sub index_latest_comments {
    my ($this,$id,$number) = @_;
    return $this->index_latest("comment",$id,$number);
}

# display a complete article

sub show {
    my $this = shift;
    my $out;
#    if (! $this->approve) {
#	return "<p class=error>Sorry, this article is not available for viewing.</p>\n";
#    }

    # show the article

    if ($this->{article}{type} =~ /comment/) {
	$out .= $this->show_comment;
    }
    elsif ($this->{article}{type} =~ /wiki/) {
	$out .= $this->show_wiki;
    }
    else {
	$out .= $this->show_article;
    }
    $out .= $this->tools($this->{article});

    my $footer;
    if ($this->{reference}) {
	if ($this->reftype =~ /e-zine|book|forum/) {
	    my $refurl = $this->link(zine=>"show",aid=>$this->{reference}{article_id});
	    $footer = "<p>[<a href=\"$refurl\">Back to \"".$this->reftitle."\"</a>]</p>\n";
	}
    }

    # show the regular article index
    $out .= $this->index("article",(archive=>$this->{input}{archive}));

    # show the index of comments

    $out .= $this->index("comment",(archive=>$this->{input}{archive}));

    # show the shortcuts out of here
    $out .= $footer;

    return $out;
}

sub show_article {
    my $this = shift;
    if (! $this->approve) {
	return "<p class=error>Sorry, this article is not available for viewing.</p>\n";
    }
    my $a = $this->{article};
    return $this->show_comment() if ($a->{type} eq "comment");

    # generic article formatting
    
    my $out = "<div class=\"Zine\">\n";
    my $title = $this->title;
    if ($a->{reference} || $a->{parent_id}) {
	$out .= "<h2>$title</h2>\n";
    }
    else {
	$out .= "<h1>$title</h1>\n";
    }
    if ($a->{subtitle}) {
	$out .= "<h3>".$this->show_data("subtitle")."</h3>\n";
    }
    if ($a->{type} =~ /book|article/ && $this->reftype =~ /book|article/) {
	my $author = $this->author;
	if ($author) {
	    $out .= "<h4>By $author</h4>\n";
	}
    }
    $out .= "\n";
    if ($a->{summary}) {
 	#$out .= "<blockquote>$a->{summary}</blockquote>\n";
	my ($align, $width);
 	$out .= "<table class=\"ZineSummary\"";
	if ($a->{article}{body}) { $out .= " class=\"ZineSummary\" align=\"right\" width=\"150\""; }
 	$out .= "><tr><td>".$this->show_data("summary")."</td></tr></table>\n";
    }
    my $body = $this->body;
    my $pic = $this->picture;
    if ($pic) {
	if (length $body < 1000) {
	    # short article; put picture at end
	    $body .= "<p>$pic</p>\n";
	}
	elsif (length $body < 4000) {
	    # medium-length article; put picture at top
	    # because it's safer to assume that the text will probably wrap 
	    # the whole image, and not mess up formatting of subsequent
	    # articles.
	    $body = "$pic\n$body";
	}
	else {
	    # long article; insert picture in mid article, but near top
	    my $pos = 600;
	    while (substr($body,$pos,1) ne " " && $pos < length $body) { 
		$pos++; 
	    }
	    substr($body,$pos,1) = " $pic";
	}
    }
    if ($body) {
	$out .= "<p>$body</p>";
    }
    $out .= $this->footer;
    $out .= "</div>\n";
    return $out;
}

sub footer {
    my $this = shift;
    my $out;
    if ($this->type =~ /article/) {
	my $date = ucfirst($this->date);
	$out = "\n$date<br>\n";
    }
    $out .= $this->show_data("about");
    if ($out) {
	return "<address>$out</address>\n";
    }
    return undef;
}

sub show_comment {
    my $this = shift;
    if (! $this->approve) {
	return "<p class=error>Sorry, this comment is not approved for viewing.</p>\n";
    }
    my ($author,$date,$body,$title,$out);
    $author = $this->author;
    $date = $this->date;
    $body = $this->body;
    $title = $this->title;
    $body = $this->body;
    $out = "<p><div class=\"ZineHeader\"><b>$title</b>, ";
    $out .= "<em>$date ";
    if ($this->{article}{type} eq "comment") {
	$out .= "by $author\n";
    }
    $out .= "</em></div>\n<div class=\"Zine\">$body</div></p>\n";
    return $out;
}

sub show_wiki {
    my ($this,$term) = @_;
    if (! $this->approve) {
	return "<p class=error>Sorry, this article is not available for viewing.</p>\n";
    }

    # remember current article
    my $currart = $this->{article};

    my $wiki_id;
    if ($this->{article}{type} eq "wiki") {
	$wiki_id = $this->{article}{article_id};
    }
    else {
	$wiki_id = $this->{article}{reference};
    }

    my $out;
    my $goto = $term || lc($this->{input}->{goto}) || "=start";

    if ($goto) {
	my @wart = $share{DB}->fetch_match("article",
					   {reference=>$wiki_id,
					    name=>"%$goto%",
					    status=>"active"},
					   "title");
	if (@wart == 0 && $goto =~ /s$/) {
	    # hmm, no hits
	    my $singular_goto = $goto;
	    $singular_goto =~ s/s$//;   # try singular
	    @wart = $share{DB}->fetch_match("article",
					    {reference=>$wiki_id,
					     name=>"%$singular_goto%",
					     status=>"active"},
					    "title");
	}
	foreach my $wart (@wart) {
	    $this->set($wart);
	    $out .= "<p style=\"border-bottom:1px solid black;\" align=\"right\">&nbsp;</p>\n";
	    $out .= "<h1 class=\"Zine\">$wart->{title}</h1>\n";
	    if ($wart->{subtitle}) {
		$out .= "<h3 class=\"Zine\">$wart->{subtitle}</h3>\n";
	    }
#	    if ($wart->{type} =~ /book|article/) {
#		my $author = $this->author($wart);
#		if ($author) {
#		    $out .= "<h4 class=\"Zine\">By $author</h3>\n";
#		}
#	    }
	    if ($wart->{summary}) {
		$out .= "<blockquote><i>$wart->{summary}</i></blockquote>\n";
	    }
	    my $body .= $this->body($wart);
	    $body =~ s/\[(.+?)\]/$this->wikilink($1)/egs;
	    $out .= "<p>$body</p>";
	    if ($wart->{about}) {
		$out .= "<br><p><i>".$this->show_data("about")."</i></p>\n";
	    }
	    $out .= $this->tools($wart);

	    $out .= $this->index("comment",(archive=>$this->{input}{archive}));

	}
	if (@wart == 0) {
	    $out .= "<p>No articles found for \"$goto\".</p>\n";
	}	    
    }

    # wiki search form

    $goto = lc($this->{input}->{goto});
    my $url = $this->link(goto=>undef);
    $out .= "<p><form method=\"POST\" action=\"$url\">
<input type=\"text\" name=\"goto\" size=\"20\" value=\"$goto\">
<input type=\"submit\" value=\" Go \">
</form></p>\n";

    # back link
    
    if ($this->{input}{goto}) {
	my $backurl = $this->link(zine=>"show",aid=>$this->{article}{reference},goto=>undef);
	$out .= "<p><a href=\"$backurl\">Back to top</a>.</p>\n";
    }

    # restore current article
    $this->set($currart);

    return $out;
}

sub wikilink {
    my ($this,$name) = @_;
    my $url = $this->link(goto=>$name);
    return "<a href=\"$url\">$name</a>";
}

sub tools {
    my ($this,$article) = @_;
    my ($out,$url,$ntool);
    if (! exists $this->{tools}) { $this->{tools} = {}; }
    my $aid = $article->{article_id};

    if ($article->{type} eq "comment" && $this->allow_comments) {
	$url = $this->link(zine=>"reply",aid=>$aid);
	$out .= "<td align=\"center\">\n<a href=\"$url\"><img src=\"$config{server}{HTMLpath}/_Modules/Zine/reply.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"reply\" title=\"reply to this\">";
	if ($config{Zine}{icon_labels}) { $out .= "<br>reply"; }
	$out .= "</a></td>\n";
	$ntool++;
    }
    #elsif ($a->{type} eq "article") {
    else {
	$url = $this->link(zine=>"new",aid=>$aid);
	if ($article->{type} =~ /article|forum/ && $this->allow_comments("article")) {
	    $out .= "<td align=\"center\">\n<a href=\"$url\"><img src=\"$config{server}{HTMLpath}/_Modules/Zine/comment.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"new comment\" title=\"new comment\">";
	    if ($config{Zine}{icon_labels}) { $out .= "<br>comment"; }
	    $out .= "</a></td>\n";
	    $ntool++;
	}
#	if (! $this->{tools}{print}) {
#	    $this->{tools}{print} = 1;
#	    $out .= "<td align=\"center\">\n<a href=\"javascript:print()\"><img src=\"$config{server}{HTMLpath}/_Modules/Zine/print.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"print\" title=\"print this $article->{type}\">";
#	    if ($config{Zine}{icon_labels}) { $out .= "<br>print"; }
#	    $out .= "</a></td>\n";
#	    $ntool++;
#	}

#	if (! $this->{admin}) {
#	    # cannot send an admin screen to a friend :(
#	    $url = $this->link(zine=>"send",aid=>$aid);
#	    $out .= "<a href=\"$url\"><img src=\"$config{server}{HTMLpath}/_Modules/Zine/email.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"email\" title=\"send this to a friend\"></a>&nbsp;\n";
#	    $ntool++;
#	}
    }

    if ($this->{admin}) {

	# new article

	if ($article->{type} !~ /comment|forum/) {
	    my $newart = 1;
	    if ($article->{type} eq "article") {
		# only show the newarticle link if this is a book
		my $ref = $share{DB}->fetch("article",$article->{reference});
		$newart = ($ref->{type} eq "book");
	    }
	    if ($newart) {
		$url = $this->link(zine=>"newarticle",aid=>$article->{article_id});
		$out .= "<td align=\"center\">\n<a href=\"$url\"><img src=\"$config{server}{HTMLpath}/_Modules/Zine/new.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"comment\" title=\"new article\">";
		if ($config{Zine}{icon_labels}) { $out .= "<br>new"; }
		$out .= "</a></td>\n";
		$ntool++;
	    }
	}

	# edit article

	$url = $this->link(zine=>"edit",aid=>$article->{article_id});
	$out .= "<td align=\"center\">\n<a href=\"$url\"><img src=\"$config{server}{HTMLpath}/_Modules/Zine/edit.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"edit\" title=\"edit\">";
	if ($config{Zine}{icon_labels}) { $out .= "<br>edit"; }
	$out .= "</a></td>\n";
	$ntool++;

	# configure article

	$url = $this->link(zine=>"conf",aid=>$article->{article_id});
	$out .= "<td align=\"center\">\n<a href=\"$url\"><img src=\"$config{server}{HTMLpath}/_Modules/Zine/tool.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"configure\" title=\"configure\">";
	if ($config{Zine}{icon_labels}) { $out .= "<br>conf"; }
	$out .= "</a></td>\n";
	$ntool++;

#	# copy article
#
#	$out .= "<td align=\"center\">\n<a href=\"$config{server}{CGIpath}/webdb.cgi?action=copy&tab=article&id=$article->{article_id}\"><img src=\"$config{server}{HTMLpath}/_Modules/Zine/copy.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"copy\" title=\"copy\">";
#	if ($config{Zine}{icon_labels}) { $out .= "<br>copy"; }
#	$out .= "</a></td>\n";
#	$ntool++;

	# delete article

	$url = $this->link(zine=>"del",aid=>$article->{article_id});
	$out .= "<td align=\"center\">\n<a href=\"$url\"><img src=\"$config{server}{HTMLpath}/_Modules/Zine/delete.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"delete\" title=\"delete\">"; 
	if ($config{Zine}{icon_labels}) { $out .= "<br>delete"; }
	$out .= "</a></td>\n";
	$ntool++;
    }
    elsif ($article->{owner} == $share{DB}->my_uid) {
	# NOT admin mode, but we may want to allow some simple admin 
	# functions in some cases, eg. blogging, moderating

	if ($share{Page} && ! $share{Page}->publishing) {

	    # new article

	    if ($article->{type} =~ /blog/) {
		my $parent = $article->{article_id};  #### parent_id ???
		my $ref = $article->{reference_id} || $parent;
		$url = $this->link(zine=>"newarticle",aid=>$article->{article_id});
		$out .= "<td align=\"center\">\n<a href=\"$url\"><img src=\"$config{server}{HTMLpath}/_Modules/Zine/new.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"comment\" title=\"new article\">";
		if ($config{Zine}{icon_labels}) { $out .= "<br>new"; }
		$out .= "</a></td>\n";
		$ntool++;
	    }

	    elsif ($article->{type} eq "comment" && $this->allow_comments &&
		   $article->{owner} == $share{DB}->my_uid) {
		# we can only edit our comments within a few minutes of posting
		my $t = new ExSite::Time;
		my $t1 = $t->write("unix_timestamp");
		$t->set($article->{ctime},"sql_timestamp");
		my $t2 = $t->write("unix_timestamp");
		if ($t1 - $t2 < $config{Zine}{edit_my_comment_timeout}) {
		    $url = $this->link(zine=>"edit",aid=>$article->{article_id});
		    $out .= "<td align=\"center\">\n<a href=\"$url\"><img src=\"$config{server}{HTMLpath}/_Modules/Zine/edit.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"edit\" title=\"edit\">";
		    if ($config{Zine}{icon_labels}) { $out .= "<br>edit"; }
		    $out .= "</a></td>\n";
		    $ntool++;
		}
	    }
	}

    }
    if ($article->{status} &&  $article->{status} ne "active") {
	$out .= "</tr>\n<tr><td colspan=$ntool><span class=error>This $article->{type} is $article->{status}.</span></td><td align=center>";
	if ($article->{status} =~ /inactive|pending|rejected/) {
	    if ($this->moderator) {
		$url = $this->link(zine=>"approve",aid=>$article->{article_id});
		$out .= "<a href=\"$url\"><img src=\"$config{server}{HTMLpath}/_Modules/Zine/approve.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"comment\" title=\"approve\">";
		$out .= "</a>\n";
		$url = $this->link(zine=>"reject",aid=>$article->{article_id});
		$out .= "<a href=\"$url\"><img src=\"$config{server}{HTMLpath}/_Modules/Zine/delete.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"comment\" title=\"reject\">";
		$out .= "</a>\n";
	    }
	}
	$out .= "</td>";
    }

    if ($out) {
	my $align = ($config{Zine}{tool_placement} =~ /right/) ? 
	    " align=\"right\"" : "";
	$out = "<table class=\"ZineTools\"$align><tr>\n$out\n</tr></table>\n";
    }
    return $out;
}

# return array of zines owned by the current user

sub my_zines {
    my ($this) = @_;
    my $uid = $share{DB}->my_uid;
    return () if (! $uid);
    my %match = (status=>"active", owner=>$uid, reference=>0);
    return $share{DB}->fetch_match("article",\%match);
}

# return array of zines avialable to the current site

sub our_zines {
    my ($this,$sid) = @_;
    if (! $sid) { $sid = $share{Page} ? $share{Page}->id("section") : 0; }
    return () if (! $sid);
    
    my %match = (status=>"active", section_id=>0, reference=>0);
    my @art;

    # fetch global zines
    @art = $share{DB}->fetch_match("article",\%match);
    # add local Zines
    $match{section_id} = $sid;
    push @art, $share{DB}->fetch_match("article",\%match);
    return wantarray ? @art : \@art;
}

# index of available zines (site-specific)

sub topview {
    my ($this,$opt) = @_;
    my %match = (status=>"active", section_id=>0, reference=>0);
    my (@art,$type);
    if ($opt && $opt !~ /=/) { 
	# request to view a particular class of top-level articles
	$type = $opt; 
    }

    # get all zines for this site
    @art = $this->our_zines();

    my $out;
    if (@art > 0) {
	foreach my $art (@art) {
	    next if ($type && $art->{type} ne $type);
	    $this->set($art);
	    my $url = $this->link(zine=>"show",aid=>$art->{article_id});
	    my $title = $this->title;
	    my $author = $this->author;
	    my $summary = $this->show_data("summary");
	    $out .= "<dt><a href=\"$url\">$title</a>&nbsp;&nbsp;&nbsp;";
	    if ($author && $art->{type} !~ /forum|wiki/) {
		$out .= "by $author";
	    }
	    $out .= "</dt>\n";
	    if ($summary) {
		$out .= "<dd><i>$summary</i>";
	    }
	    $out .= "<p>\n";
	}
	$out .= "</dl>\n";
    }
    return $out;
}

# index of all zines

sub topview_admin {
    my ($this,$opt) = @_;
    if (! $share{DB}->is_manager) {
	return "<p class=error>Permission denied.</p>\n";
    }

    my (@art,$match);
    if ($share{DB}->is_admin) {
	$match = {reference=>0};
    }
    else {
	$match = {reference=>0,owner=>$share{DB}->my_uid()};
    }
    if ($opt) { $match->{type} = $opt; }
    @art = $share{DB}->fetch_match("article",$match);

    my $out;
    if (@art > 0) {
	foreach my $art (@art) {
	    $this->set($art);
	    my $url = $this->link(zine=>"show",aid=>$art->{article_id});
	    my $title = $this->title;
	    my $author = $this->author;
	    my $summary = $this->show_data("summary");
	    my $site;
	    if ($art->{section_id}) {
		my $s = $share{DB}->fetch("section",$art->{section_id});
		$site = $s->{title}.": ";
	    }
	    $out .= "<dt>$site<a href=\"$url\">$title</a>&nbsp;&nbsp;&nbsp;";
	    if ($author && $art->{type} !~ /forum|wiki/) {
		$out .= "by $author";
	    }
	    $out .= "</dt>\n";
	    if ($summary) {
		$out .= "<dd><i>$summary</i></dd>\n";
	    }
	    $out .= $this->tools($art);
	    $out .= "<p>\n";
	}
	$out .= "</dl>\n";
    }
    return $out;
}

sub send {
    my $this = shift;
    my $url = $ENV{HTTP_REFERER};
    
    return "<p class=error>Sorry, the send article function is not implemented at this time.</p>\n";
}

#==========================================================================
# posting methods
# compose: build a form to post a message

# newmsg: wrapper for compose

sub newmsg {
    my $this = shift;
    my $subject = $this->title;
    if ($subject !~ /^Re:/i) {
	$subject = "Re: $subject";
    }
    return $this->makeform({type=>"comment",title=>$subject},$this->{article});
}

# reply: wrapper for compose, which inserts content from parent message

sub reply {
    my $this = shift;
    my $art = $this->{article};
    # add "Re:" to subject
    my $subject = $this->title;
    if ($subject !~ /^Re:/i) {
	$subject = "Re: $subject";
    }
    # quote parent article
    my $quote .= "&gt;".$this->{article}{body};
    $quote =~ s/\n/\n\&gt\;/g;
    $quote = $this->author." wrote:\n".$quote;
    my $out = $this->makeform({type=>"comment",title=>$subject,body=>$quote},
			      $this->{article});

#    my $out = $this->compose({
#	subject=>$subject,
#	body=>$quote,
#	thread=>$art->{thread} || $this->{article_id},
#	reference=>$art->{reference},
#	parent=>$this->{article_id},
#    });
    if ($out =~ /<form/i) {
	$out .= "<p>Please edit any quoted text for brevity, before submitting your comments.</p>\n";
    }
    return $out;
}

# notify an author/owner when their article has been commented on

sub notify {
    my ($this,$data) = @_;
    return if (! $config{Zine}{notify});
    return if (! $data->{parent_id});
    $this->fetch($data->{parent_id});
    my $email = $this->author_email;
    return if (! $email);
    my $url = $this->link(aid=>$data->{article_id},zine=>"show");
    my $purl = $this->link(aid=>$data->{parent_id},zine=>"show");
    my $turl;
    if ($data->{thread}) {
	$turl = $this->link(aid=>$data->{thread},zine=>"show");
    }
    my $zurl = $this->link(aid=>$data->{reference},zine=>"show");
    my $msg = "
$data->{author} has commented on your message '".this->title."' at
$config{server}{server}

---------------------------------------------------------------------
From: $data->{author}
Subject: $data->{title}

$data->{body}
---------------------------------------------------------------------

This message alone may be viewed (and replied to) at:
$config{server}{server}$url

Your original message (including replies) may be viewed at:
$config{server}{server}$purl\n";
    if ($turl) {  ### fixme: assumes threaded comment index
	$msg .= "
The whole thread may be viewed at:
$config{server}{server}$turl\n";
    }
    $msg .= "
The whole discussion may be viewed at:
$config{server}{server}$zurl
";
    return &SendMail("morgan\@exware.com",
		     $config{site}{administrator},
		     "[$config{site}{name}] $data->{title}",
		     $msg);
}

#==========================================================================
# we have our own form processing tools for Zine

sub label {
    my ($this,$type,$col) = @_;
    return ($type eq "comment") ?
	$comment_label{$col->{column}} || $col->{label} || $col->{column} :
	$col->{label} || $col->{column};
}

sub makeform {
    my ($this,$data,$parent) = @_;
    my $db = $share{DB};
    if (! $data->{type}) { $data->{type} = "article"; }
    if (! $parent) { $parent = $this->fetch($data->{parent_id}); }

    # security checks
    my $stat = $this->allow_post($data);
    return $stat if ($stat);

    if (! $data->{parent_id}) { $data->{parent_id} = $parent->{article_id}; }
    if (! $data->{reference}) { 
	if ($data->{type} eq "comment" && $this->type ne "comment") {
	    # reset the reference article if this is a top-level comment
	    $data->{reference} = $parent->{article_id};
	}
	else {
	    # maintain the same reference article
	    $data->{reference} = $parent->{reference} || $parent->{article_id}; 
	}
    }
    if (! $data->{thread} && $data->{type} eq "comment") { 
	$data->{thread} = $parent->{thread} || $parent->{article_id};
    }

    my $type = $data->{type};
    if ($type eq "article" && $parent->{type} eq "blog") {
	$type .= "_blog";
    }
    elsif ($type eq "article" && $parent->{type} eq "photoblog") {
	$type .= "_photoblog";
    }
    elsif ($type eq "article" && $parent->{type} eq "wiki") {
	$type .= "_wiki";
    }

    my $columns = $formtype{$type};
    my @hide;
    if ($this->{admin}) {
	$data->{owner} = $db->my_uid;
	$data->{section_id} = $db->this_site()->{section_id};
	if ($db->is_admin) {
	    push @$columns, "owner";
	    push @$columns, "section_id";
	}
	else {
	    @hide = ("owner", "section_id");
	}
    }

    my $out;
    if ($data->{article_id}) {
	$out = "<h3>Edit $article_type{$type}</h3>\n";
    }
    else {
	$out = "<h3>New $article_type{$type}</h3>\n";
    }

    my $map = $db->{map}->get_map("article");

    my $form;
    foreach my $col (@$columns) {
	if ($col eq "content_type" && $db->{htmleditor}) {
	    # don't prompt for content type if we're using the HTMLeditor
	    next;
	}
	if ($col eq "author" && ! $data->{author} && $db->authorize()) {
	    $data->{author} = $db->my_name();
	}
        my %column = $db->{map}->get_column("article",$col);

	my $label = $this->label($type,\%column);
	my $reqd = $column{validate} =~ /hard|soft/ ? "(required)" : "";
 	$form .= "<p>$label $reqd<br>\n";

	if ($col eq "body" && $type =~ /article/ && $db->use_wysiwyg) {
	    # try to use the HTML editor for normal articles
	    my $page = $this->{input}{_id};
	    my $section = $this->{reference}{section_id} || $this->{article}{section_id};
	    $form .= $db->input_htmleditor(name=>$col,
					   editor_height=>350,
					   editor_width=>550,
					   table=>"article",
					   column=>"body",
					   record=>$data->{article_id},
					   page=>$page,
					   section=>$section,
					   links=>1,
					   editor=>{
					       op_image=>($db->owns),
					       op_link=>1,
					       op_table=>1,
					       op_dyncontent=>0,
					   });
	    my $edtype = $db->use_wysiwyg ? "HTML" : "mixed";
	    $form .= $db->input_html(name=>"content_type",
				     type=>"hidden",
				     value=>$edtype,
				     );
	}
	else {
	    $form .= $db->input_exsite(name=>$col,
				       size=>$column{size},
				       datatype=>$column{datatype},
				       value=>$data->{$col},
				       table=>"article",
				       column=>$col,
				       record=>$data->{article_id},
				       );
	}
	$form .= "</p>\n";
    }
    foreach my $col (@hide) {
        my %column = $db->{map}->get_column("article",$col);
	my $label;
	if ($type eq "comment") {
	    $label = $comment_label{$col} || $column{label} || $column{column};
	}
	else {
	    $label = $column{label} || $column{column};
	}
	$form .= $db->input_html(name=>$col,
				type=>"hidden",
				value=>$data->{$col},
				);
	$form .= "\n";
    }

    my $url = $this->link(zine=>"submit");
    if ($db->{htmleditor}) {
	$out = $db->editor_javascript();
#	$out .= "<form METHOD=\"POST\" ACTION=\"$config{server}{CGIpath}/debug.cgi\" onSubmit=\"validate()\">\n";
	$out .= "<form METHOD=\"POST\" ENCTYPE=\"multipart/form-data\" ACTION=\"$url\" onSubmit=\"validate()\" id=\"exsiteform\">\n";
	$out .= "<input type=\"hidden\" name=\"$db->{htmleditor}\">\n";
    }
    else {
	$out .= "<form METHOD=\"POST\" ENCTYPE=\"multipart/form-data\" ACTION=\"$url\">\n";
    }
    $out .= $form;

    # add protected control info 
    my $crypt = new ExSite::Crypt;
    my $ctrlinfo = "$data->{type};$data->{article_id};$data->{parent_id};$data->{reference};$data->{thread}";
    my $ctrl_crypt = $crypt->encrypt($ctrlinfo);
    $out .= $db->input_html(name=>"ctrlinfo",type=>"hidden",value=>$ctrl_crypt);
    #### reply screen?

    $out .= $db->make_form_buttons;
    $out .= "</form>\n";

    return $out;
}

#==========================================================================
# configure the article

sub confform {
    my ($this) = @_;
    my $db = $share{DB};
    my $data = $this->{article};

    # security checks
    my $stat = $this->allow_post($data);
    return $stat if ($stat);

    my $columns = ["type","status","content_type","indexlimit","indexage",
		   "indextype","indexformat","comment_policy"];

    my $type = ucfirst($data->{type});
    my $url = $this->link(zine=>"submit");
    my $out = "<h3>Configure $type</h3>
<form METHOD=\"POST\" ENCTYPE=\"multipart/form-data\" ACTION=\"$url\">\n";

    my $map = $db->{map}->get_map("article");

    foreach my $col (@$columns) {
        my %column = $db->{map}->get_column("article",$col);
	my $label = $column{label} || $column{column};
	my $reqd = $column{validate} =~ /hard|soft/ ? "(required)" : "";
 	$out .= "<p>$label $reqd<br>\n";
	$out .= $db->input_exsite(name=>$col,
				  size=>$column{size},
				  datatype=>$column{datatype},
				  value=>$data->{$col},
				  table=>"article",
				  column=>$col,
				  record=>$data->{article_id},
				  );
	$out .= "</p>\n";
    }

    # add protected control info 
    my $crypt = new ExSite::Crypt;
    my $ctrlinfo = "$data->{type};$data->{article_id};$data->{parent_id};$data->{reference};$data->{thread}";
    my $ctrl_crypt = $crypt->encrypt($ctrlinfo);
    $out .= $db->input_html(name=>"ctrlinfo",type=>"hidden",value=>$ctrl_crypt);
    #### reply screen?

    $out .= $db->make_form_buttons;
    $out .= "</form>\n";

    return $out;
}

sub doform {
    my $this = shift;
    my $in = new ExSite::Input;
    my $data = $in->post;
    my $db = $share{DB};
    my ($error,@err);
    my $submit = $data->{submit_action};
    if ($submit eq "Cancel") { return (); } 
    delete $data->{submit_action};

    # decode control info
    my $crypt = new ExSite::Crypt;
    my $ctrlinfo = $crypt->decrypt($data->{ctrlinfo});
    if ($ctrlinfo !~ /^[-\w]*;\w*;\w*;\w*;\w*$/) {
	# tampering detected - abort
	#$error = "bad control data: $ctrlinfo";
	$error = "bad control data";
	$this->error($error);
	return ($error);
    }
    delete $data->{ctrlinfo};
    my @ctrlinfo = split(/;/,$ctrlinfo);
    my $ctype = shift @ctrlinfo;
    if (! $data->{type}) { $data->{type} = $ctype; }
    ($data->{article_id},$data->{parent_id},
     $data->{reference},$data->{thread}) = @ctrlinfo;

    foreach my $key (qw(parent_id reference thread)) {
	if (! defined $data->{$key}) { $data->{$key} = 0; }
    }

    # type 
    if (!$data->{type}) {
	# no type - better abort
	return $this->error("undefined article type");
    }
    my $type = $data->{type};
    my $parent;
    if ($data->{parent_id}) {
	$parent = $db->fetch("article",$data->{parent_id});
	if ($type eq "article" && $parent->{type} eq "blog") {
	    $type .= "_blog";
	}
	elsif ($type eq "article" && $parent->{type} eq "photoblog") {
	    $type .= "_photoblog";
	}
	elsif ($type eq "article" && $parent->{type} eq "wiki") {
	    $type .= "_wiki";
	}
    }

    # tidy up the record data
    my $sc = $config{form}{sepchar};

    # fix body field
    if (exists $data->{"body${sc}html"}) {
    	# Unescape %7B to { ...etc
    	require ExSite::Util;
	$data->{body} = &restore_cms_tags($data->{"body${sc}html"});
	delete $data->{"body${sc}html"};
	$data->{body} = $share{DB}->clean_text($data->{body});
    }
    
    # owner info
    if (exists $data->{"owner${sc}key"}) {
	$data->{owner} = $data->{"owner${sc}key"};
	delete $data->{"owner${sc}key"};
    }
    else {
	my $uid = $share{DB}->my_uid;
	if ($uid) { $data->{owner} = $uid; }
    }
    # section info
    if (exists $data->{"section_id${sc}key"}) {
	$data->{section_id} = $data->{"section_id${sc}key"};
	delete $data->{"section_id${sc}key"};
    }
    else {
	##### auto-set section ??
	
    }

    # clean all text input fields
    foreach my $f (qw(author title subtitle summary body about caption)) {
	if (exists $data->{$f}) {
	    my $allow_html = ($type =~ /comment/) ?
		$config{Zine}{allow_html_in_comments} : undef;
	    # undef means we defer to basic ExSite rules for whether or not
	    # to allow html;  ie. config setting report.text.honor_html
	    $data->{$f} = $share{DB}->clean_text($data->{$f},$allow_html);
	}
    }

    # encode picture, if any
    if ($data->{"picture${sc}filename"}) { 
	my $fname = $data->{"picture${sc}filename"};
	if ($fname !~ /\.(gif|png|jpe?g)$/i) {
	    # wrong file type
	    $error = "picture must be GIF, PNG, or JPEG file type";
	    $this->error($error);
	    push @err, $error;
	}
	if (@err > 0) {
	    return @err;
	}

	my $fdata = undef;
	my $size = 0;
	# read file contents
	my $continue = 1;
	while ($continue) {
	    $continue = read($data->{"picture${sc}filename"},$fdata,1024,$size);
	    $size += $continue;
	}

	my $img = new ExSite::Image($fname,$fdata);
	my $max_size = $config{Zine}{max_picture_size} || 48;
	my $max_dim = $config{Zine}{max_picture_dim} || 500;
	my ($img_x,$img_y) = $img->dim;
	if ($img_x > $max_dim || $img_y > $max_dim || $size/1024 > $max_size) {
	    # picture too big
	    $img->shrink({dim=>$max_dim,size=>$max_size});
	}
	$data->{picture} = $img->encode;
	# thumbnail the pic
	if ($img->thumb) {
	    $data->{thumb} = $img->encode;
	}
    }
    if (exists $data->{"picture${sc}filename"}) {
	delete $data->{"picture${sc}filename"};
    }

    if (! $data->{status}) { $data->{status} = "active"; } 

    # check for required fields
    
    foreach my $req (@{$required{$type}}) {
	if (exists $data->{$req}) {  # field is present
	    if (! $data->{$req}) {   # data is missing
		$error = "$req is missing";
		$this->warn($error);
		push @err,$error;
	    }
	}
    }   
    if (@err > 0) {
	return @err;
    }

    # all is good - save the article

    if ($data->{article_id}) {
	$error = $db->update("article",$data);
	if (! $error) { 
	    # update our internal copy of the article
	    foreach my $key (keys %$data) { 
		$this->{article}{$key} = $data->{$key};
	    }
	}
    }
    else {
	my $null = "NULL";
	$data->{ctime} = \$null;  # forces auto-setting of MySQL timestamp
	my $aid = $db->insert("article",$data);
	if (!$aid) { 
	    $error = $db->{query_error};
	}
	else {
	    $data->{article_id} = $aid;
	}
    }
    if ($error) {
	$this->error($error);
	return ($error);
    }
    $this->run_publisher($data);
    return ();
}

sub post_doform {
    my $this = shift;
    # this is a dummy;  it's purpose is to be overloaded by other
    # article handling systems
    return;
}

# status : update article status only

sub status {
    my ($this,$action) = @_;
    my ($db,$stat,$error);
    $db = $share{DB};
    # security checks
    $stat = $this->allow_post($this->{article});
    return $stat if ($stat);
    if ($action eq "approve") {
	$error = $db->update("article",{status=>"approved"},{article_id=>$this->{article_id}});
    }
    elsif ($action eq "reject") {
	$error = $db->update("article",{status=>"approved"},{article_id=>$this->{article_id}});
    }
    return $error if ($error);
    my $p = $share{Page} || $share{PageBuf};
    $p->redirect($ENV{HTTP_REFERER});
    return undef;
}

#==========================================================================
# delete the article and its children

sub delete {
    my ($this) = @_;
    my $db = $share{DB};
    my $data = $this->{article};
    
    my $out;
    if ($this->{admin} && $this->owns) {
	# deletions are restricted to the control panel and owner
	$db->trash_r("article",$this->{article_id},1);
	$out .= "<p>".ucfirst($data->{type})." ".$data->{article_id}.
	    " was moved to the trash.</p>\n";
    }
    else {
	$out .= "<p>Sorry, you do not have permission to delete this article.</p>\n";
    }
    my $url = $this->link(zine=>"show",aid=>$data->{parent_id});
    $out .= "<p><a href=\"$url\">Continue</a></p>\n";
    return $out;
}

#==========================================================================
# standard article queries

sub fetch ($$) {
    my ($this,$id) = @_;
    if ($id != $this->{article_id}) {
	$this->{article_id} = $id;
	$this->{article} = $share{DB}->fetch("article",$id);
	if ($this->{article}{reference}) {
	    $this->{reference} =
		$share{DB}->fetch("article",$this->{article}{reference});
	}
	else {
	    $this->{reference} = undef;
	}
    }
    return $this->{article};
}

sub set ($$) {
    my ($this,$art) = @_;
    $this->{article_id} = $art->{article_id};
    $this->{article} = $art;
    $this->{reference} = $share{DB}->fetch("article",$art->{reference});
    return $art;
}

sub title {
    my $this = shift;
    return $this->{article}{title};
}

sub reftitle {
    my $this = shift;
    return $this->{reference}{title};
}

sub author {
    my $this = shift;
    if ($this->{article}{author}) {
	my $author = $this->{article}{author};
	if ($this->{article}{type} eq "comment" && 
	    $config{Zine}{mark_unauthenticated_comments} && 
	    ! $this->{article}{owner}) {
	    my $mark = $config{Zine}{unauthenticated_comment_mark};
	    return "$author $mark";
	}
	else {
	    return $author;
	}
    }
    elsif ($this->type eq "comment") {
	if ($this->{article}{owner}) {
	    my %owner = $share{DB}->fetch("member",$this->{article}{owner});
	    return $owner{name} || $owner{login};
	}
	else {
	    return "Anonymous";
	}
    }
    return "";
}

sub author_email {
    my $this = shift;
    if ($this->{article}{owner}) {
	my %owner = $share{DB}->fetch("member",$this->{article}{owner});
	return $owner{email};
    }
    return undef;
}

sub picture {
    my $this = shift;
    my $out;
    if ($this->{article}{picture}) {
	my $img = new ExSite::Image($this->{article}{picture});
	my ($width,$height) = $img->dim;
	my $filename = "$this->{article_id}_$img->{filename}";
	if (! -e $this->diskpath."/".$filename) {
	    # not published; go to the database
	    $filename = "$config{server}{CGIpath}/getdata.cgi?tab=article&col=picture&id=$this->{article}{article_id}";
	}
	else {
	    $filename = $this->htmlpath."/".$filename;
	}
	$out = "<table width=\"$width\"";
	if (length $this->{article}{body} > 1000) {
	    # check image size
	    my ($x,$y) = $img->dim;
	    if ($x < ($config{Zine}{img_max_wrap_size} || 400)) {
		$out .= " align=\"right\"";
	    }
	}
	$out .= "><tr><td><img src=\"$filename\" height=\"$height\" width=\"$width\"></td></tr>";
	if ($this->{article}{caption}) {
	    $out .= "\n<tr><td><span class=\"caption\">$this->{article}{caption}</span></td></tr>";
	}
	$out .= "</table>";
    }
    return $out;
}

sub thumb {
    my ($this,$align) = @_;
    my $out;
    if ($this->{article}{thumb}) {	
	my $img = new ExSite::Image($this->{article}{thumb});
	my $filename = "$this->{article_id}T_$img->{filename}";
	if (! -e $this->diskpath."/".$filename) {
	    # not published; go to the database
	    $filename = "$config{server}{CGIpath}/getdata.cgi?tab=article&col=thumb&id=$this->{article}{article_id}";
	}
	else {
	    $filename = $this->htmlpath."/".$filename;
	}
	$out = "<img class=\"ZineThumb\" src=\"$filename\" border=\"0\"";
    }
    elsif ($this->{article}{picture}) {
	my $img = new ExSite::Image($this->{article}{picture});
	my $filename = "$this->{article_id}_$img->{filename}";
	if (! -e $this->diskpath."/".$filename) {
	    # not published; go to the database
	    $filename = "$config{server}{CGIpath}/getdata.cgi?tab=article&col=picture&id=$this->{article}{article_id}";
	}
	else {
	    $filename = $this->htmlpath."/".$filename;
	}
	my $thumbdim = $config{Zine}{thumbdim} || 100;
	$out .= "<img class=\"ZineThumb\" src=\"$filename\" width=\"$thumbdim\" border=\"0\"";
    }
    if ($out) { 
	if ($align) {
	    $out .= " style=\"padding-right:4px;\" align=\"$align\"";
	}
	$out .= ">";
    }
    return $out;
}

sub body {
    my $this = shift;
    my $body = $this->{article}{body};
    if ($this->{article}{content_type} =~ /html/i) {
	$body = $this->{article}{body};
	$body =~ s/<!--/<!-- /g; # kill any hidden CMS tags
    }
    else {
	my $nl = $config{report}{text}{honor_nl};
	#$config{report}{text}{honor_nl} = 1;
	#$body = $this->show_data("body");
	if ($this->{article}{type} eq "comment") {
	    $body =~ s/\n/<br>\n/g;       # convert NL to <br>
	}
	else {
	    # insert paragraph tags at any double-linebreaks
	    $body =~ s/\r\n\r\n/<P>\n/g;  # handles CR-LF linebreaks
	    $body =~ s/\n\n/<P>\n/g;      # handles LF linebreaks
	}
	#$config{report}{text}{honor_nl} = $nl;
    }
    # try to impose some style rules on the quoted sections, in case
    # the designer wants to highlight them
    if ($this->{article}{type} eq "comment") {
	$body =~ s/\n>([^\n]*?)(\r|\n)/\n<span class=\"quote\">&gt;$1<\/span>\n/g;
    }
    return $body;
}

# basic timestamp formatting

sub timestamp {
    my ($this,$dtype) = @_;
    if (! $dtype) { $dtype = "ctime"; }  # alternative is mtime
    return $this->show_data($dtype);
}

# article date

sub date {
    my ($this) = @_;
    my $timetype = ($this->{article}{mtime} ne $this->{article}{ctime}) ? 
	"mtime" : "ctime";
    my $timestamp = $this->timestamp($timetype);
    return $timetype eq "mtime" ? 
	"updated on $timestamp" :
	"posted on $timestamp";
}

# returns the article type;

sub type {
    my $this = shift;
    return $this->{article}{type};
}

# returns the reference and current article type;

sub extended_type {
    my $this = shift;
    my $ref = $this->reftype;
    return "$ref $this->{article}{type}";
}

# returns the article type class.  Either "comment" (ie. external source) 
# or "article" (internal source).
### FIXME: are wiki articles another class?

sub typeclass {
    my $this = shift;
    return $this->{article}{type} eq "comment" ? "comment" : "article";
}

# returns the reference article type;  determines general formatting rules

sub reftype {
    my $this = shift;
    if (ref $this->{reference}) {
	return $this->{reference}{type};
    }
    return $this->{article}{type};
}

# return age of article in days (does not account for leap years)

sub age {
    my $this = shift;
    my @mdays = (31,28,31,30,31,30,31,31,30,31,30,31);

    my (undef,undef,undef,undef,undef,$year,undef,$yday,undef) = 
	localtime(time);
    my $now = ($year + 1900) * 365 + $yday + 1;

    # assume timestamp format YYYYMMDDHHMMSS
    $this->{article}{ctime} =~ /^(\d{4})(\d{2})(\d{2})/;
    my $then = $1*365 + $3;
    my $imonth = 1;
    while ($imonth < $2) {
	$then += $mdays[$imonth-1];
	$imonth++;
    }

    return ($now - $then);
}

sub show_data {
    my $this = shift;
    my $col = shift;
    return $share{DB}->show_data_noauth("article",$col,
				 $this->{article}{$col},$this->{article_id});
}

# diskpath to article's published files

sub diskpath {
    my $this = shift;
    return $config{server}{HTMLroot}.$this->htmlpath;
}

sub htmlpath {
    my $this = shift;
    my $zpath = "$config{server}{HTMLpath}/_Modules/Zine";
    if ($this->{article}{reference}) {
	$zpath .= "/$this->{article}{reference}";
    }
    else {
	$zpath .= "/$this->{article}{article_id}";
    }
    return $zpath;
}

#------------------------------------------------------------------------
# article security

# owns : tests whether the viewer has permission to modify/update the article

sub owns {
    my $this = shift;
    my $db = $share{DB};
    return 1 if ( $db->is_manager() ); 
    my $uid = $db->my_uid;
    return 0 if ( ! $uid );
    return ($uid == $this->{article}{owner});
}

# approve : test whether the user has permission to view the article

sub approve { 
    my $this = shift;
    my $db = $share{DB};

    # managers/admins see all
    return 1 if ($db->is_manager);
    
    # article privacy check
    return 0 if ($this->{article}{privacy} =~ /members/ && ! $db->is_member);

    # article status check (must be active or unset [default is active])
    return 1 if (! $this->{article}{status} || 
		 $this->{article}{status} eq "active");
}

# moderator : test whether the user has permission to moderate comments

sub moderator {
    my $this = shift;
    my $db = $share{DB};
    return 1 if ( $db->is_manager() ); 
    my $article;
    if ($this->{article}{type} eq "comment") {
	$article = $this->{reference};
    }
    else {
	$article = $this->{article};
    }
    my $uid = $db->my_uid;
    return 0 if ( ! $uid ); 
    return ($uid == $article->{owner});
}

# allow_comments : test whether comments are accepted on the current article

sub allow_comments {
    my ($this,$class) = @_;
    # custom article setting
    my $com = $this->comment_policy($class);
    return ($com ne "forbidden");
}

# allow_post : test whether user is allowed to post;  returns an error
# string if DISallowed, undef if allowed.  Article data must be provided.

sub allow_post {
    my ($this,$data) = @_;
    if ($data->{article_id} > 0) {
	# we appear to be editing an existing article
	if ($data->{type} eq "comment") {
	    if (! $share{DB}->is_admin) {
		if ($data->{owner} == $share{DB}->my_uid()) {
		    my $t = new ExSite::Time;
		    my $t1 = $t->write("unix_timestamp");
		    $t->set($data->{ctime},"sql_timestamp");
		    my $t2 = $t->write("unix_timestamp");
		    if ($t1 - $t2 > $config{Zine}{edit_my_comment_timeout}) {
			return "<p>This comment is now frozen.</p>\n";
		    }
		    # else, edit comment is allowed
		    return undef;
		}
		return "<p>You don't have permission to edit this comment.</p>\n";
	    }
	}
	if ($data->{type} eq "article" && ! $this->owns) {
	    # user does not own the Zine
	    return "<p>Sorry, you do not have permission to manage this article.</p>\n";
	}
	if ($data->{type} !~ /article|comment/ && ! $this->{admin}) {
	    # Zine management only from control panel
	    return "<p>Zines can only be managed from the administrator control panel.</p>\n";
	}
    }
    else {
	# we appear to be creating a new article
	if ($data->{type} eq "comment" && ! $this->allow_comments) {
	    # comments are disabled under this Zine
	    if ($config{Zine}{$this->reftype}{comment_policy}{member}) {
		return "<p>Sorry, only members can post comments here.</p>\n";
		# login link might be nice...
	    }
	    return "<p>Sorry, comments are not permitted.</p>\n";
	}
	if ($data->{type} eq "article" && ! $this->owns) {
	    # user does not own the Zine
	    return "<p>Sorry, you do not have permission to post articles here.</p>\n";
	}
	if ($data->{type} !~ /article|comment/ && ! $this->{admin}) {
	    # Zine creation only from control panel	
	    return "<p>Sorry, you do not have permission to create new Zines.</p>\n";
	}
    }
    return undef;
}

1;



