package Modules::Catalog;
#---------------------------------------------------------------------
# Catalog: The catalog module manages a browsable catalog of products and/or
# services that are offered by a business.
#
# Usage:
#
# <!--&Catalog(<options>)-->
#
# Options:
#
# action=default
#	- Default action is set to 'browse'
# action=browse
# 	- Browse catalog
# action=search
#	- Search form
# action=chart
#	- Application chart
# product_id=<product_id>
#	- Show product details
# disable_product_detail=1|0
#	- Don't show product detail screen
#	(This option overwrites the conf setting)
# enable_requestquote=1|0
#	- Enable 'request a quote' plugin in Catalog
#	(This option overwrites the conf setting)
# show_link_openwindow=1|0
#	- Open product detail screen in a new window
#	(This option overwrites the conf setting)
#---------------------------------------------------------------------
use strict;

use MIME::Base64;
use ExSite::Config;
use ExSite::Misc;
use Modules::BaseDCD;
use Modules::Catalog::Tree;
use ExSite::ReportBuilder;

use vars qw(@ISA $row_class);
push(@ISA,"Modules::BaseDCD");

#---------------------------------------------------------------------
sub new {
	my $this = shift;
	my ($obj,$class);
	$obj = {};
	$class = ref($this) || $this;
	bless $obj, $class;
	$class =~ /^(.+?::)+?(.+)$/;
	$obj->{class_name} = $2;
	$obj->{db} = $share{DB};
	$obj->{cgi} = $config{server}{server}.$config{server}{CGIpath};
	$obj->{html} = $config{server}{server}.$config{server}{HTMLpath};
	$obj->read;  # inherits BaseDCD::read()
	$obj->read_conf;
	$obj->{row_class} = 0;
	$obj->init();
	return $obj;
}
sub ioctl{
	my $this = shift;
	$_ = shift;   # $_ is the ioctl request
	if (/ModuleName/){
		return "Catalog";
	}elsif (/ControlPanel/){
		return \&control_panel;
	}elsif (/ModuleInfo/){
        	return "The catalog module manages a browsable catalog of products and/or number of services that are offered by a business.";
	}elsif (/Publish/){
		return \&publish;
	}elsif (/DynContentParam/){
		return $this->getDynContentParameter(shift);
	}elsif (/isRestricted/){
		return $config{Catalog}{isRestricted};
	}elsif (/isService/){
		return $config{Catalog}{isService};
	}		
	return;
}
sub write{
	my ($this,$options) = @_;
	my %opt = &DecodeString($options);
	$this->module_option(\%opt);
	my $action = $this->{input}{action} || $opt{action};
	$this->action($action);
	my $out;
	# Get section id
	my $sectionid = $this->section_id();
	# Number of new product to show
	my $num_to_show = $opt{number_to_show} || $this->{input}{number_to_show} || 0;	
	# Set category path
	$this->path($this->{input}{path} || $opt{path});
	# Category search
	if ($opt{action} =~ /^search$/i){
		$this->action($opt{action});
		return $this->catalogSearch();
	# Display new products
	}elsif ($action eq "what_is_new"){
		return $this->whatIsNew($sectionid,$num_to_show);
	# View product details
	}elsif ($this->{input}{product_id} || $opt{product_id}){
		$out = $this->productDetail(($this->{input}{product_id} || $opt{product_id}),$sectionid);
	# Add a new link
	}elsif ($action eq "new_product"){
		$out = $this->resourceNew();
	}elsif ($opt{action} =~ /^chart$/i){
		return $this->catalogChart($this->section_id());
	# Process new/edit link data
	}elsif ($action eq "post_link"){
		$out = $this->processPost();
#		if ($this->{db}->is_admin()){
		if ($this->is_admin()){
#			$this->republish();
		}
#	# View category details
#	}elsif ($action =~ /category_detail/ && $this->path()){
#		return $this->categoryDetail($this->path());
	# Show sub directory
	}elsif ($sectionid && $this->path()){
		$out = $this->catalog(undef,$sectionid);
	# Doing catalog search
	}elsif ($action eq "dir_search" && $this->{input}{search_field}){
		#$out = $this->directorySearch($this->{input}{search_field});
		return $this->directorySearch($this->{input}{search_field});
	# Show sub directory
	}elsif ($sectionid){
		$out = $this->catalog(undef,$sectionid);
	}
	return $out;
}
# Get section id
sub section_id{
	my ($this,$id) = @_;
	# Save section id in the object
	if (defined($id)){
		if ($this->is_int($id)){
			$this->{section_id} = $id;
		}
	}
	# Attempt to set a section id
	if (!$this->{section_id}){
		# Using built in get_section_id()
		$this->{section_id} = $this->get_section_id();
	}
	return $this->{section_id} || 0;
}
# Are we using the thumbnail hover over effects in the top level display?
sub toplevel_show_thumbnail{
	my ($this,$value) = @_;
	return 0 if $this->admin_mode();
	return $this->getModuleSetting("toplevel_show_thumbnail");
}
# Save form action
sub action{
	my ($this,$val) = @_;
	if (defined($val)){
		$this->{action} = $val || "";
	}
	return $this->{action};
}
# Save module option
sub module_option{
	my ($this,$opt) = @_;
	# Ssave module option in the object
	if (ref($opt) =~ /hash/i){
		$this->{module_option} = $opt;
	}
	if (ref($this->{module_option}) !~ /hash/i){
		return {};
	}
	return $this->{module_option};
}
# Get module setting
sub getModuleSetting{
	my ($this,$param) = @_;
	my %option = %{$this->module_option()};
	if (exists($option{$param})){
		return $option{$param};
	}
	if ($config{Catalog}{$param}){
		return $config{Catalog}{$param};
	}
	return undef;
}
# Disable the product detail screen?
sub disableProductDetail{
	my ($this) = @_;
	return $this->getModuleSetting("disable_product_detail");
}
# Open product detail screen in a new window
sub showProductNewWindow{
	my ($this) = @_;
	return $this->getModuleSetting("show_link_openwindow");
}
# Is request a quote plugin enabled on the product page?
sub enableRequestQuote{
	my ($this) = @_;
	return $this->getModuleSetting("enable_requestquote");
}

#---------------------------------------------------------------------
# Control panel
sub control_panel{
	my $this = shift;
	# Turn on admin mode
	$this->admin_mode(1);
	# Save module path
	$this->path($this->{input}{path});
	my $action = $this->{input}{action};
	# Save action
	$this->action($action);
	my $out;
	$this->section_id();
	# Reset sort key
	if ($action =~ /^reset_sort$/i){
		$this->resetSort();
	}
	# Add new join record
	if ($action =~ /join_new/i && $this->{input}{topic_id} && $this->{input}{join_type}){
		$out = $this->categoryJoinNewForm($this->{input}{topic_id},$this->{input}{join_type});
	# Edit join record
	}elsif ($action =~ /join_edit_form/i && $this->{input}{join_id} && $this->{input}{join_type}){
		$out = $this->categoryJoinEditForm($this->{input}{join_id},$this->{input}{join_type});
	# Delete join record
	}elsif ($action =~ /join_delete/i && $this->{input}{join_id}){
		$out - $this->categoryJoinDelete($this->{input}{join_id});
	# Edit category form
	}elsif ($action =~ /^topic_edit_form$/i && $this->{input}{topic_id}){
		$out = $this->categoryEditForm($this->{input}{topic_id});
	# Add new category
	}elsif ($action =~ /^new_category$/i){
		$out = $this->processNewCategory();
	# Process attachment insert or update
	}elsif ($action =~ /^(insert|update)$/i){
		$out = $this->processAttachment();
	# Show attachment details
	}elsif ($action eq "product_attachment" && $this->{input}{resource_id}){
		$out = $this->productAttachment($this->{input}{id},$this->{input}{resource_id});
	# Product edit form
	}elsif ($action =~ /resource_(record|category|attachment)_form/i){
		$out = $this->productEditForm($1,$this->{input}{record_id});
	# Delete a product
	}elsif ($action =~ /resource_(record|category|attachment)_delete/i && $this->{input}{record_id}){
		$out = $this->productDelete($1,$this->{input}{record_id});
	# Edit a resource
	}elsif ($action eq "resource_edit" && $this->{input}{resource_id}){
		$out = $this->productEdit($this->{input}{resource_id});
	# Edit a topic
	}elsif ($action eq "topic_edit" && $this->{input}{topic_id}){
		$out = $this->categoryEdit($this->{input}{topic_id});
	# Process post using do
	}elsif ($action eq "process_doform"){
		$out = $this->processDoForm();
	# Delete a topic
	}elsif ($action eq "topic_delete" && $this->{input}{topic_id}){
		$out = $this->categoryDelete($this->{input}{topic_id});
#		$this->republish();
	# Add a new topic
	}elsif ($action eq "topic_new"){
		$out = $this->categoryNew($this->section_id());
	# Delete a topic
	}elsif ($action eq "topic_delete" && $this->{input}{topic_id}){
		$out = $this->categoryDelete($this->{input}{topic_id});
#		$this->republish();
	# Delete a resource
	}elsif ($action eq "resource_delete" && $this->{input}{resource_id}){
		$out = $this->productDelete("record",$this->{input}{resource_id});
#		$this->republish();
	# Display catalog
	}elsif ($this->section_id()){
		$out = $this->write();
	}else{
		$out .= $this->set_section_id();
	}
	return $out;
}
#----------------------------------------------------------------------

sub init{
	my ($this) = @_;
	# Clean cache
	&clean_cache();
	$this->uninstallCatalogObject();
	return;
}

# Module init function
sub build_cache{
	my ($this) = @_;

	return if !$this->section_id();

	my $db = $this->{db};	
	
	my %site = $db->fetch("section",$this->section_id());
	$this->save_cache_section(\%site);	
	
	my @sectionids;
	push(@sectionids,$site{section_id});

	my @where;
	foreach my $sid(@sectionids){
		push(@where,"catalog_category.section_id = $sid");
	}

	# Fetch global cateogry(categories with no section_id is set
	if ($this->enableCategorySharing()){
		push(@where,"catalog_category.section_id = 0");
		push(@where,"catalog_category.section_id is NULL");
		push(@where,"catalog_category.section_id = ''");
		
		push(@sectionids,0);
	}

	# Cache search specific data
	if ($this->skip_default_cache()){
		# Don't have to cache any data. Data get cached when running custom routines.(see sub skip_default_cache for more info)
	# Cache single promo product data
	}elsif ($this->promotion_mode()){
		# Cache routine specifically written for Catalog::CatalogPromotion module
		$this->build_cache_promotion();
	# Cache product data only
	}elsif ($this->product_only_mode()){
		$this->build_cache_product();
	# Cache sublevel and product data
	}elsif ($this->sublevel_mode()){
		$this->build_cache_sub_level_category(\@sectionids);
	# Cache top level category display data
	}elsif ($this->toplevel_mode()){
		$this->build_cache_top_level_category(\@sectionids);
	# For everything else, we cache everything
	}else{
		$this->build_cache_all_products(\@where);
		$this->build_cache_all_categories(\@sectionids);
	}

	$share{catalog}{catalog_cache} = 1;
	return;
}
# Cache promo product data
sub build_cache_promotion{
	my ($this) = @_;
	my $productid = $this->product_id();
	return if !$productid;
	my $sql = <<END;
select
	p.catalog_product_id as id_product_1,
	p.code as code_product_1,
	p.title as title_product_1,
	p.summary as summary_product_1,
	p.details as details_product_1,
	p.price as price_product_1,
	p.status as status_product_1,
	p.options as options_product_1,
	p.sortkey as sortkey_product_1,
	pa.catalog_product_attachment_id as id_product_attachment_1,
	pa.catalog_product_id as product_id_product_attachment_1,
	pa.type as type_product_attachment_1,
	pa.description as description_product_attachment_1,
	pa.heading as heading_product_attachment_1,
	SUBSTRING(pa.upload,1,100) as upload_product_attachment_1,
	SUBSTRING(pa.thumbnail,1,100) as thumbnail_product_attachment_1
from
	catalog_product as p
	left join catalog_product_attachment as pa on p.catalog_product_id = pa.catalog_product_id
where
	p.catalog_product_id = $productid
END
	if ($this->admin_mode()){
		$sql .= " and (p.status like 'enabled' || p.status = '' || p.status is null)";
	}
	
	my $db = $this->{db};
	my @records = $db->custom_query($sql);
	
	foreach my $record(@records){
		# Product
		my %product = (
			catalog_product_id=>$record->{id_product_1},
			code=>$record->{code_product_1},
			title=>$record->{title_product_1},
			summary=>$record->{summary_product_1},
			details=>$record->{details_product_1},
			price=>$record->{price_product_1},
			status=>$record->{status_product_1},
			options=>$record->{options_product_1},
			sortkey=>$record->{sortkey_product_1},
		);
		# Product attachment
		my %product_attachment = (
			catalog_product_attachment_id=>$record->{id_product_attachment_1},
			catalog_product_id=>$record->{product_id_product_attachment_1},
			type=>$record->{type_product_attachment_1},
			description=>$record->{description_product_attachment_1},
			heading=>$record->{heading_product_attachment_1},
			upload=>$record->{upload_product_attachment_1},
			thumbnail=>$record->{thumbnail_product_attachment_1},
		);
		$this->save_cache_product(\%product);
		$this->save_cache_product_attachment(\%product_attachment);
	}
	return;
}
# Are we in category edit mode?
# This mode requires the system to fetch all categories
sub category_edit_mode{
	my ($this) = @_;
	if ($this->action() =~ /resource_category/i){
		return 1;
	}
	return 0;
}
# Are we display product only? 
sub product_only_mode{
	my ($this) = @_;
	if ($this->category_edit_mode()){
		return 0;
	}
	if (!$this->path() && $this->product_id()){
		return 1;
	}
	return 0;
}
# Are we caching a product info for Catalog::CatalogPromotion
sub promotion_mode{
	my ($this,$value) = @_;
	# This routine is overloaded by Catalog::CatalogPromotion
	return 0;
}
# Are we caching top level data?
# 	Top level mode cache routine caches all the top and 1st level sub category data. 
sub toplevel_mode{
	my ($this) = @_;
	# If we were to show thunbmails along with the categories we need to cache product data as well
	if (($this->topleveldisplaystyle_list() && $this->toplevel_show_thumbnail()) && !$this->admin_mode()){
		return 0;
	}
	# Path shouldn't be defined when we are working on the top level directory display
	if ($this->path()){
		return 0;
	}
	if ($this->action() =~ /browse/i || !$this->action()){
		return 1;
	}
	return 0;
}
# Are we displaying a list of new product?
sub what_is_new_mode{
	my ($this) = @_;
	if ($this->action() =~ /what_is_new/i){
		return 1;
	}
	return 0;
}
# Are we running custom caching routines?
sub skip_default_cache{
	my ($this) = @_;
	# We cache data based on the searching query
	if ($this->search_mode()){
		return 1;
	}
	# We cache data based on the 'what is new' query
	if ($this->what_is_new_mode()){
		return 1;
	}
	return 0;
}
# Are we caching sub level category data?
sub sublevel_mode{
	my ($this) = @_;
	if ($this->category_edit_mode()){
		return 0;
	}
	# If a path is specified, we are in sub directory mode
	if ($this->path()){
		return 1;
	}
	# When displaying product info, we cache all sub category and product info
	if ($this->product_id()){
		return 1;
	}
	return 0;
}
# Are we caching search data?
sub search_mode{
	my ($this) = @_;
	if ($this->is_search()){
		return 1;
	}
	return 0;
}
# Cache category data
sub save_cache_category{
	# $category: category data hash
	my ($this,$category) = @_;
	my $id = $category->{catalog_category_id};
	# Invalid id
	return if !$id;
	# We've cached the record already
	return if (exists($share{catalog}{cache}{catalog_category}{category_id}{$id}));
	# Cache category by category types
	my $category_type;
	if ($category->{type} eq "category" or !$category->{type}){
		$category_type = "category";
	}else{
		$category_type = $category->{type};
	}
	$share{catalog}{cache}{catalog_category}{category_id}{$id} = $category;
	$share{catalog}{cache}{catalog_category}{category_type}{$category_type} = 1;
	push(@{$share{catalog}{cache}{catalog_category}{$category_type}},$category);
	return;
}
# Cache category join data
sub save_cache_category_join{
	# $join: join data hash
	my ($this,$join) = @_;
	my $id = $join->{catalog_category_join_id};
	# Invalid id
	return if !$id;
	# We've cached the record already
	return if (exists($share{catalog}{cache}{catalog_category_join}{join_id}{$id}));
	$share{catalog}{cache}{catalog_category_join}{join_id}{$id} = $join;
	my $parent = $join->{parent};
	my $child = $join->{child};
	# Cache parent join data
	if ($parent){
		push(@{$share{catalog}{cache}{catalog_category_join}{parent}{$parent}},$join);
	}
	# Cache child join data
	if ($child){
		push(@{$share{catalog}{cache}{catalog_category_join}{child}{$child}},$join);
	}
	return;
}
# Cache section data
sub save_cache_section{
	# $site: section data hash
	my ($this,$site) = @_;
	my $id = $site->{section_id};
	# Invalid id
	return if !$id;
	# We've cached the record already
	return if (exists($share{catalog}{cache}{section}{section_id}{$id}));
	$share{catalog}{cache}{section}{section_id}{$id} = $site;
	return;
}
# Cache product join data
sub save_cache_product_join{
	# $join: product join data hash
	my ($this,$join) = @_;
	my $id = $join->{catalog_product_category_join_id};
	# Invalid id
	return if !$id;
	# We've cached the record already
	return if (exists($share{catalog}{cache}{catalog_product_category_join}{join_id}{$id}));
	$share{catalog}{cache}{catalog_product_category_join}{join_id}{$id} = $join;
	my $product_id = $join->{catalog_product_id};
	my $category_id = $join->{catalog_category_id};
	# Cache by product
	if ($product_id){
		push(@{$share{catalog}{cache}{catalog_product_category_join}{product}{$product_id}},$join);
	}
	# Cache by category
	if ($category_id){
		push(@{$share{catalog}{cache}{catalog_product_category_join}{category}{$category_id}},$join);
	}
	return;
}
# Cache product data
sub save_cache_product{
	# $product: product data hash
	my ($this,$product) = @_;
	my $id = $product->{catalog_product_id};
	# Invalid id
	return if !$id;
	# We've cached the record already
	return if (exists($share{catalog}{cache}{catalog_product}{product_id}{$id}));
	$share{catalog}{cache}{catalog_product}{product_id}{$id} = $product;
	return;
}
# Cache product attachment data
sub save_cache_product_attachment{
	# $attachment: product attachment data hash
	my ($this,$attachment) = @_;
	my $id = $attachment->{catalog_product_attachment_id};
	# Invalid id
	return if !$id;
	# We've cached the record already
	return if (exists($share{catalog}{cache}{catalog_product_attachment}{attachment_id}{$id}));
	my $productid = $attachment->{catalog_product_id};
	my $attachment_type = $attachment->{type};
	# Hash - All attachments indexed by attachment id			
	$share{catalog}{cache}{catalog_product_attachment}{attachment_id}{$id} = $attachment;
	my $heading = $attachment->{heading} || "_null";
	if ($attachment_type eq "primary-image"){
		# Array - Primary image
		push(@{$share{catalog}{cache}{catalog_product_attachment}{primary}{$productid}},$id);
		push(@{$share{catalog}{cache}{catalog_product_attachment}{primary_heading}{$productid}{$heading}},$id);
	}elsif ($attachment_type eq "secondary-image"){
		# Array - Secondary image			
		push(@{$share{catalog}{cache}{catalog_product_attachment}{secondary}{$productid}},$id);
		push(@{$share{catalog}{cache}{catalog_product_attachment}{secondary_heading}{$productid}{$heading}},$id);
	}elsif ($attachment_type eq "download"){
		# Array - Documents			
		push(@{$share{catalog}{cache}{catalog_product_attachment}{download}{$productid}},$id);
		push(@{$share{catalog}{cache}{catalog_product_attachment}{download_heading}{$productid}{$heading}},$id);
	}
	return;
}
# Cache product data
sub build_cache_product{
	my ($this) = @_;
	my $db = $this->{db};
	my $productid = $this->product_id();
	# Fetch a product info and attachments
	my $sql = <<END;
select
	c.catalog_category_id as id_category_1,
	c.title as title_category_1,
	c.description as description_category_1,
	pcj.catalog_product_category_join_id as id_product_join_1,
	pcj.catalog_category_id as category_product_join_1,
	pcj.catalog_product_id as product_product_join_1,
	p.catalog_product_id as id_product_1,
	p.code as code_product_1,
	p.title as title_product_1,
	p.summary as summary_product_1,
	p.details as details_product_1,
	p.price as price_product_1,
	p.status as status_product_1,
	p.options as options_product_1,
	p.sortkey as sortkey_product_1,	
	pa.catalog_product_attachment_id as id_product_attachment_1,
	pa.catalog_product_id as product_id_product_attachment_1,
	pa.type as type_product_attachment_1,
	pa.description as description_product_attachment_1,
	pa.heading as heading_product_attachment_1,
	SUBSTRING(pa.upload,1,100) as upload_product_attachment_1,
	SUBSTRING(pa.thumbnail,1,100) as thumbnail_product_attachment_1
from
	catalog_product as p 
	left join catalog_product_attachment as pa on p.catalog_product_id = pa.catalog_product_id 
	left join catalog_product_category_join as pcj on pcj.catalog_product_id = p.catalog_product_id 
	left join catalog_category as c on c.catalog_category_id = pcj.catalog_category_id 	
where
	p.catalog_product_id = $productid
END
	if (!$this->admin_mode()){
		$sql .= " and (p.status like 'enable%' or p.status is null or p.status = '')";
	}

	my @records = $db->custom_query($sql);
	foreach my $record(@records){
		# Category
		my %category = (
			catalog_category_id=>$record->{id_category_1},
			title=>$record->{title_category_1},
			type=>$record->{type_category_1},
			status=>$record->{status_category_1},
			rank=>$record->{rank_category_1},
			description=>$record->{description_1},
		);

		# Category join
		my %category_join = (
			catalog_category_join_id=>$record->{id_join_1},
			parent=>$record->{parent_join_1},
			child=>$record->{child_join_1},
		);
		# Product
		my %product = (
			catalog_product_id=>$record->{id_product_1},
			code=>$record->{code_product_1},
			title=>$record->{title_product_1},
			summary=>$record->{summary_product_1},
			details=>$record->{details_product_1},
			price=>$record->{price_product_1},
			status=>$record->{status_product_1},
			options=>$record->{options_product_1},
			sortkey=>$record->{sortkey_product_1},
		);
		# Product join
		my %product_join = (
			catalog_product_category_join_id=>$record->{id_product_join_1},
			catalog_category_id=>$record->{category_product_join_1},
			catalog_product_id=>$record->{product_product_join_1},
		);
		# Product attachment
		my %product_attachment = (
			catalog_product_attachment_id=>$record->{id_product_attachment_1},
			catalog_product_id=>$record->{product_id_product_attachment_1},
			type=>$record->{type_product_attachment_1},
			description=>$record->{description_product_attachment_1},
			heading=>$record->{heading_product_attachment_1},
			upload=>$record->{upload_product_attachment_1},
			thumbnail=>$record->{thumbnail_product_attachment_1},
		);

		$this->save_cache_category(\%category);
		$this->save_cache_category_join(\%category_join);
		$this->save_cache_product(\%product);
		$this->save_cache_product_join(\%product_join);
		$this->save_cache_product_attachment(\%product_attachment);
	}
	return;	
}
# Build cache for displaying sub level catalog
sub build_cache_sub_level_category{
	# $sectionid: section id(array reference)
	my ($this,$sectionids) = @_;
	my @sectionids = @$sectionids;
	my $db = $this->{db};
	my @paths = split(/\//,$this->path());
	my $current_path = pop(@paths);
	shift(@paths);
	my $sql;
	# Cache parent categories
	if (scalar(@paths) > 0){	
		$sql = "select * from catalog_category where catalog_category_id in (".join(", ",@paths).")";
		my @categories = $db->custom_query($sql);
		foreach my $category(@categories){
			$this->save_cache_category($category);
		}
	}
	# Cache current category, sub categories and product info
	$sql = <<END;
select 
	c1.catalog_category_id as id_category_1,
	c1.title as title_category_1,
	c1.type as type_category_1,
	c1.status as status_category_1,	
	c1.rank as rank_category_1,
	c1.description as description_1,
	c2.catalog_category_id as id_category_2,
	c2.title as title_category_2,
	c2.type as type_category_2,
	c2.status as status_category_2,	
	c2.rank as rank_category_2,
	c2.description as description_2,
	cj1.catalog_category_join_id as id_join_1,
	cj1.parent as parent_join_1,
	cj1.child as child_join_1,
	pcj1.catalog_product_category_join_id as id_product_join_1,
	pcj1.catalog_category_id as category_product_join_1,
	pcj1.catalog_product_id as product_product_join_1,
	pcj2.catalog_product_category_join_id as id_product_join_2,
	pcj2.catalog_category_id as category_product_join_2,
	pcj2.catalog_product_id as product_product_join_2,
	p1.catalog_product_id as id_product_1,
	p1.code as code_product_1,
	p1.title as title_product_1,
	p1.summary as summary_product_1,
	p1.details as details_product_1,
	p1.price as price_product_1,
	p1.status as status_product_1,
	p1.options as options_product_1,
	p1.sortkey as sortkey_product_1,	
	p2.catalog_product_id as id_product_2,
	p2.code as code_product_2,
	p2.title as title_product_2,
	p2.summary as summary_product_2,
	p2.details as details_product_2,
	p2.price as price_product_2,
	p2.status as status_product_2,
	p2.options as options_product_2,
	p2.sortkey as sortkey_product_2,
	pa1.catalog_product_attachment_id as id_product_attachment_1,
	pa1.catalog_product_id as product_id_product_attachment_1,
	pa1.type as type_product_attachment_1,
	pa1.description as description_product_attachment_1,
	pa1.heading as heading_product_attachment_1,
	SUBSTRING(pa1.upload,1,100) as upload_product_attachment_1,
	SUBSTRING(pa1.thumbnail,1,100) as thumbnail_product_attachment_1,
	pa2.catalog_product_attachment_id as id_product_attachment_2,
	pa2.catalog_product_id as product_id_product_attachment_2,
	pa2.type as type_product_attachment_2,
	pa2.description as description_product_attachment_2,
	pa2.heading as heading_product_attachment_2,
	SUBSTRING(pa2.upload,1,100) as upload_product_attachment_2,
	SUBSTRING(pa2.thumbnail,1,100) as thumbnail_product_attachment_2
from 
	catalog_category as c1
	left join catalog_category_join as cj1 on c1.catalog_category_id = cj1.parent 
	left join catalog_category as c2 on c2.catalog_category_id = cj1.child 
	left join catalog_product_category_join as pcj1 on c1.catalog_category_id = pcj1.catalog_category_id 
	left join catalog_product_category_join as pcj2 on c2.catalog_category_id = pcj2.catalog_category_id 
	left join catalog_product as p1 on p1.catalog_product_id = pcj1.catalog_product_id 
	left join catalog_product as p2 on p2.catalog_product_id = pcj2.catalog_product_id 
	left join catalog_product_attachment as pa1 on pa1.catalog_product_id = p1.catalog_product_id 
	left join catalog_product_attachment as pa2 on pa2.catalog_product_id = p2.catalog_product_id 	
where 
	c1.catalog_category_id = $current_path
order by 
	title_category_1 asc, 
	title_category_2 asc
END
	my @records = $db->custom_query($sql);

	foreach my $record(@records){
		# Reorganize and cache data
		
		# Category
		my %parent = (
			catalog_category_id=>$record->{id_category_1},
			title=>$record->{title_category_1},
			type=>$record->{type_category_1},
			status=>$record->{status_category_1},
			rank=>$record->{rank_category_1},
			description=>$record->{description_1},
		);
		# Category
		my %child = (
			catalog_category_id=>$record->{id_category_2},
			title=>$record->{title_category_2},
			type=>$record->{type_category_2},
			status=>$record->{status_category_2},
			rank=>$record->{rank_category_2},
			description=>$record->{description_2},
		);
		# Category join
		my %category_join = (
			catalog_category_join_id=>$record->{id_join_1},
			parent=>$record->{parent_join_1},
			child=>$record->{child_join_1},
		);
		# Product
		my %parent_product = (
			catalog_product_id=>$record->{id_product_1},
			code=>$record->{code_product_1},
			title=>$record->{title_product_1},
			summary=>$record->{summary_product_1},
			details=>$record->{details_product_1},
			price=>$record->{price_product_1},
			status=>$record->{status_product_1},
			options=>$record->{options_product_1},
			sortkey=>$record->{sortkey_product_1},
		);
		# Product join
		my %parent_product_join = (
			catalog_product_category_join_id=>$record->{id_product_join_1},
			catalog_category_id=>$record->{category_product_join_1},
			catalog_product_id=>$record->{product_product_join_1},
		);
		# Product attachment
		my %parent_product_attachment = (
			catalog_product_attachment_id=>$record->{id_product_attachment_1},
			catalog_product_id=>$record->{product_id_product_attachment_1},
			type=>$record->{type_product_attachment_1},
			description=>$record->{description_product_attachment_1},
			heading=>$record->{heading_product_attachment_1},
			upload=>$record->{upload_product_attachment_1},
			thumbnail=>$record->{thumbnail_product_attachment_1},
		);
		# Product
		my %child_product = (
			catalog_product_id=>$record->{id_product_2},
			code=>$record->{code_product_2},
			title=>$record->{title_product_2},
			summary=>$record->{summary_product_2},
			details=>$record->{details_product_2},
			price=>$record->{price_product_2},
			status=>$record->{status_product_2},
			options=>$record->{options_product_2},
			sortkey=>$record->{sortkey_product_2},
		);
		# Product join
		my %child_product_join = (
			catalog_product_category_join_id=>$record->{id_product_join_2},
			catalog_category_id=>$record->{category_product_join_2},
			catalog_product_id=>$record->{product_product_join_2},
		);
		# Product attachment
		my %child_product_attachment = (
			catalog_product_attachment_id=>$record->{id_product_attachment_2},
			catalog_product_id=>$record->{product_id_product_attachment_2},
			type=>$record->{type_product_attachment_2},
			description=>$record->{description_product_attachment_2},
			heading=>$record->{heading_product_attachment_2},
			upload=>$record->{upload_product_attachment_2},
			thumbnail=>$record->{thumbnail_product_attachment_2},
		);
		$this->save_cache_category(\%parent);
		$this->save_cache_category(\%child);
		$this->save_cache_category_join(\%category_join);
		$this->save_cache_product(\%parent_product);
		$this->save_cache_product_join(\%parent_product_join);
		$this->save_cache_product_attachment(\%parent_product_attachment);
		$this->save_cache_product(\%child_product);
		$this->save_cache_product_join(\%child_product_join);
		$this->save_cache_product_attachment(\%child_product_attachment);
	}
	return;
}

# Cache top level category display
sub build_cache_top_level_category{
	# $sectionids: section ids(array reference)
	my ($this,$sectionids) = @_;
	# Cache 1st level and 2nd level categories
	my $sql = <<END;
select 
	c2.catalog_category_id as id_1,
	c2.title as title_1,
	c2.type as type_1,
	c2.status as status_1,	
	c2.rank as rank_1,	
	c3.catalog_category_id as id_2,
	c3.title as title_2,
	c3.type as type_2,
	c3.status as status_2,	
	c3.rank as rank_2,
	j2.catalog_category_join_id as join_id_1,
	j2.parent as parent_1,
	j2.child as child_1 
from 
	catalog_category as c3 
	left join catalog_category_join as j2 on c3.catalog_category_id = j2.child 
	left join catalog_category as c2 on c2.catalog_category_id = j2.parent 
	left join catalog_category_join as j1 on c2.catalog_category_id = j1.child 
	left join catalog_category as c1 on c1.catalog_category_id = j1.parent 
	where j1.parent is NULL
END
	$sql .= " and (c3.section_id = ".join(" || c3.section_id = ",@$sectionids).")";
	$sql .= " order by title_1 asc, title_2 asc";

	my $db = $this->{db};

	my @records = $db->custom_query($sql);
	foreach my $record(@records){
		my %parent = (
			catalog_category_id=>$record->{id_1},
			title=>$record->{title_1},
			type=>$record->{type_1},
			status=>$record->{status_1},
			rank=>$record->{rank_1},
		);
		my %child = (
			catalog_category_id=>$record->{id_2},
			title=>$record->{title_2},
			type=>$record->{type_2},
			status=>$record->{status_2},
			rank=>$record->{rank_2},
		);
		my %join = (
			catalog_category_join_id=>$record->{join_id_1},
			parent=>$record->{parent_1},
			child=>$record->{child_1},
		);
		$this->save_cache_category(\%parent);
		$this->save_cache_category(\%child);
		$this->save_cache_category_join(\%join);		
	}
	return;
}

# Cache all categories
sub build_cache_all_categories{	
	# $sectionids: section id(array reference)
	my ($this,$sectionids) = @_;
	my $db = $this->{db};
	my @sectionids = @$sectionids;
	# Cache category information
	my $sql = <<END;
select
	catalog_category.catalog_category_id,
	catalog_category.section_id,
	catalog_category.title,
	catalog_category.type,
	catalog_category.status,
	catalog_category.description,
	catalog_category.rank
from
	catalog_category
where
END
	$sql .= " section_id = ".join(" or section_id = ",@sectionids);
	$sql .= " order by catalog_category.section_id asc, catalog_category.title asc";
	my @categories = $db->custom_query($sql);

	my @categoryids;
	# Cache every category
	foreach my $category(@categories){
		my $categoryid = $category->{catalog_category_id};		
		$this->save_cache_category($category);
		push(@categoryids,$categoryid);
	}
	# Fetch category joins
	my $cid_list = join(", ",@categoryids);
	$sql = <<END;
select 
	*
from
	catalog_category_join
where
	parent in ($cid_list) or child in ($cid_list)
END
	my @joins = $db->custom_query($sql);
	foreach my $join(@joins){
		# Cache category joins
		$this->save_cache_category_join($join);
	}
	return;
}

# Cache all product info
sub build_cache_all_products{
	# $where: where clause(array reference)
	my ($this,$where) = @_;
	my $db = $this->{db};
	my @where = @$where;
	my $where_clause = "(".join(" or ",@where).")";
	# Fetch product joins
	my $sql = <<END;
select
	catalog_product_category_join.*
from
	catalog_product_category_join
	left join catalog_category on catalog_product_category_join.catalog_category_id = catalog_category.catalog_category_id
where
	$where_clause
END
	# Cache product category joins
	my @joins = $db->custom_query($sql);

	my %join;
	my @product_id;
	my @category_id;
	# Cache product category join records by both category and product
	foreach my $join(@joins){
		my $product_id = $join->{catalog_product_id};
		my $category_id = $join->{catalog_category_id};
		$this->save_cache_product_join($join);
		if ($product_id){
			push(@product_id,$product_id);
		}
		if ($category_id){
			push(@category_id,$category_id);
		}
	}

	# Cache products
	if (@product_id > 0){

		# Product id list
		my $pid_list = join(", ",@product_id);
		# Fetch products
		$sql = <<END;
select
	*
from
	catalog_product
where
	catalog_product_id in ($pid_list)
END

		my @products = $db->custom_query($sql);
		foreach my $product(@products){
			$this->save_cache_product($product);
		}
	}	
	# Cache all product attachments
	if (@product_id > 0){
	
		# Product id list
		my $pid_list = join(", ",@product_id);
		# Fetch product attachments
		$sql = <<END;
select
	catalog_product_attachment_id,	
	catalog_product_id,
	type,
	description,
	heading,
	SUBSTRING(upload,1,100) as upload,
	SUBSTRING(thumbnail,1,100) as thumbnail	
from 
	catalog_product_attachment
where
	catalog_product_id in ($pid_list)
END

		my @attachments = $db->custom_query($sql);
		foreach my $attachment(@attachments){
			# Cache product attachments
			$this->save_cache_product_attachment($attachment);
		}
	}
	return;
}
# Get cached data
sub cache_data{
	my ($this,$table,$type,$id) = @_;
	# If no cache found, build catalog cache

	# Check the cache flag and see if we need to build data cache
	if (!$share{catalog}{catalog_cache}){
		$this->build_cache();
	}else{
		# Verify that there are data existed in the cache
		if (scalar(keys(%{$share{catalog}{cache}})) < 1){
			$this->build_cache();
		}
	}
	my $admin_mode = $this->admin_mode();
	my $cache = $share{catalog}{cache};
	if ($table eq "catalog_product_category_join"){
		if ($type eq "join_id"){
			return $cache->{$table}{join_id}{$id} || {};
		}
		if (exists($cache->{$table}{$type}{$id})){			
			return $cache->{$table}{$type}{$id} || [];
		}
		return [];
	}
	if ($table eq "catalog_product"){
		if ($type eq "all"){
			return $cache->{$table}{product_id} || {};
		}
		if (exists($cache->{$table}{$type}{$id})){
			return $cache->{$table}{$type}{$id} || {}
		}
		return {};
	}
	if ($table eq "catalog_product_attachment"){
		if ($type =~ /(primary|secondary|download)_heading/i){
			return $cache->{$table}{$type}{$id} || {};
		}
		if ($type =~ /primary|secondary|download/i){
			return $cache->{$table}{$type}{$id} || [];
		}
		if ($type eq "attachment_id"){
			return $cache->{$table}{$type}{$id} || {};
		}
	}
	if ($table eq "catalog_category"){
		if ($type eq "all"){
			return $cache->{$table}{category_id} || {};
		}elsif ($type eq "category_id"){
			return $cache->{$table}{$type}{$id} || {};
		}elsif ($type eq "category_type"){
			return $cache->{$table}{$type} || {};
		}else{
			return $cache->{$table}{$type} || [];
		}
	}
	if ($table eq "catalog_category_join"){
		if ($type =~ /parent|child/i){
			return $cache->{$table}{$type}{$id} || [];
		}
		if ($type eq "join_id"){
			return $cache->{$table}{$type}{$id} || {};
		}
		if ($type eq "all"){
			return $cache->{$table}{join_id} || {};
		}
	}
	if ($table eq "section"){
		return $cache->{$table}{$type}{$id} || {};
	}	
	return;
}
# Enable category sharing between sections
sub enableCategorySharing{
	my ($this) = @_;
	return $this->getModuleSetting("enable_category_sharing");
}
# Get section data
sub getSection{
	# $id: section id
	my ($this,$id) = @_;
	return {} if !$this->is_int($id);
	return $this->cache_data("section","section_id",$id);
}
# Get product category join records by product id
sub getProductJoinByProduct{
	# $id: product id
	my ($this,$id) = @_;
	return [] if !$this->is_int($id);
	return $this->getProductJoinRecord($id,"product");	
}
# Get product category join records by category id
sub getProductJoinByCategory{
	# $id: category id
	my ($this,$id) = @_;
	return [] if !$this->is_int($id);
	return $this->getProductJoinRecord($id,"category");
}
# Get product join data
sub getProductJoin{
	# $id: product join id
	my ($this,$id) = @_;
	return $this->cache_data("catalog_product_category_join","join_id",$id) || {};
}
# Get product join data by category or product
sub getProductJoinRecord{
	my ($this,$id,$type) = @_;
	return $this->cache_data("catalog_product_category_join",$type,$id) || [];	
}
# Get all categories
sub getAllCategory{
	my ($this) = @_;
	my %categories = %{$this->cache_data("catalog_category","all")};
	my @categories;
	foreach my $categoryid(keys(%categories)){
		push(@categories,$categories{$categoryid});
	}
	return \@categories;
}
# Get all products
sub getAllProduct{
	my ($this) = @_;
	my %products = %{$this->cache_data("catalog_product","all")};
	my @products;
	foreach my $productid(keys(%products)){
		push(@products,$products{$productid});
	}
	return \@products;
}
# Get category Type
sub getCategoryType{
	my ($this) = @_;
	my %types = %{$this->cache_data("catalog_category","category_type")};
	my @types;
	foreach my $key(keys(%types)){
		push(@types,$key);
	}
	return \@types;
}
# Get all category joins
sub getAllCategoryJoin{
	my ($this) = @_;
	my %joins = %{$this->cache_data("catalog_category_join","all")};
	my @joins;
	foreach my $id(keys(%joins)){
		push(@joins,$joins{$id});
	}
	return \@joins;
}
# Get sub categories
sub getSubCategory{
	# $id: category id
	my ($this,$id) = @_;
	my @joins= @{$this->cache_data("catalog_category_join","parent",$id)};
	my @categories;
	foreach my $join(@joins){
		my %category = %{$this->getCategory($join->{child})};
		push(@categories,\%category);
	}
	return \@categories;
}
sub getCategoryJoin{
	# Category join id
	my ($this,$id) = @_;
	return {} if !$id;
	return $this->cache_data("catalog_category_join","join_id",$id) || {};
}
# Get join records by child id
sub getJoinByChild{
	# $id: category join id
	my ($this,$id) = @_;
	return $this->getCategoryJoinRecord($id,"child");
}
# Get join records by parent id
sub getJoinByParent{
	# $id: cagetory join id
	my ($this,$id) = @_;
	return $this->getCategoryJoinRecord($id,"parent");
}
sub getCategoryJoinRecord{
	my ($this,$id,$type) = @_;
	return [] if !$id;
	return [] if !$type;
	return $this->cache_data("catalog_category_join",$type,$id) || [];
}
# Get a product's category id
sub getProductCategoryID{
	# $data: product record
	my ($this,$data) = @_;
	my $product_id = $data->{catalog_product_id};
	return 0 if !$product_id;
	my @joins = @{$this->cache_data("catalog_product_category_join","product",$product_id)};
	return 0 if @joins < 1;
	# If a product is attached to multiple categories, we pick the last category from the database results
	return $joins[-1]{catalog_category_id} || 0;
}
# Get a product record
sub getProduct{
	# $id: product id
	my ($this,$id) = @_;
	return {} if !$id;
	return $this->cache_data("catalog_product","product_id",$id) || {};
}
# Get a category record
sub getCategory{
	# $id: category id
	my ($this,$id) = @_;
	return {} if !$id;
	my %record = %{$this->cache_data("catalog_category","category_id",$id)};
	return \%record;
}
# Get category by category type
sub getCategoryByType{
	my ($this,$type) = @_;
	$type = !$type ? "category" : $type;
	return $this->cache_data("catalog_category",$type) || [];
}
# Get product attachment by group
sub getProductAttachmentGroupRecord{
	# $type: primary|secondary|download
	# $id: product id
	my ($this,$type,$id) = @_;
	my %group = %{$this->cache_data("catalog_product_attachment",$type,$id)};
	my %attachments;
	foreach my $heading(keys(%group)){
		my @attachments;
		foreach my $attachmentid(@{$group{$heading}}){
			my %attach = %{$this->getAttachment($attachmentid)};
			push(@attachments,\%attach);
		}
		@attachments = @{$this->sortProductAttachment(\@attachments)};
		$attachments{$heading} = \@attachments;
	}
	return \%attachments;
}
# Get product attachment record
sub getProductAttachmentRecord{
	# $type: primary|secondary|download
	# $id: product id
	my ($this,$type,$id) = @_;
	my @ids = @{$this->cache_data("catalog_product_attachment",$type,$id)};
	my @list;
	foreach my $attachmentid(@ids){
		my %attachment = %{$this->getAttachment($attachmentid)};
		push(@list,\%attachment);
	}
	@list = @{$this->sortProductAttachment(\@list)};
	return \@list;
}
# Sort product attachments
sub sortProductAttachment{
	my ($this,$data) = @_;
	@$data = sort {$a->{description} cmp $b->{description}} @$data;
	return $data;
}
# Get all product attachments
sub getProductAttachmentGroupAll{
	# $id: product id
	my ($this,$id) = @_;
	my %download = %{$this->getProductAttachmentGroupRecord("download_heading",$id)};
	my %secondary = %{$this->getProductAttachmentGroupRecord("secondary_heading",$id)};
	my %attachments = %download;
	foreach my $key (keys(%secondary)){
		if (exists($attachments{$key})){
			# Verify that the data in $attachments{$key} is an array. If the data isn't an array, we initialize the array.
			if (ref($attachments{$key}) !~ /array/i){
				$attachments{$key} = [];
			}
			if (exists($secondary{$key})){
				my @secondary;
				# Verify that the data in $secondary{$key} is an array. If the data isn't an array, we set an empty array.
				if (ref($secondary{$key}) =~ /array/i){
					@secondary = @{$secondary{$key}};
				}else{
					@secondary = ();
				}
				# Merge attachment and images into 1 array list when they are grouped in the same heading
				my @merge = (@{$attachments{$key}},@secondary);
				$attachments{$key} = \@merge;
			}
		}else{
			# Verify that the data in $secondary{$key} is an array. If the data isn't an array, we set an empty array.
			if (ref($secondary{$key}) =~ /array/i){
				$attachments{$key} = $secondary{$key};
			}else{
				$attachments{$key} = [];
			}
		}
	}
	return \%attachments;
}
# Get downloadable attachments only
sub getProductAttachmentGroupDownload{
	# $id: product id
	my ($this,$id) = @_;
	my %attachments = %{$this->getProductAttachmentGroupRecord("download_heading",$id)};
	return \%attachments;
}
# Get attachment images only
sub getProductAttachmentGroupImageSecondary{
	# $id: product id
	my ($this,$id) = @_;
	my %attachments = %{$this->getProductAttachmentGroupRecord("secondary_heading",$id)};
	return \%attachments;
}
# Get image attachments.  Excluding primary images
sub getProductAttachmentAll{
	# $id: product id
	my ($this,$id) = @_;
	my @attachments = (@{$this->getProductAttachmentRecord("secondary",$id)},@{$this->getProductAttachmentRecord("download",$id)});
	return \@attachments;
}
# Get downloadable documents.
sub getProductAttachmentDownload{
	# $id: product id
	my ($this,$id) = @_;
	my @downloads = @{$this->getProductAttachmentRecord("download",$id)};
	return \@downloads;
}
# Get image attachments. Excluding downloads and primary images
sub getProductAttachmentImageSecondary{
	# $id: product id
	my ($this,$id) = @_;	
	my @images = (@{$this->getProductAttachmentRecord("secondary",$id)});	
	return \@images;	
}
# Get image attachments. Excluding downloads
sub getProductAttachmentImage{
	# $id: product id
	my ($this,$id) = @_;	
	my @images = (@{$this->getProductAttachmentRecord("primary",$id)},@{$this->getProductAttachmentRecord("secondary",$id)});	
	return \@images;	
}
# Get product by category
sub getProductByCategory{
	# $id: category id
	my ($this,$id) = @_;
	my @joins = @{$this->cache_data("catalog_product_category_join","category",$id)};
	my @product_list;
	my $admin_mode = $this->admin_mode();
	foreach my $join(@joins){
		my %product = %{$this->getProduct($join->{catalog_product_id})};
		if (!$admin_mode){
			# Don't show disabled products to the public
			if ($product{status} =~ /disable/i){
				next;
			}
		}
		push(@product_list,\%product);
	}
	# Use sort key to sort products
	foreach my $product(@product_list){
		$product->{sort_key_field} = $product->{sortkey}.$product->{title};
	}
	@product_list = sort {$a->{sort_key_field} cmp $b->{sort_key_field}} @product_list;
	return \@product_list;
}
# Get the first product image.
# Image selection condition:
#	1. Get any primary image
#	2. If no primary image is found, get any secondary image
sub getProductAttachmentSingleImage{
	# $id: category id
	my ($this,$id) = @_;
	my @products = @{$this->getProductByCategory($id)};
	my @attachment_list;
	foreach my $product(@products){
		my @images = @{$this->getProductAttachmentImage($product->{catalog_product_id})};
		my %attachment;
		if (@images > 0){
			%attachment = %{$images[0]};
		}else{
			# No image found, we need to set the catalog_product_id so the system
			# would generate a valid product detail url based on the attachment record
			$attachment{catalog_product_id} = $product->{catalog_product_id};
		}
		push(@attachment_list,\%attachment);
	}
	return \@attachment_list;
}
sub getAttachment{
	# $id: attachment id
	my ($this,$id) = @_;
	return $this->cache_data("catalog_product_attachment","attachment_id",$id) || {};
}
# Get primary images for a product
sub getProductPrimaryImage{
	# $id: product id
	my ($this,$id) = @_;
	my @primary_images = @{$this->cache_data("catalog_product_attachment","primary",$id)};
	my @image_list;
	foreach my $image(@primary_images){
		my %attachment = %{$this->getAttachment($image)};
		push(@image_list,\%attachment);
	}
	return \@image_list;
}
# This routine is written to fetch cached data.
# It only fetches category, product and attachment information
sub getRecord{
	# $table: table
	# $id: record id
	my ($this,$table,$id) = @_;
	my %lookup = (
		catalog_product=>"getProduct",
		catalog_category=>"getCategory",
		catalog_product_attachment=>"getAttachment",
		catalog_category_join=>"getCategoryJoin",
		catalog_product_category_join=>"getProductJoin",
	);
	# $id isn't a integer
	return {} if !$this->is_int($id);
	# No table specified
	return {} if !$table;
	# No lookup specified
	return {} if !exists($lookup{$table});
	# Run internal data fetching routine
	my $func = $lookup{$table};
	return $this->$func($id) || {};
}
# Get a random product image from a random product in a category
sub getSingleRandomProductImageByCategory{
	# $categoryid: category id
	my ($this,$categoryid) = @_;
	# Get all products from a category
	my @products = @{$this->getProductByCategory($categoryid)};
	my $null_graphic = "<img src=\"$config{server}{server}$config{server}{HTMLpath}/_Modules/Catalog/null.gif\" border=\"0\">";
	# No product found.
	if (@products < 1){
		return $null_graphic;
	}
	my $product_count = scalar(@products);
	# Randomly chosen a product
	my $product_index = int((rand() * rand() * 10 * $product_count) % $product_count);
	my $product = $products[$product_index];
	# Get all images from a product
	my @images = @{$this->getProductAttachmentImage($product->{catalog_product_id})};
	if (@images < 1){
		return $null_graphic;
	}
	my $image_count = scalar(@images);
	# Randomly chosen an image
	my $image_index = int((rand() * rand() * 10 * $image_count) % $image_count);
	# Image url
	my $image_url = $this->getProductAttachment($images[$image_index]);
	if (!$this->isImage($image_url)){
		return $null_graphic;
	}
	my $image = "<img src=\"$image_url\" border=\"0\" height=\"250\">";
	return $image;
}

# Find sub categories
sub findSubCategory{
	# $catid: category id
	my ($this,$catid) = @_;
	# Get sub categories
	my @list = @{$this->getSubCategory($catid)};
	my @categories;
	foreach my $category(@list){
		$category = $this->updateTitle($category);
		# Don't show 'disabled' categories in page.cgi
		if (!$this->isCategoryViewable($category)){
			next;
		}
		push(@categories,$category);
	}
	@categories = @{$this->sortArrayCategory(\@categories)};
	return \@categories;
}
# Find parent categories
sub findParentCategory{
	# $sectionid: section id
	# $type: category type
	my ($this,$sectionid,$type) = @_;
	my @categories = @{$this->getCategoryByType($type)};
	# Fetch all join tables
	my @joins = @{$this->getAllCategoryJoin()};
	# Sort join records by 'child' column
	my %categories = %{&keywise("child",\@joins)};
	my @parents;
	# We use the 'child' value to determine whether a category is a parent category or not
	foreach my $cat(@categories){
		if (!exists($categories{$cat->{catalog_category_id}}) || 
			(exists($categories{$cat->{catalog_category_id}}) && !$categories{$cat->{catalog_category_id}}{parent})){
			$cat = $this->updateTitle($cat);
			# Don't show 'disabled' category in page.cgi
			if (!$this->isCategoryViewable($cat)){
				next;
			}
			push(@parents,$cat);
		}
	}
	@parents = @{$this->sortArrayCategory(\@parents)};
	return \@parents;
}
# Sort an array of category by rank then by title
sub sortArrayCategory{
	# $list: array reference to a list of categories
	my ($this,$list) = @_;
	@$list = sort {$a->{rank} <=> $b->{rank} || $a->{title} cmp $b->{title}} @$list;
	return $list;
}

#----------------------------------------------------------------------

# Is this product viewable by the public?
sub isProductViewable{
	my ($this,$data) = @_;
	# Always show a product in admin mode
	return 1 if $this->admin_mode();
	$data = $data || {};
	# We show the product if the status isn't set or the status is set to 'enabled'
	if (keys(%$data) > 1 && (!$data->{status} || $data->{status} =~ /enable/i)){
		return 1;
	}
	return 0;
}
# Is this category viewable by public?
sub isCategoryViewable{
	my ($this,$data) = @_;
	# Always show a product in admin mode
	return 1 if $this->admin_mode();
	$data = $data || {};
	# We show the category if the status isn't set or the status is set to 'enabled'
	if (keys(%$data) > 1 && (!$data->{status} || $data->{status} =~ /enable/i)){
		return 1;
	}
	return 0;
}

#----------------------------------------------------------------------
# Dynamic content parameter dialog
sub getDynContentParameter{
	# $sectionid: section id
	my ($this,$sectionid) = @_;
	return if !$sectionid;
	my @lists = (
		{label=>"Browse",value=>"action=browse"},
		{label=>"Search",value=>"action=search"},
		{label=>"New Products",value=>"action=what_is_new"},
	);
	my $out = <<END;
<table>
<tr>
	<td>
		<b>Select an option:</b>
	</td>
	<td>
		<select name="param">
		<option value="">&lt;Default&gt;</option>
END
	foreach my $list(@lists){
		$out .= <<END;
		<option value="$list->{value}">$list->{label}</option>
END
	}
	$out .= <<END;
		</select>
	</td>
</tr>
</table>
END
	return $out;
}
#----------------------------------------------------------------------
# Category detail template
sub getCategoryDetailTemplate{
	# $type: template id
	my ($this,$templateid) = @_;
	my $use_template = 0;
	if ($this->getModuleSetting("enable_category_detail_template")){
		$use_template = 1;
	}
	return $this->getTemplate($templateid,$use_template);
}
# Product detail template
sub getProductDetailTemplate{
	# $type: template id
	my ($this,$templateid) = @_;
	my $use_template = 0;
	if ($this->getModuleSetting("enable_product_detail_template")){
		$use_template = 1;
	}
	return $this->getTemplate($templateid,$use_template);
}
# 'What is new' page template
sub getWhatIsNewTemplate{
	# $type: template type
	my ($this,$templateid) = @_;
	my $use_template = 0;
	if ($this->getModuleSetting("enable_what_is_new_template")){
		$use_template = 1;
	}
	return $this->getTemplate($templateid,$use_template);
}
# Search form template
sub getSearchTemplate{
	my ($this,$templateid) = @_;
	my $use_template = 0;
	if ($this->getModuleSetting("enable_search_template")){
		$use_template = 1;
	}
	return $this->getTemplate($templateid,$use_template);
}
# Function that gets specific templates
sub getTemplate{
	# $templateid: id of a template
	# $template_flag: a flag to determine whether to use template or not
	my ($this,$templateid,$template_flag) = @_;
	my %template = %{$this->templateData()};
	return if !exists($template{$templateid});

	# The custom template option is enabled.  We are reading the file from the disk
	if ($template_flag){
		my $diskpath = "$config{server}{HTMLroot}$config{server}{HTMLpath}/_Modules/Catalog/template";
		my $file = "${diskpath}/$template{$templateid}{filename}";
		if ($this->is_admin()){
			$file .= "Admin";
		}
		$file .= ".html";
		if (-e $file){
			open(FILE,$file);
			my @lines = <FILE>;
			close(FILE);
			return join("",@lines);
		}
	}
	
	# Get the default template
	if (exists($template{$templateid}{default})){
		return $template{$templateid}{default};
	}	
	return;
}
# Template data
sub templateData{
	my ($this) = @_;
	my %template_data;
	
	# Templates are index by template id then template filename and default template
	# $template_data 	=> <template_id> 	=> filename(Filename of the template, excluding the file extension)
	#						=> default(Default template in html. This input is optional.)
	
	# These templates determine default template by user access level
	$template_data{CategoryDetail}{filename} = "CategoryDetail";
	$template_data{ProductDetail}{filename} = "ProductDetail";
	$template_data{CategoryDetailProductList}{filename} = "CategoryDetailProductList";
	$template_data{CategoryDetailDescription}{filename} = "CategoryDetailDescription";
	$template_data{CategoryDetailTitle}{filename} = "CategoryDetailTitle";

	# Search template
	$template_data{CatalogSearchForm}{filename} = "CatalogSearchForm";
	$template_data{CatalogSearchForm}{default} = $this->html_searchFormTemplate();
	
	# These templates use only 1 default template
	$template_data{CategoryDetailThumbnailList}{filename} = "CategoryDetailThumbnailList";
	$template_data{CategoryDetailThumbnailList}{default} = <<END;
<div class="CatalogProductThumbnail">
<!--[[thumbnail]]-->
</div>
<div style="display:none;">
<!--[[thumbnail_preload]]-->
</div>
END
	$template_data{CategoryDetailThumbnailItem}{filename} = "CategoryDetailThumbnailItem";
	$template_data{CategoryDetailThumbnailItem}{default} = <<END;
<div class="CatalogProductThumbnailWrapper" onMouseOver="document.getElementById(<!--[[product_id]]-->).src='<!--[[mouseover]]-->';" onMouseOut="document.getElementById(<!--[[product_id]]-->).src='<!--[[mouseout]]-->';"><a href="<!--[[link_url]]-->"><img id="<!--[[product_id]]-->" src="<!--[[image_url]]-->" border="0" title="<!--[[image_label]]-->" alt="<!--[[image_label]]-->"></a></div>
END
	$template_data{CategoryDetailProductListDIVWrapper}{filename} = "CategoryDetailProductListDIVWrapper";
	$template_data{CategoryDetailProductListDIVWrapper}{default} = <<END;
<div class="CategoryProductWrapper">
	<!--[[CategoryProducts]]-->
</div>
END
	$template_data{CategoryDetailProductListThumbnail}{filename} = "CategoryDetailProductListThumbnail";
	$template_data{CategoryDetailProductListThumbnail}{default} = <<END;
<div class="CatalogProductThumbnail"><!--[[ProductThumbnail]]--></div>
END
	$template_data{ProductDetailSummary}{filename} = "ProductDetailSummary";
	$template_data{ProductDetailSummary}{default} = <<END;
<span class="ProductSummary"><!--[[ProductSummary]]--></span><br>
END
	$template_data{ProductDetailOption}{filename} = "ProductDetailOption";
	$template_data{ProductDetailOption}{default} = <<END;
<span class="ProductOption"><!--[[ProductOption]]--></span><br>
END
	$template_data{ProductDetailDetail}{filename} = "ProductDetailDetail";
	$template_data{ProductDetailDetail}{default} = <<END;
<span class="ProductDetail"><!--[[ProductDetail]]--></span><br>
END
	$template_data{ProductDetailPrice}{filename} = "ProductDetailPrice";
	$template_data{ProductDetailPrice}{default} = <<END;
<span class="ProductPrice"><!--[[ProductPrice]]--></span>
END
	$template_data{ProductDetailAttachmentThumbnailImageDescription}{filename} = "ProductDetailAttachmentThumbnailImageDescription";
	$template_data{ProductDetailAttachmentThumbnailImageDescription}{default} = <<END;
<br><!--[[ProductAttachmentDescription]]-->
END
	$template_data{ProductDetailAttachmentThumbnailDownloadDescription}{filename} = "ProductDetailAttachmentThumbnailDownloadDescription";
	$template_data{ProductDetailAttachmentThumbnailDownloadDescription}{default} = <<END;
 - <!--[[ProductAttachmentDescription]]-->
END
	$template_data{ProductDetailAttachmentDIV}{filename} = "ProductDetailAttachmentDIV";
	$template_data{ProductDetailAttachmentDIV}{default} = <<END;
<div class="ProductAttachment"><!--[[ProductAttachment]]--></div>
END
	$template_data{ProductDetailAttachmentDIVWrapper}{filename} = "ProductDetailAttachmentDIVWrapper";
	$template_data{ProductDetailAttachmentDIVWrapper}{default} = <<END;
<div class="ProductAttachmentWrapper">
	<!--[[ProductAttachments]]-->
</div>
END
	$template_data{ProductDetailAttachmentDownloadDIVWrapper}{filename} = "ProductDetailAttachmentDownloadDIVWrapper";
	$template_data{ProductDetailAttachmentDownloadDIVWrapper}{default} = <<END;
<div class="ProductAttachmentDownloadWrapper">
	<!--[[ProductAttachments]]-->
</div>
END
	$template_data{ProductDetailAttachmentImageDIVWrapper}{filename} = "ProductDetailAttachmentImageDIVWrapper";
	$template_data{ProductDetailAttachmentImageDIVWrapper}{default} = <<END;
<div class="ProductAttachmentImageWrapper">
	<!--[[ProductAttachments]]-->
</div>
END
	$template_data{ProductDetailAttachmentGroupWrapper}{filename} = "ProductDetailAttachmentGroupWrapper";
	$template_data{ProductDetailAttachmentGroupWrapper}{default} = <<END;
<div class="ProductAttachmentHeading">
	<!--[[ProductHeading]]-->
</div>
<div class="<!--[[CssClass]]-->">
	<!--[[ProductAttachments]]-->
</div>
END
	
	$template_data{ProductDetailPrimaryImageDescription}{filename} = "ProductDetailPrimaryImageDescription";
	$template_data{ProductDetailPrimaryImageDescription}{default} = <<END;
<br><!--[[ProductPrimaryImageDescription]]-->
END
	$template_data{ProductDetailPrimaryImageDIV}{filename} = "ProductDetailPrimaryImageDIV";
	$template_data{ProductDetailPrimaryImageDIV}{default} = <<END;
<div class="ProductPrimaryImage"><!--[[ProductPrimaryImage]]--></div>
END
	$template_data{ProductDetailPrimaryImageDIVWrapper}{filename} = "ProductDetailPrimaryImageDIVWrapper";
	$template_data{ProductDetailPrimaryImageDIVWrapper}{default} = <<END;
<div class="ProductPrimaryImageWrapper">
	<!--[[ProductPrimaryImages]]-->
</div>
END
	$template_data{WhatIsNewProductImage}{filename} = "WhatIsNewProductImage";
	$template_data{WhatIsNewProductImage}{default} = <<END;
<div class="WhatIsNewProductImage">
<a href="<!--[[ProductLink]]-->"><!--[[ProductImage]]--></a>
</div>
END
	$template_data{WhatIsNewProductDetail}{filename} = "WhatIsNewProductDetail";
	$template_data{WhatIsNewProductDetail}{default} = <<END;
<div class="WhatIsNewProductDetail">
<div class="WhatIsNewProductTitle"><a href="<!--[[ProductLink]]-->"><!--[[ProductTitle]]--></a></div>
<!--[[ProductSummary]]-->
</div>
END
	$template_data{WhatIsNewProductInfo}{filename} = "WhatIsNewProductInfo";
	$template_data{WhatIsNewProductInfo}{default} = <<END;
<div class="WhatIsNewProductInfo" onClick="location.href='<!--[[ProductLink]]-->'">
<!--[[ProductThumbnail]]-->
<!--[[ProductInfo]]-->
</div>
END
	$template_data{WhatIsNewProductWrapper}{filename} = "WhatIsNewProductWrapper";
	$template_data{WhatIsNewProductWrapper}{default} = <<END;
<div class="WhatIsNewProductWrapper">
<!--[[Products]]-->
</div>
END
	$template_data{WhatIsNewCategoryInfo}{filename} = "WhatIsNewCategoryInfo";
	$template_data{WhatIsNewCategoryInfo}{default} = <<END;
<div class="WhatIsNewCategoryInfo" style="margin-top: 10px;">
<div class="WhatIsNewCategoryTitle"><a href="<!--[[CategoryLink]]-->"><!--[[CategoryTitle]]--></a></div>
<!--[[CategoryProducts]]-->
</div>
END
	$template_data{WhatIsNewCategoryWrapper}{filename} = "WhatIsNewCategoryWrapper";
	$template_data{WhatIsNewCategoryWrapper}{default} = <<END;
<div class="WhatIsNewCategoryWrapper">
<!--[[CategoryList]]-->
</div>
END
	return \%template_data;
}
#----------------------------------------------------------------------
# Get data for the 'what is new' screen
sub getWhatIsNewData{
	my ($this,$sectionid,$limit) = @_;
	my $public_sql;
	# We only show 'disabled' products in the control panel
	if (exists($share{Page})){
		$public_sql = " and (p.status not like 'disabled' or p.status is null) ";
	}
	# Fetch all the feature products(feature field set to 'yes')
	my $sql = <<END;
select 
	c.catalog_category_id as id_category_1,
	c.type as type_category_1,
	c.title as title_category_1,
	c.rank as rank_category_1,
	c.status as status_category_1,
	c.section_id as section_category_1,
	pcj.catalog_product_category_join_id as id_product_join_1,
	pcj.catalog_category_id as category_product_join_1,
	pcj.catalog_product_id as product_product_join_1,
	p.catalog_product_id as id_product_1,
	p.code as code_product_1,
	p.title as title_product_1,
	p.summary as summary_product_1,
	p.details as details_product_1,
	p.price as price_product_1,
	p.feature as feature_product_1,
	p.status as status_product_1,
	p.options as options_product_1,
	p.sortkey as sortkey_product_1,	
	pa.catalog_product_attachment_id as id_product_attachment_1,
	pa.catalog_product_id as product_id_product_attachment_1,
	pa.type as type_product_attachment_1,
	pa.description as description_product_attachment_1,
	pa.heading as heading_product_attachment_1,
	SUBSTRING(pa.upload,1,100) as upload_product_attachment_1,
	SUBSTRING(pa.thumbnail,1,100) as thumbnail_product_attachment_1
from
	catalog_product as p 
	left join catalog_product_attachment as pa on pa.catalog_product_id = p.catalog_product_id 
	left join catalog_product_category_join as pcj on pcj.catalog_product_id = p.catalog_product_id 
	left join catalog_category as c on c.catalog_category_id = pcj.catalog_category_id 
where
	c.section_id = $sectionid and 
	p.feature like 'yes' 
	$public_sql
order by
	c.rank asc,
	c.title asc, 
	p.catalog_product_id desc
END

	my @records = $this->{db}->custom_query($sql);
	my %return;
	
	foreach my $record(@records){
		if (scalar(keys(%return)) > $limit){
			last;
		}
		# Category
		my %category = (
			catalog_category_id=>$record->{id_category_1},
			title=>$record->{title_category_1},
			type=>$record->{type_category_1},
			status=>$record->{status_category_1},
			rank=>$record->{rank_category_1},
		);
		# Product
		my %product = (
			catalog_product_id=>$record->{id_product_1},
			code=>$record->{code_product_1},
			title=>$record->{title_product_1},
			summary=>$record->{summary_product_1},
			details=>$record->{details_product_1},
			price=>$record->{price_product_1},
			status=>$record->{status_product_1},
			options=>$record->{options_product_1},
			sortkey=>$record->{sortkey_product_1},
			feature=>$record->{feature_product_1},
		);
		# Product join
		my %product_join = (
			catalog_product_category_join_id=>$record->{id_product_join_1},
			catalog_category_id=>$record->{category_product_join_1},
			catalog_product_id=>$record->{product_product_join_1},
		);
		# Product attachment
		my %product_attachment = (
			catalog_product_attachment_id=>$record->{id_product_attachment_1},
			catalog_product_id=>$record->{product_id_product_attachment_1},
			type=>$record->{type_product_attachment_1},
			description=>$record->{description_product_attachment_1},
			heading=>$record->{heading_product_attachment_1},
			upload=>$record->{upload_product_attachment_1},
			thumbnail=>$record->{thumbnail_product_attachment_1},
		);

		$this->save_cache_category(\%category);
		$this->save_cache_product(\%product);
		$this->save_cache_product_join(\%product_join);
		$this->save_cache_product_attachment(\%product_attachment);
	
		if (!$product{catalog_product_id}){
			next;
		}
		$product{rank} = $category{rank};
		$product{category_title} = $category{title};
		$product{category_id} = $category{catalog_category_id};
		$product{sort_field} = $category{sortkey}.$category{title};
		$return{$product{catalog_product_id}} = \%product;
	}
	my @return;
	foreach my $product_id(keys(%return)){
		push(@return,($return{$product_id}));
	}
	return \@return;
}
# What's new product page
sub whatIsNew{
	# $sectionid: section id
	# $show: number of products to show
	my ($this,$sectionid,$show) = @_;
	# No section id found
	return if !$sectionid;
	# Get the number of product to show
	if ($show < 1){
		$show = $this->getModuleSetting("number_to_show");
	}
	# No products need to be displayed
	return if $show < 1;

	my @products = @{$this->getWhatIsNewData($sectionid,$show)};
	my %category_product;
	my $h_index = "category_title";
	my %category_rank;
	# Index products by its category titles
	foreach my $product(@products){
		$category_rank{$product->{rank}} = $product->{category_title};
		$category_product{$product->{$h_index}}{title} = $product->{category_title};
		$category_product{$product->{$h_index}}{id} = $product->{category_id};
		# Group products in an array
		push(@{$category_product{$product->{$h_index}}{data}},$product);
	}
	# Order new product using sorted category titles
	my @titles;
	foreach my $rank(sort keys(%category_rank)){
		push(@titles,$category_rank{$rank});
	}
	my $new_list;
	# Order products by its category name then display them	
	foreach my $title(@titles){
		my $categorytitle = $category_product{$title}{title};
		my $categoryid = $category_product{$title}{id};
		my @c_products = $category_product{$title}{data} ? sort {$a->{sort_field} cmp $b->{sort_field}} @{$category_product{$title}{data}} : ();
		my $product_list = $this->whatisnewProduct(\@c_products);
		my $category_link = $this->link(action=>undef,path=>$this->getPath($categoryid));
		# Gather product data
		my %category_data = (
			CategoryLink=>$category_link || "",
			CategoryTitle=>$categorytitle || "",
			CategoryProducts=>$product_list || "",
		);
		$new_list .= $this->merge($this->getWhatIsNewTemplate("WhatIsNewCategoryInfo"),\%category_data);
	}
	# Output product list
	return $this->merge($this->getWhatIsNewTemplate("WhatIsNewCategoryWrapper"),{CategoryList=>$new_list});
}
# Create display for individual product
sub whatisnewProduct{
	# $products: an array of product hashes
	my ($this,$products) = @_;
	my $attachment_list;
	foreach my $product(@$products){
		my $productid = $product->{catalog_product_id};
		# Find an image to be displayed beside the product
		# 1. Use a primary image
		# 2. If a primary image isn't found, we randomly select a secondary image
		my @attachments = @{$this->getProductAttachmentImage($productid)};
		my $link = $this->link(action=>undef,product_id=>$productid);
		# Create a thumbnail for the product
		my $thumbnail;		
		if (@attachments > 0){			
			my %attachment = %{$attachments[0]};
			my $thumbnail_image = $this->getProductThumbnail(\%attachment);
			my %image_data = (
				ProductImage=>$thumbnail_image || "",
				ProductLink=>$link || "",
			);
			$thumbnail = $this->html_whatisnewImage(\%image_data);
		}
		# Create product information display
		my %product_data = (
			ProductTitle=>$this->html_productTitle($product) || "",
			ProductSummary=>$this->html_productSummary($product) || "",
			ProductLink=>$link || "",
		);
		my $product_info = $this->merge($this->getWhatIsNewTemplate("WhatIsNewProductDetail"),\%product_data);
		my %attachment_data = (
			ProductLink=>$link || "",
			ProductInfo=>$product_info || "",
			ProductThumbnail=>$thumbnail || "",
		);
		# Save product display(image + product info) in $attachment_list
		$attachment_list .= $this->merge($this->getWhatIsNewTemplate("WhatIsNewProductInfo"),\%attachment_data);
	}
	return $this->merge($this->getWhatIsNewTemplate("WhatIsNewProductWrapper"),{Products=>$attachment_list});
	
}
# Display product image in the what's new template
sub html_whatisnewImage{
	# $data: merge data(hash reference)
	my ($this,$data) = @_;
	# We don't need to display anything since there is no product image
	return if !$data->{ProductImage};
	return $this->merge($this->getWhatIsNewTemplate("WhatIsNewProductImage"),$data);
}
# Does the thumbnail exist?
sub thumbnailExist{
	# $type: color or black and white thumbnail
	# $data: attachment data(hash reference)
	my ($this,$type,$data) = @_;
	my $filepath = $this->getDiskPath();
	my $categoryid = $this->getProductCategoryID($data);
	my $product_path = $filepath.$this->getPath($categoryid)."/_$data->{catalog_product_id}";
	my $filename = $this->bwSquareThumbnailFilename($data);
	if ($type =~ /color/i){
		$filename = $this->colorSquareThumbnailFilename($data);
	}
	my $file = $product_path."/".$filename;
	return 1 if (-e $file);
	return 0;
}
# Get thumbnail image
sub getSquareThumbnail{
	# $data: product(hash reference)
	# $type: thumbnail type(color or black and white)
	my ($this,$data,$type) = @_;
	if ($this->thumbnailExist($type,$data)){
		my $sectionid = $this->section_id();
		my $categoryid = $this->getProductCategoryID($data);
		my $path = $this->getPath($categoryid)."/_$data->{catalog_product_id}";
		my $filename = $this->squareThumbnailFilename($data,$type);
		return $this->getWebPublishDirectory()."/${sectionid}${path}/${filename}";
	}
	return $this->getNullImage();
}
# Get null/spacer image
sub getNullImage{
	my ($this) = @_;
	return "$config{server}{server}$config{server}{HTMLpath}/_Modules/Catalog/$config{Catalog}{no_image_graphic}";
}
# Get color thumbnail product image
sub getColorSquareThumbnail{
	# $data: attachment data(hash reference)
	my ($this,$data) = @_;
	return $this->getSquareThumbnail($data,"color");
}
# Get black and white thumbnail product image
sub getBWSquareThumbnail{
	# $data: attachment data(hash reference)
	my ($this,$data) = @_;
	return $this->getSquareThumbnail($data,"bw");
}
# Set/get product id
sub product_id{
	my ($this,$id) = @_;
	# Setting product id
	if (defined($id)){
		$this->{product_id} = $id || 0;
	}
	# No product id found. Find a default product id.
	if (!$this->{product_id}) {
		# Check the CMS commands for product id
		if ($this->{module_option}{product_id} && !$this->{product_id}) {
			$this->{product_id} = $this->{module_option}{product_id} || 0;
		}
		# Check the query string for product id
		if ($this->{input}{product_id} && !$this->{product_id}){
			$this->{product_id} = $this->{input}{product_id} || 0;
		}
	}
	return $this->{product_id};
}
# Are we in category search mode?
sub is_search{
	my ($this) = @_;
	return 1 if $this->{input}{action} =~ /dir_search/i;
	return 0;
}
# Get thumbnail product list
sub thumbnailProductList{
	# $path: category path(format: /1/2/3/4/5)
	my ($this,$path) = @_;
	return if !$this->enableThumbnailProductList();
	my $productid = $this->product_id();
	my @path = split(/\//,$path);
	my $categoryid = $path[-1];
	
	# No category found
	return if !$categoryid;

	my @products = @{$this->getProductAttachmentSingleImage($categoryid)};

	# No products found
	return if @products < 1;

	my %product;
	# Index products to get rid of duplicate images
	foreach my $product(@products){
		$product{$product->{catalog_product_id}} = $product;
	}
	my $preload;
	my $thumbnails;
	foreach my $product(@products){
		my $id = $product->{catalog_product_id};
		my $imageurl;
		# Use color thumbnail image if we are viewing a selected product
		if ($id == $productid){
			$imageurl = $this->getColorSquareThumbnail($product{$id});
		}else{
			$imageurl = $this->getBWSquareThumbnail($product{$id});
		}
		my $mouseover = $this->getColorSquareThumbnail($product{$id});
		my $mouseout = $imageurl;
		my %t_data = (
			link_url=>$this->link(product_id=>$id,path=>$this->path()),
			image_url=>$imageurl,
			product_id=>$id,
			mouseover=>$mouseover,
			mouseout=>$mouseout,
			#image_label=>$product{$id}{title},
			image_label=>$this->getProduct($id)->{title},
		);
		$thumbnails .= $this->merge($this->html_thumbnailProductItem(),\%t_data);
		$preload .= <<END;
<img src="$mouseover">
END
	}
	my %m_data = (
		thumbnail=>$thumbnails,
		thumbnail_preload=>$preload,
	);
	return $this->merge($this->html_thumbnailProductList(),\%m_data);
}
# Thumbnail product template
sub html_thumbnailProductItem{
	my ($this) = @_;
	my $template = $this->getCategoryDetailTemplate("CategoryDetailThumbnailItem");
	return $template;
}
# Thumbnail product list template
sub html_thumbnailProductList{
	my ($this) = @_;
	my $template = $this->getCategoryDetailTemplate("CategoryDetailThumbnailList");
	return $template;
}

# Reset sort keys
# 'sort key' is a field in the product record
sub resetSort{
	my ($this) = @_;
	my $querypath = $this->path();
	my @path = split(/\//,$querypath);
	my $path = $path[-1];
	return if !$path;
	# Get all product ids
	my $sql;
	# Figure out the products that need to be updated
	$sql = <<END;
select 
	b.catalog_product_id as id 
from 
	catalog_product_category_join a 
	left join catalog_product b on a.catalog_product_id = b.catalog_product_id 
where 
	a.catalog_category_id = $path and (b.sortkey is not null or b.sortkey != '')
END
	my @product_ids = $this->{db}->custom_query($sql);
	my @ids;
	foreach my $product_id(@product_ids){
		push(@ids,$product_id->{id});
	}
	# Reset product sort key
	$sql = "update catalog_product set sortkey = '' where catalog_product_id = ".join(" or catalog_product_id = ",@ids);
	$this->{db}->custom_query($sql);
	return;
}

# Modified sub do from Form.pm
# - added republish() calls
sub processDoForm{
	my ($this) = @_;
	my $reply;
	my $out;
	my $redirect = 1;
	my $action;
	if (!$this->{db}->{data}) {
        	if (! $this->{db}->get_data()) {
        		$this->{db}->error("do form: Nothing to do!");
            	}
	}
	if (!$action){
		$action = $this->{db}->get_action();
	}
	$this->{db}->{action} = $action;

	# fetch the destination URL

	if (exists $this->{db}->{data}->{reply}){
        	$reply = $this->{db}->{data}->{reply};
        	delete $this->{db}->{data}->{reply};
        	if ($reply =~ /doform\.cgi/){
        	    	# recursion is perilous
        		$reply = "";
        	}
	}

	# cancel, if requested

	if ($this->{db}->{submit_action} eq "Cancel"){
        	# bail
        	if ($this->{db}->{handler}->{cancel_form}) {
        		my $out = &{$this->{db}->{handler}->{cancel_form}}($this);
        		if ($out){
                		return $out;
        		}
        	}
        	if ($reply){
        		print "Location: $reply\n\n";
        		return;
        	}else{
        		return "<H3>Operation cancelled.</H3>\n";
        	}
	}

	# do everything that touches the database

	$this->{db}->do_db($action);
	if ($this->{db}->errorcheck){
        	# uh-oh!
            	$out = "<H3>Problem:</H3>\n<p>";
            	$out .= $this->{db}->show_diagnostics("error","html");
            	$out .= "</p>\n<P><A HREF=javascript:history.back()>Go back to form.</A></P>\n";
	}elsif ($reply){            	
	    	# republish the whole web directory
#	    	$this->republish();
	    	# success; return the user to the original page
            	print "Location: $reply\n\n";
	}elsif ($this->{db}->{output}){
            	# success; return the status report to the user
            	return $this->{output};
	}else{
            	# success; generic confirmation message
            	$out = "<H3>Done!</H3>";
	}
	return $out;
}
#----------------------------------------------------------------------
# Catalog publish directory
sub getPublishDirectory{
	my ($this) = @_;
	# The directory is configurable in the conf file
	if ($this->getModuleSetting("publish_directory")){
		return $this->getModuleSetting("publish_directory");
	}
	return "/_Modules/Catalog/top";
}
# Catalog publishing directory on the disk
sub getDiskPublishDirectory{
	my ($this) = @_;	
	return "$config{server}{HTMLroot}$config{server}{HTMLpath}".$this->getPublishDirectory();
}
# Catalog publishing directory on the website
sub getWebPublishDirectory{
	my ($this) = @_;
	return "$config{server}{server}$config{server}{HTMLpath}".$this->getPublishDirectory();
}
# Get disk path
sub getDiskPath{
	my ($this) = @_;
	my $sectionid = $this->section_id();
	return if !$sectionid;
	return $this->getDiskPublishDirectory()."/$sectionid";
}

# Publish catalog
sub publish{
	# $opt: option hash
	my ($this,$opt) = @_;
	my @sections;
	if ($this->{input}{section_id}){
		push(@sections,$this->{input}{section_id});
	}else{
		my @sites = $this->{db}->my_real_sites();
		foreach my $site(@sites){
			push(@sections,$site->{section_id});
		}
	}
	if (scalar(@sections) < 1){
		if ($this->section_id()){
			push(@sections,$this->section_id());
		}else{
			return "<p>Nothing to publish!</p>";
		}
	}

	my $out;
	$out .= $this->mkdir($this->getDiskPublishDirectory());

	foreach my $sectionid(@sections){
		$this->section_id($sectionid);
		# Publish mode begins
		$this->{publishing} = 1;
		my $filepath = $this->getDiskPath();
		# Create file directory
		$out .= $this->mkdir($filepath);
		# Find categories that have products with attachments
		my $sql = <<END;
select
	c.catalog_category_id as catalog_category_id,
	c.section_id as category_section_id,
	pa.catalog_product_attachment_id as catalog_product_attachment_id,
	pa.catalog_product_id as catalog_product_id,
	pa.type as type,
	pa.upload as upload,
	pa.thumbnail as thumbnail
from
	catalog_product_attachment as pa
	left join catalog_product as p on pa.catalog_product_id = p.catalog_product_id
	left join catalog_product_category_join as pcj on pcj.catalog_product_id = p.catalog_product_id
	left join catalog_category as c on c.catalog_category_id = pcj.catalog_category_id
where	
	(p.status != 'disabled' or p.status is null or p.status = '')
END
		$sql .= " and (c.section_id = $sectionid ";
		if ($this->enableCategorySharing()){
			$sql .= " or c.section_id = 0";
		}
		$sql .= ")";
		my @attachments = $this->{db}->custom_query($sql);
		
		# Publish attachments
		foreach my $attachment(@attachments){
		
			# Cache category data for publishing purposes
			my %c_data = (
				catalog_category_id=>$attachment->{catalog_category_id},
			);
			$this->save_cache_category(\%c_data);
		
			my $categoryid = $attachment->{catalog_category_id};
			my $diskpath = $this->getDiskPath;
			my $directorypath = $diskpath.$this->getPath($categoryid);
			$out .= $this->mkdir($directorypath);
			$out .= $this->publishProductAttachment($categoryid,$attachment);
		}
	}
	$this->{publishing} = 0;
	return if $opt->{silence};
	return $out;	
}
# Publish product attachment
sub publishProductAttachment{
	# $categoryid: category id
	# $attachment: attachment record
	my ($this,$categoryid,$attachment) = @_;		
	my $filepath = $this->getDiskPath;
	my $product_path = $filepath.$this->getPath($categoryid)."/_$attachment->{catalog_product_id}";
	my $out = $this->mkdir($product_path);
	# Publish attachments to product directory
	my (undef,$attach_data) = split(/#/,$attachment->{upload});
	my $attachment_filename = $this->attachmentFilename($attachment);
	# Only publish a valid file
	if ($attachment_filename){
		$out .= $this->writeToDisk($product_path,$attachment_filename,decode_base64($attach_data));
	}
	if ($attachment->{thumbnail}){
		my (undef,$th_data) = split(/#/,$attachment->{thumbnail});
		my $thumb_filename = $this->thumbnailFilename($attachment);
		# Only publish a valid file
		if ($thumb_filename){
			$out .= $this->writeToDisk($product_path,$thumb_filename,decode_base64($th_data));
		}		
	}
	$out .= $this->publishSquareThumbnailImage($attachment,$product_path);
	return $out;
}
# Enable dasal style category list
sub enableThumbnailProductList{
	my ($this) = @_;
	return $this->getModuleSetting("enable_thumbnail_product_list") || 0;
}
# Publish square thumbnail image
sub publishSquareThumbnailImage{
	# $data: attachment record
	# $path: disk path to write the image
	my ($this,$data,$path) = @_;
	my $disable_square_thumbnail = $this->getModuleSetting("disable_square_thumbnail") || 0;
	my %thumbnail_function = (
		1=>{
			bw=>"scale_thumb_by_width_bw",
			color=>"scale_thumb_by_width",
		},
		0=>{
			bw=>"square_thumb_bw",
			color=>"square_thumb",
		},
	);
	my $thumbnail_size = $this->getModuleSetting("product_thumbnail_size") || 10;
	my $out;
	# We only publish thumbnails if we are display product in a thumbnailed list
	if ($this->enableThumbnailProductList()){
		my $file_data = $data->{thumbnail} || $data->{upload};
		require ExSite::Image;
		my $color_thumb_filename = $this->colorSquareThumbnailFilename($data);	
		# Only publish a valid file
		if ($color_thumb_filename){
			my $color_thumb = new ExSite::Image($file_data);
			my $func = $thumbnail_function{$disable_square_thumbnail}{color};
			$color_thumb->$func($thumbnail_size);
			$out .= $this->writeToDisk($path,$color_thumb_filename,$color_thumb->get());
		}
		my $bw_thumb_filename = $this->bwSquareThumbnailFilename($data);
		# Only publish a valid file
		if ($bw_thumb_filename){
			my $bw_thumb = new ExSite::Image($file_data);
			my $func = $thumbnail_function{$disable_square_thumbnail}{bw};
			$bw_thumb->$func($thumbnail_size);	
			$out .= $this->writeToDisk($path,$bw_thumb_filename,$bw_thumb->get());
		}
	}
	return $out;
}
sub getSingleImageThumbnailUrl{
	# $productid: product id
	# $sectionid: section id
	my ($this,$productid,$sectionid) = @_;
	my @products = @{$this->getProductAttachmentImage($productid)};
	# No attachment found
	if (scalar(@products) < 1){
		return;
	}
	my %product = %{$products[-1]};
	return $this->getImageUrlThumbnail(\%product,$sectionid);
}
# Get primary thumbnail image url
sub getPrimaryImageThumbnailUrl{
	# $productid: product id
	# $sectionid: section id
	my ($this,$productid,$sectionid) = @_;
	my @products = @{$this->getProductPrimaryImage($productid)};
	if (scalar(@products) < 1){
		return;
	}
	my %product = %{$products[-1]};
	return $this->getImageUrlThumbnail(\%product,$sectionid);
}
# Get thumbnail image url
sub getImageUrlThumbnail{
	# $data: attachment record
	# $sectionid: section id
	my ($this,$data,$sectionid) = @_;
	return $this->getImageUrl($data,$sectionid,"thumbnail");
}
# get attachment url
sub getImageUrlAttachment{
	# $data: attachment record
	# $sectionid: section id
	my ($this,$data,$sectionid) = @_;
	return $this->getImageUrl($data,$sectionid,"attachment");
}
# Get image url
sub getImageUrl{
	# $data: attachment record
	# $sectionid: section id
	# $type: image type i.e. thumbnail or attachment
	my ($this,$data,$sectionid,$type) = @_;
	my ($filename) = split(/#/,$data->{upload});
	return if (!$this->isImage($filename));
	# Predefine a set of routines to run based on the image type
	my %func = (
		thumbnail=>{
			exist=>"productThumbnailExist",
			filename=>"thumbnailFilename",
		},
		attachment=>{
			exist=>"productAttachmentExist",
			filename=>"attachmentFilename",
		}
	);
	my $exist_func = $func{$type}{exist};
	if (!$this->$exist_func($data)){		
		return "$this->{cgi}/getdata.cgi/$filename?tab=catalog_product_attachment&id=$data->{catalog_product_attachment_id}&col=$type";
	}else{
		my $filename_func = $func{$type}{filename};
		return $this->getAttachmentPath($data,$sectionid)."/".$this->$filename_func($data);
	}
}
# Get attachment url
sub getAttachmentPath{
	# $data: product id
	# $sectionid: section id
	my ($this,$data,$sectionid) = @_;
	my $path = $this->getPath($this->getProductCategoryID($data));
	my $out = $this->getWebPublishDirectory()."/${sectionid}${path}/_$data->{catalog_product_id}";
	return $out;
}
# Get product attachment url
sub getProductAttachment{
	# $data: product record
	my ($this,$data) = @_;
	my ($filename) = split(/#/,$data->{upload});
	if (!$this->productAttachmentExist($data)){		
		return "$this->{cgi}/getdata.cgi/$filename?tab=catalog_product_attachment&id=$data->{catalog_product_attachment_id}&col=upload";
	}
	my $path = $this->getAttachmentPath($data,$this->section_id());
	return $path."/".$this->attachmentFilename($data);
}
# Get product thumbnail
sub getProductThumbnail{
	# $data: product record
	my ($this,$data) = @_;
	my $image_link;
	if (!$this->productThumbnailExist($data)){
		# Thumbnail isn't published.  Use show_data to generate the image link
		$image_link = $this->{db}->show_data("catalog_product_attachment","thumbnail",$data->{upload},$data->{catalog_product_attachment_id});
	}else{
		# Published thumbnails
		my $path = $this->getAttachmentPath($data,$this->section_id())."/".$this->thumbnailFilename($data);
		$image_link = "<img src=\"$path\" border=\"0\">";
	}
	my $imageurl = $this->getProductAttachment($data);
	# Open product images in a new window
	if ($data->{type} !~ /download/i){
		$imageurl = "javascript:popup_image('$imageurl')";
	}
	my $image_html = "<a href=\"$imageurl\"";
	# Open the downloadable document in a new window
	if ($data->{type} =~ /download/i){
		$image_html .= " target=\"_blank\"";
	}
	$image_html .= ">$image_link</a>";
	return $image_html;
}
# Is the attachment published?
sub productAttachmentExist{
	# $data: product record
	my ($this,$data) = @_;
	return $this->attachmentExist("upload",$data);
}
# Is the thumbnail published?
sub productThumbnailExist{
	# $data: product record
	my ($this,$data) = @_;
	return $this->attachmentExist("thumbnail",$data);
}
# Is attachment exists
sub attachmentExist{
	# $type: attachment type
	# $data: attachment record
	my ($this,$type,$data) = @_;
	# Get disk path
	my $filepath = $this->getDiskPath();
	# Category id
	my $categoryid = $this->getProductCategoryID($data);
	# Product disk path
	my $product_path = $filepath.$this->getPath($categoryid)."/_$data->{catalog_product_id}";
	my $filename = $this->genFilename($type,$data);
	my $file = $product_path."/".$filename;
	return 1 if (-e $file);
	return 0;	
}
# Get the attachment filename
sub attachmentFilename{
	# $data: attachment record
	my ($this,$data) = @_;
	return $this->genFilename("upload",$data);
}
# Get the thumbnail filename
sub thumbnailFilename{
	# $data: attachment record
	my ($this,$data) = @_;
	return $this->genFilename("thumbnail",$data);
}
# Generate a filename for square color thumbnail
sub colorSquareThumbnailFilename{
	# $data: attachment record
	my ($this,$data) = @_;
	return $this->squareThumbnailFilename($data,"color");
}
# Generate a filename for square black and white thumbnail
sub bwSquareThumbnailFilename{
	# $data: attachment record
	my ($this,$data) = @_;
	return $this->squareThumbnailFilename($data,"bw");
}
# Generate a filename for a square thumbnail
sub squareThumbnailFilename{
	# $data: attachment record
	# $type: bw|color (black and white or color thumbnail)
	my ($this,$data,$type) = @_;
	my $thumbnail_size = $this->getModuleSetting("product_thumbnail_size") || 10;
	my $type = "${type}_${thumbnail_size}x${thumbnail_size}";
	return $this->genFilename($type,$data);	
}
# Get file name
sub getFilename{
	# $data: attachment record
	my ($this,$data) = @_;
	return "invalid_filename" if !$data;
	my ($filename,undef) = split("#",$data);
	$filename =~ /(.+)\.(.+)$/;
	my $fname = &clean_filename($1);
	$fname =~ s/\//_/g;
	return $fname;
}
# Get file extension
sub getFileExt{
	# $data: attachment record
	my ($this,$data) = @_;
	return "invalid_ext" if !$data;
	my ($filename,undef) = split("#",$data);
	$filename =~ /(.+)\.(.+)$/;
	return $2;
}
# Generate filename
sub genFilename{
	# $type: upload|thumbnail|bw|color : filename type
	# $data: attachment record
	my ($this,$type,$data) = @_;
	my $col = $type;
	if ($type =~ /bw|color/i){
		# To create a square thumbnail filename, we prefer to use the data in the 'upload' field to generate a filename.  
		# However, if the 'upload' field doesn't contain any data, we use the 'thumbnail field.
		if ($data->{upload}){
			$col = "upload";
		}else{
			$col = "thumbnail";
		}
	}
	return if !$data->{$col};
	my $filename = $this->getFilename($data->{$col});
	my $ext = $this->getFileExt($data->{$col});
	if ($type =~ /bw|color/i){
		$ext = $this->getFileExt($data->{thumbnail});
	}
	return "${filename}_${type}_$data->{catalog_product_attachment_id}.${ext}";
}
# Readjust link in the publishable contents
sub fixFileContent{
	# $in: query data(hash reference)
	my ($this,$in) = @_;
	# Remove 'mofule=catalog' query string
	$in =~ s/module=catalog//gi;
	# Change publish.cgi to webdir.cgi
	$in =~ s/publish\.cgi/catalog\.cgi/gi;
	# Change ctrl-panel.cgi/WebDir to webdir.cgi
	$in =~ s/ctrl-panel\.cgi(\/.+?)?\?/catalog\.cgi\?/gi;
	$in =~ s/\?&/\?/gi;
	$in =~ s/&\"/\"/gi;
	return $in;
}
# Replublish the whole web directory
sub republish{
	my ($this) = @_;
	my $oldEnv = $ENV{QUERY_STRING};
	$ENV{QUERY_STRING} = "module=Catalog";
	# Run publish.cgi
	`$config{server}{CGIroot}/publish.cgi`;
	# Restore query string
	$ENV{QUERY_STRING} = $oldEnv;
	return;
}
# Published file check 
# Display the published web directory contents for public only
sub checkPublishFile{
	# $dirpath: directory path
	my ($this,$dirpath) = @_;
	# We are publishing the pages
	return 0 if $this->{publishing};
	# The user is an admin
	return 0 if $this->admin_mode();
	my $dir = $this->getDiskPath.$dirpath;
	my $file = $dir."/index.html";
	# File exists
	return 1 if (-e $file);
	return 0;
}
# Fix sublevel file path
sub fixSubLevelPath{
	# $dirpath: directory path
	my ($this,$dirpath) = @_;
	if ($dirpath =~ /\//){
		my @paths = split(/\//,$dirpath);
		$dirpath = $paths[-1];
	}
	return $this->getPath($dirpath);
}
# Published file check
sub checkPublishSubLevel{
	# $dirpath: directory path
	my ($this,$dirpath) = @_;
	$dirpath = $this->fixSubLevelPath($dirpath);
	return $this->checkPublishFile($dirpath);
}
# Published file check
sub checkPublishTopLevel{
	my ($this) = @_;
	return $this->checkPublishFile();
}
sub productPublishFile{
	# $productid: product id
	my ($this,$productid) = @_;
	my %product = %{$this->getProduct($productid)};
	my $dir = $this->getPath($this->getProductCategoryID(\%product))."/_$productid";
	return $this->getFilePublished($dir);
}
# Check for the existance of a published file
sub checkPublishProduct{
	# $productid: product id
	my ($this,$productid) = @_;
	# We are publishing the pages
	return 0 if $this->{publishing};
	return 0 if $this->is_admin() && $ENV{SCRIPT_NAME} =~ /ctrl-panel\.cgi/i;
	my %product = %{$this->getProduct($productid)};
	my $dir = $this->getDiskPath.$this->getPath($this->getProductCategoryID(\%product))."/_$productid";
	my $file = $dir."/index.html";
	# File exists
	return 1 if (-e $file);
	return 0;
}
# Get published file contents
sub getFilePublished{
	# $dirpath: directory path
	my ($this,$dirpath) = @_;
	# filename
	my $filename = $this->getDiskPath.$dirpath."/index.html";
	my $out;
	if (open(IFILE,"<$filename")){
		my @lines = <IFILE>;		
		close(IFILE);
		# Output file contents
		$out .= join("",@lines);
	}else{
		$this->{db}->error("Catalog:: Failed to load $filename : $!");
	        $out .= "<span class=\"error\">Error: $!</span>\n";
	}	
	return $out;
}
# Get published file contents
sub topLevelPublishFile{
	my ($this) = @_;
	return $this->getFilePublished();
}
# Get published file contents
sub subLevelPublishFile{
	# $dirpath: directory path
	my ($this,$dirpath) = @_;
	$dirpath = $this->fixSubLevelPath($dirpath);
	return $this->getFilePublished($dirpath);
}
sub directoryTaintedCheck{
	my ($this,$directory) = @_;
	$directory =~ /^((\/{1}[a-zA-Z0-9_-]+)+)$/;
	my $checked = $1;
	return $checked;
}
sub fileTaintedCheck{
	my ($this,$filename) = @_;
	$filename =~ /^([a-zA-Z0-9_-]+\.[a-zA-Z0-9_-]+)$/;
	my $checked = $1;
	return $checked;
}
# Create a directory
sub mkdir{
	# $directory: directory to publish files
	my ($this,$directory) = @_;
	# Tainted check
	my $clean_directory = $this->directoryTaintedCheck($directory);
	
	if (!$clean_directory){
		$this->error("mkdir $clean_directory: failed: $directory - Invalid Directory");
		return "<span class=\"error\">FAILED: $directory - Invalid Directory</span>\n"
	}
	
	# Directory existed.
	return if (-e $clean_directory);
	my $out = "<p>Create directory $clean_directory ...";
	if (mkdir($clean_directory)){
		$out .= "succeed.";
	}else{
		$this->error("mkdir $clean_directory: failed: $!");
		$out .= "<span class=\"error\">FAILED: $!</span>\n";
	}
	$out .= "</p>";
	return $out;
}
# Write files to the disk
sub writeToDisk{
	# $directory: directory to write the file
	# $file: filename
	# $content: file content
	my ($this,$directory,$file,$content) = @_;
	# Tainted check
	my $clean_directory = $this->directoryTaintedCheck($directory);
	my $clean_filename = $this->fileTaintedCheck($file);
	
	if (!$clean_directory){
		$this->{db}->error("Catalog::publish: $directory - Invalid Directory");
		return "<span class=\"error\">FAILED: $directory - Invalid Directory</span>";
	}
	if (!$clean_filename){
		$this->{db}->error("Catalog::publish: $file - Invalid Filename");
		return "<span class=\"error\">FAILED: $file - Invalid Filename</span>";
	}
	
	my $out = "<p>Writing $file ...";
	my $clean_file = $clean_directory."/".$clean_filename;
	if (open(OFILE,">$clean_file")){
		# Write files
		print OFILE $content;
		close(OFILE);
		$out .= "succeed.";
		}else{
		# Error
		$this->{db}->error("Catalog::publish: publish $clean_file failed: $!");
	        $out .= "<span class=\"error\">FAILED: $!</span>\n";
	}
	$out .= "</p>";
	return $out;
}
# Process attachment form data
sub processAttachment{
	my ($this) = @_;
	my $db = $this->{db};
	# Save Catalog settings into the db object so the handler can access these settings
	$db->{catalog_image_size} = $this->getModuleSetting("image_size");
	$db->{catalog_thumbnail_size} = $this->getModuleSetting("thumbnail_size");
	# Install handlers
	$this->installCatalogObject();
	$db->handler("pre_insert",\&custom_pre_insert);
	$db->handler("pre_update",\&custom_pre_update);
	$db->do();
	return;
}
# Is this file an image?
sub isImage{
	# $filename: filename
	my ($this,$filename) = @_;
	return 1 if $filename =~ /(jpe?g|png|gif)$/i;
	return 0;
}
# File upload
sub file_upload{
	# $db: db object
	# $data: record data
	my ($db,$data) = @_;
	# Only create thumbnails for images
	my ($filename) = split(/#/,$data->{upload});
	if ($data->{upload} && &isImage(undef,$filename)){		
		foreach my $field(
			{type=>"image",name=>"upload",size=>$db->{catalog_image_size},flag=>"-1"},
			{type=>"thumbnail",name=>"thumbnail",size=>$db->{catalog_thumbnail_size}},
		){
			my $file_data = $data->{upload};
			if ($field->{type} eq "thumbnail"){
				# Set image data for $file_data
				if ($data->{thumbnail}){
					$file_data = $data->{thumbnail};
				}
			}
			require ExSite::Image;
			my $image = new ExSite::Image($file_data);
			if ($image){
				if ($field->{type} eq "thumbnail"){
					# If there is no thumbnail, we need to auto thumbnail the original image
					if (!$data->{thumbnail}){
						$image->square_thumb($field->{size});
					}
				}else{
					$image->scale($field->{size},$field->{size},$field->{flag});
				}
				$image->{filename} = $share{catalog}{object}->modifyFilename($image->{filename});
				$data->{$field->{name}} = $image->encode();
			}
		}
	}
	return;
}
# Set document type
sub set_document_type{
	# $data: record data
	my ($data) = @_;
	return if $data->{type};
	my ($filename) = split(/#/,$data->{upload});
	if (&isImage(undef,$filename)){
		$data->{type} = "secondary-image";
	}else{
		$data->{type} = "download";
	}
	return;
}
# 'pre_update' handler
sub custom_pre_update{
	# $table: $table
	# $data: record data
	# $cond: conditional
	my ($this,$table,$data,$cond) = @_;
	if ($table eq "catalog_product_attachment"){
		# Fix file upload i.e. fix invalid filename..etc
		&file_upload($this,$data);
		# Set document type
		&set_document_type($data);
	}
	return;
}
# 'pre_sert' handler
sub custom_pre_insert{
	# $table: table
	# $data: record data
	my ($this,$table,$data) = @_;
	if ($table eq "catalog_product_attachment"){
		# Fix file upload i.e. fix invalid filename..etc
		&file_upload($this,$data);
		# Set document type
		&set_document_type($data);
	}
	return;
}
# Create a form for adding/updating product attachment
sub productAttachment{
	# $attachment_id: attachment id
	# $product_id: product id
	my ($this,$attachment_id,$product_id) = @_;
	my %product = %{$this->getProduct($product_id)};
	my $form_type = $attachment_id ? "Edit" : "Add";
	my %data = $this->getAttachment($attachment_id);
	$data{catalog_product_id} = $product_id;
	my $edit_form = $this->{db}->make(
			table=>"catalog_product_attachment",
			data=>\%data,
			hide=>["catalog_product_id"],
			action=>$this->link(tab=>undef,id=>undef,action=>undef),
			extra=>{reply=>$this->link(action=>"resource_edit",tab=>undef,id=>undef)},
		);
	my %mdata = (
		ProductLabel=>"Product: $product{title}",
		FormLabel=>"$form_type Attachment",
		Form=>$edit_form,
	);
	return $this->html_productRecordForm(\%mdata);
}
sub html_productRecordForm{
	# $data: merge data
	my ($this,$data) = @_;
	my $template = <<END;
<h3><!--[[ProductLabel]]--></h3>
<h3><!--[[FormLabel]]--></h3>
<!--[[Form]]-->
END
	return $this->merge($template,$data);
}
# Get adminstrator's email address
sub adminEmail{
	my ($this) = @_;
	# Default admin's email address from existe.conf
	return $config{site}{administrator};
}
# Searchable fields
sub searchField{
	my ($this) = @_;
	my @fields = (
	{id=>"product_code",field=>"catalog_product.code"},
	{id=>"product_title",field=>"catalog_product.title"},
	{id=>"product_summary",field=>"catalog_product.summary"},
	{id=>"product_details",field=>"catalog_product.details"},
	{id=>"category_title",field=>"catalog_category.title"},
	{id=>"category_description",field=>"catalog_category.description"},
	);
	return \@fields;
}
# Remove 's' from a word(Improves search flexiblilty)
# i.e. if someone types in 'cars', we search for 'car'.
sub fixSearchTerm{
	# $terms: search terms
	my ($this,$terms) = @_;
	my @list;
	foreach my $term(@$terms){
		$term =~ s/s$//g;
		push(@list,$term);
	}
	return \@list;
}
sub getSearchResult{
	# $terms: search terms
	my ($this,$terms) = @_;
	return [] if scalar(@$terms) < 1;
	my $db = $this->{db};
	my $sectionid = $this->section_id();
	# Get catalog_product and category info
	my $sql = <<END;
select
	catalog_category.catalog_category_id as id_category_1,
	catalog_category.title as title_category_1,
	catalog_category.type as type_category_1,
	catalog_category.status as status_category_1,	
	catalog_product_category_join.catalog_product_category_join_id as id_product_join_1,
	catalog_product_category_join.catalog_category_id as category_product_join_1,
	catalog_product_category_join.catalog_product_id as product_product_join_1,
	catalog_product.catalog_product_id as id_product_1,
	catalog_product.code as code_product_1,
	catalog_product.title as title_product_1,
	catalog_product.summary as summary_product_1,
	catalog_product.details as details_product_1,
	catalog_product.status as status_product_1,
	catalog_product_attachment.catalog_product_attachment_id as id_product_attachment_1,
	catalog_product_attachment.catalog_product_id as product_id_product_attachment_1,
	catalog_product_attachment.description as description_product_attachment_1
from
	catalog_category,
	catalog_product_category_join,
	catalog_product  
	left join catalog_product_attachment on catalog_product_attachment.catalog_product_id = catalog_product.catalog_product_id
where 
	catalog_category.catalog_category_id = catalog_product_category_join.catalog_category_id and 
	catalog_product_category_join.catalog_product_id = catalog_product.catalog_product_id and
	catalog_category.section_id = $sectionid
END
	if (!$this->is_admin()){
		$sql .= " and (catalog_product.status like \"enabled\" or catalog_product.status is null or catalog_product.status = '')";
	}
	$sql .= " and (";
	my @searchfields = @{$this->searchField()};
	my @searchterms = @$terms;
	my @where;
	# Add search input to the where clause
	foreach my $term(@searchterms){
		foreach my $field(@searchfields){
			push(@where,"$field->{field} like '%$term%'");
		}
	}
	$sql .= join(" or ",@where);
	$sql .= ") ";
	$sql .= " order by catalog_product.catalog_product_id";
	
	my @results;
	my @records = $db->custom_query($sql);
	foreach my $record(@records){
		# Cache data
		my %category = (
			catalog_category_id=>$record->{id_category_1},
			title=>$record->{title_category_1},
			type=>$record->{type_category_1},
			status=>$record->{status_category_1},
		);
		my %product = (
			catalog_product_id=>$record->{id_product_1},
			code=>$record->{code_product_1},
			title=>$record->{title_product_1},
			summary=>$record->{summary_product_1},
			details=>$record->{details_product_1},
			status=>$record->{status_product_1},
		);
		my %product_join = (
			catalog_product_category_join_id=>$record->{id_product_join_1},
			catalog_category_id=>$record->{category_product_join_1},
			catalog_product_id=>$record->{product_product_join_1},
		);
		my %product_attachment = (
			catalog_product_attachment_id=>$record->{id_product_attachment_1},
			catalog_product_id=>$record->{product_id_product_attachment_1},
			description=>$record->{description_product_attachment_1},
		);
		$this->save_cache_category(\%category);
		$this->save_cache_product(\%product);
		$this->save_cache_product_join(\%product_join);
		$this->save_cache_product_attachment(\%product_attachment);
		
		# Reorganize data in a way that sub directorySearch can use
		######################################################################
	 	# directorySearch is expecting data to be returned in this format:
	 	# @data = (
	 	#	{
		#		catalog_product_id=>xxxxx,
		#		code=>xxxxx,
		#		title=>xxxxx,
		#		summary=>xxxxx,
		#		detail=>xxxxx,
		#		category_id=>xxxxx,
		#		category_title=>=>xxxxx,
		#		category_description=>xxxxx,
		#	}
		#	.... so on
	 	# )
		######################################################################
		
		# Verify that the data is valid
		if ($category{catalog_category_id} && $product{catalog_product_id}){
			my %return = %product;
			$return{category_id} = $category{catalog_category_id};
			$return{category_title} = $category{title};
			$return{category_description} = $category{description};
			push(@results,\%return);
		}
	}
	
	return \@results;
}
# Search function
sub directorySearch{
	# $phrase: key phrase entered on the search field
	my ($this,$phrase) = @_;	
	# Validate search phrase
	# remove spaces
	$phrase =~ s/^\s+?(.*?)/$1/g;
	$phrase =~ s/(.+?)\s+$/$1/g;
	# escape quotes
	$phrase =~ s/"/\\"/g;
		
	my @searchterms = split(/ /,$phrase);
	@searchterms = @{$this->fixSearchTerm(\@searchterms)};			
	
	my @results = @{$this->getSearchResult(\@searchterms)};
	
	# Reorganize data
	my %result_data;
	foreach my $result(@results){
		$result_data{$result->{catalog_product_id}} = $result;
		$result_data{$result->{catalog_product_id}}{score} = 0;
		# Gather category data
		my %temp = (
			name=>$result->{category_title},
			id=>$result->{category_id},
			description=>$result->{category_description},
		);

		# Push category data into an array
		if (!exists($result_data{$result->{catalog_product_id}}{category}{$result->{category_id}})){
			$result_data{$result->{catalog_product_id}}{category}{$result->{category_id}} = 1;
			push(@{$result_data{$result->{catalog_product_id}}{topics}},\%temp);
		}
		# Delete useless hash keys
		delete $result_data{$result->{catalog_product_id}}{category_title};
		delete $result_data{$result->{catalog_product_id}}{category_id};
		delete $result_data{$result->{catalog_product_id}}{category_description};
	}
	# Fetch attachments
	my @attachments = @{$this->getAllProduct()};
	# Order attachments by product id
	my %attachments = %{&keywise("catalog_product_id",\@attachments)};
	# Attach attachments to products
	foreach my $key(keys(%result_data)){
		if ($attachments{$key}{attachment_description}){
			if ($this->string_match(\@searchterms,$attachments{$key}{attachment_description})){
				push(@{$result_data{$key}{attachment}},$attachments{$key}{attachment_description});
			}
		}
	}
	# Sort results
	my %result = %{$this->sortSearchData(\%result_data,\@searchterms)};
	my $noresult = "<p>Your search - <b>$phrase</b> - did not match any products.</p>";
	# Gather merge field info
	my %data;
	# Search form
	$data{search} = $this->html_searchForm();
	# Search results
	$data{result} = keys(%result) > 0 ? "<p>Results for your search - <b>$phrase</b></p>".$this->searchResult(\%result) : $noresult;
	# Add new link	
	$data{new_link} = $this->html_newProduct();
	$data{next_page} = $this->html_nextpage(scalar(keys(%result)));
	return $this->html_resultListTemplate(\%data);
}
# Sort search result
sub sortSearchResult{
	# $list: search result
	my ($this,$list) = @_;
	my $sort_col = "final_score";
	my @result = sort {$b->{$sort_col} cmp $a->{$sort_col}} @$list;
	my $prev = 0;
	my $count;
	my $quality_count = $result[0]{total_terms};
	# Calculate relevance:
	#	
	# e.g. Total search terms 2
	#
	# Result Quality(keywords found)	score	relevance
	# 1	 2/2				10	100%
	# 2	 2/2				2	60% = (2/10*100) * 1/2) + (1/2 * 100) = (<score>/<highest score> * 100) * 1/<search terms>) + ((<number of terms found> - 1)/<search terms * 100)
	# 3	 1/2				20	50%
	# 4	 1/2				2	 5% = (2/20*100) * 1/2) + (0/2 * 100)
	for(my $i = 0; $i < @result; $i++){
		if (!$prev){
			$count = $result[$i]{score};
			$prev = keys(%{$result[$i]{quality}});
			$result[$i]{relevance} = sprintf("%i%",$result[$i]{quality_score} * 100);;
			next;
		}
		if ($prev != keys(%{$result[$i]{quality}})){
			$prev = keys(%{$result[$i]{quality}});
			$count = $result[$i]{score};
			$result[$i]{relevance} = sprintf("%i%",$result[$i]{quality_score} * 100);
		}else{
			my $new_quality = keys(%{$result[$i]{quality}}) - 1;
			my $relevance = $new_quality / $quality_count * 100;
			$relevance += ($result[$i]{score} / $count * 100) * (1 / $quality_count);
			$result[$i]{relevance} = sprintf("%i%",$relevance);
		}
	}
	return @result;
}
# Calculate score
sub calculateScore{
	# $rec: a search result
	# $terms: search terms
	# $string: string to run search on
	# $score: result score
	my ($this,$rec,$terms,$string,$score) = @_;
	foreach my $term(@$terms){
		my @search = ($term);
		my $match = $this->string_match(\@search,$string);
		if ($match > 0){
			$rec->{score} += $match * $score;
			$rec->{quality}{$term} = 1;
		}
	}
	return $rec;
}
# Calculate search scores
sub calculateQualityScore{
	# $rec: a search result
	# $term_count: number of search terms
	my ($this,$rec,$term_count) = @_;
	my $quality = 0;
	if ($rec->{quality}){
		$quality = (keys(%{$rec->{quality}})) / $term_count;
	}
	my $score = $rec->{score};
	$rec->{total_terms} = $term_count;
	$rec->{quality_score} = sprintf("%.4f",$quality);
	$rec->{final_score} = sprintf("%.4f",$quality).":".sprintf("%04i",$score);
	return $rec;
}
# Sort search result
sub sortSearchData{
	# $result: search results
	# $terms: search terms
	my ($this,$results,$terms) = @_;
	my %data = %$results;
	my %score = %{$this->searchScore()};
	my @search_terms = @$terms;
	foreach my $dat(keys(%data)){
		$data{$dat}{score} += 0;
		# Attachments
		foreach my $attachment (@{$data{$dat}{attachment}}){
			$data{$dat} = $this->calculateScore($data{$dat},\@search_terms,$attachment,$score{attachment_description});
		}
		# Categories
		foreach my $category(@{$data{$dat}{topics}}){
			foreach my $col("title","description"){
				my $score_label = "category_".$col;
				$data{$dat} = $this->calculateScore($data{$dat},\@search_terms,$category->{$col},$score{$score_label});
			}
		}
		# Products
		foreach my $field("code","title","summary","details"){
			my $score_label = "product_".$field;
			$data{$dat} = $this->calculateScore($data{$dat},\@search_terms,$data{$dat}{$field},$score{$score_label});
		}
		if ($data{$dat}{score} < 1){
			delete $data{$dat};
			next;
		}
		my $term_count = @$terms;
		$data{$dat} = $this->calculateQualityScore($data{$dat},$term_count);
	}
	return \%data;	
}
# Calculate the record number for next page
sub getNextPageIndex{
	my ($this) = @_;
	# Set default to 20
	return 20 if !$this->{input}{next_page};
	return $this->{input}{next_page} + $this->getResultNo;
}
# 'Next' link text
sub getNextPageLinkText{
	my ($this) = @_;
	return $this->getModuleSetting("next_page_link_text") || "Next &gt;&gt;";
}
# Construct the 'Next' link for the search result page
sub html_nextpage{
	# $count: number of records found after the search
	my ($this,$count) = @_;
	my $nextindex = $this->getNextPageIndex();
	return if $nextindex > $count;
	my $link_url = $this->link(next_page=>$nextindex,action=>$this->{input}{action},search_field=>$this->{input}{search_field});
	my $link_text = $this->getNextPageLinkText();
	return <<END;
<a href="$link_url">$link_text</a>
END
}
# Define points to be assigned to each result
sub searchScore{
	my ($this) = @_;
	my %score = (
		product_code=>1,
		product_title=>5,
		product_summary=>2,
		product_details=>2,
		category_title=>5,
		category_description=>1,
		attachment_description=>1,
	);
	return \%score;
}
# Look for matching works in a string
sub string_match{
	# $words: search keywords
	# $string: a string to be searched on
	my ($this,$words,$string) = @_;
	my $found = 0;
	# Split a string into words with the space character
	my @string_words = split(/ /,$string);
	foreach my $word(@$words){
		foreach my $s_word(@string_words){
			next if !$s_word;
			# Compare keyword against the split string
			if ($s_word =~ /$word/is){
				$found++;
			}
		}
	}
	return 0 if $found < 1;
	return $found;
}
# Search result
sub searchResult{
	# $result: search result(hash reference)
	my ($this,$result) = @_;
	my @result_data = @{$this->formatResult($result)};
	return join("",@result_data);
}
# Fetch resources
sub fetchProduct{
	# $categoryid: category record
	my ($this,$categoryid) = @_;
	return [] if !$categoryid;
	my @list = @{$this->getProductByCategory($categoryid)};
	my @products;
	foreach my $product(@list){
		$product = $this->updateTitle($product);
		# Don't show 'disabled' product in page.cgi
		if (!$this->isProductViewable($product)){
			next;
		}
		push(@products,$product);
	}
	return \@products;
}
# Fetch topics
sub fetchCategory{
	# $category: category record
	# $sectionid: section id
	# $type: category type
	my ($this,$category,$sectionid,$type) = @_;	
	my @list = @{$this->getCategoryByType($type)};
	my @categories;
	foreach my $category(@list){
		$category = $this->updateTitle($category);
		# Don't show 'disabled' category in page.cgi
		if (!$this->isCategoryViewable($category)){
			next;
		}
		push(@categories,$category);
	}	
	return \@categories;
}
# Fix topic path
sub topicList{
	# $path: path from the query e.g. '/1/2/3/4/5/6'
	my ($this,$path) = @_;
	my @oldlist = split(/\//,$path);
	my @newlist;
	# Remove the first element from array
	for(my $i = 1; $i < @oldlist; $i++){
		push(@newlist,$oldlist[$i]);
	}
	return \@newlist;
}
# Catalog search form
sub catalogSearch(){
	my ($this) = @_;
	return $this->html_searchForm();
}
# Display catalog
sub catalog{
	# $dirpath: catalog path(option - mainly used for publishing web directories to the disk)
	# $sectionid: section id
	my ($this,$dirpath,$sectionid) = @_;
	if ($dirpath){
		# Make sure that value of $dirpath begins with a '/'.. e.g. '/12323'
		$dirpath = $dirpath !~ /^\// ? "/$dirpath" : $dirpath;
	}
	# Get path info from the query
	my $path = $dirpath || $this->path();
	# Fix topic path
	my @categorylist = @{$this->topicList($path)};
	# The current category locates on the last element of the array
	my $categoryid = $categorylist[-1];
	my $category_type = $this->{input}{category_type};
	# If there is only one category, the catalog automatically jumps to that category
	if (!$path){
		my @categories = @{$this->fetchCategory($categoryid,$sectionid,$category_type)};
		if (@categories == 1) {
			$categoryid = $categories[-1]{catalog_category_id};
			$path = "/$categoryid";
		}
	}

	if (!$categoryid){
		# Top level listing
		return $this->toplevel($sectionid,$categoryid,$category_type);
	}else{
		# Sub level listing
		return $this->sublevel($sectionid,$categoryid,$path);
	}
	return;
}
# Get category path
sub path{
	# $dir_path: category path
	my ($this,$dir_path) = @_;
	# Set the path
	if (defined($dir_path)){
		$dir_path =~ s/;//g;
		$this->{path} = $dir_path;
	}
	return $this->{path};
}
# Get category path
sub getPath{
	# $id: category id
	my ($this,$id) = @_;
	# Recursively searching for a category's parent paths
	my @paths = $this->pathSearch($id);
	my $out;
	foreach my $path(@paths){
		$out .= "/$path->{catalog_category_id}";
	}
	return $out;
}
# Search for paths
sub pathSearch{
	# $id: category id
	my ($this,$id) = @_;
	my %rec = %{$this->getCategory($id)};
	# Recursively searching for parent topics of a topic
	if ($rec{parent}){
		return ($this->pathSearch($rec{parent}),\%rec);
	}else{
		return (\%rec);
	}
	return ();
}
# Are we in admin mode? i.e. running the module from ctrl-panel.cgi
sub admin_mode{
	# $flag: accepts value 0 or 1.
	my ($this,$flag) = @_;
	# Set the admin flag
	if (defined $flag){
		$this->{admin_mode} = $flag;
	}
	return $this->{admin_mode};
}
# Are we displaying the 'Top' link?
sub isTopLinkOff{
	my ($this) = @_;
	# Always enable top link for admin side
	return 0 if $this->admin_mode();
	return 1 if ($this->getModuleSetting("turn_top_link_off") || 0) > 0;
	return 0;
}
# get the 'Top' link in the directory path
sub getTopLink{
	my ($this) = @_;
	my @path;
	# Are we showing the 'Top' link?
	if (!$this->isTopLinkOff()){
		my $path_url = $this->link(path=>undef,product_id=>undef,action=>undef,resource_id=>undef);
		@path = ("<a href=\"$path_url\">Top</a>");
	}
	return \@path;
}
# Show path info 
sub sublevelPath{
	# $listdata: topic list(array reference)
	my ($this,$listdata) = @_;
	# The category that we are viewing right now
	my $currentdir = $listdata->[-1];
	# Doing a recursive search on parent categories if there is only one category in the array, @listdata
	# (This happens when the user clicks on the category link from the search results)
	if (@$listdata == 1){
		my @checklist = $this->pathSearch($currentdir->{catalog_category_id});
		# Reset the topic list
		$listdata = \@checklist;
	}	
	my %current_topic = ();
	if (@$listdata > 0){
		%current_topic = %{pop(@$listdata)};
	}
	# 'Top' refers to the toplevel(root) directory
	my @pathlist = @{$this->getTopLink()};
	# $toppath contains the path info.  It grows after each foreach loop.
	my $toppath;
	foreach my $data(@$listdata){
		# Update path info
		$toppath .= "/$data->{catalog_category_id}";
		# Create clickable link topics
		my $link_url = $this->link(path=>$toppath,product_id=>undef,action=>undef,resource_id=>undef);
		my $link = "<a href=\"$link_url\">$data->{title}</a>";
		push(@pathlist,$link);
	}
	# Add the selected topic at the end
	push(@pathlist,$currentdir->{title});
	my %template_data = (
		title=>$currentdir->{title},
		description=>$this->{db}->show_data("catalog_category","description",$currentdir->{description},$currentdir->{catalog_category_id}),
		admin_option=>$this->adminTopicOption(\%current_topic)
	);
	my $out = "<p class=\"CatalogPath\">".join(" &gt; ",@pathlist)."</p>";
	return $out;
}
# Construct product category path
sub productSublevelPath{
	# $listdata: topic list(array reference)
	# $product: product record
	my ($this,$listdata,$product) = @_;
	# The category that we are viewing right now
	my $currentdir = $listdata->[-1];
	# Doing a recursive search on parent categories if there is only one category in the array, @listdata
	# (This happens when the user clicks on the category link from the search results)
	if (@$listdata == 1){
		my @checklist = $this->pathSearch($currentdir->{catalog_category_id});
		# Reset the topic list
		$listdata = \@checklist;
	}
	# 'Top' refers to the toplevel(root) directory
	my @pathlist = @{$this->getTopLink()};
	# $toppath contains the path info.  It grows after each foreach loop.
	my $toppath;
	foreach my $data(@$listdata){
		# Update path info
		$toppath .= "/$data->{catalog_category_id}";
		# Create clickable link topics
		my $link = "<a href=\"".$this->link(path=>$toppath,product_id=>undef,action=>undef,resource_id=>undef)."\">$data->{title}</a>";
		push(@pathlist,$link);
	}
	# Add the product name at the end
	my $product_title = $this->html_productCode($product).$this->html_productTitle($product);
	push(@pathlist,$product_title);
	my $out = "<p class=\"CatalogPath\">".join(" &gt; ",@pathlist)."</p>";
	return $out;
}
# Construct product detail page
sub productDetail{
	# $productid: product id
	# $sectionid: section id
	my ($this,$productid,$sectionid) = @_;
	my %product = %{$this->getProduct($productid)};
	if (!$this->isProductViewable(\%product)){
		return "<p>Sorry, we were unable to find the product you requested.</p>";
	}
	my %resource = %{$this->updateTitle(\%product)};
	my $path = $this->path() || "/".$this->getProductCategoryID(\%resource);
	if ($this->checkPublishProduct($productid)){
		return $this->productPublishFile($productid);
	}
	# Fix topic path
	my @topicList = @{$this->topicList($path)};
	# The selected topic locates on the last element of the array
	my @topicListData;
	foreach my $id(@topicList){
		my %rec = %{$this->getCategory($id)};
		%rec = %{$this->updateTitle(\%rec)};
		push(@topicListData,\%rec);
	}	
	my %data;
	$data{CategoryDetailPath} = $this->productSublevelPath(\@topicListData,\%resource);
	$data{product_admin_option} = $this->productAdminOptionOther(\%resource);
	$data{ProductDetailCode} = $this->html_productCode(\%resource);
	$data{ProductDetailTitle} = $this->html_productTitle(\%resource);
	$data{ProductDetailSummary} = $this->html_productSummary(\%resource);
	$data{ProductDetailPrice} = $this->html_productPrice(\%resource);
	$data{ProductDetailOption} = $this->html_productOption(\%resource);
	$data{ProductDetailDetail} = $this->html_productDetail(\%resource);
	$data{ProductDetailAttachmentAll} = $this->html_productAttachmentAll(\%resource);
	$data{ProductDetailAttachmentDownload} = $this->html_productAttachmentDownload(\%resource);
	$data{ProductDetailAttachmentImage} = $this->html_productAttachmentImage(\%resource);
	$data{ProductDetailPrimaryImage} = $this->html_productImage(\%resource);
	# Thumbnail product list
	$data{CategoryDetailThumbnailList} = $this->thumbnailProductList($this->path());
	if ($this->enableRequestQuote()){
		$data{request_a_quote_link} = "<div class=\"ProductRequestQuote\"><!--&RequestQuote(product=$productid)--></div>";
	}
	return $this->html_productDetailTemplate(\%data);
}
# Construct product details page
sub html_productDetailTemplate{
	my ($this,$data) = @_;
	my $template = $this->getProductDetailTemplate("ProductDetail");
	# Use default templates
	if (!$template){
		$template = $this->is_admin() ? $this->html_productDetailTemplateAdmin() : $this->html_productDetailTemplatePublic();
	}
	$template .= $this->html_productDetailJavascript();
	return $this->merge($template,$data);
}
# Product detail template for admin
sub html_productDetailTemplateAdmin{
	my ($this) = @_;
	my $out = $this->javascript();
	$out .= <<END;
<style type="text/css">
.ProductTitle{
	color: #6f6f6f;
	font-size: 14px;
}
.ProductSummary{
	font-size: 12px;
}
.ProductOption{
	font-size: 12px;
}
.ProductDetail{
	font-size: 12px;
}
.ProductPrimaryImage{
	float: right;
	text-align: center;
}
.ProductPrimaryImageWrapper{
	width: 450;
}
.ProductAttachment{
	width: 150;
	float: left;
	text-align: center;
}
</style>
<p>
<!--[[CategoryDetailPath]]-->
<h2 class="ProductTitle"><!--[[ProductDetailCode]]--> <!--[[ProductDetailTitle]]--> <!--[[product_admin_option]]--></h2>
<!--[[ProductDetailPrimaryImage]]-->
<!--[[ProductDetailSummary]]-->
<!--[[ProductDetailOption]]-->
<!--[[ProductDetailDetail]]-->
<!--[[ProductDetailAttachmentAll]]-->
<!--[[ProductDetailPrice]]-->
<!--[[request_a_quote_link]]-->
</p>
END
	return $out;
}
# Product details template for public
sub html_productDetailTemplatePublic{
	my ($this) = @_;
	return <<END;
<p>
<!--[[CategoryDetailPath]]-->
<h2 class="ProductTitle"><!--[[ProductDetailCode]]--> <!--[[ProductDetailTitle]]--></h2>
<!--[[ProductDetailPrimaryImage]]-->
<!--[[ProductDetailSummary]]-->
<!--[[ProductDetailOption]]-->
<!--[[ProductDetailDetail]]-->
<!--[[ProductDetailAttachmentAll]]-->
<!--[[ProductDetailPrice]]-->
<!--[[request_a_quote_link]]-->
</p>
END
}
# Get sort fields
sub getSortField(){
	my ($this) = @_;
	my $table = "catalog_product";	
	my $ncol = $this->{db}->{map}->get_ncol($table);
	my %col_table;
	# Get all table fields from the dbmap
	for(my $i = 0; $i < $ncol; $i++){
		my %column = $this->{db}->{map}->get_column($table,$i);
		$col_table{$column{column}} = 1;
	}
	# Check and see if the field defined in the conf file exists in the dbmap
	my $sort_field = $this->getModuleSetting("sort_field");
	if (exists($col_table{$sort_field})){
		return $sort_field;
	}else{
		# Return title by default
		return "title";
	}
	return "title";
}
# Sort product by sortkey
sub sortProductBySortKey{
	# $cateories: categories
	my ($this,$products) = @_;
	foreach my $product(@$products){
		my $sortkey = $product->{sortkey};
		# Which field to sort
		my $sortcol = $this->getSortField();
		my $datavalue = $product->{$sortcol};
		# Concatenate sortkey with title
		$product->{sort_index} = $sortkey.$datavalue;
	}
	@$products = sort {$a->{sort_index} cmp $b->{sort_index}} @$products;
	return $products;
}
# Update category title
sub updateTitle{
	# $rec: category record
	my ($this,$rec) = @_;
	# Add a label '(DISABLED)' beside  the product title
	if ($rec->{status} =~ /^disable/i){
		if ($rec->{title} !~ /\(disabled\)/i){
			$rec->{title} = "(DISABLED) $rec->{title}";
		}
	}
	return $rec;
}

# Sub level web directory
sub sublevel{
	# $topicid: selected topic
	# $dirpath: web directory path
	my ($this,$sectionid,$topicid,$dirpath) = @_;
	my %category = %{$this->getCategory($topicid)};
	
	if (!$this->isCategoryViewable(\%category)){
		return "<p>Sorry, we were unable to find the category you requested.</p>";
	}
	
	# Show published files
	if ($this->checkPublishSubLevel($dirpath)){
		return $this->subLevelPublishFile($dirpath);
	}
	# Fetch resources
	my @resources = @{$this->fetchProduct($topicid)};
	@resources = @{$this->sortProductBySortKey(\@resources)};
	# Get topic path info
	my @topicList = @{$this->topicList($dirpath)};
	my @topicListData;
	# Fetch topic info
	foreach my $id(@topicList){
		my %rec = %{$this->getCategory($id)};
		%rec = %{$this->updateTitle(\%rec)};
		push(@topicListData,\%rec);
	}
	# Fetch selected topic's child topics
	my @subtopics = @{$this->findSubCategory($topicid)};
	# Gather merge field data
	my %data;
	# Search form
	# Topic path
	$data{CategoryDetailPath} = $this->sublevelPath(\@topicListData);
	# Child topics
	my $subtopics = $this->html_sublevelDirectory(\@subtopics,$dirpath,$topicListData[-1]);
	$data{CategoryDetailSubCategory} = $subtopics;
	# Set a flag to determine whether to show 'No product found' message or not.
	$this->showProductFlag($subtopics);
	# Resources
	$data{CategoryDetailProductList} = $this->html_sublevelResource(\@resources);
	# Suggest a link
	$data{CategoryDetailNewLink} = $this->html_newProduct();
	# Thumbnail product list
	$data{CategoryDetailThumbnailList} = $this->thumbnailProductList($this->path());
	# Category title
	$category{admin_option} = $this->adminTopicOption(\%category);
	$data{CategoryDetailTitle} = $this->html_categoryTitle(\%category);
	# Category description
	$data{CategoryDetailDescription} = $this->categoryDescription(\%category);

	my $sublevel_html = $this->html_sublevelTemplate(\%data);
	return <<END;
<script language="javascript" src="$this->{html}/_ExSite/js/misc.js"></script>
$sublevel_html
END
	return $this->html_sublevelTemplate(\%data);
}

# Set internal flag that displays the 'No products found' message
sub showProductFlag{
	# $subcategories: sub categories
	my ($this,$subcategories) = @_;
	if ($subcategories && $this->getModuleSetting("subdirectory_no_product_message")){
		$this->{no_product_message} = 1;
	}else{
		$this->{no_product_message} = 0;
	}
	return;
}
# Root web directory
sub toplevel{
	# $sectionid: section id
	# $categoryid
	# $category_type
	my ($this,$sectionid,$categoryid,$category_type) = @_;
	# Show published files
	if ($this->checkPublishTopLevel()){
		return $this->topLevelPublishFile();
	}
	# Fetch all root topics
	my @categories = @{$this->findParentCategory($sectionid,$category_type)};
	my %subcategories;
	foreach my $category(@categories){
		my $category_id = $category->{catalog_category_id};
		my @subcategories = @{$this->findSubCategory($category_id)};
		# Index child topics by parent topoic id
		foreach my $sub(@subcategories){
			$subcategories{$category->{catalog_category_id}}{$sub->{title}} = $sub;
		}
	}
	# Gather merge field data
	my %data;
	# Root web directory
	$data{topic} = $this->html_toplevelDirectory(\@categories,\%subcategories);
	$data{select_category_type} = $this->html_selectCategoryType();
	$data{product_image} = $this->html_toplevelProductImage();
	# Admin reports
	if ($this->is_admin()){
		$data{new_link} = $this->adminTopLevelNewLink();
	}
	return $this->html_toplevelTemplate(\%data);
}
# Category type selector
sub html_selectCategoryType{
	my ($this) = @_;
	# Select categories	
	my @options = @{$this->getCategoryType()};
	my $select_value;
	foreach my $option(@options){
		next if !$option;
		my $val = $option;
		my $selected = $this->{input}{category_type} eq $val ? " selected" : "";
		$select_value .= <<END;
	<option value="$val"${selected}>$val</option>
END
	}
	# Location to redirect a page
	my $location_url = $this->link(category_type=>undef);
	return <<END;
<script language="javascript">
<!--
function SwitchCategory(id){
	var mySelect = document.getElementById(id);
	location.href = "${location_url}&category_type=" + mySelect.value;
}
//-->
</script>
<form>
Select a category type: 
<select id="select_category" name="category_type" onChange="SwitchCategory(this.id)">
	<option value="">default</option>
$select_value
</select>
</form>
END
}
# Url to publish the catalog
sub getPublishUrl{
	my ($this) = @_;
	my $sectionid = $this->section_id();
	return "$config{server}{CGIpath}/publish.cgi?module=Catalog&section_id=$sectionid";
}
# Top level category listing admin options
sub adminTopLevelNewLink{
	my ($this) = @_;
	my $link_topic_new = $this->link(action=>"topic_new");
	my $link_new_product = $this->link(action=>"new_product");
	my $publish_url = $this->getPublishUrl();
	return <<END;
<center>
<a href="$link_topic_new">Add new category</a>
<br>
<a href="$link_new_product">Add new product</a>
<br>
<a target="_blank" href="$publish_url">Publish catalog</a>
</center>
END
}
# Reorganize the data for outputting html
sub displayDirectory{
	# $topics: list of topics(array reference
	# $column: number of columns in the table
	my ($this,$topics,$column) = @_;	
	require POSIX;
	# Figure out the number of rows in the table
	my $row = POSIX::ceil(@$topics/$column);
	# Record position in the array, $topics
	my $rec_pos = 0;
	# Organize the data column by column
	my %coldata;
	# Loop throught each column
	for(my $i = 0; $i < $column; $i++){
		# Use modulus to figure out the number of records per column
		my $looprow = (((@$topics % $column) - 1) >= $i) || (@$topics % $column == 0) ? $row : $row - 1;
		# Populate each column with data
		for(my $j = 0; $j < $looprow; $j++){
			# Column number
			my $hashindex = $j + 1;
			# Merge field
			my $dataindex = "row".($i + 1);
			my $topic = $topics->[$rec_pos];
			# Merge field data
			$coldata{$hashindex}{$dataindex} = $topic->{display_data};
			# Increment the index
			$rec_pos++;
		}		
	}
	# Once we have the data formatted column by column, we need to convert
	# the data from a hash to an array.  We merge data by row not by column.
	# Each array represents a row in the table.
	my @rowdata;
	# Numericly sort the hash keys
	foreach my $hindex(sort {$a <=> $b} keys(%coldata)){
		my %hdata = %{$coldata{$hindex}};
		push(@rowdata,\%hdata);
	}
	return \@rowdata;
}

# Process new category form input
sub processNewCategory{
	my ($this) = @_;
	my $parent_id;
	my %category;
	my $reply;
	foreach my $key (keys(%{$this->{input}})){
		if ($key =~ /^category_select$/i){
			$parent_id = $this->{input}{$key};
		}elsif ($key =~ /^category_(.+)/i){
			my $hashkey = $1;
			$category{$hashkey} = $this->{input}{$key};
		}
		$reply = $this->{input}{reply};
	}
	# Only set the type for root level categorys
#	if ($parent_id){
#		delete $category{type};
#	}
	my $new_id = $this->{db}->insert("catalog_category",\%category);
	return "<p>Unable to create a new category..</p>" if !$new_id;
	my $new_joinid = $this->{db}->insert("catalog_category_join",{parent=>$parent_id,child=>$new_id});
	return "<p>Unable to join 2 categories..</p>" if !$new_joinid;
	return $this->redirect($reply);
}

# Create a new category
sub categoryNew{
	# $section_id: section id
	my ($this,$section_id) = @_;
	my $reply = $this->link(action=>undef,topic_id=>undef);	
	return $this->html_newCategoryForm($section_id,$reply);
}
# Create the input field selecting section id
sub html_categorySectionSelect{
	my ($this,$section_id,$column) = @_;
	if ($this->enableCategorySharing()){
		my @sites = $this->{db}->my_real_sites();
		my $input_select;
		# Only access level 3 user is allowed to make a category global
		if ($this->{db}->authorize() >= 3){
			$input_select .= <<END;
	<option value="0">All sites</option>
END
		}
		foreach my $site(@sites){
			my $selected = $site->{section_id} == $section_id ? " selected" : "";
			$input_select .= <<END;
	<option value="$site->{section_id}"${selected}>$site->{title}</option>
END
		}
		return <<END;
<select name="$column">
$input_select
</select>
END
	}else{
		return "<input type=\"hidden\" name=\"$column\" value=\"$section_id\">";
	}
}
# New category form
sub html_newCategoryForm{
	# $section_id: section id
	# $reply: reply url
	my ($this,$section_id,$reply) = @_;
	my $type = $this->{input}{category_type};
	my $category_select = $this->html_categorySelect($this->path());
	my $form_action = $this->link(action=>undef,topic_id=>undef);
	my $section_select = $this->html_categorySectionSelect($section_id,"category_section_id");
	if ($section_select !~ /hidden/i){
		$section_select = <<END;
<p>
<span class="label">Section</span><br>
$section_select
</p>
END
	}
	return <<END;
<form method="post" action="$form_action">
<h2>New Category</h2>
$section_select
<input type="hidden" name="action" value="new_category">
<input type="hidden" name="reply" value="$reply">
<p><span class="label">Parent Category</span><br>$category_select</p>
<p><span class="label">Title</span><br><input type="text" size="60" name="category_title" value=""></p>
<p><span class="label">Description</span><br><textarea wrap="virtual" rows="5" cols="60" name="category_description"></textarea></p>
<p><span class="label">Type</span><br><input type="text" size="60" name="category_type" value=""></p>
<p><input type="submit" value="Submit">&nbsp;&nbsp;&nbsp;&nbsp;<input type="button" value="Cancel" onclick="history.back()"></p>
</form>
END
}
# Category selector
sub html_categorySelect{
	# $sectionid: section id
	# $path: category path
	my ($this,$path) = @_;
	my @paths = split(/\//,$path);
	my $catid = @paths[-1];
	my $selector = $this->getCategoryList($catid);
	return <<END;
<select name="category_select">
$selector
</select>
END
}
# Show the detail of a record
sub recordList{
	# $table: table
	# $id: record id
	# $child: child
	# $type: parent|child
	#	- type of secondary records to display.
	#	- 'child': display records in child tables
	#	- 'parent': currently, we are using 'parent' to display a catalog's parent join tables.  
	#		You'll need to add your logic if you decided to this option
	my ($this,$table,$id,$child,$type) = @_;
	return if !$child;
	return if (keys(%$child) < 1);
	my $db = $this->{db};
	my %children;
	$type = $type || "child";
	# Logic for fetching associated records. i.e. parent tables or child tables
	if ($type eq "parent"){
		if ($table eq "catalog_category"){
			# The following section of code is done specifically to fetch a category's parent categories.
			push(@{$children{catalog_category}},$id);
			my @list = @{$this->getJoinByChild($id)};
			foreach my $lst(@list){
				next if !$lst->{parent};
				push(@{$children{catalog_category_join}},$lst->{catalog_category_join_id});
			}
		}
	}else{
		%children = %{$db->find_descendants($table,$id)};
	}
	# Index child table records
	my @c_table;
	if (keys(%$child) > 0){
		foreach my $ctab(keys(%$child)){
			push(@c_table,$ctab);
		}	
	}
	my $out;
	foreach my $tab(@c_table){
		my @rec;
		if (exists($children{$tab})){
			my @c_id = @{$children{$tab}};		
			foreach my $c_id(@c_id){
				my $rec = $this->getRecord($tab,$c_id);
				my @row;
				my @option;
				foreach my $col(@{$child->{$tab}{cols}}){
					my $r_data;
					if ($tab eq "catalog_category_join" && $col eq "option"){
						@option = (
							#{url=>$this->link(topic_id=>$rec->{$type},action=>"topic_edit_form"),id=>"edit",label=>"Edit Category"},
							#{url=>$this->link(topic_id=>$rec->{$type},action=>"topic_delete"),id=>"delete",label=>"Delete Category"},
							{url=>$this->link(join_id=>$rec->{catalog_category_join_id},join_type=>$type,action=>"join_edit_form"),id=>"edit",label=>"Edit Category Join Record"},
							{url=>$this->link(join_id=>$rec->{catalog_category_join_id},join_type=>$type,action=>"join_delete"),id=>"delete",label=>"Delete Category join Record"},
						);
						$r_data = $this->reportOptions(\@option);
					}elsif ($tab =~ /catalog_product_(category_join|attachment)/i && $col eq "option"){
						@option = (
							{url=>$this->link(record_id=>$c_id,action=>"$child->{$tab}{option}{action}_form"),id=>"edit",label=>"Edit $child->{$tab}{title}"},
							{url=>$this->link(record_id=>$c_id,action=>"$child->{$tab}{option}{action}_delete"),id=>"delete",label=>"Delete $child->{$tab}{title}"},
						);
						$r_data = $this->reportOptions(\@option);
					}else{
						$r_data = $db->show_data($tab,$col,$rec->{$col},$c_id) || "n/a";
					}
					push(@row,$r_data);
				}
				push(@rec,\@row);
			}
		}
		my %report = (
			title=>$child->{$tab}{title},
			data=>\@rec,
			headers=>$child->{$tab}{headers},
			tools=>$child->{$tab}{tools},
		);
		$out .= "<p>".$this->reportbuilder(\%report)."</p>";		
	}
	return $out;
}

# Edit a category
sub categoryEdit{
	# $category_id: category id
	my ($this,$category_id) = @_;
	return if !$this->is_int($category_id);
	my %option = (all_edit=>0,edit=>1,delete=>1);
	$config{db_ops}{edit}{url} = $this->link(action=>"topic_edit_form",topic_id=>$category_id);
	
	my $db = $this->{db};
	my $table = "catalog_category";
	my $title = "Category";	
	my @options = (
		{url=>$this->link(action=>"topic_edit_form"),id=>"edit",label=>"Edit Category"},
		{url=>$this->link(action=>"topic_delete",path=>undef),id=>"delete",label=>"Delete Category"},
	);
	# Show category record
	my $parent_category = $this->recordList($table,$category_id,
		{
			"catalog_category_join"=>{
				title=>"Parent Category",
				headers=>["Category","Options"],
				cols=>["parent","option"],
				#tools=>$this->reportOptions([{topic_id=>undef,url=>$this->link(action=>"topic_new"),id=>"new",label=>"New Category"}]),
				tools=>$this->reportOptions([{topic_id=>$category_id,url=>$this->link(action=>"join_new",join_type=>"parent"),id=>"new",label=>"New Category Join"}]),
			}
		},"parent",
	);

	my @topicList = @{$this->topicList($this->getPath($category_id))};
	my @topicListData;
	# Fetch topic info
	foreach my $id(@topicList){
		my %rec = %{$this->getCategory($id)};
		%rec = %{$this->updateTitle(\%rec)};
		push(@topicListData,\%rec);
	}
	
	my $out;
	$out .= $this->sublevelPath(\@topicListData);
	$out .= "\n<p>";
	$out .= $this->recordInfo($table,$category_id,{
		title=>"Category",
		tools=>$this->reportOptions(\@options),
		foot=>$parent_category,
	});
	$out .= "</p>\n";
	return $out;
}
# Edit a product
sub productEdit{
	# $product_id: product id
	my ($this,$product_id) = @_;
	return "<p>Sorry, you can't to update an invalid product.</p>" if !$this->is_int($product_id);
	my $db = $this->{db};
	my $option = {all_edit=>0,edit=>1,delete=>1};
	my $title = "Product";
	my $table = "catalog_product";
	# Category path
	my %product = %{$this->getProduct($product_id)};
	my $path = $this->path() || "/".$this->getProductCategoryID(\%product);
	# Fix topic path
	my @topicList = @{$this->topicList($path)};
	# The selected topic locates on the last element of the array
	my @topicListData;
	foreach my $id(@topicList){
		my %rec = %{$this->getCategory($id)};
		push(@topicListData,\%rec);
	}
	my @options = (
		{url=>$this->link(action=>"resource_record_form",record_id=>$product_id),id=>"edit",label=>"Edit Product"},
		{url=>$this->link(),id=>"delete",label=>"Delete Product"},
	);
	my $out = $this->productSublevelPath(\@topicListData,\%product);
	$out .= $this->recordInfo($table,$product_id,{
		title=>"Product",
		tools=>$this->reportOptions(\@options),
		foot=>$this->recordList(
			$table,$product_id,
			{
			"catalog_product_category_join"=>
				{
					title=>"Category",
					headers=>["Category","Options"],
					cols=>["catalog_category_id","option"],
					option=>{action=>"resource_category"},
					tools=>$this->reportOptions([{url=>$this->link(action=>"resource_category_form"),id=>"new",label=>"New Category"}]),
				},
			"catalog_product_attachment"=>
				{
					title=>"Attachment",
					headers=>["Heading","Type","Description","Thumbnail","Options"],
					cols=>["heading","type","description","thumbnail","option"],
					option=>{action=>"resource_attachment"},
					tools=>$this->reportOptions([{url=>$this->link(action=>"product_attachment",tab=>"catalog_product_attachment"),id=>"new",label=>"New Attachment"}]),
				}

			}
		),
	});	
	return $out;
}


# Create a new resource
sub resourceNew{
	my ($this) = @_;
	my $reply = $this->link(action=>undef);
	my $form = $this->html_productForm();
	# Gather merge field data
	my $newCategoryList = $this->getCategoryList();
	my %data = (
		category_list_1=>$newCategoryList,
		category_list_2=>$newCategoryList,
		category_list_3=>$newCategoryList,
	);
	return $this->merge($form,\%data);
}
# Get column data using ExSite::DBMap
sub getRecordColumnData{
	# $table: table
	my ($this,$table) = @_;
	my @colmap;
	my $db = $this->{db};
	my $ncol = $db->{map}->get_ncol($table);
	for (my $icol = 0; $icol < $ncol; $icol++) {
		my %colmap = $db->{map}->get_column($table,$icol);
		if (($colmap{display} !~ /no/)){
	  		push @colmap, \%colmap;
		}
	}
	return \@colmap;
}
# Get report column
sub getReportColumn{
	# $table: table
	# $col: column
	my ($this,$table,$col) = @_;
	return [] if !$table || !$col;
	# Get table columns
	my $display_col = $this->getRecordColumnData($table);
	my @cols;
	foreach my $c(@$display_col){
		push(@cols,$c->{$col});
	}
	return \@cols;
}
# Get column labels
sub getReportColumnLabel{
	# $table: table
	my ($this,$table) = @_;
	return $this->getReportColumn($table,'label');
}
# Get table columns
sub getReportColumnDB{
	# $table: table
	my ($this,$table) = @_;
	return $this->getReportColumn($table,'column');
}
# Display a table record information
sub recordInfo{
	# $table: table
	# $id: record id
	# $opt: option hash(reference)
	#	title: report title
	# 	foot: footer portion of the report
	#	tools: tool options
	my ($this,$table,$id,$opt) = @_;
	my $cols = $this->getReportColumnLabel($table);
	my $db_cols = $this->getReportColumnDB($table);
	$this->{db}->handler("show_data",\&handler_show_data);
	my $db = $this->{db};
	my %record = %{$this->getRecord($table,$id)};
	my @rec;	
	foreach my $col(@$db_cols){
		push(@rec,$db->show_data($table,$col,$record{$col}));
	}
	my %report_option = (
		title=>$opt->{title},
		data=>\@rec,
		headers=>$cols,
		foot=>$opt->{foot},
		tools=>$opt->{tools},
	);
	my $report_html = $this->reportbuilder(\%report_option);
	return <<END;
<p>
$report_html
</p>
END
}
# show_data handler
sub handler_show_data{
	my ($this,$table,$column,$data,$id) = @_;
	if ($table eq "catalog_category"){
		if ($column eq "section_id"){
			if (!$data){
				return "All Sites";
			}
		}
	}
	return undef;
}

# Category join handler
sub handler_category_join{
	# %opt: option hash
	#	table: table
	#	column: table column
	#	value: default selected value
	#	record: record id
	my ($this,%opt) = @_;
	if ($opt{table} eq "catalog_category_join" && $opt{column} =~ /parent|child/i){
		my $table = $opt{table};
		my $column = $opt{column};
		my $select = $opt{value};
		my $record = $opt{record};
		my $out = "<select name=\"${table}#${record}#${column}#key\">";
		$out .= $share{catalog}{object}->getCategoryList_tree($share{catalog}{object}->section_id(),$select,{terminate_id=>$share{catalog}{input}{topic_id}});
		$out .= "</select>";
		return $out;
	}
	return undef;
}
# Create a form for entering a new category join record
sub categoryJoinNewForm{
	# $join_id: id of the join record
	# $join_type: parent|child(current 'option' isn't used
	my ($this,$topic_id,$join_type) = @_;
	# Default the $join_type to "parent"
	$join_type = "parent";
	return "<p>Invalid category id</p>" if !$this->is_int($topic_id);
	return "<p>Invalid join type</p>" if $join_type !~ /parent|child/;
	my $f = $this->{db};
	my $table = "catalog_category_join";
	my @hide = ("child");
	if ($join_type eq "child"){
		@hide = ("parent");
	}
	my %join_rec = (
		join("",@hide)=>$topic_id,
	);
	my %extra = (
		reply=>$config{server}{server}.$this->link(join_id=>undef,join_type=>undef,action=>"topic_edit"),
	);
	$this->installCatalogObject();
	$f->handler("input_exsite",\&handler_category_join);	
	my $form = $f->make(table=>$table,hide=>\@hide,data=>\%join_rec,extra=>\%extra);
	$this->uninstallCatalogObject();
	return <<END;
<h2>New $join_type category</h2>
$form
END
}
# Create a form for editing a category join record
sub categoryJoinEditForm{
	# $join_id: id of the join record
	# $join_type: parent|child(current 'option' isn't used
	my ($this,$join_id,$join_type) = @_;
	# Default the $join_type to "parent"
	$join_type = "parent";
	return "<p>Invalid join record id</p>" if !$this->is_int($join_id);
	return "<p>Invalid join type</p>" if $join_type !~ /parent|child/;
	my $f = $this->{db};
	my $table = "catalog_category_join";
	my $id = $join_id;
	my @hide = ("child");
	if ($join_type eq "child"){
		@hide = ("parent");
	}
	$this->installCatalogObject();
	$f->handler("input_exsite",\&handler_category_join);	
	my %extra = (
		reply=>$config{server}{server}.$this->link(join_id=>undef,join_type=>undef,action=>"topic_edit"),
	);
	my %join_rec = $this->getCategoryJoin($join_id);
	my $form = $f->make(table=>$table,id=>$id,hide=>\@hide,data=>\%join_rec,extra=>\%extra);
	$this->uninstallCatalogObject();
	return <<END;
<h2>Edit $join_type category</h2>
$form
END
}
# Delete a category join record
sub categoryJoinDelete{
	# $id: record id
	my ($this,$id) = @_;
	return $this->delete("catalog_category_join",$id);
}
# Edit topic form
sub categoryEditForm{
	my ($this,$id) = @_;
	# Is $id an integer?
	return if !$this->is_int($id);
	return $this->customForm("catalog_category",$id,"Edit Category");
}
# Edit product related forms
sub productEditForm{
	# $type: type of form to create(catalog_product, catalog_product_category_join or catalog_product_attachment)
	# $id: record id
	my ($this,$type,$id) = @_;
	my $title_action = $id ? "Edit" : "New";
	# Predefine editable tables
	my %table = (
		record=>{table=>"catalog_product",label=>"Product"},
		category=>{table=>"catalog_product_category_join",label=>"$title_action Category Visibility"},
		attachment=>{table=>"catalog_product_attachment",label=>"$title_action Attachment"},
	);
	# Attempt to edit a table that's not exists in the predefine editable tables
	return if !exists($table{$type}{table});
	my %option;
	if (!$id){
		# Get $id from the query string if one isn't provided
		if ($table{$type}{table} !~ /^catalog_product$/i){
			$option{data} = {catalog_product_id=>$this->{input}{resource_id}};
		}
	}
	return $this->customForm($table{$type}{table},$id,$table{$type}{label},\%option);
}

# Custom input form
sub customForm{
	# $table: table
	# $id: record id
	# $label: form table
	# $opt: option hash(reference)
	#	data: set hidden data
	my ($this,$table,$id,$label,$opt) = @_;
	my $db = $this->{db};
	# Each table has its own form reply url
	my %reply = (
		catalog_category=>$this->link(section_id=>$this->section_id(),path=>$this->path(),action=>"topic_edit"),
		catalog_product_category_join=>$this->link(section_id=>$this->section_id(),path=>$this->path(),action=>"resource_edit",record_id=>undef),
		catalog_product_attachment=>$this->link(section_id=>$this->section_id(),path=>$this->path(),action=>"resource_edit",record_id=>undef),
		catalog_product=>$this->link(section_id=>$this->section_id(),path=>$this->path(),action=>"resource_edit",record_id=>undef),
	);
	# Reply url
	my $reply = $config{server}{server};
	$reply .= $reply{$table} || $this->link(section_id=>$this->section_id(),path=>$this->path(),action=>undef,topic_id=>undef,tab=>undef,id=>undef,record_id=>undef);
	# Define make() parameters
	my %param = (
		table=>$table,
		record=>$id,
		action=>$this->link(action=>undef),
		extra=>{
			reply=>$reply,
			action=>"process_doform",
		},
		hide=>[],
	);
	
	if (!$id){
		$param{data} = $opt->{data};
	}
	
	if ($table eq "catalog_product_join"){
		$param{extra}{catalog_product_id} = $this->{input}{product_id};
	}elsif ($table eq "catalog_category_join"){
		$param{extra}{catalog_category_id} = $this->{input}{topic_id};
	}
	
	# Allow user update existing attachment thumbnail?
	if ($table eq "catalog_product_attachment"){
		if (!$this->allowThumbnailUpload()){
			push(@{$param{hide}},"thumbnail");
		}
	}
	
	if ($table eq "catalog_category"){
		if (!$this->enableCategorySharing()){
			push(@{$param{hide}},"section_id");
		}		
	}else{
		push(@{$param{hide}},"section_id");
	}
	
	$this->installCatalogObject();
	# Install handlers
	$this->{db}->handler("input_exsite",\&handler_input_exsite);
	$this->{db}->handler("user_owns",\&handler_user_owns);
	$this->{db}->handler("approve",\&handler_approve);
	my $form = $db->make(%param);
	my $product_label;
	if ($table eq "catalog_product_attachment"){
		my %attachment = %{$this->getAttachment($id)};
		my %product = %{$this->getProduct($attachment{catalog_product_id})};
		$product_label = "<h3>Product: $product{title}</h3>";
	}
	
	my %mdata = (
		ProductLabel=>$product_label,
		FormLabel=>$label,
		Form=>$form,
	);
	$this->uninstallCatalogObject();
	return $this->html_productRecordForm(\%mdata);	
}
# Save catalog object in the share variable
sub installCatalogObject{
	my ($this) = @_;
	$share{catalog}{object} = $this;
	return;
}
# Remove catalog object fromt he share variable
sub uninstallCatalogObject{
	my ($this) = @_;
	if (exists($share{catalog}{object})){
		$share{catalog}{object} = undef;
		delete $share{catalog}{object};
	}
	return;
}
# input_exsite handler
sub handler_input_exsite{
	# %opt: option hash
	#	table: table
	#	column: column
	#	value: selected value
	#	record: record id
	my ($this,%opt) = @_;
	if ((
		($opt{table} eq "catalog_category_join" && $opt{column} eq "parent") || 
		($opt{table} eq "catalog_product_category_join" && $opt{column} eq "catalog_category_id")
	) && $config{$share{catalog}->{class_name}}{edit}{categorylist}{treemode}){
		my $table = $opt{table};
		my $column = $opt{column};
		my $select = $opt{value};
		my $record = $opt{record};
		my $out = "<select name=\"${table}#${record}#${column}#key\">";
		$out .= $share{catalog}{object}->getCategoryList($select);
		$out .= "</select>";
		return $out;
	} elsif ($opt{table} eq "catalog_product_category_join" && $opt{column} eq "catalog_category_id"){
		my $orig_section_id = $share{catalog}{object}->section_id();
		my @sites = $this->my_real_sites();
		my $table = $opt{table};
		my $column = $opt{column};
		my $select = $opt{value};
		my $record = $opt{record};
		my $out = $share{catalog}{object}->getCategoryList($select);
		return <<END;
<select name="${table}#${record}#${column}#key">
$out
</select>
END
	}
	if ($opt{table} eq "catalog_category" && $opt{column} eq "section_id"){
		my $input_name = "$opt{table}$config{form}{sepchar}$opt{record}$config{form}{sepchar}section_id";
		return $share{catalog}{object}->html_categorySectionSelect($opt{value},$input_name);
	}
	return undef;
}
# user_owns handler
# custom uthorization chekcing routine
sub handler_user_owns{
	my ($this,$table,$record) = @_;
	if ($table =~ /catalog_(category|product(_category_join|_attachment)?)/i){
		return $share{catalog}{object}->is_admin();
	}
	return undef;
}
# approve handler
# custom uthorization chekcing routine
sub handler_approve{
	my ($this,$op,$table,$record) = @_;
	if ($table =~ /catalog_(category|product(_category_join|_attachment)?)/i){
		return $share{catalog}{object}->is_admin();
	}
	return undef;

}
# Delete a product
sub productDelete{
	# $table: table
	# $id: record id
	my ($this,$table,$id) = @_;
	# Only allow deletion from these tables
	my %table  = (
		record=>{table=>"catalog_product",label=>"Product"},
		category=>{table=>"catalog_product_category_join",label=>"Edit Category"},
		attachment=>{table=>"catalog_product_attachment",label=>"Edit Attachment"},
	);
	return if !$this->is_int($id);
	# Invalid table
	return if !exists($table{$table}{table});
	$this->delete($table{$table}{table},$id);
	return;
}
# Delete a topic
sub categoryDelete{
	# $id: category id
	my ($this,$id) = @_;
	$this->delete("catalog_category",$id);
	return;
}
# Remove illegal characters(%20,',"...etc) to prevent the filename being url encoded
sub modifyFilename{
	# $filename: filename
	my ($this,$filename) = @_;
	return &clean_filename($filename);
}
# Get attachment file
sub getAttachmentFile{
	my ($this,$file) = @_;
	# $file: filename
	return $this->readUploadFile($file);
}
# Get thumbnail data
sub getAttachmentThumbnail{
	# $file: filename
	my ($this,$file) = @_;
	return $this->readUploadFile($file,1);
}
# Read upload file submitted via the form
sub readUploadFile{
	# $file: filename
	# $mode(accepts value 1 or 0): are we in process image mode? if so, we'll only process image files(i.e. jpg, gif, jpeg and png)
	my ($this,$file,$mode) = @_;
	my @path = split(/[:\\\/]/,$file);
	# Check for image files.  
	if ($mode){
		if ($path[-1] !~ /jpg|gif|jpeg|png$/i){
			return;
		}
	}
	my $fdata = undef;
	my $size = 0;
	my $continue = 1;
	while($continue){
		$continue = read($file,$fdata,1024,$size);
		$size += $continue;
	}
	my $filename;
        if ($path[-1]){
		$filename = $path[-1];
		# Fix filename replace illegal characters from the filename
		$filename = $this->modifyFilename($filename);
	}
	my $sc = $config{form}->{sepchar};
	# Encode the file
	my $mimedata = encode_base64($fdata,"");
	my $upload_data = "${filename}${sc}${mimedata}";
	return $upload_data;
}
# Process document files
sub processDocument{
	my ($this,$file,$description,$thumbnail,$type) = @_;
	# Get file data
	my $upload_data = $this->getAttachmentFile($file);
	# Get thumbnail data
	my $thumbnail_data = $this->getAttachmentThumbnail($thumbnail);
	# Prepare the data for insertion
	my %data = (
		upload=>$upload_data,
		description=>$description,
		thumbnail=>$thumbnail_data,
		type=>$type,
	);
	return \%data;
}
# Process post data for adding/editing a resource
sub processPost{
	my ($this) = @_;
	# Post query
	my %input = %{$this->{input}};
	# Category info
	my %category;
	# Store resource data
	my %record;
	# Reply url
	# File uploads
	my @uploads;
	# Process form data
	foreach my $key(keys(%input)){
		# All resource related data are store in the resource_.... keys
		next if $key !~ /^(resource|upload)/i;
		if ($key =~ /^upload_(\d)_(.+)/i){
			my $ul_id = $1;
			if (!$input{"upload_${ul_id}_file"}){
				delete $input{"upload_${ul_id}_file"};
				delete $input{"upload_${ul_id}_description"};
				next;
			}
			my %rec = %{$this->processDocument($input{"upload_${ul_id}_file"},$input{"upload_${ul_id}_description"},$input{"upload_${ul_id}_thumbnail"},$input{"upload_${ul_id}_type"})};
			# Delete the following keys because we've saved the data in %rec already.
			delete $input{"upload_${ul_id}_file"};
			delete $input{"upload_${ul_id}_description"};
			delete $input{"upload_${ul_id}_thumbnail"};
			delete $input{"upload_${ul_id}_type"};
			push(@uploads,\%rec);
		# Category information
		}elsif ($key =~ /^resource_category_(\d+)/i){
			my $cat_id = $1;
			next if !$input{$key};
			# To prevent the system from inserting duplicate links into the directory 
			# when the user selects duplicate categories
			$category{$input{$key}} = 1;
		# Resource information
		}elsif ($key =~ /resource_(.+)/i){
			my $col = $1;
			# Post data use 'resource_id' to store the record id.
			# The resource table uses 'webdir_resource_id' to store the record id.
			# Adjustment needs to be made for record id, but everthing else should remain the same.
			$col = $col eq "id" ? "catalog_product_id" : $col;
			$record{$col} = $input{$key};
		}			
	}
	# Put the selected categories into an array
	my @category = keys(%category);
	# The user didn't select a category on the form
	if (@category < 1){
		# Error message
		$input{custom_message} = "<p>Error: You must select a category for the link.</p>";
		# Gather merge field info
		$input{category_list_1} = $this->getCategoryList($input{resource_category_1});
		$input{category_list_2} = $this->getCategoryList($input{resource_category_2});
		$input{category_list_3} = $this->getCategoryList($input{resource_category_3});
		my $form = $this->html_productForm();
		# return the user back to the form
		return $this->merge($form,\%input);
	}
	# record id
	my $rec_id;
	# insert a new record
	if (!$record{catalog_product_id}){
		# For admin, the status is set to 'active'
		# For eveyone else, the status is set to 'suggested
#		$record{status} = $this->is_admin() ? "active" : "suggested";
		# Remove the primary key
		delete $record{catalog_product_id};
		# Insert the record
		$rec_id = $this->{db}->insert("catalog_product",\%record);
		$record{catalog_product_id} = $rec_id;
	# update a record
	}else{
		$rec_id = $record{catalog_product_id};
		# Update the record
		$this->{db}->update("catalog_product",\%record,{catalog_product_id=>$rec_id});	
		# Find all categories that a resource is linked to and delete them
		my @join_records = $this->getProductCategoryID({catalog_product_id=>$rec_id});
		# When updating a record, we reinsert the join records again, because users may reajust the join table information from the form
		foreach my $join(@join_records){
			$this->delete("catalog_product_category_join",$join->{catalog_product_id});
		}
	}
	my $admin_email = $this->adminEmail();
	my $error = <<END;
<p>
Error: The system failed to add a new product to the catalog. <br>
Please contact the administrator(<a href="mailto:$admin_email">$admin_email</a>).<br>
<input type="button" value="&lt; Back" onclick="history.back()">
</p>
END
	# Return an error message if update/insert fails
	return $error if !$rec_id;
	# Link a resource to topics
	foreach my $cat_id(@category){
		my %join = (
			catalog_product_id=>$rec_id,
			catalog_category_id=>$cat_id,
		);
		$this->{db}->insert("catalog_product_category_join",\%join);
	}
	$this->{db}->{catalog_image_size} = $this->getModuleSetting("image_size");
	$this->{db}->{catalog_thumbnail_size} = $this->getModuleSetting("thumbnail_size");
	$this->installCatalogObject();
	$this->{db}->handler("pre_insert",\&custom_pre_insert);
	$this->{db}->handler("pre_update",\&custom_pre_update);
	# Insert attachments
	foreach my $upload(@uploads){
		$upload->{catalog_product_id} = $rec_id;
		$this->{db}->insert("catalog_product_attachment",$upload);
	}
	$this->uninstallCatalogObject();
	# Republish web directory
	$this->republish();
	my $reply = $config{server}{server}.$this->link(action=>"resource_edit",resource_id=>$rec_id);
	$this->redirect($reply);
	return;
}
# Report option
sub reportOptions{
	# $options: report options(array reference)
	my ($this,$options) = @_;
	# Output html for report options
	my $out;
	foreach my $op (@$options){
		$out .= "&nbsp;&nbsp;<a href=\"$op->{url}\">".$this->getIcon($op->{id},$op->{label})."</a>";
	}
	return $out;
}
# Figure out if a category and its sub category have products or not.
sub findProducts{
	# $categoryid: category id
	my ($this,$categoryid) = @_;
	# Find the category in the tree first
	my $node = $this->findTreeNode($categoryid);
	# The category doesn't exist in the tree
	return 0 if !$node;
	# Count the category and its sub categories
	return 0 + $this->countProductTree($node,0);
}
# Recursively counting the number of categories that have products
sub countProductTree{
	# $node: the node we are on right now
	# $count: running total of the number of categories that have products
	my ($this,$node,$count) = @_;
	# Count child nodes
	foreach my $subnode(@{$node->next()}){
		$count += $this->countProductTree($subnode,($count + $subnode->HasProduct()));
	}
	# Count the current node
	return $count + $node->HasProduct();
}
# Find a category(node) in a category tree
sub findTreeNode{
	# $categoryid: category id
	my ($this,$categoryid) = @_;
	# If we don't have a category tree cached in the object, we need to build one
	if (!$this->{category_tree}){
		my $rootnode = new Modules::Catalog::Tree();
        	$rootnode->productlist_html("");
	        $rootnode->data({section_id=>$this->section_id()});
		$rootnode->id(0);
		my $category_type = $this->{input}{category_type};
		# Build a tree
		my $cat_tree = $this->buildTree(0,$rootnode,0,{no_product=>0,type=>$category_type});
		$this->{category_tree} = $cat_tree;
	}
	# Find the category!
	return $this->findTreeNodeRecursive($this->{category_tree},$categoryid);
}
# Recursive function to find a specific category in a tree
sub findTreeNodeRecursive{
	# $node: the node we are on 
	# $categoryid: the category we are looking for
	my ($this,$node,$categoryid) = @_;
	# Foudn it!
	if ($node->id() == $categoryid){
		return $node;
	}
	my $found;
	# Search child node
	foreach my $subnode(@{$node->next()}){
		$found = $this->findTreeNodeRecursive($subnode,$categoryid);
		# We've found the node category we are looking for. Terminate the loop.
		if ($found){
			last;
		}
	}
	return $found;
}
# Admin topic options
sub adminTopicOption{
	# $category: category data
	my ($this,$category) = @_;
	# Find out the number of products under a category including sub categories
	my $product_count = $this->findProducts($category->{catalog_category_id});
	# Report options
	my @ops = (
		{id=>"edit",url=>$this->link(action=>"topic_edit",topic_id=>$category->{catalog_category_id},resource_id=>undef),label=>"Edit '$category->{title}'"},
	);
	$ops[1]{id} = "delete";
	$ops[1]{label} = "Delete '$category->{title}'";
	$ops[1]{url} = "javascript:deleteTopic($category->{catalog_category_id})";
	if ($product_count){
		$ops[1]{url} = "javascript:alert('Sorry, the system does not allow users to delete a category when there are products attached to it or its sub categories.\\nPlease move these products to another category first.')";
	}
	return $this->reportOptions(\@ops);
}

# Resource admin options(active/disabled resources)
sub productAdminOptionOther{
	# $resource: hash reference
	my ($this,$resource) = @_;
	my @ops = (
		{id=>"edit",url=>$this->link(action=>"resource_edit",resource_id=>$resource->{catalog_product_id}),label=>"Edit '$resource->{title}'"},
		{id=>"delete",url=>"javascript:deleteResource($resource->{catalog_product_id})",label=>"Delete '$resource->{title}'"},
	);
	return $this->reportOptions(\@ops);
}
# Get icons
sub getIcon{
	# $id: icon id
	# $label: icon label
	my ($this,$id,$label) = @_;
	# Predefine icon data
	my %icon = (
		edit=>{label=>"Edit",image=>"edit12.gif"},
		approve=>{label=>"Approve",image=>"approve12.gif"},
		disable=>{label=>"Disable",image=>"disable12.gif"},
		reactivate=>{label=>"Reactivate",image=>"approve12.gif"},
		delete=>{label=>"Delete",image=>"rm12.gif"},
		new=>{label=>"New",image=>"new12.gif"},
	);
	$label = $label || $icon{$id}{label};
	my %data = (
		image_url=>"$this->{html}/_Modules/$this->{class_name}/$icon{$id}{image}",
		image_label=>$label,
		image_border=>0,
		image_hspace=>2,
		image_align=>"middle",
	);
	return $this->html_icon(\%data);
}
# Get attachment display type
sub getAttachmentDisplayType{
	my ($this,$data) = @_;
	if ($data->{thumbnail}){
		return "image";
	}
	return "link";
}
# Get attachment link
sub getAttachmentLink{
	my ($this,$data) = @_;
	my $type = $this->getAttachmentDisplayType($data);
	my $link = "<a href=\"";
	if ($type =~ /image/){
		# The attachment uses a thumbnail
		my $imageurl = $this->getProductAttachment($data);
		my $theimage = $this->getProductThumbnail($data);
		my $target;
		my $url;
		if ($data->{type} =~ /download/i){
			# Downloadable document with a thumbnail
			$url = $imageurl;
			$target = " target=\"_blank\"";
		}else{
			# Popup image
			$url = "javascript:popup_image('$imageurl')";
		}
		# Assemble the link
		$link .= "${url}\"${target}>${theimage}";
	}else{
		# Downloadable document without a thumbnail
		my $downloadurl = $this->getProductAttachment($data);
		# Assemble the link
		$link .= "$downloadurl\" target=\"_blank\">Download";
	}
	$link .= "</a>";
	return $link;
}
#------------------------------------------------------------------------------------------------
# Icon template
sub html_icon{
	# $data: merge data
	my ($this,$data) = @_;
	my $template = <<END;
<img src="<!--[[image_url]]-->" border="<!--[[image_border]]-->" hspace="<!--[[image_hspace]]-->" align="<!--[[image_align]]-->" title="<!--[[image_label]]-->" alt="<!--[[image_label]]-->">
END
	return $this->merge($template,$data);
}
# Template for showing a single resource
sub html_sublevelResourceTemplate{
	my ($this) = @_;
	my $template = $this->getCategoryDetailTemplate("CategoryDetailProductList");
	if (!$template){
		if ($this->is_admin()){
			$template = $this->html_sublevelResourceTemplateAdmin();
		}else{
			$template = $this->html_sublevelResourceTemplatePublic();
		}
	}
	return $template;
}

# Template for showing a single resource
sub html_searchProductTemplate{
	my ($this) = @_;
	return $this->html_searchProductTemplateAdmin() if $this->is_admin();
	return $this->html_searchProductTemplatePublic();
}
# Template for showing a list of results
sub html_resultListTemplate{
	# $data: merge data
	my ($this,$data) = @_;
	my $template = $this->is_admin() ? $this->html_resultListTemplateAdmin() : $this->html_resultListTemplatePublic();
	return $this->merge($template,$data);
}
# 'Suggest a link' link
sub html_newProduct{
	my ($this) = @_;
	my $link_topic_new = $this->link(action=>"topic_new");
	my $link_new_product = $this->link(action=>"new_product");
	my $link_reset_sort = $this->link(action=>"reset_sort");
	my $publish_url = $this->getPublishUrl();
	return <<END;
<table border="0" align="right">
<tr>
	<td>
		<a href="$link_topic_new">Add new category</a> | 
		<a href="$link_new_product">Add new product</a> | 
		<a href="$link_reset_sort">Reset sort order</a> | 
		<a href="$publish_url" target="_blank">Publish catalog</a>
	</td>
</tr>
</table>
END
}
sub html_categoryTitleAdmin{
	my ($this) = @_;
	return "<h1><!--[[title]]--> <!--[[admin_option]]--></h1>";
}
sub html_categoryTitlePublic{
	my ($this) = @_;
	return "<h1><!--[[title]]--></h1>";
}
# Directory title
sub html_categoryTitle{
	# $data: merge data
	my ($this,$data) = @_;
	my $template = $this->getCategoryDetailTemplate("CategoryDetailTitle");
	if (!$template){
		$template = $this->is_admin() ? $this->html_categoryTitleAdmin() : $this->html_categoryTitlePublic();
	}
	return $this->merge($template,$data);
}
# Category description for admin mode
sub html_categoryDescriptionAdmin{
	my ($this) = @_;
	return "<p><!--[[description]]--></p>";
}
# Category description for public mode
sub html_categoryDescriptionPublic{
	my ($this) = @_;
	return "<p><!--[[description]]--></p>";
}
# Get category description field
sub html_categoryDescription{
	my ($this,$data) = @_;
	my $template = $this->getCategoryDetailTemplate("CategoryDetailDescription");
	if (!$template){
		$template = $this->is_admin() ? $this->html_categoryDescriptionAdmin() : $this->html_categoryDescriptionPublic();
	}
	return $this->merge($template,$data);
}
# Product link
sub html_productLink{
	my ($this,$rec) = @_;
	my $url = $this->html_productUrl($rec);
	my $target;
	# Open the product link im a new window	
	if ($this->showProductNewWindow()){
		$target = " target=\"_blank\"";
	}
	if ($this->disableProductDetail()){
		return $rec->{title};
	}else{
		return "<a href=\"$url\"${target}>$rec->{title}</a>";
	}
}
# Product url
sub html_productUrl{
	# $rec: product record
	my ($this,$rec) = @_;
	my $path = $this->path();
	if (!$rec->{topics}){
		$rec->{topics} = [];
	}
	if (!$path && @{$rec->{topics}} > 0){
		$path = "/".$rec->{topics}[0]{id};
	}
	return $this->link(path=>$path,product_id=>$rec->{catalog_product_id},action=>undef,search_field=>undef,next_page=>undef);
}

# Template for showing a single resource - public
sub html_searchProductTemplatePublic{
	my ($this) = @_;
	return <<END;
<p>
	<div class="CatalogTitle"><span class="CatalogCode"><!--[[resource_code]]--></span><span class="CatalogName"><!--[[resource_link]]--></span> (<span class="CatalogRelevance">Relevance: <!--[[resource_percent]]--></span>)</div>
	<div class="CatalogCategory">Category: <!--[[resource_category]]--></div>
	<div class="CatalogSummary"><!--[[resource_summary]]--></div>
	<div class="CatalogPrice"><!--[[resource_price]]--></div>
</p>
END
#<div class="CatalogOptions"><!--[[resource_option]]--></div>
}
# Template for showing a single resource - admin
sub html_searchProductTemplateAdmin{
	my ($this) = @_;
	return <<END;
<p>
	<div class="CatalogTitle"><span class="CatalogCode"><!--[[resource_code]]--></span><span class="CatalogName"><!--[[resource_link]]--></span> (<span class="CatalogRelevance">Relevance: <!--[[resource_percent]]--></span>)</div>
	<div class="CatalogCategory">Category: <!--[[resource_category]]--></div>
	<div class="CatalogSummary"><!--[[resource_summary]]--></div>	
	<div class="CatalogPrice"><!--[[resource_price]]--></div>	
</p>
END
#<div class="CatalogOptions"><!--[[resource_option]]--></div>
}
# Search relevancy in percentage
sub resourcePercentScore{
	# $highest: total score
	# $score: actual score
	my ($this,$highest,$score) = @_;
	$highest = $highest || 1;
	return sprintf("%.2f%",(($score/$highest) * 100));
}
# The number of search results to show on a page
sub getResultNo{
	my ($this) = @_;
	return $this->getModuleSetting("search_result") || 20;
}
# Get begin and end offset for displaying search results.  
sub getBeginEndOffset{
	# $recs: Search results(array reference)
	my ($this,$recs) = @_;
	my @results = @$recs;
	# Verify that $begin & $end aren't using negative values
	my $begin = ($this->{input}{next_page} - 1) > 0 ? $this->{input}{next_page} - 1 : 0;
	my $end = 0;	
	if (($this->{input}{next_page} - 1) > 0){ 
		$end =  $this->{input}{next_page} + $this->getResultNo() - 1;
	}else{
		if ($this->getResultNo() > @$recs){
			$end = @$recs - 1;
		}else{
			$end = $this->getResultNo() - 1;
		}
	}
	# Invalid next page index
	if ($begin > @results ||$begin < 0){
		$begin = 0;
		$end = $this->getResultNo() - 1;
		$this->{input}{next_page} = $this->getResultNo();
	}
	# $end value can't exceed the number of the results
	if ($end > @results){
		$end = @results - 1;
	}
	# $begin value can't be greater than $end
	if ($begin > $end){
		$end = $begin;
	}
	return ($begin,$end);
}
# Format search Results
sub formatResult{
	# $result: search result(hash reference)
	my ($this,$result) = @_;
	my @newResult;
	my @results;
	foreach my $key(keys(%$result)){
		push(@results,$result->{$key});
	}
	@results = $this->sortSearchResult(\@results);
	my ($begin,$end) = $this->getBeginEndOffset(\@results);
	foreach my $reslt(@results[$begin..$end]){
		my %res = %$reslt;
		my @topics;
		foreach my $topic(@{$res{topics}}){
			#my $url = $this->link(path=>$this->getPath($topic->{id}),action=>undef,search_field=>undef,next_page=>undef);
			my $url = $this->link(path=>"/$topic->{id}",action=>undef,search_field=>undef,next_page=>undef);
			push(@topics,"<a href=\"$url\">$topic->{name}</a>");
		}
		
		# Gather merge field data
		$res{resource_score} = $res{score};
		$res{resource_percent} = $res{relevance};
		$res{resource_link} = $this->html_productLink(\%res);
		$res{resource_admin_option} = $this->productAdminOptionOther(\%res);
		$res{resource_category} = join(", ",@topics);
		$res{resource_description} = $res{description};
		$res{resource_url} = $this->html_productUrl(\%res);		
		$res{resource_code} = $this->html_productCode(\%res);
		$res{resource_summary} = $res{summary};
		$res{resource_price} = $this->html_productPrice(\%res);
		
		my $template = $this->html_searchProductTemplate();
		# Store formatted results into an array
		push(@newResult,$this->merge($template,\%res));
	}
	return \@newResult;
}
# Allow user to upload custom thumbnails?
sub allowThumbnailUpload{
	my ($this) = @_;
	return $this->getModuleSetting("allow_attachment_thumbnail_upload") || 0;
}

# List resources
sub html_sublevelResource{
	# $products: list of resources(array reference)
	my ($this,$products) = @_;
	my $out;
	# No products found
	if (@$products < 1){
		if ($this->{no_product_message}){
			return;
		}else{		
			return $out."<p>No products found.</p>";
		}
	}
	my $product_list;
	my $template = $this->html_sublevelResourceTemplate();
	foreach my $res(@$products){
		# Gather data for merge fields
		my %data = (
			ProductAdminOption=>$this->productAdminOptionOther($res) || "",
			ProductThumbnail=>$this->html_productThumbnail($res) || "",
			ProductCode=>$this->html_productCode($res) || "",
			ProductLink=>$this->html_productLink($res) || "",
			ProductSummary=>$res->{summary} || "",
			ProductPrice=>$this->html_productPrice($res) || "",			
		);
		# Populate the output html
		$out .= $this->merge($template,\%data);
	}
	return $this->html_categoryDetailProductDIVWrapper($out);
}
# Template for showing a single resource - admin
sub html_sublevelResourceTemplateAdmin{
	my ($this) = @_;
	return <<END;
<div class="CatalogProduct">
	<!--[[ProductThumbnail]]-->
	<div class="CatalogTitle"><span class="CatalogCode"><!--[[ProductCode]]--></span><span class="CatalogName"><!--[[ProductLink]]--></span> <!--[[ProductAdminOption]]--></div>
	<div class="CatalogSummary"><!--[[ProductSummary]]--></div>
	<div class="CatalogPrice"><!--[[ProductPrice]]--></div>
</div>
END
}
# Template for showing a single resource - public
sub html_sublevelResourceTemplatePublic{
	my ($this) = @_;
	return <<END;
<div class="CatalogProduct">
	<!--[[ProductThumbnail]]-->
	<div class="CatalogTitle"><span class="CatalogCode"><!--[[ProductCode]]--></span><span class="CatalogName"><!--[[ProductLink]]--></span></div>
	<div class="CatalogSummary"><!--[[ProductSummary]]--></div>
	<div class="CatalogPrice"><!--[[ProductPrice]]--></div>
</div>
END
}

sub html_categoryDetailProductDIVWrapper{
	# $data: merge data
	my ($this,$data) = @_;
	return if !$data;
	my $template = $this->getProductDetailTemplate("CategoryDetailProductListDIVWrapper");
	return $this->merge($template,{CategoryProducts=>$data});
}
sub html_productThumbnail{
	# $data: merge data
	my ($this,$data) = @_;
	# Pick the first primary image
	my @primary_images = @{$this->getProductPrimaryImage($data->{catalog_product_id})};
	return if @primary_images < 1;
	my $thumb = $primary_images[0];	
	my $image_link = $this->getProductThumbnail($thumb);
	my $template = $this->getCategoryDetailTemplate("CategoryDetailProductListThumbnail");
	return $this->merge($template,{ProductThumbnail=>$image_link});
}

# Template for root web directory
sub html_toplevelTemplate{
	# $data: merge data
	my ($this,$data) = @_;
	my $template = $this->is_admin() ? $this->html_toplevelTemplateAdmin() : $this->html_toplevelTemplatePublic();
	return $this->merge($template,$data);
}

# Template for root web directory
sub html_sublevelTemplate{
	# $data: merge data
	my ($this,$data) = @_;
	my $template = $this->getCategoryDetailTemplate("CategoryDetail");
	if (!$template){
		$template = $this->is_admin() ? $this->html_sublevelTemplateAdmin() : $this->html_sublevelTemplatePublic();
	}
	return $this->merge($template,$data);
}
# Get the text to be displayed before the search input
sub html_searchText{
	my ($this) = @_;
	return $this->getModuleSetting("search_text") || "Search";
}
# Get the 'GO' button for search function
sub getGoButton{
	my ($this) = @_;
	my $go_button = $this->getModuleSetting("go_button_image") || "go_button1.gif";
	return $go_button;
}
# Template for the search form
sub html_searchFormTemplate{
	my ($this) = @_;
	# Get the go button
	my $go_icon = $this->getGoButton();
	my $go_button_url = $this->{html}."/_Modules/Catalog/$go_icon";	
	my $diskpath;
	my $sectionid = $this->section_id();
	if ($sectionid){
		# Figure out the actual location of the files
		require ExSite::Section;
		my $s = new ExSite::Section(id=>$sectionid);
		$diskpath = $s->diskpath();
	}
	my $submit_button;
	if (-e $diskpath."/_Modules/Catalog/$go_icon"){
		$submit_button = "<input type=\"image\" src=\"$go_button_url\" border=\"0\">";
	}else{
		$submit_button = "<input type=\"submit\" value=\"GO\">";
	}	
	return <<END;
<span class="CatalogSearchForm">
<form method="post" action="<!--[[CatalogSearchUrl]]-->">
<input type="hidden" name="action" value="dir_search">
<table border="0" cellpadding="0" cellspacing="0">
<tr>
	<td valign="middle" nowrap>
		<span class="CatalogSearchLabel" align="middle"><!--[[CatalogSearchText]]--></span>&nbsp;
		<input class="CatalogSearchInput" type="text" size="18" name="<!--[[CatalogSearchInput]]-->" value="">&nbsp;
	</td>
	<td valign="middle">
		$submit_button
	</td>
</tr>
</table>
</form>
</span>
END
}

# Web directory search form
sub html_searchForm{
	my ($this) = @_;
	# MySite module Fix: the search field confuses the Html editor.
	return if ($this->{input}{ecmd} =~ /update/i);
	my $css_style = "border-width:0px;padding:0px;border:0px;";	
	my $form_action = $this->link(path=>undef,product_id=>undef,action=>undef);		
	my %merge_data = (
		CatalogSearchUrl=>$form_action,
		CatalogSearchInput=>"search_field",
		CatalogSearchText=>$this->html_searchText(),
		ImagePath=>"$config{server}{server}$config{server}{HTMLpath}/_Modules/Catalog",
	);
	my $template = $this->getCategoryDetailTemplate("CatalogSearchForm");
	return $this->merge($template,\%merge_data);
}


# Template for table rows
sub html_sublevelRowTemplate{
	# $cols: column data
	my ($this,$cols) = @_;
	$cols = !$cols ? 3 : $cols;
	my $out;
	for(my $i = 0; $i < $cols; $i++){
		my $merge_key = "row".($i + 1);
		$out .= "<td valign=\"top\"><!--[[$merge_key]]--></td>";
	}
	return $out;
}
# Root webdirectory topic link - public
sub html_directoryCellTitleLinkPublic{
	my ($this) = @_;
	return "<a href=\"<!--[[url]]-->\"<!--[[javascript_event]]-->><!--[[topic]]--></a>";
}
# Root webdirectory topic link - admin
sub html_directoryCellTitleLinkAdmin{
	my ($this) = @_;
	return "<a href=\"<!--[[url]]-->\"><!--[[topic]]--></a> <!--[[admin_option]]-->";
}
# Root webdirectory topic link
sub html_directoryCellTitleLink{
	# $data: category data
	my ($this,$data) = @_;
	my $template = $this->is_admin() ? $this->html_directoryCellTitleLinkAdmin() : $this->html_directoryCellTitleLinkPublic();
	return $this->merge($template,$data);
}
# Category path
sub html_path{
	# $category_id: category id
	my ($this,$category_id) = @_;
	my $out = $this->path()."/".$category_id;
	return $out;
}
# Create clickable link topic
sub html_sublevelCell{
	# $category: category data
	# $dirpath: category path
	my ($this,$category,$dirpath) = @_;
	my $categoryid = $category->{catalog_category_id};
	my $path = "$dirpath/$categoryid";
	my %fields = (
		url=>$this->link(path=>$this->html_path($categoryid)),
		topic=>$category->{title},
		admin_option=>$this->adminTopicOption($category)
	);
	if ($this->toplevel_show_thumbnail()){
		$fields{javascript_event} = " onMouseOver=\"ShowCategoryImage($categoryid)\"";
	}
	return $this->html_directoryCellTitleLink(\%fields);
}
# Category description
sub categoryDescription{
	# $data: category data
	my ($this,$data) = @_;
	return if !$data->{description};
	#description=>$this->{db}->show_data("catalog_category","description",$data->{description},$data->{catalog_category_id})
	my %mdata = (		
		description=>$data->{description},
	);
	return $this->html_categoryDescription(\%mdata);
}
# Show the child topics
sub html_sublevelDirectory{
	# $categories: list of topics(array reference)
	# $dirpath: category path
	# $parent_topic:  parent topic
	my ($this,$categories,$dirpath,$parent_topic) = @_;

	# No topics found
	return if @$categories < 1;
	# Create clickable link for every topics
	for(my $i = 0; $i < @$categories; $i++){
		$categories->[$i]{display_data} = $this->html_sublevelCell($categories->[$i],$dirpath);
	}
	# The number of columns the table is going to show
	my $column = $this->getModuleSetting("sublevel_display_column") || 3;
	# Convert topic data into merge field data
	my @rowdata = @{$this->displayDirectory($categories,$column)};
	# Get template for table rows
	my $rowtemplate = $this->html_sublevelRowTemplate($column);
	my @topiclist;
	foreach my $category(sort {$a->{title} cmp $b->{title}} @$categories){
		push(@topiclist,$category->{display_data});
	}
	my $subdir_list = $this->html_listSubDirectory(\@topiclist);
	my $image_swap = $this->html_categoryImageHover($categories);
	# Output the page
	
	# Showing random category pictures
	if ($this->toplevel_show_thumbnail()){
		return <<END;
<table>
<tr>
	<td valign="top">
		$subdir_list
	</td>
	<td style="padding-left: 50px;">
	$image_swap
	</td>
</tr>
</table>
END
	# Don't show random category pictures
	}else{
		return <<END;
$subdir_list
END
	}
}
# List sub categories
sub html_listSubDirectory{
	# $links: link data
	my ($this,$links) = @_;
	my $out;
	# Sub directory display style
	if ($this->getModuleSetting("subdirectory_display") =~ /bullet/i){
		$out = "\n<ul>\n\t<li>".join("</li>\n\t<li>",@$links)."</li>\n</ul>\n";
	}elsif ($this->getModuleSetting("subdirectory_display") =~ /^vertical$/i){
		$out = join("\n<br>",@$links);
	}else{
		# Comma delimited list
		$out = join(", ",@$links);
	}
	return <<END;
<div class="CatalogSubCategory">
$out
</div>
END
}
# Show the root topics
sub html_toplevelDirectory{
	# $topics: list of root topics(array reference)
	# $subtopics: list of child topics(hash reference)
	my ($this,$topics,$subtopics) = @_;
	# No root topics found
	return "<p>No categories found</p>" if @$topics < 1;
	# The number of child topics to be shown under a root topic
	my $subtopicPerCell = $this->getModuleSetting("toplevel_childtopic_display") || 2;
	# Create clickable link topics
	for(my $i = 0; $i < @$topics; $i++){
		$topics->[$i]{display_data} = $this->html_toplevelCell($subtopicPerCell,$topics->[$i],$subtopics->{$topics->[$i]{catalog_category_id}});
	}
	
	my $row_data = $this->html_toplevelData($topics);
	return $this->html_toplevelDisplay($row_data);
}

# Default number of columns to display
sub getToplevelDisplayColumn{
	my ($this) = @_;
	my $count = $this->getModuleSetting("toplevel_display_column") || 3;
	return $count;
}
# Top level category data
sub html_toplevelData{
	# $categories: categories
	my ($this,$categories) = @_;
	my @rowdata;
	if ($this->topleveldisplaystyle_list()){
		foreach my $category(@$categories){
			push(@rowdata,{data=>$category->{display_data}});
		}
	}else{
		# The number of columns the table is going to show
		my $column = $this->getToplevelDisplayColumn();
		# Convert topic data into merge field data
		@rowdata = @{$this->displayDirectory($categories,$column)};
	}
	return \@rowdata;
}
# Top level category display
sub html_toplevelDisplay{
	# $rdata: row data
	my ($this,$rdata) = @_;
	
	my $rowtemplate;
	my $row;
	foreach my $r(@$rdata){
		my $row_template;
		if ($this->topleveldisplaystyle_list()){
			$rowtemplate = "\n<li><!--[[data]]--></li>";
		}else{
			my $column = $this->getToplevelDisplayColumn();
			$rowtemplate = $this->html_toplevelRowTemplateTable($column)
		}
	}
	
	my $row_out;
	foreach my $rowdat(@$rdata){
		my $r_out = $this->merge($rowtemplate,$rowdat);
		if (!$this->topleveldisplaystyle_list()){
			my $row_width = sprintf("%i%",(100/$this->getToplevelDisplayColumn()));
			$row_out .= "\n<tr>$r_out\n</tr>";
		}else{
			$row_out .= $r_out;
		}
	}
	
	my $template;
	if ($this->topleveldisplaystyle_list()){
		$template .= <<END;
<ul>
<!--[[directory_data]]-->
</ul>
END
	}else{
		$template .= <<END;
<table class="Catalog" cellspacing="10" cellpadding="5">
<!--[[directory_data]]-->
</table>
END
	}
	return $this->merge($template,{directory_data=>$row_out});
}
# Display for each cell in the root web directory page
sub html_toplevelCell{
	# $limit: number of child topics to show
	# $topic: current topic(hash reference)
	# $subtopics: list of child topics(hash reference)
	my ($this,$limit,$topic,$subtopics) = @_;
	# Convert child topics from hashes to arrays
	my @stopics = sort keys(%$subtopics);
	# No limit for list display
	if ($this->topleveldisplaystyle_list()){
		$limit = scalar(@stopics);
	}
	
	# Sub category ids
	my @sub_ids;
	
	# Store child topic html code in an array
	my @sub_topics;
	for(my $i = 0; $i < $limit; $i++){
		# Exit the loop when the number of child records is less than the $limit
		last if (@stopics < ($i + 1));
		# The child topic that we are walking on
		my $currtopic = $subtopics->{$stopics[$i]};
		my $currtopicid = $currtopic->{catalog_category_id};
		my $link_url = $this->link(path=>"/$topic->{catalog_category_id}/$currtopicid");
		# Create clickable link
		my $subtopiclink = "<a href=\"$link_url\"";
		$subtopiclink .= ">$currtopic->{title}</a>";
		push(@sub_topics,$subtopiclink);
		push(@sub_ids,$currtopicid);
	}
	# Output the page
	# Root topic clickable link
	my %fields = (
		url=>$this->link(path=>"/$topic->{catalog_category_id}"),
		topic=>$topic->{title},
		admin_option=>$this->adminTopicOption($topic),
	);
	# Show product thumbnails in category list mode
	if ($this->topleveldisplaystyle_list() && $this->toplevel_show_thumbnail()){
		$fields{javascript_event} = " OnMouseOver=\"ShowCategoryImage($topic->{catalog_category_id})\"";
	}
	my $topiclink = $this->html_directoryCellTitleLink(\%fields);
	my $out = "<b>$topiclink</b>";
	if (@sub_topics > 0){
		# list mode
		if ($this->topleveldisplaystyle_list()){
			$out .= $this->html_displayCategoryListMode($topic->{catalog_category_id},\@sub_ids);
		}else{
			# table mode
			$out .= "<br>";
			if (@sub_topics > 0){
				$out .= join(", ",@sub_topics)."...";
			}
		}
	}
	return $out;
}
# Template for table rows(root web directory)
sub html_toplevelRowTemplateTable{
	# $cols: columns
	my ($this,$cols) = @_;	
	$cols = !$cols ? 3 : $cols;
	my $width = sprintf("%i",(100/$cols));
	my $out;
	for(my $i = 0; $i < $cols; $i++){
		my $merge_key = "row".($i + 1);
		$out .= "\n\t<td valign=\"top\"";
		if ($width){
			$out .= " width=\"$width%\"";
		}
		$out .= "><!--[[$merge_key]]--></td>";
	}
	return $out;
}

# Show product image on the topleve category list
sub html_toplevelProductImage{
	my ($this) = @_;
	return if !$this->topleveldisplaystyle_list();
	return if !$this->toplevel_show_thumbnail();
	my @categories = @{$this->getAllCategory()};
	return $this->html_categoryImageHover(\@categories);
}

sub html_categoryImageHover{
	my ($this,$categories) = @_;
	return if !$this->toplevel_show_thumbnail();
	my @image_list;
	foreach my $category(@$categories){
		my $categoryid = $category->{catalog_category_id};
		push(@image_list,{id=>$categoryid,url=>$this->getSingleRandomProductImageByCategory($categoryid)});
	}
	my @image_data;
	my @image_preload;
	foreach my $image(@image_list){
		my $id = $image->{id};
		my $url = $image->{url};
		push(@image_data,"image_list[$id] = '$url';");
		push(@image_preload,$url);
	}
	my $image_data = join("\n",@image_data);
	my $image_preload = join("\n",@image_preload);
	my $defaultid = $categories->[0]->{catalog_category_id} || 0;
	return <<END;
<script language="javascript">
<!--
var image_list = new Array();
$image_data
var default_id = $defaultid;
ShowCategoryImage(default_id);
function ShowCategoryImage(id){
	var image_div = document.getElementById("catalog_image_list");
	image_div.innerHTML = image_list[id];
}
//-->
</script>
<div id="catalog_image_list" class="CatalogProductImage">
</div>
<div style="display: none;">
$image_preload
</div>
END
}
# Display top level in list mode
sub html_displayCategoryListMode{
	# $categoryid: category id
	# $ids: sub category ids
	my ($this,$categoryid,$ids) = @_;
	$this->{category_lookup}{$categoryid} = 1;
	my $out;
	foreach my $id (@$ids){
		$this->{category_lookup}{$id} = 1;
		$out .= $this->html_displayCategoryListModeRecursive($id,"/$categoryid/$id");
	}
	return $out;
}
# Display category recursively in list format
sub html_displayCategoryListModeRecursive{
	# $categoryid: category id
	# $path: category path
	my ($this,$categoryid,$path) = @_;
	return if !$this->is_int($categoryid);
	# Fetch category record
	my %category = %{$this->getCategory($categoryid)};
	my @records = @{$this->getSubCategory($categoryid)};
	my $admin_option;
	if ($this->is_admin()){
		$admin_option = $this->adminTopicOption(\%category);
	}
	my $category_url = $this->link(path=>$path);
	my $category_image;
	# Show product thumbnails
	if ($this->toplevel_show_thumbnail()){
		$category_image = " OnMouseOver=\"ShowCategoryImage($categoryid)\"";
	}
	my $out = <<END;
<ul>
	<li><a href="$category_url"${category_image}>$category{title}</a>$admin_option
END
	foreach my $rec(@records){
		my $c_id = $rec->{child};
		next if (exists($this->{category_lookup}{$c_id}));
		$this->{category_lookup}{$c_id} = 1;
		$out .= $this->html_displayCategoryListModeRecursive($c_id,$path."/$c_id");
	}
	$out .= <<END;
	</li>
</ul>
END
	return $out;
}
# Product title
sub html_productTitle{
	my ($this,$data) = @_;
	return $data->{title};
}
# Product summary
sub html_productSummary{
	# $data: product data
	my ($this,$data) = @_;
	my $text = $data->{summary};
	return if !$text;
	my $template = $this->getProductDetailTemplate("ProductDetailSummary");
	return $this->merge($template,{ProductSummary=>$text});
	}
# Product option
sub html_productOption{
	# $data: product data
	my ($this,$data) = @_;
	my $text = $data->{option};
	return if !$text;
	my $template = $this->getProductDetailTemplate("ProductDetailOption");
	return $this->merge($template,{ProductOption=>$text});
}
# Product price
sub html_productPrice{
	# $data: product data
	my ($this,$data) = @_;
	my $price = $data->{price};
	my $no_price_message = "";
	if (!$price || $price <= 0){		
		if ($this->getModuleSetting("no_price_message")){
			$no_price_message = $this->getModuleSetting("no_price_message");
		}else{
			return;
		}
	}
	my $text = "Price: ".($no_price_message || sprintf("\$%.2f",$price));
	my $template = $this->getProductDetailTemplate("ProductDetailPrice");
	return $this->merge($template,{ProductPrice=>$text});
}
# Product code
sub html_productCode{
	# $data: product data
	my ($this,$data) = @_;
	return undef if !$data->{code};
	return $data->{code}." ";
}
# Product details
sub html_productDetail{
	# $data: product data
	my ($this,$data) = @_;
	my $text = $data->{details};
	return if !$text;
	my @tags = ("<br>","<table>","<tr>","</table>","<li>");
	my $string = join("|",@tags);
	# Insert a return character after the tags in (@tags)
	$text =~ s/($string)/\n$1/g;
	my $template = $this->getProductDetailTemplate("ProductDetailDetail");
	return $this->merge($template,{ProductDetail=>$text});
}
# Product attachment wrapper(groups attachments)
sub html_productAttachmentDIVWrapper{
	# $data: attachment data
	# $type: attachment type
	my ($this,$data,$type) = @_;
	return if !$data;
	return if !$type;
	return $data if $this->useAttachmentHeading();
	my %type = (
		all=>"ProductDetailAttachmentDIVWrapper",
		download=>"ProductDetailAttachmentDownloadDIVWrapper",
		image=>"ProductDetailAttachmentImageDIVWrapper",
	);
	# Invalid attachment type
	return if !exists($type{$type});
	my $template = $this->getProductDetailTemplate($type{$type});
	return $this->merge($template,{ProductAttachments=>$data});
}
# All attachment
sub html_productAttachmentAll{
	my $this = shift;
	my $type = "all";
	my $a_data = $this->html_productAttachment(@_,$type);
	return $this->html_productAttachmentDIVWrapper($a_data,$type);
}
# Downloadable attachments
sub html_productAttachmentDownload{
	my $this = shift;
	my $type = "download";
	my $a_data = $this->html_productAttachment(@_,$type);
	return $this->html_productAttachmentDIVWrapper($a_data,$type);
}
# Image attachment
sub html_productAttachmentImage{
	my $this = shift;
	my $type = "image";
	my $a_data = $this->html_productAttachment(@_,$type);
	return $this->html_productAttachmentDIVWrapper($a_data,$type);
}
sub useAttachmentHeading{
	my ($this) = @_;
	return 0 if $this->admin_mode();
	return $this->getModuleSetting("enable_attachment_heading") || 0;
}
sub html_productAttachment{
	# $data: product data
	# $type: type of attachment
	my ($this,$data,$type) = @_;
	if ($this->useAttachmentHeading()){
		return $this->html_productAttachmentHeadingEnable($data,$type);
	}
	return $this->html_productAttachmentHeadingDisable($data,$type);
}

sub html_productAttachmentHeadingEnable{
	# $data: product data
	# $type: type of attachment
	my ($this,$data,$type) = @_;
	# A set of default functions to call according to the $type variable
	my %type = (
		all=>"getProductAttachmentGroupAll",
		download=>"getProductAttachmentGroupDownload",
		image=>"getProductAttachmentGroupImageSecondary",
	);
	return if !$type;
	# Invalid attachment type
	return if !exists($type{$type});
	# Get the name of the function to call
	my $attachment_func = $type{$type};
	my %headings = %{$this->$attachment_func($data->{catalog_product_id})};
	return if keys(%headings) < 1;
	my %image_data;
	foreach my $heading (keys(%headings)){
		my @image_data;
		# Organize attachment data into an @image_data array
		foreach my $image(@{$headings{$heading}}){
			my $image_type = $this->getAttachmentDisplayType($image);
			my $image_link = $this->getAttachmentLink($image);
			my $image_description = $image->{description};
			push(@image_data,{type=>$image_type,link=>$image_link,description=>$image_description});
		}
		$image_data{$heading} = \@image_data;
	}
	# Output attachment data as html
	my @headings;
	if (exists($image_data{_null})){
		push(@headings,"_null");
	}
	foreach my $heading(sort keys(%image_data)){
		next if $heading eq "_null";
		push(@headings,$heading);
	}
	my %heading_class = (
		all=>"ProductAttachmentWrapper",
		download=>"ProductAttachmentDownloadWrapper",
		image=>"ProductAttachmentImageWrapper",
	);
	my $attachment_data;
	foreach my $heading(@headings){
		my $attach_data;
		foreach my $idata(@{$image_data{$heading}}){
			my $attachment = $idata->{link};
			# Get attachment description
			if ($idata->{type} =~ /image/i){
				$attachment .= $this->html_productAttachmentImageDescription($idata->{description});
			}else{
				$attachment .= $this->html_productAttachmentDownloadDescription($idata->{description});
			}
			$attach_data .= $this->html_productAttachmentDIV($attachment);
		}
		my %data = (
			CssClass=>$heading_class{$type},
			ProductHeading=>($heading eq "_null" ? "" : $heading),
			ProductAttachments=>$attach_data,
		);
		$attachment_data .= $this->html_productAttachmentGroupWrapper(\%data);
		
	}
	return $attachment_data;
}
sub html_productAttachmentGroupWrapper{
	my ($this,$data) = @_;
	my $template = $this->getProductDetailTemplate("ProductDetailAttachmentGroupWrapper");
	return $this->merge($template,$data);
}
# Product attachment based on the attachment type
sub html_productAttachmentHeadingDisable{
	# $data: product data
	# $type: type of attachment
	my ($this,$data,$type) = @_;
	# A set of default functions to call according to the $type variable
	my %type = (
		all=>"getProductAttachmentAll",
		download=>"getProductAttachmentDownload",
		image=>"getProductAttachmentImageSecondary",
	);
	return if !$type;
	# Invalid attachment type
	return if !exists($type{$type});
	# Get the name of the function to call
	my $attachment_func = $type{$type};
	my @images = @{$this->$attachment_func($data->{catalog_product_id})};
	return if @images < 1;
	my @image_data;
	# Organize attachment data into an @image_data array
	foreach my $image(@images){
		my $image_type = $this->getAttachmentDisplayType($image);
		my $image_link = $this->getAttachmentLink($image);
		my $image_description = $image->{description};
		push(@image_data,{type=>$image_type,link=>$image_link,description=>$image_description});
	}
	# Output attachment data as html
	my $attachment_data;
	foreach my $idata(@image_data){
		my $attachment = $idata->{link};
		# Get attachment description
		if ($idata->{type} =~ /image/i){
			$attachment .= $this->html_productAttachmentImageDescription($idata->{description});
		}else{
			$attachment .= $this->html_productAttachmentDownloadDescription($idata->{description});
		}
		$attachment_data .= $this->html_productAttachmentDIV($attachment);
	}
	return $attachment_data;
}
# Product description for image
sub html_productAttachmentImageDescription{
	# $data: data hash
	my ($this,$data) = @_;
	return if !$data;
	my $template = $this->getProductDetailTemplate("ProductDetailAttachmentThumbnailImageDescription");
	return $this->merge($template,{ProductAttachmentDescription=>$data});
}
# Product description for downloadable documents
sub html_productAttachmentDownloadDescription{
	# $data: data hash
	my ($this,$data) = @_;
	return if !$data;
	my $template = $this->getProductDetailTemplate("ProductDetailAttachmentThumbnailDownloadDescription");
	return $this->merge($template,{ProductAttachmentDescription=>$data});
}
# Product attachment
sub html_productAttachmentDIV{
	# $data: data hash
	my ($this,$data) = @_;
	return if !$data;
	my $template = $this->getProductDetailTemplate("ProductDetailAttachmentDIV");
	return $this->merge($template,{ProductAttachment=>$data});
}
# Product image
sub html_productImage{
	# $data: data hash
	my ($this,$data) = @_;
	my @images = @{$this->getProductPrimaryImage($data->{catalog_product_id})};
	my $out;
	foreach my $image(@images){
		my ($filename) = split(/#/,$image->{upload});
		next if !$this->isImage($filename);
		my $imageurl = $this->getProductAttachment($image);
		my $image_description = $this->html_productPrimaryImageDescription($image->{description});
		my $image_link = <<END;
<a href="javascript:popup_image('$imageurl')"><img src="$imageurl" border="0" width="200"></a>
END
		$out .= $this->html_productPrimaryImageDIV($image_link.$image_description);
	}
	if ($out){
		return $this->html_productPrimaryImageDIVWrapper($out);
	}
	return;
}
# Primary image description
sub html_productPrimaryImageDescription{
	# $data: data hash
	my ($this,$data) = @_;
	return if !$data;
	my $template = $this->getProductDetailTemplate("ProductDetailPrimaryImageDescription");
	return $this->merge($template,{ProductPrimaryImageDescription=>$data});
}
# Primary image idv
sub html_productPrimaryImageDIV{
	# $data: data hash
	my ($this,$data) = @_;
	return if !$data;
	my $template = $this->getProductDetailTemplate("ProductDetailPrimaryImageDIV");
	return $this->merge($template,{ProductPrimaryImage=>$data});
}
# Primary image div wrapper
sub html_productPrimaryImageDIVWrapper{
	# $data: data hash
	my ($this,$data) = @_;
	return if !$data;
	my $template = $this->getProductDetailTemplate("ProductDetailPrimaryImageDIVWrapper");
	return $this->merge($template,{ProductPrimaryImages=>$data});
}
#----------------------------------------------------------------------
# Admin resource style
sub html_adminResourceStyle{
	my ($this) = @_;
	return <<END;
<style type="text/css">
.CatalogUrl{
	color: #6f6f6f;
	font-size: 12px;
}
.CatalogDescription{
	font-size: 12px;
}
.CatalogNotifySelect{
	font-size: 8px;
	color: #6f6f6f;
}
.CatalogProduct{
	width: 100%;
	overflow: auto;
	padding: 5px;
}
.CatalogProductThumbnail{
	float: right;
}
.CatalogProductWrapper{
	width: 100%;
}
</style>
END
}
# Template for showing a list of results - public
sub html_resultListTemplatePublic{
	my ($this) = @_;
	return <<END;
<div class="Catalog">
<!--[[result]]-->
<!--[[next_page]]-->
</div>
END
}
# Template for showing a list of results - admin
sub html_resultListTemplateAdmin{
	my ($this) = @_;
	my $out = $this->html_adminResourceStyle();
	$out .= <<END;
<div class="Catalog">
<!--[[result]]-->
<!--[[next_page]]-->
</div>
END
	return $out;
}
# Template for root web directory - admin
sub html_toplevelTemplateAdmin{
	my ($this) = @_;
	my $template = $this->javascript();
	# List display style
	if ($this->topleveldisplaystyle_list()){
		$template .= <<END;
<div class="Catalog">
<!--[[select_category_type]]-->
<!--[[topic]]-->
<!--[[new_link]]-->
</div>
END
	}else{
		$template .= <<END;
<div class="Catalog">
<center>
<!--[[select_category_type]]-->
<!--[[topic]]-->
</center>
<!--[[new_link]]-->
</div>
END
	}
	return $template;
}
# Template for root web directory - public
sub html_toplevelTemplatePublic{
	my ($this) = @_;
	# List display style
	if ($this->topleveldisplaystyle_list()){
		return <<END;
<div class="Catalog">
<table class="CatalogCategoryList">
<tr>
<td class="CatalogCategoryDirectory">
<!--[[topic]]-->
</td>
<td class="CatalogCategoryImage" style="padding-left: 50px;">
<!--[[product_image]]-->
</td>
</tr>
</table>
</div>
END
	}else{
		return <<END;
<div class="Catalog">
<center>
<!--[[topic]]-->
</center>
</div>
END
	}
}
# Using list display for top level catalog listing
sub topleveldisplaystyle_list{
	my ($this) = @_;
	if ($this->getModuleSetting("toplevel_display_style") =~ /list/i){
		return 1;
	}
	return 0;
}
# Javascript
sub html_productDetailJavascript{
	my ($this) = @_;
	return "\n<script language=\"javascript\" src=\"$this->{html}/_ExSite/js/misc.js\"></script>";
}

# Sublevel web directory page - public
sub html_sublevelTemplateAdmin{
	my ($this) = @_;
	my $spacer = "<img src=\"$this->{html}/_ExSite/images/null.gif\" width=\"40\" height=\"1\">";
	my $template = $this->javascript();
	$template .= $this->html_adminResourceStyle();
	$template .= <<END;
<div class="Catalog">
<!--[[CategoryDetailNewLink]]-->
<!--[[CategoryDetailPath]]-->
<!--[[CategoryDetailTitle]]-->
<!--[[CategoryDetailDescription]]-->
<!--[[CategoryDetailSubCategory]]-->
<!--[[CategoryDetailProductList]]-->
</div>
END
	return $template;
}
# Sublevel web directory page - public
sub html_sublevelTemplatePublic{
	my ($this) = @_;
	my $spacer = "<img src=\"$this->{html}/_ExSite/images/null.gif\" width=\"40\" height=\"1\">";
	return <<END;
<div class="Catalog">
<!--[[CategoryDetailPath]]-->
<!--[[CategoryDetailTitle]]-->
<!--[[CategoryDetailDescription]]-->
<!--[[CategoryDetailSubCategory]]-->
<!--[[CategoryDetailProductList]]-->
</div>
END
}
# Form for adding/editing a product
sub html_productForm{
	# $id: product id
	my ($this,$id) = @_;
	my $posturl = $this->link(action=>undef);
	my $out;
	$out .= $this->{db}->editor_javascript();
	$out .= <<END;
<div class="Catalog">
<h2>New product</h2>
<!--[[custom_message]]-->
<form id="exsiteform" method="post" enctype="multipart/form-data" action="$posturl" onSubmit="validate()">
<input type="hidden" name="resource_details" value="">
<input type="hidden" name="reply" value="<!--[[reply]]-->">
<input type="hidden" name="action" value="post_link">
<input type="hidden" name="resource_id" value="<!--[[resource_id]]-->">
<p><span class="label">Category 1</span><br>
<select name="resource_category_1">
<!--[[category_list_1]]-->
</select>
</p>
<p><span class="label">Category 2</span><br>
<select name="resource_category_2">
<!--[[category_list_2]]-->
</select>
</p>
<p><span class="label">Category 3</span><br>
<select name="resource_category_3">
<!--[[category_list_3]]-->
</select>
</p>
<p><span class="label">Code</span><br><input type="text" size="80" name="resource_code" value="<!--[[resource_code]]-->"></p>
<p><span class="label">Title</span><br><input type="text" size="80" name="resource_title" value="<!--[[resource_title]]-->"></p>
<p><span class="label">Summary</span><br><textarea name="resource_summary" cols="80" rows="5"><!--[[resource_summary]]--></textarea></p>
<p><span class="label">Details</span><br>
END
	my %product = %{$this->getProduct($id)};
	$out .= $this->{db}->input_htmleditor(
			name=>"resource_details",
			column=>"details",
			table=>"catalog_product",
			record=>$id,
			value=>$product{details},
		);
	# Are we allowing users to upload their own thumbnail images?
	my $allow_thumbnail_upload = $this->allowThumbnailUpload();
	$out .= <<END;
<p><span class="label">Price</span><br>\$ <input type="text" size="6" name="resource_price" value="<!--[[resource_price]]-->"></p>
<p><span class="label">Status</span><br><select name="resource_status"><option value="enabled">enabled</option><option value="disabled">disabled</option></select></p>
<p><span class="label">Is this a new product?</span><br><input type="radio" name="resource_feature" value="Yes" checked>Yes&nbsp;&nbsp;&nbsp;&nbsp;<input type="radio" name="resource_feature" value="No">No</p>
<p><span class="label">Options</span><br><input type="text" size="80" name="resource_options" value="<!--[[resource_options]]-->"></p>
<p><span class="label">Attachment 1</span><br><input type="file" size="80" name="upload_1_file" value=""></p>
END
	$out .= $this->html_documenttype(1);
	if ($allow_thumbnail_upload){
		$out .= <<END;
<p><span class="label">Attachment 1 Thumbnail</span><br><input type="file" size="80" name="upload_1_thumbnail" value=""></p>
END
	}
	$out .= <<END;
<p><span class="label">Attachment 1 Description</span><br><textarea name="upload_1_description" cols="80" rows="5"></textarea></p>
<p><span class="label">Attachment 2</span><br><input type="file" size="80" name="upload_2_file" value=""></p>
END
	$out .= $this->html_documenttype(2);
	if ($allow_thumbnail_upload){
		$out .= <<END;
<p><span class="label">Attachment 2 Thumbnail</span><br><input type="file" size="80" name="upload_2_thumbnail" value=""></p>
END
	}
	$out .= <<END;
<p><span class="label">Attachment 2 Description</span><br><textarea name="upload_2_description" cols="80" rows="5"></textarea></p>
<p><span class="label">Attachment 3</span><br><input type="file" size="80" name="upload_3_file" value=""></p>
END
	$out .= $this->html_documenttype(3);
	if ($allow_thumbnail_upload){
		$out .= <<END;
<p><span class="label">Attachment 3 Thumbnail</span><br><input type="file" size="80" name="upload_3_thumbnail" value=""></p>
END
	}
	$out .= <<END;
<p><span class="label">Attachment 3 Description</span><br><textarea name="upload_3_description" cols="80" rows="5"></textarea></p>
<input type="submit" value="Submit">&nbsp;&nbsp;<input type="reset" value="Reset">
</form>
</div>
END
	return $out;	
}
# Attachment inputs
sub html_documenttype{
	# $id: label
	my ($this,$id) = @_;
	return if $id !~ /\d+/;
	my $select = <<END;
<select name="upload_${id}_type">
<option value="">== attachment type==</option>
END
	foreach my $type("primary-image","secondary-image","download"){
		$select .= <<END;
<option value="$type">$type</option>
END
	}
	$select .= "\n</select>";
	return <<END;
<p><span class="label">Attachment $id Type</span><br>$select</p>
END
}
#--------------------------------------------------------------------
# Get all categories in the drop down list format - list
sub html_getCategoryList_list{
	# $data: list of records
	# $select: selected value
	my ($this,$data,$select) = @_;	
	# Aphabetically sorted list
	my $selector;
	foreach my $cat (@$data){
		my $id = $cat->{catalog_category_id};
		my $selected = $select == $id ? " selected" : "";
		$selector .= <<END;
<option value="${id}"${selected}>$cat->{topic}</option>
END
	}
	return <<END;
<option value="">== Select ==</option>
$selector
END
}
# Get all categories in the drop down list format
sub getCategoryList{
	# $select: selected value
	my ($this,$select) = @_;
	my $sectionid = $this->section_id();
	return if !$this->section_id();
	# Check and see if the treemode is on
	if ($config{$this->{class_name}}{edit}{categorylist}{treemode}){
		return $this->getCategoryList_tree($this->section_id(),$select);
	}else{
		my @categories = sort {$a->{title} cmp $b->{title}} @{$this->getAllCategory()};
		return $this->html_getCategoryList_list(\@categories,$select);
	}
	return;
}
# Set global terminate id(i.e. category id)
sub nodeTerminateID{
	# $id: category id
	my ($this,$id) = @_;
	# $id is not a valid integer
	return $this->{terminate_id} || 0 if !$this->is_int($id);
	if (defined $id){
		$this->{terminate_id} = $id;
	}
	return $this->{terminate_id};
}
# Are we top processing a node?
sub isStopNode{
	# $id: category id
	my ($this,$id) = @_;
	return 1 if $this->nodeTerminateID() == $id;
	return 0;
}
# Build tree select input
sub getCategoryList_tree{
	# $sectionid: section id
	# $default: default selected value
	# $opt: hash reference
	# 	terminate_id: stop the loop at this category id
	my ($this,$sectionid,$default,$opt) = @_;
	# Set the category id to stop the loop
	$this->nodeTerminateID($opt->{terminate_id});
	
	# Build the complete category tree
	my $rootnode = new Modules::Catalog::Tree();
        $rootnode->productlist_html("");
        $rootnode->data({section_id=>$sectionid});
	$rootnode->id(0);	
	my $category_type = $this->{input}{category_type};
	
	# Build a tree
	my $cat_tree = $this->buildTree(0,$rootnode,0,{no_product=>1,type=>$category_type});
	my $selector;
	if (@{$cat_tree->next()} > 0){
		foreach my $node(@{$cat_tree->next()}){
			if ($this->isStopNode($node->id())){
				next;
			}
			$selector .= $this->outputCatalogTreeInput($node,$default);	
		}
	}
	# Reset terminate id
	$this->nodeTerminateID(0);
	return <<END;
<option value="0">== select ==</option>
$selector
END
}
# Output the <option> portion of the <select> input tag
sub outputCatalogTreeInput{
	# $tree: category tree object
	# $default: default select value
	# $sep: separator. $sep is used to indent the list display based on the node depth
	my ($this,$tree,$default,$sep) = @_;
	my $go = $config{$this->{class_name}}{edit}{categorylist}{treemode_separator};
	my $option_value = $tree->data->{catalog_category_id};
	my $option_label = $sep.$tree->data->{title};
	my $selected = $default == $tree->data->{catalog_category_id} ? " selected" : "";
	my $section_label = $this->getSection($tree->data->{section_id})->{label} ? " [".$this->getSection($tree->data->{section_id})->{label}."]" : "";
	my $out = "\n<option value=\"${option_value}\"${selected}>${option_label}${section_label}</option>";
	# Only the parent categories to prevent circular references
	foreach my $node(@{$tree->next()}){
		# nodeTerminateID() tells us when we've reached a child node
		if ($this->isStopNode($node->id())){
			next;
		}
		$out .= $this->outputCatalogTreeInput($node,$default,$go.$sep);
	}
	return $out;	
}
#----------------------------------------------------------------------

# Delete confirmation popup
sub javascript{
	my ($this) = @_;
	my $link_resource_delete = $this->link(action=>"resource_delete");
	my $link_topic_delete = $this->link(action=>"topic_delete");
	return <<END;
<script language="javascript">
<!--
	function deleteResource(id){
		if (confirm('Are you sure want to delete this product?')){
			location.href = '${link_resource_delete}&resource_id=' + id;
		}
	}
	function deleteTopic(id){
		var msg = 'Are you sure you want to delete this category?';
		if (confirm(msg)){
			location.href = '${link_topic_delete}&topic_id=' + id;
		}
	}
//-->
</script>
END
}

# Page redirection
sub redirect{
	# $reply: reply url
        my ($this,$reply) = @_;
	$reply = $reply || $this->link(action=>undef);
        my $p = $share{Page} || $share{PageBuf};
        if ($p){
                $p->redirect($reply);
        }else{
                print "location: $reply\n\n";
        }
        return;
}
# Data merge
sub merge{
	# $view: template with merge fields
	# $data: data hash contains data to be merged into the template
        my ($this,$view,$data) = @_;
	return if !$view || !$data;
	# Merge field format: <!--[[<name>]]-->
        $view =~ s/\<\!\-\-\[\[((\w+(\_| )?)+\w*)\]\]\-\-\>/$data->{$1}/g;
        return $view;
}
sub searchDescendant{
	# $table: table
	# $id: record id
	my ($this,$table,$id) = @_;
	my %data;
	if ($table eq "catalog_category"){
		# Clear variable
		$this->{catalog_delete} = {};
		# Recursively searching for descendant records
		$this->searchDescendantRecursive($table,$id);
		# Finding left over join records
		$this->findLeftOverJoin($table,$id);
		%data = %{$this->{catalog_delete}};
	}
	return \%data;
}
# Fine left over category join records
sub findLeftOverJoin{
	# $table: table
	# $id: record id
	my ($this,$table,$id) = @_;
	return if !$this->is_int($id);
	return if !$this->is_int($id);
	if ($table eq "catalog_category"){
		my @joins = @{$this->getJoinByChild($id)};
		foreach my $join(@joins){
			push(@{$this->{catalog_delete}{catalog_category_join}},$join->{catalog_category_join_id});
		}
	}
	return;
}
# Recursively fetching descendant records
sub searchDescendantRecursive{
	# $table: table
	# $id: record id
	my ($this,$table,$id) = @_;
	return if !$id;	
	# Verify that $id is an integer
	return if !$this->is_int($id);
	if ($table eq "catalog_category"){
		push(@{$this->{catalog_delete}{catalog_category}},$id);
		# Fetch products
		my @products = @{$this->fetchProduct($id)};
		foreach my $p (@products){
			push(@{$this->{catalog_delete}{catalog_product}},$p->{catalog_product_id});
		}
		# Fetch category join
		my @cat_join = @{$this->getJoinByParent($id)};
		foreach my $join(@cat_join){
			push(@{$this->{catalog_delete}{catalog_category_join}},$join->{catalog_category_join_id});
			$this->searchDescendantRecursive($table,$join->{child});
		}
	}
	return;
}
# Exsite handler: find_descendants handler
#
# This handler is written to ensure that the join records are included in the data fetch
sub handler_find_descendants{
	# $table: table
	# $id: record id
	my ($this,$table,$id) = @_;
	if ($table eq "catalog_category"){
		my %data = %{$share{catalog}{object}->searchDescendant($table,$id)};
		return \%data;
	}
	if ($table eq "catalog_product"){
		my @records = @{$share{catalog}{object}->getProductJoinByProduct($id)};
		foreach my $record(@records){
			push(@{$this->{catalog_delete}{catalog_product_category_join}},$record->{catalog_product_category_join_id});
		}
	}
	return undef;
}
# Delete a record
sub delete{
	# $table: table
	# $id: record id
	my ($this,$table,$id) = @_;
	return if !$id || !$table;
	# Verify that $id is an integer
	return if !$this->is_int($id);
	my $skip_flag = $this->is_admin();
	$this->installCatalogObject();
	$this->{db}->handler('find_descendants',\&handler_find_descendants);
	$this->{db}->trash_r($table,$id,$skip_flag);
	$this->uninstallCatalogObject();
	# Redirect the user to the reply url
	$this->redirect($this->link(action=>undef,resource_id=>undef,topic_id=>undef,product_id=>undef));
	return;
}

# Is the user an admin?
sub is_admin{
	my ($this) = @_;
	# Not in control panel
	return 0 if $ENV{SCRIPT_NAME} !~ /ctrl-panel\.cgi/i;
	# In publishing mode
	return 0 if $this->{publishing};
	# User has the ownership of a section
	my $sectionid = $this->section_id();
	return 1 if $this->{db}->owns($sectionid);
	return 0;
}
# Is the input an integer?
sub is_int{
	my ($this,$value) = @_;
	return 1 if $value =~ /\d+/;
	return 0;
}
# Build report using ExSite::ReportBuilder
sub reportbuilder{
	# $opt: option hash
	#	title=>report title
	#	data=>row data
	#	tools=>tool bar
	#	headers=>report headers
	#	foot=>footer html
	#	width=>table width
        my ($this,$opt) = @_;
        my $rb = $this->{rb} || new ExSite::ReportBuilder();
        $rb->init(
                title=>$opt->{title},
                data=>$opt->{data},
                tools=>$opt->{tools},
                headers=>$opt->{headers},
                foot=>$opt->{foot},
                width=>$opt->{width},         
	);
	return $rb->make();
}
#----------------------------------------------------------------------
sub cacheCategory{
	# $sectionid: section id
	# $type: category type
	my ($this,$sectionid,$type) = @_;
	my @categories = @{$this->getCategoryByType($type)};
	# Fetch category join records
	my @joins = @{$this->getAllCategoryJoin()};
	my %category_child = %{&keywise("child",\@joins)};
	my %category_parent = %{&keywise("catalog_category_id",\@categories)};
	my @parents;
	my %parent_key;
	# Index category and join records
	foreach my $cat(@categories){
		if (!exists($category_child{$cat->{catalog_category_id}}) || 
			(exists($category_child{$cat->{catalog_category_id}}) && !$category_child{$cat->{catalog_category_id}}{parent})){
			$cat = $this->updateTitle($cat);
			push(@parents,$cat);
		}
	}
	foreach my $join(@joins){
		next if (!exists($category_parent{$join->{child}}));
		push(@{$parent_key{$join->{parent}}},$category_parent{$join->{child}});
	}
	# Cache data
	$this->{tree_cache}{child_record} = \%parent_key;
	$this->{tree_cache}{parent} = \@parents;
	$this->{tree_cache_complete} = 1;
	return;
}
# Build category tree
sub buildTree{
	# $cat_id: category id
	# $old_tree: tree object
	# $level: tree depth
	# $opt: 
	#	no_product => 1 or 0 (product flag)
	my ($this,$cat_id,$oldtree,$level,$opt) = @_;
	$level = $level ? $level : 1;
	# Fetch sub categories
	my @sub_cats = ();
	if (!$this->{tree_cache_complete}){
		$this->cacheCategory($oldtree->data->{section_id},$opt->{type});
	}
	if (!$cat_id){
		@sub_cats = @{$this->{tree_cache}{parent}};
	}else{
		@sub_cats = $this->{tree_cache}{child_record}{$cat_id} ? @{$this->{tree_cache}{child_record}{$cat_id}} : ();
	}
	# Fetch product
	my @products = $opt->{no_product} ? () : @{$this->fetchProduct($cat_id)};
	# Figure out tree depth
	$oldtree->depth($level);
	# Set product flag
	$oldtree->HasProduct((@products ? 1 : 0));
	# Save path and set path
	my $oldpath = $this->path();
	$this->path("/".$oldtree->id());
	# Product list(in html format)
	$oldtree->productlist_html($this->productList(\@products));
	$oldtree->product_data(\@products);
	$level += 1;
	foreach my $subcat(@sub_cats){
		# Populate child nodes		
		my $newnode = new Modules::Catalog::Tree();
		$newnode->prev($oldtree);
		$newnode->data($subcat);
		$oldtree->AddChild($this->buildTree($subcat->{catalog_category_id},$newnode,$level,$opt));
	}
	# Restore path
	$this->path($oldpath);
	return $oldtree;
}
# Application chart
sub catalogChart{
	# $sectionid: section id
	# $cat_id: category id
	my ($this,$sectionid,$cat_id) = @_;
	return $this->catalogChart_style($sectionid,$cat_id,"application");
}

# Application chart
sub catalogChart_style{
	# $sectionid: section id
	# $cat_id: category id
	# $type: category type
	my ($this,$sectionid,$cat_id,$type) = @_;
	return "<p>You've got an empty catalog.</p>" if !$sectionid;

	# Category id
	my $catid = 0;
	# Fetch products
	my @products = ();
	# Create root tree node
	my $rootnode = new Modules::Catalog::Tree();
	$rootnode->productlist_html("");		
	$rootnode->data({section_id=>$sectionid});
	$rootnode->id(0);

	# Build a tree
	my $cat_tree = $this->buildTree(0,$rootnode,0,{type=>$type});
	# Figure out table rowspan for every node
	$cat_tree->FigureOutTableSpan($cat_tree,0);

	$this->{max_depth} = $cat_tree->MaxDepth();

	# Output the chart
	my $chart_html = $this->createChart($cat_tree,$cat_tree->MaxDepth());
	
	# Output root node's product links
	my $product_html;
	if ($cat_tree->HasProduct()){
		my $product_colspan_value = $cat_tree->MaxDepth() - $cat_tree->depth() + 1;
		my $product_colspan;
		if ($product_colspan_value > 1){
			$product_colspan = " colspan=\"$product_colspan\""
		}
		my $product_info = $cat_tree->productlist_html();
		$product_html = <<END;
	<td${product_colspan} class="CatalogChartProduct">${product_info}</td>
</tr>
END
	}
	

	my $out = <<END;
<table border="0" cellspacing="0" cellpadding="0" class="CatalogChartTable">
<tr class="Header">
$chart_html
$product_html
END
	$out =~ s/<tr>\n+$//gis;
	$out .= <<END;
</table>
<br>
END
	return $out;
}
# Figure out table row class
sub getRowClass{
	my ($this) = @_;
	my %class = (
		1=>"RowEven",
		0=>"RowOdd",
	);
	# Set a default value for $row_class. 
	# $row_class is used to alternate css style.
	if (!$row_class){
		$row_class = 1;
	}
	# Alternate table row class
	my $flag = 1;
	if ($row_class % 2 == 1){
		$flag = 0;
	}
	$row_class += 1;
	return " class=\"$class{$flag}\"";
}
# Alternating table row css class
sub applyRowStyle{
	# $node: tree node
	# $depth: node depth
	# $out: html buffer
	my ($this,$node,$depth,$out) = @_;
	# Don't apply row style if the depth is 1
	return $out if $node->depth() < 2;
	if ($out =~ /<tr>\n+$/gis){
		$out =~ s/<tr>\n+$//gis;
		my $node_attrib;
		# Attribute for top level categories
		if ($node->depth() == 2){
			# Main categories
			$node_attrib = " class=\"Header\"";
		# Attribute for sub categories
		}else{
			# Sub categories
			$node_attrib = $this->getRowClass();
		}
		$out .= <<END;
<tr${node_attrib}>
END
	}
	return $out;
}
# CSS class for category title
sub getTitleClass{
	my ($this,$node) = @_;
	return "CatalogChartTitle".($node->depth() - 2);
}
# Create a chart using the tree structure
sub createChart{
	# $node: tree node
	# $depth: tree depth
	my ($this,$node,$depth) = @_;	
	my $out = "\n";
	# Apply h3 for top level categories
	if ($node->depth() == 2){
		my $node_colspan = $this->getCellColSpan($node,$this->{max_depth});
		my $node_attrib = $node_colspan ? " colspan=\"$node_colspan\"" : "";
		my $node_title = $node->Title();
		$out .= <<END;
	<td${node_attrib}><h3>${node_title}</h3></td>
</tr>
<tr>
END
		$row_class = 1;
	}else{
		# Display sub categories
		if ($node->prev()){
			my $node_class = $this->getTitleClass($node);
			my $node_rowspan;
			# Get row span for a node
			if ($node->RowSpan()){
				my $rowspan = $node->RowSpan();
				$node_rowspan = $rowspan > 1 ? " rowspan=\"$rowspan\"" : "";
			}
			my $node_title = $node->Title();
			$out .= <<END;
	<td class="$node_class"${node_rowspan}>$node_title</td>
END
		}
	
	}
	# Output child nodes
	foreach my $nod(@{$node->next()}){
		if ($nod->next()){
			$out = $this->applyRowStyle($nod,$nod->depth(),$out);
			# Recursively generate output for child nodes
			$out .= $this->createChart($nod,$depth);
		}			
	}
	# We've reached the last node
	if (!$node->ChildCount()){
		# Output current category's(node) product information
		my $product_cell_attrib = " valign=\"top\" align=\"center\"";
		my $product_cell_colspan = $this->getColSpan($node,$depth);
		my $product_html = $node->productlist_html() || "n/a";
		my ($prev,$childnode) = $node->GetParentWithChildNode();		
		my $cell_row_content;
		# Stack previous category's product information under the current category(node)
		if ($prev){
			my @children = @{$prev->next()};
			my $arrayindex = scalar(@children) - 1;
			# We display the product information after all sub categories
			if ($prev->HasProduct() && $children[$arrayindex]->id() == $childnode->id()){
				my $child_cell_colspan = $this->getColSpanRecursive($prev,$depth);
				my $child_cell_html = $prev->productlist_html();
				$cell_row_content .= <<END;
<tr>
	<td${product_cell_attrib}${child_cell_colspan} class="CatalogChartProduct">${child_cell_html}</td>
</tr>
END
			}
		}	
		$out .= <<END;
	<td${product_cell_attrib}${product_cell_colspan} class="CatalogChartProduct">${product_html}</td>
</tr>
$cell_row_content
<tr>
END
	}
	return $out;
}
# Calculate the colspan for a parent node backward.
# (going from the last child node all the way to the parent)
# See henderson's application chart for more details
sub getColSpanRecursive{
	# $node: tree node
	# $depth: branch depth
	my ($this,$node,$depth) = @_;
	return if !$node || !$depth;
	my $prev = $node->prev();
	if (!$node->HasProduct()){
		my $prev = $node->prev();
		# We stop the recursive routine when we've reached the parent node
		return if !$prev;
		$this->getColSpan($prev,$depth);
	}else{
		# If a node has a product, we display the product under these conditions:
		# 1. display the product cell in another column if the node is the last node in the tree
		# 2. display the product cell in another row if the node isn't the last node in the tree
		my $span = $depth - $node->depth() + 1;
		return if $span < 2;
		return " colspan=\"$span\"";
	}
}
# Get column span
sub getCellColSpan{
	# $node: tree node
	# $depth: branch depth
	my ($this,$node,$depth) = @_;
	return if !$node || !$depth;
	my $colspan = $depth - $node->depth() + 1;
	return $colspan;
}
# Get column span
sub getColSpan{
	# $node: tree node
	# $depth: branch depth
	my ($this,$node,$depth) = @_;
	return if !$node || !$depth;
	my $span = $this->getCellColSpan($node,$depth);
	# Only output the colspan tag if the span value is bigger than 1
	return if $span < 2;
	return " colspan=\"$span\"";
}

#-------------------------------------------------------------------------------------

# Generate product link(in html format) for the application chart
sub productList{
	# $products: an array of products
	my ($this,$products) = @_;
	my @links;
	foreach my $product(@$products){
		my $prod_url = $this->html_productUrl($product);
		my $prod_target;
		if ($this->showProductNewWindow()){
			$prod_target = " target=\"_blank\"";
		}
		my $prod_code = $product->{code} ? $product->{code} : "n/a";
		# Construct the link
		my $prod_link = "<a href=\"$prod_url\"${prod_target}>$prod_code</a>";
		push(@links,$prod_link);
	}
	return join(", ",@links) || "n/a";
}
# Get default category type
sub getDefaultCategoryType{
	my ($this) = @_;
	return "catalog";
}

#----------------------------------------------------------------------
sub clean_cache{
	$share{catalog}{cache} = undef;
	delete $share{catalog}{cache};
	$share{catalog}{catalog_cache} = 0;
	return;
}
# Clear catalog cache
END {
	&clean_cache();
}
#----------------------------------------------------------------------
1;
